#!/bin/sh
LOGFILE=/tmp/samurai-graph-$$.log

# set directory
CURRENT_PATH=`pwd`
SCRIPT_PATH=`dirname $0`
CONTRIB_PATH=$SCRIPT_PATH/..
TOOLS_PATH=$SCRIPT_PATH/../../tools
FREEHEP_PATH=$SCRIPT_PATH
FREEHEP_EXPORT_PATH=$FREEHEP_PATH/freehep-export
FREEHEP_VECTORGRAPHICS_PATH=$FREEHEP_PATH/vectorgraphics/src

# function for getting absolute directory
get_absolute_dir() {
    current_dir=`pwd`
    cd "$1"
    absolute_dir=`pwd`
    cd $current_dir
    echo "$absolute_dir"
}

# set absolute directory
SCRIPT_PATH=`get_absolute_dir $SCRIPT_PATH`
CONTRIB_PATH=`get_absolute_dir $CONTRIB_PATH`
TOOLS_PATH=`get_absolute_dir $TOOLS_PATH`
FREEHEP_PATH=`get_absolute_dir $FREEHEP_PATH`
FREEHEP_EXPORT_PATH=`get_absolute_dir $FREEHEP_EXPORT_PATH`
FREEHEP_VECTORGRAPHICS_PATH=`get_absolute_dir $FREEHEP_VECTORGRAPHICS_PATH`

# set done file
DONEFILE=$CONTRIB_PATH/_build_freehep_done

# set maven
MAVEN_HOME=$TOOLS_PATH/maven/apache-maven-2.2.1
MAVEN_HOME=`get_absolute_dir $MAVEN_HOME`
MVN=$MAVEN_HOME/bin/mvn
build_freehep () {
  cd $SCRIPT_PATH
  if [ ! -f $DONEFILE ]; then
  
    echo -n "[freehep] creating freehep-export jar file... "
    cd $FREEHEP_EXPORT_PATH
    $MVN > $LOGFILE 2>&1
    if [ $? != 0 ]; then
      echo error
      cat $LOGFILE
      rm -f $LOGFILE
      exit 1
    else
      echo done
    fi
  
    echo -n "[freehep] creating vectorgraphics jar files... "
    cd $FREEHEP_VECTORGRAPHICS_PATH
    $MVN  install assembly:attached > $LOGFILE 2>&1
    if [ $? != 0 ]; then
      echo error
      cat $LOGFILE
      rm -f $LOGFILE
      exit 1
    else
      echo done
    fi
  
    touch $DONEFILE
    rm -f $LOGFILE
  fi
  cd $CURRENT_PATH
}

clean_freehep () {
  cd $SCRIPT_PATH
  rm -f $DONEFILE
  
  echo -n "[freehep] deleting freehep-export jar file ..."
  cd $FREEHEP_EXPORT_PATH
  $MVN clean > $LOGFILE 2>&1
  echo -n "[freehep] deleting vectorgraphics jar files ..."
  cd $FREEHEP_VECTORGRAPHICS_PATH
  $MVN clean > $LOGFILE 2>&1
  
  if [ $? != 0 ]; then
    echo error
    cat $LOGFILE
    rm -f $LOGFILE
    exit 1
  else
    rm -f $LOGFILE
    echo done
  fi
  cd $CURRENT_PATH
}

case "$1" in
rebuild)
  clean_freehep
  build_freehep
  ;;
build)
  build_freehep
  ;;
clean)
  clean_freehep
  ;;
*)
  echo "Usage: ${0##*/}: { build | rebuild | clean }" 2>&1
  exit 65
  ;;
esac

exit 0
