package jp.riken.brain.ni.samuraigraph.application;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JOptionPane;
import javax.swing.UIManager;

/**
 * The main class of this application.
 */
public class SGDrawingServer implements SGIApplicationConstants,
        SGIApplicationTextConstants {

    // the name of look and feel
    private static String mLookAndFeel = null;

    /*
    // the name of a data file
    private static String mDataFileName = null;

    // the name of a property file
    private static String mPropertyFileName = null;

    // the name of a data set file
    private static String mDataSetFileName = null;

    // the name of a script file
    private static String mScriptFileName = null;

    // a flag whether to use the command input
    private static boolean mCommandFlag = false;

    // a flag whether to start in developer mode
    private static boolean mDevFlag = false;
    */

    /**
     * main thread
     */
    private static SGMainFunctions mAppMain = null;

    private static SGApplicationProperties mAppProp = null;

    /**
     * The main method.
     * 
     * @param args
     *            An array of arguments
     */
    public static void main(String args[]) {
/*
    	StringBuffer sb = new StringBuffer();
    	for (int ii = 0; ii < args.length; ii++) {
    		sb.append(args[ii]);
    		sb.append(", ");
    	}
        JOptionPane.showOptionDialog(null,
                sb.toString(), "Input",
                JOptionPane.DEFAULT_OPTION,
                JOptionPane.INFORMATION_MESSAGE, null, null, null);
*/
    	
        // interpret command lines
        Map<String, Object> paramMap = interpretCommands(args);

        // load dynamic constant values from property file
        mAppProp = new SGApplicationProperties();
        if (mAppProp.getStatus() == false) {
            System.exit(1); // fatal error
        }

        // set look and feel
        if (setLookAndFeel() == false) {
            System.exit(1); // fatal error
        }

        // create a Main thread
        SGMainFunctions m = new SGMainFunctions(mAppProp, paramMap);
        
        /*
        // create a Main thread
        SGMainFunctions m = null;
        if (mPropertyFileName != null) {
            m = new SGMainFunctions(mAppProp, mCommandFlag, mPropertyFileName, FILETYPE_PROPERTY);
        } else if (mDataSetFileName != null) {
            m = new SGMainFunctions(mAppProp, mCommandFlag, mDataSetFileName, FILETYPE_DATASET);
        } else if (mDataFileName != null) {
            m = new SGMainFunctions(mAppProp, mCommandFlag, mDataFileName, FILETYPE_DATA);
        } else if (mScriptFileName != null) {
            m = new SGMainFunctions(mAppProp, mCommandFlag, mScriptFileName, FILETYPE_SCRIPT);
        } else {
            m = new SGMainFunctions(mAppProp, mCommandFlag);
        }
        */

        try {
            // wait till the end of Main thread
            m.waitInit();
            mAppMain = m;

            // register event handler for macos x
            MacOSXRegistration();

//            // create Connect thread
//            if (mCommandFlag) {
//                if (mScriptFileName != null) {
//                    new Connect(m, mScriptFileName);
//                } else {
//                    new Connect(m);
//                }
//            }

        } catch (Exception ex) {
            JOptionPane.showOptionDialog(null,
                    "Failed to start up Samurai Graph.", TITLE_ERROR,
                    JOptionPane.DEFAULT_OPTION,
                    JOptionPane.WARNING_MESSAGE, null, null, null);
        }
    }

    // interpret the command lines
    private static Map<String, Object> interpretCommands(String[] commands) {

    	Map<String, Object> paramMap = new HashMap<String, Object>();
    	
    	String propertyFileName = null;
    	String dataSetFileName = null;
    	String scriptFileName = null;
    	boolean commandFlag = false;
    	boolean devFlag = false;
    	List<String> dataFileNameList = new ArrayList<String>();
        for (int ii = 0; ii < commands.length; ii++) {
        	if (commands[ii].equals("-prop") || commands[ii].equals("-p")) {
                if (ii < commands.length - 1) {
                    propertyFileName = commands[ii + 1];
                    ii++;
                }
            } else if (commands[ii].equals("-dataset") || commands[ii].equals("-a")) {
                if (ii < commands.length - 1) {
                	dataSetFileName = commands[ii + 1];
                    ii++;
                }
            } else if (commands[ii].equals("-i")) {
            	commandFlag = true;
            } else if (commands[ii].equals("-s")) {
                if (ii < commands.length - 1) {
                	commandFlag = true;
            		scriptFileName = commands[ii + 1];
                    ii++;
                }
            } else if (commands[ii].equals("-dev")) {
            	devFlag = true;
            } else {
            	dataFileNameList.add(commands[ii]);
            }
        }
        
        // set values to the map
        String fileName = null;
        int fileType = FILETYPE_NONE;
        if (propertyFileName != null) {
        	fileName = propertyFileName;
        	fileType = FILETYPE_PROPERTY;
        }
        if (dataSetFileName != null) {
        	fileName = dataSetFileName;
        	fileType = FILETYPE_DATASET;
        }
        if (dataFileNameList.size() != 0) {
        	fileName = dataFileNameList.get(0);
        	fileType = FILETYPE_DATA;
        }
        if (scriptFileName != null) {
        	fileName = scriptFileName;
        	fileType = FILETYPE_SCRIPT;
        }
        if (fileName != null) {
            paramMap.put(SGMainFunctions.KEY_FILE_NAME, fileName);
            paramMap.put(SGMainFunctions.KEY_FILE_TYPE, Integer.valueOf(fileType));
        }
        paramMap.put(SGMainFunctions.KEY_COMMAND_MODE_FLAG, Boolean.valueOf(commandFlag));
        paramMap.put(SGMainFunctions.KEY_DEVELOPER_MODE_FLAG, Boolean.valueOf(devFlag));

        /*
        // set to the attributes
        if (propertyFileName != null) {
        	mPropertyFileName = propertyFileName;
        }
        if (dataSetFileName != null) {
        	mDataSetFileName = dataSetFileName;
        }
        if (scriptFileName != null) {
        	mScriptFileName = scriptFileName;
        }
        if (dataFileNameList.size() != 0) {
        	mDataFileName = dataFileNameList.get(0);
        }
        mCommandFlag = commandFlag;
        mDevFlag = devFlag;
        */
        
        return paramMap;
    }
    
//    /**
//     * Returns the class name of look and feel.
//     * 
//     * @param name -
//     *            the short name of look and feel
//     * @return the class name of look and feel
//     */
//    private static String getLookAndFeelClassName(String name) {
//        if (name == null) {
//            throw new IllegalArgumentException("name==null");
//        }
//        UIManager.LookAndFeelInfo[] lafArray = UIManager
//                .getInstalledLookAndFeels();
//        for (int ii = 0; ii < lafArray.length; ii++) {
//            String name_ = lafArray[ii].getName().toLowerCase();
//            if (name_.equals(name.toLowerCase())) {
//                mLookAndFeel = lafArray[ii].getClassName();
//            }
//        }
//        return null;
//    }

    // set look and feel
    private static boolean setLookAndFeel() {
        try {
            if (mLookAndFeel == null) {
                // avoid NPE at non en_US locale for JRE 1.6
                UIManager.getInstalledLookAndFeels();
                UIManager.setLookAndFeel(UIManager
                        .getSystemLookAndFeelClassName());
            } else {
                UIManager.setLookAndFeel(mLookAndFeel);
            }
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    private static void MacOSXRegistration() {
        boolean isMacOSX = (System.getProperty("os.name").toLowerCase()
                .startsWith("mac os x"));
        if (isMacOSX) {
            try {
                Class osxAdapter = Class
                        .forName("jp.riken.brain.ni.samuraigraph.platform.macosx.SGMacOSXAdapter");
                Method registerMethod = osxAdapter.getMethod(
                        "registerMacOSXApplication", (Class[]) null);
                if (registerMethod != null) {
                    registerMethod.invoke(osxAdapter, (Object[]) null);
                }
            } catch (NoClassDefFoundError e) {
                // This will be thrown first if the OSXAdapter is loaded on a
                // system without the EAWT
                // because OSXAdapter extends ApplicationAdapter in its def
                System.err
                        .println("This version of Mac OS X does not support the Apple EAWT.  Application Menu handling has been disabled ("
                                + e + ")");
            } catch (ClassNotFoundException e) {
                // This shouldn't be reached; if there's a problem with the
                // OSXAdapter we should get the
                // above NoClassDefFoundError first.
                System.err
                        .println("This version of Mac OS X does not support the Apple EAWT.  Application Menu handling has been disabled ("
                                + e + ")");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * quit handler for application adapter
     * 
     */
    public static boolean quitHandler() {
        if (mAppMain == null) {
            return false;
        }
        mAppMain.exit();
        return true;
    }

    /**
     * about handler for application adapter
     * 
     */
    public static boolean aboutHandler() {
        if (mAppMain == null) {
            return false;
        }
        return mAppMain.showAboutDialog(null);
    }

    /**
     * openFile handler for application adapter
     * 
     */
    public static boolean openFileHandler(final String fname) {
        if (mAppMain == null) {
            return false;
        }
        return mAppMain.openFile(null, fname);
    }

//    /**
//     * @author okumura
//     */
//    static class Connect extends Thread {
//
//        /**
//         * Main object.
//         */
//        private SGMainFunctions mMain;
//
//        /**
//         * The script file name.
//         */
//        private String mScriptFileName = null;
//
//        /**
//         * Create a thread object with a server socket.
//         * 
//         * @param main
//         *            the main class object
//         */
//        private Connect(SGMainFunctions main) {
//            super();
//            this.mMain = main;
//            this.start();
//        }
//
//        /**
//         * Create a thread object with a server socket.
//         * 
//         * @param main
//         *            the main class object
//         * @param script
//         *            the script file name
//         */
//        private Connect(SGMainFunctions main, final String script) {
//            super();
//            this.mMain = main;
//            this.mScriptFileName = script;
//            this.start();
//        }
//
//        // the prompt
//        private static final String PROMPT = "$ ";
//        
//        // a symbol for the file input stream
//        private static final String FILE_INPUT = "<<";
//
//        // the maximum number for the recursion of file input
//        private static final int FILE_RECURSION_DEPTH_MAX = 10;
//
//        // the header for a comment line
//        private static final String COMMENT_HEADER = "#";
//
//        // the header for a comment line
//        private static final String COMMENT_HEADER_2 = "//";
//
//        // a text string that represents the start of a block comment
//        private static final String BLOCK_COMMENT_START = "/*";
//
//        // a text string that represents the end of a block comment
//        private static final String BLOCK_COMMENT_END = "*/";
//        
//        // a flag whether the current position is in a block comment
//        private boolean mBlockComment = false;
//
//        private void readRecursively(BufferedReader br, BufferedWriter bw,
//                int depth, final boolean showPrompt, final boolean inFile
//                ) throws IOException {
//
//            // check whether the recursion depth is within range
//            depth++;
//            if (depth > FILE_RECURSION_DEPTH_MAX) {
//                bw.write("Recursion is too deep.\n");
//                bw.flush();
//                return;
//            }
//
//            // infinite loop as a server process
//            while (true) {
//            	
//                if (showPrompt) {
//                    // display the prompt
//                    bw.write(PROMPT);
//                    bw.flush();
//                }
//
//                // get a line
//                String line = br.readLine();
//                if (line == null) {
//                    break;
//                }
//
//                // checks whether a dialog is open
//            	if (this.mMain.isDialogOpen()) {
//                    bw.write("Dialog is open.\n");
//                    bw.flush();
//            		continue;
//            	}
//            	
//            	// checks whether a text field is shown
//            	if (this.mMain.isTextFieldShown()) {
//                    bw.write("Text field is shown.\n");
//                    bw.flush();
//            		continue;
//            	}
//
//                // trim the line
//                String tLine = line.trim();
//
//                // skip an empty line
//                if (tLine.length() == 0) {
//                    continue;
//                }
//
//                // skip the comment line
//                if (tLine.startsWith(COMMENT_HEADER) || tLine.startsWith(COMMENT_HEADER_2)) {
//                    continue;
//                }
//
//                // start or end a block comment only when the file input sream is opened
//                if (inFile) {
//                    if (tLine.startsWith(BLOCK_COMMENT_START)) {
//                    	this.mBlockComment = true;
//                    	continue;
//                    } else if (tLine.startsWith(BLOCK_COMMENT_END)) {
//                    	this.mBlockComment = false;
//                    	continue;
//                    }
//
//                    // skip the current line if it is in a block comment
//                    if (this.mBlockComment) {
//                    	continue;
//                    }
//                }
//                
//                // interpret as a file path
//                if (tLine.startsWith(FILE_INPUT)) {
//                    String rest = tLine.substring(FILE_INPUT.length());
//                    String trimmed = rest.trim();
//                    File file = new File(trimmed);
//                    if (!file.exists()) {
//                        String errmsg = getFileNotFoundString(file);
//                        bw.write(errmsg);
//                        bw.flush();
//                        continue;
//                    }
//                    BufferedReader br2 = new BufferedReader(
//                            new FileReader(file));
//                    readRecursively(br2, bw, depth, false, true);
//                    br2.close();
//                    
//                    continue;
//                }
//
//                // parse the line and execute the command
//                final int ret = this.mMain.exec(tLine);
//                
//                // output the status
//                StringBuffer status = new StringBuffer();
//                if (ret == SGMainFunctions.STATUS_FAILED) {
//                    status.append("failed: ");
//                } else if (ret == SGMainFunctions.STATUS_NOT_FOUND) {
//                    status.append("not found: ");
//                } else if (ret == SGMainFunctions.STATUS_SUCCEEDED) {
//                    status.append("succeeded: ");
//                } else if (ret == SGMainFunctions.STATUS_PARTIALLY_FAILED) {
//                	status.append("partially failed: ");
//                }
//                status.append(tLine);
//                status.append('\n');
//                bw.write(status.toString());
//                bw.flush();
//            }
//        }
//
//        /**
//         * Runs the thread.
//         */
//        public void run() {
//            // standard output stream
//            BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(
//                    System.out));
//            
//            // read the input script file
//            if (this.mScriptFileName != null) {
//                File sf = new File(this.mScriptFileName);
//                BufferedReader brs = null;
//                try {
//                    brs = new BufferedReader(
//                            new FileReader(sf));
//                    try {
//                        // read input recursively
//                        this.readRecursively(brs, bw, 0, false, true);
//                    } catch (IOException ex) {
//                    }
//                } catch (FileNotFoundException e) {
//                    String errmsg = getFileNotFoundString(sf);
//                    try {
//                        bw.write(errmsg);
//                        bw.flush();
//                    } catch (IOException e1) {
//                    }
//                } finally {
//                    if (brs != null) {
//                        try {
//                            brs.close();
//                        } catch (IOException e) {
//                        }
//                    }
//                }
//            }
//            
//            // standard input stream
//            BufferedReader br = new BufferedReader(new InputStreamReader(
//                    System.in));
//            try {
//                // read input recursively
//                this.readRecursively(br, bw, 0, true, false);
//            } catch (IOException ex) {
//                return;
//            }
//        }
//    }
//
//    private static String getFileNotFoundString(File file) {
//        StringBuffer sb = new StringBuffer();
//        sb.append("file not found: ");
//        sb.append(file.getPath());
//        sb.append('\n');
//        return sb.toString();
//    }
    
}
