package jp.riken.brain.ni.samuraigraph.application;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Frame;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.prefs.Preferences;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import jp.riken.brain.ni.samuraigraph.application.SGDataCreator.FileColumn;
import jp.riken.brain.ni.samuraigraph.base.SGBufferedFileReader;
import jp.riken.brain.ni.samuraigraph.base.SGData;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfoSet;
import jp.riken.brain.ni.samuraigraph.base.SGDialog;
import jp.riken.brain.ni.samuraigraph.base.SGDrawingWindow;
import jp.riken.brain.ni.samuraigraph.base.SGFigure;
import jp.riken.brain.ni.samuraigraph.base.SGIConstants;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElement;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementAxis;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementGraph;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementLegend;
import jp.riken.brain.ni.samuraigraph.base.SGIProgressControl;
import jp.riken.brain.ni.samuraigraph.base.SGIPropertyFileConstants;
import jp.riken.brain.ni.samuraigraph.base.SGIRootObjectConstants;
import jp.riken.brain.ni.samuraigraph.base.SGPlotTypeConstants;
import jp.riken.brain.ni.samuraigraph.base.SGPluginsQueryMessage;
import jp.riken.brain.ni.samuraigraph.base.SGProperties;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyMap;
import jp.riken.brain.ni.samuraigraph.base.SGUserProperties;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityText;
import jp.riken.brain.ni.samuraigraph.data.SGArrayData;
import jp.riken.brain.ni.samuraigraph.data.SGArrayDataDuplicationDialog;
import jp.riken.brain.ni.samuraigraph.data.SGDataColumn;
import jp.riken.brain.ni.samuraigraph.data.SGDataDuplicationDialog;
import jp.riken.brain.ni.samuraigraph.data.SGDataTypeConstants;
import jp.riken.brain.ni.samuraigraph.data.SGDataUtility;
import jp.riken.brain.ni.samuraigraph.data.SGDefaultColumnTypeUtility;
import jp.riken.brain.ni.samuraigraph.data.SGIDataColumnNameConstants;
import jp.riken.brain.ni.samuraigraph.data.SGIDataColumnTypeConstants;
import jp.riken.brain.ni.samuraigraph.data.SGISXYTypeData;
import jp.riken.brain.ni.samuraigraph.data.SGISXYTypeMultipleData;
import jp.riken.brain.ni.samuraigraph.data.SGISXYZTypeData;
import jp.riken.brain.ni.samuraigraph.data.SGIVXYTypeData;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFData;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFDataDuplicationDialog;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFFile;
import jp.riken.brain.ni.samuraigraph.data.SGSXYMultipleData;
import jp.riken.brain.ni.samuraigraph.data.SGSXYMultipleDimensionNetCDFData;
import jp.riken.brain.ni.samuraigraph.data.SGSXYMultipleVariableNetCDFData;
import jp.riken.brain.ni.samuraigraph.data.SGSXYZData;
import jp.riken.brain.ni.samuraigraph.data.SGSXYZNetCDFData;
import jp.riken.brain.ni.samuraigraph.data.SGTextDataFile;
import jp.riken.brain.ni.samuraigraph.data.SGVXYData;
import jp.riken.brain.ni.samuraigraph.data.SGVXYNetCDFData;
import jp.riken.brain.ni.samuraigraph.data.SGVariable;
import jp.riken.brain.ni.samuraigraph.figure.SGIFigureTypeConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGXYFigure;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import ucar.nc2.Attribute;
import ucar.nc2.NetcdfFile;
import foxtrot.Task;
import foxtrot.Worker;

/**
 * The main thread.
 */
class SGMainFunctions implements ActionListener, SGIConstants,
        SGIUpgradeConstants, SGIApplicationCommandConstants,
        SGIApplicationConstants, SGIPropertyFileConstants,
        SGIPreferencesConstants, SGIApplicationTextConstants,
        SGIRootObjectConstants, SGIImageConstants, SGIArchiveFileConstants,
        SGIDataColumnTypeConstants, WindowListener {

    // only used for debug
    static boolean USE_FOXTROT = true;

    /**
     * Application Properties
     */
    SGApplicationProperties mAppProp = null;

    /**
     * Data set manager
     */
    SGDataSetManager mDataSetManager;

    /**
     * Proxy manager.
     */
    SGProxyManager mProxyManager;

    /**
     * Upgrade manager.
     */
    SGUpgradeManager mUpgradeManager;

    /**
     * Data creator.
     */
    SGDataCreator mDataCreator;

    /**
     * Window manager
     */
    SGWindowManager mWindowManager = null;

    /**
     * Clip Board manager.
     */
    SGClipBoardManager mClipBoardManager = null;

    /**
     * Property manager
     */
    SGCommandManager mPropertyManager = null;

    /**
     * Property file manager.
     */
    SGPropertyFileManager mPropertyFileManager = null;

    /**
     * Figure creator.
     */
    SGFigureCreator mFigureCreator = null;
    
    /**
     * Plugin manager.
     */
    SGPluginManager mPluginManager = null;

    /**
     * Initializer.
     */
    private Initializer mInit = null;

    /**
     * The standard output stream.
     */
    private BufferedWriter mStdoutWriter = null;
    
    /**
     * The standard input stream.
     */
    private BufferedReader mStdinReader = null;
    
    private static final int DATA_ADDITION_TOOL_BAR = 0;
    
    private static final int DATA_ADDITION_DRAG_AND_DROP = 1;

    static final String KEY_FILE_TYPE = "fileType";
    
    static final String KEY_FILE_NAME = "fileName";
    
    static final String KEY_COMMAND_MODE_FLAG = "commandMode";
    
    static final String KEY_DEVELOPER_MODE_FLAG = "developerMode";

//    /**
//     * Create a thread object.
//     * 
//     */
//    public SGMainFunctions(final SGApplicationProperties prop, final boolean command) {
//        super();
//        this.mAppProp = prop;
//        this.mInit = new Initializer(command);
//        this.mInit.start();
//    }
//
//    /**
//     * Create a thread object.
//     * 
//     */
//    public SGMainFunctions(final SGApplicationProperties prop,
//    		final boolean command, final String path, final int type) {
//        super();
//        this.mAppProp = prop;
//        this.mInit = new Initializer(command, path, type);
//        this.mInit.start();
//    }

    /**
     * Create a thread object.
     * 
     */
    public SGMainFunctions(final SGApplicationProperties prop, final Map<String, Object> paramMap) {
        super();
        this.mAppProp = prop;
        this.mInit = new Initializer(paramMap);
        this.mInit.start();
    }
    
    /**
     * SGMainFunctions :: Initializer class
     * 
     * @author okumura
     */
    private class Initializer extends Thread {
    	
    	private boolean mCommandModeFlag = false;
    	
        private String mStartupFilePath = null;

        private int mStartupFileType = FILETYPE_NONE;

//        private Initializer(final boolean command) {
//        	super();
//        	this.mCommand = command;
//        }
//
//        private Initializer(final boolean command, final String path, final int type) {
//        	this(command);
//            this.mStartupFilePath = path;
//            this.mStartupFileType = type;
//        }
        
        private Initializer(final Map<String, Object> paramMap) {
        	super();
        	Set<String> keys = paramMap.keySet();
        	
        	// command mode
        	if (keys.contains(KEY_COMMAND_MODE_FLAG)) {
        		Boolean b = (Boolean) paramMap.get(KEY_COMMAND_MODE_FLAG);
        		this.mCommandModeFlag = b.booleanValue();
        	}
        	
        	// input file
        	if (keys.contains(KEY_FILE_TYPE) && keys.contains(KEY_FILE_NAME)) {
        		Integer fileType = (Integer) paramMap.get(KEY_FILE_TYPE);
        		if (fileType.intValue() != FILETYPE_NONE) {
        			String fileName = (String) paramMap.get(KEY_FILE_NAME);
        			this.mStartupFilePath = fileName;
        		}
        		this.mStartupFileType = fileType.intValue();
        	}
        	
        	// developer mode
        	boolean devMode = false;
        	if (keys.contains(KEY_DEVELOPER_MODE_FLAG)) {
        		Boolean b = (Boolean) paramMap.get(KEY_DEVELOPER_MODE_FLAG);
        		devMode = b.booleanValue();
        	}
        	
        	// set user properties
    		SGUserProperties p = SGUserProperties.getInstance();
    		p.setProperty("dev", Boolean.toString(devMode));
        }

        public void run() {
            
            // create and show the splash window
            final SGSplashWindow sw = this.createSplashWindow();

            // remove temporary files used in upgrade
            this.removeTemporaryFiles();

            if (sw == null) {
                exitApplication(1);
            }
            sw.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            sw.setProgressValue(0.0f);
            sw.setVisible(true);
            // sw.show();
            
            // create instances in attributes
            SGMainFunctions.this.mDataCreator = new SGDataCreator();
            sw.setProgressValue(0.15f);
            SGMainFunctions.this.mPropertyFileManager = new SGPropertyFileManager(SGMainFunctions.this);
            SGMainFunctions.this.mDataSetManager = new SGDataSetManager(SGMainFunctions.this,
            		sw, 0.15f, 0.50f);
            sw.setProgressValue(0.50f);
            SGMainFunctions.this.mProxyManager = new SGProxyManager();
            sw.setProgressValue(0.60f);
            SGMainFunctions.this.mUpgradeManager = new SGUpgradeManager(
                    SGMainFunctions.this.mProxyManager,
                    SGMainFunctions.this.mAppProp);
            SGMainFunctions.this.mWindowManager = new SGWindowManager(SGMainFunctions.this,
            		sw, 0.60f, 0.80f);
            sw.setProgressValue(0.80f);
            SGMainFunctions.this.mPropertyManager = new SGCommandManager(SGMainFunctions.this);
            SGMainFunctions.this.mClipBoardManager = new SGClipBoardManager(SGMainFunctions.this);

            // create the figure element
            SGMainFunctions.this.mFigureCreator = new SGFigureCreator(true);
            if (!SGMainFunctions.this.mFigureCreator.validateClasses()) {
                exitApplication(1);
            }
            sw.setProgressValue(0.9f);
            
            SGMainFunctions.this.mPluginManager = new SGPluginManager(SGMainFunctions.this);
            boolean result = SGMainFunctions.this.mPluginManager.loadPlugins(SGIApplicationConstants.APPLICATION_PLUGIN_DIRECTORY);
            if (result==false) {
                String filename = SGMainFunctions.this.mPluginManager.getFirstExceptionJarFilename();
                SGUtility.showErrorMessageDialog(null, "Plugins load error.\n"+filename, TITLE_ERROR);
            }

            sw.setProgressValue(1.0f);

            final SGDrawingWindow wnd = SGMainFunctions.this.mWindowManager
                    .createNewWindow();
            SGMainFunctions.this.mWindowManager.setCurrentWindow(wnd);
            wnd.setVisible(true);

            // hide the splash window
            sw.setVisible(false);
            // sw.hide();
            sw.setCursor(Cursor.getDefaultCursor());

            // take the procedure below to get rid of warning messages as follows:
            // "log4j:WARN No appenders could be found for logger (ucar.nc2.NetcdfFile)."
            // "log4j:WARN Please initialize the log4j system properly."
            org.apache.log4j.BasicConfigurator.configure();
            org.apache.log4j.Logger logger = org.apache.log4j.Logger.getRootLogger();
            logger.setLevel(org.apache.log4j.Level.OFF);
            
            if (this.mStartupFileType == FILETYPE_PROPERTY) {
                final File f = new File(this.mStartupFilePath);

                try {
                    SwingUtilities.invokeAndWait(new Runnable() {
                        public void run() {
                            SGMainFunctions.this.mPropertyFileManager
                                    .showMultiDataFileChooserDialog(f, wnd);
                        }
                    });
                } catch (InterruptedException e) {
                } catch (InvocationTargetException e) {
                }
                
            } else if (this.mStartupFileType == FILETYPE_DATASET) {
                // disable window
                wnd.setWaitCursor(true);

                final File f = new File(this.mStartupFilePath);
                if (f.exists()) {
                    try {
                        SwingUtilities.invokeAndWait(new Runnable() {
                            public void run() {
                                if (SGMainFunctions.this.mDataSetManager.loadDataSetFromEventDispatchThread(wnd, f) == false) {
                                    SGUtility.showErrorMessageDialog(wnd, MSG_DATA_SET_FILE_INVALID,
                                            TITLE_ERROR);
                                }
                            }
                        });
                    } catch (InterruptedException e) {
                    } catch (InvocationTargetException e) {
                    }
                } else {
                    // file not found
                    SGUtility.showErrorMessageDialog(wnd, MSG_FILE_OPEN_FAILURE,
                            TITLE_FILE_OPEN_FAILURE);
                }
                
                // enable window
                wnd.setWaitCursor(false);
            } else if (this.mStartupFileType == FILETYPE_DATA) {
                
                File f = new File(this.mStartupFilePath);
                final ArrayList<File> fList = new ArrayList<File>();
                fList.add(f);
                
                try {
                    SwingUtilities.invokeAndWait(new Runnable() {
                        public void run() {
                            if (SGApplicationUtility.isNetCDFFile(mStartupFilePath)) {
                                // add netCDF data
                                onNetCDFDataFilesDropped(null, wnd, fList);
                            } else {
                                // add text data
                                onTextDataFilesDropped(null, wnd, fList);
                            }
                        }
                    });
                } catch (InterruptedException e) {
                } catch (InvocationTargetException e) {
                }
                
            } else if (this.mStartupFileType == FILETYPE_SCRIPT) {
                // setup standard output stream
                mStdoutWriter = new BufferedWriter(new OutputStreamWriter(
                        System.out));
                
                // read the input script file
                readCommand(this.mStartupFilePath);
                
                // setup standard input stream
                mStdinReader = new BufferedReader(new InputStreamReader(
                        System.in));
                try {
                    // read input recursively
                	startReadingInput();
                } catch (IOException ex) {
                    return;
                }
            }
            
            if (this.mCommandModeFlag && this.mStartupFileType != FILETYPE_SCRIPT) {
                // setup standard output stream
                mStdoutWriter = new BufferedWriter(new OutputStreamWriter(
                        System.out));
        		
                // setup standard input stream
                mStdinReader = new BufferedReader(new InputStreamReader(
                        System.in));
                try {
                    // read input recursively
                	startReadingInput();
                } catch (IOException ex) {
                    return;
                }
            }

            // update the preferences
            this.updatePreferences();

            // decide whether to upgrade
            SGMainFunctions.this.mUpgradeManager.upgradeWithCheckDate(wnd);
        }

        /**
         * create splash window
         * 
         * @return
         */
        private SGSplashWindow createSplashWindow() {
            // set the image and the text
            Class inClass = getClass();
            String name = RESOURCES_DIRNAME + "Splash.png";
            URL url = inClass.getResource(name);
            Image image = Toolkit.getDefaultToolkit().getImage(url);
            if (image == null) {
                return null;
            }
            SGSplashWindow sw = new SGSplashWindow(image,
                    SGMainFunctions.this.mAppProp.getVersionString());
            return sw;
        }

        /**
         * update registry for update check cycle
         */
        private void updatePreferences() {
            Preferences pref = Preferences.userNodeForPackage(this.getClass());

            // update the upgrade cycle at the first installation
            final int cycle = pref.getInt(PREF_KEY_UPGRADE_CYCLE, -1);
            if (cycle == -1) {
                pref.putInt(PREF_KEY_UPGRADE_CYCLE, DEFAULT_UPGRADE_CYCLE);
            }

            // update the date
            // final String date = pref.get( PREF_KEY_DATE, null );
            pref.putLong(PREF_KEY_DATE, System.currentTimeMillis());

            // remove old keys
            pref.remove(PREF_KEY_MAJOR_VERSION_NUMBER);
            pref.remove(PREF_KEY_MINOR_VERSION_NUMBER);
            pref.remove(PREF_KEY_MICRO_VERSION_NUMBER);
        }

        /**
         * Remove temporary files used in upgrade.
         */
        private void removeTemporaryFiles() {
            String filename = SGApplicationUtility.getPathName(TMP_DIR, HELPER_TEMP_DIR_NAME);
            File temp = new File(filename);
            try {
                temp = temp.getCanonicalFile();
            } catch (IOException ex) {
                return;
            }
            if (temp.exists()) {
                SGApplicationUtility.deleteRecursively(temp);
            }
        }
    }

    public boolean waitInit() {
        try {
            this.mInit.join();
        } catch (InterruptedException e) {
            return false;
        }
        this.mInit = null;
        return true;
    }

    /**
     * Closes the window without confirmation.
     * 
     * @param id
     *           the window ID of a window to close
     * @param true if succeeded
     */
    public boolean closeWindowWithoutConfirmation(final int id) {
        return this.mWindowManager.closeWindowWithoutConfirmation(id);
    }

    /**
     * Execute a command.
     * 
     * @param line
     *            the command line
     * @return the status
     */
    public int exec(final String line) {
    	return this.mPropertyManager.exec(line);
    }
    
	// Checks whether a modal dialog is shown.
    boolean isDialogOpen() {
    	List<SGDrawingWindow> wndList = this.mWindowManager.getWindowList();
    	for (SGDrawingWindow wnd : wndList) {
    		if (wnd.isModalDialogShown()) {
    			return true;
    		}
    	}
    	return false;
    }

    // Checks whether a text field is shown.
    boolean closeTextField() {
    	List<SGDrawingWindow> wndList = this.mWindowManager.getWindowList();
    	for (SGDrawingWindow wnd : wndList) {
    		if (wnd.closeTextField() == false) {
    			return false;
    		}
    	}
    	return true;
    }
    
    /**
     * A wizard dialog to select the figure ID.
     */
    private SGFigureIDSelectionWizardDialog mFigureIDSelectionWizardDialog = null;

    /**
     * A wizard dialog to select a single data file.
     */
    private SGSingleDataFileChooserWizardDialog mSingleDataFileChooserWizardDilaog = null;

    /**
     * A wizard dialog to select the data type.
     */
    private SGArrayDataTypeSelectionWizardDialog mArrayDataTypeSelectionWizardDialog = null;

    /**
     * A wizard dialog to select data columns.
     */
    private SGArrayDataColumnSelectionWizardDialog mDataColumnSelectionWizardDialog = null;

    /**
     * A wizard dialog to select the data type.
     */
    private SGNetCDFDataTypeSelectionWizardDialog mNetCDFDataTypeSelectionWizardDialog = null;

    /**
     * A wizard dialog to setup the netCDF data.
     */
    private SGNetCDFDataSetupWizardDialog mNetCDFSetupWizardDialog = null;
    
    /**
     * A wizard dialog to select the plot type if the data type is scalar-xy.
     */
    private SGPlotTypeSelectionWizardDialog mPlotTypeSelectionWizardDialog = null;

    /**
     * Creates the wizard dialogs for data addition.
     * 
     * @param owner
     *            the owner of wizard dialogs
     */
    private void createDataAdditionWizardDialogs(
            final SGDrawingWindow owner) {

        // if the owner window is the same, do nothing
        if (this.mFigureIDSelectionWizardDialog != null) {
            SGDrawingWindow curOwner = this.mFigureIDSelectionWizardDialog
                    .getOwnerWindow();
            if (curOwner.equals(owner)) {
                return;
            }
        }

        // creates dialogs
        this.mFigureIDSelectionWizardDialog = new SGFigureIDSelectionWizardDialog(
                owner, true);
        this.mSingleDataFileChooserWizardDilaog = new SGSingleDataFileChooserWizardDialog(
                owner, true);
        this.mArrayDataTypeSelectionWizardDialog = new SGArrayDataTypeSelectionWizardDialog(
                owner, true);
        this.mDataColumnSelectionWizardDialog = new SGArrayDataColumnSelectionWizardDialog(
                owner, true);
        this.mNetCDFDataTypeSelectionWizardDialog = new SGNetCDFDataTypeSelectionWizardDialog(
                owner, true);
        this.mNetCDFSetupWizardDialog = new SGNetCDFDataSetupWizardDialog(
                owner, true);
        this.mPlotTypeSelectionWizardDialog = new SGPlotTypeSelectionWizardDialog(
                owner, true);

        // sets the connection between wizard dialogs
        this.mFigureIDSelectionWizardDialog.setPrevious(null);
        this.mFigureIDSelectionWizardDialog
                .setNext(this.mSingleDataFileChooserWizardDilaog);
        this.mSingleDataFileChooserWizardDilaog
                .setPrevious(this.mFigureIDSelectionWizardDialog);

        // for text data
        this.mArrayDataTypeSelectionWizardDialog.setNext(
        		this.mDataColumnSelectionWizardDialog);
        this.mDataColumnSelectionWizardDialog.setPrevious(
        		this.mArrayDataTypeSelectionWizardDialog);
        
        // for netCDF data
        this.mNetCDFDataTypeSelectionWizardDialog.setNext(
        		this.mNetCDFSetupWizardDialog);
        this.mNetCDFSetupWizardDialog.setPrevious(
        		this.mNetCDFDataTypeSelectionWizardDialog);
        
        //
        // Remaining connections are selected depending on the method of data addition
        //
        
        // sets the selected file name
        String path = this.getCurrentFileDirectory();
        this.mSingleDataFileChooserWizardDilaog.setSelectedFile(path,
                this.mCurrentDataFileName);

        // add action listener
        this.mFigureIDSelectionWizardDialog.addActionListener(this);
        this.mSingleDataFileChooserWizardDilaog.addActionListener(this);
        this.mArrayDataTypeSelectionWizardDialog.addActionListener(this);
        this.mDataColumnSelectionWizardDialog.addActionListener(this);
        this.mNetCDFDataTypeSelectionWizardDialog.addActionListener(this);
        this.mNetCDFSetupWizardDialog.addActionListener(this);
        this.mPlotTypeSelectionWizardDialog.addActionListener(this);

        // add window listener
        this.mFigureIDSelectionWizardDialog.addWindowListener(this);
        this.mSingleDataFileChooserWizardDilaog.addWindowListener(this);
        this.mArrayDataTypeSelectionWizardDialog.addWindowListener(this);
        this.mDataColumnSelectionWizardDialog.addWindowListener(this);
        this.mNetCDFDataTypeSelectionWizardDialog.addWindowListener(this);
        this.mNetCDFSetupWizardDialog.addWindowListener(this);
        this.mPlotTypeSelectionWizardDialog.addWindowListener(this);

        // packs the dialogs
        this.mFigureIDSelectionWizardDialog.pack();
        this.mSingleDataFileChooserWizardDilaog.pack();
    }

    /**
     * Sets up the connection of wizard dialogs.
     * 
     * @param fileType
     *            the file type (array data or netcdf)
     * @param method
     *            the method of data addition (toolbar or dad)
     */
    private void setupDataTypeSelectionWizardDialogConnection(final int fileType, 
    		final int method) {

    	// select the data type wizard dialog
    	final SGWizardDialog dataTypeDialog;
        switch (fileType) {
        case FILETYPE_DATA:
        	dataTypeDialog = this.mArrayDataTypeSelectionWizardDialog;
            break;
        case FILETYPE_NETCDF:
        	dataTypeDialog = this.mNetCDFDataTypeSelectionWizardDialog;
            break;
        default:
            throw new IllegalArgumentException("Invalid type for data file: "
                    + fileType);
        }

        // setup the connection
        switch (method) {
        case DATA_ADDITION_TOOL_BAR:
            this.mSingleDataFileChooserWizardDilaog.setNext(dataTypeDialog);
            dataTypeDialog.setPrevious(this.mSingleDataFileChooserWizardDilaog);
            
            // packs
            this.mSingleDataFileChooserWizardDilaog.pack();
            dataTypeDialog.pack();
            break;
        case DATA_ADDITION_DRAG_AND_DROP:
            dataTypeDialog.setPrevious(null);
            
            // packs
            this.mSingleDataFileChooserWizardDilaog.pack();
            dataTypeDialog.pack();
            break;
        default:
            throw new IllegalArgumentException("data file type is invalid: "
                    + fileType);
        }
        
    }
    
    /**
     * Sets up the connection of wizard dialogs.
     * If the data type is Scalar-XY, Plot type selection dialog is connected.
     * 
     * @param fileType
     *            the file type (array data or netcdf)
     * @param dataType
     *            the method of data addition (toolbar or dad)
     */
    void setupPlotTypeSelectionWizardDialogConnection(final int fileType, 
            final String dataType) {
        
        // select the data column type wizard dialog
        final SGWizardDialog dataColumnTypeDialog;
        switch (fileType) {
        case FILETYPE_DATA:
            dataColumnTypeDialog = this.mDataColumnSelectionWizardDialog;
            break;
        case FILETYPE_NETCDF:
            dataColumnTypeDialog = this.mNetCDFSetupWizardDialog;
            break;
        default:
            throw new IllegalArgumentException("Invalid type for data file: "+ fileType);
        }

        if (SGDataUtility.isSXYTypeData(dataType)) {
            dataColumnTypeDialog.setNext(this.mPlotTypeSelectionWizardDialog);
            this.mPlotTypeSelectionWizardDialog.setPrevious(dataColumnTypeDialog);
            this.mPlotTypeSelectionWizardDialog.setNext(null);
            
            dataColumnTypeDialog.pack();
            this.mPlotTypeSelectionWizardDialog.pack();
        } else {
            dataColumnTypeDialog.setNext(null);
            dataColumnTypeDialog.pack();
        }
    }

    /**
     * Path name of the selected data file.
     */
    // private String mCurrentFileDirectory = null;
    
    /**
     * 
     */
    String mCurrentDataFileName = null;

    /**
     * 
     */
    String mCurrentPropertyFileName = null;

    /**
     * 
     */
    String mCurrentImageFileName = null;

    /**
     * 
     */
    String mCurrentArchiveFileName = null;
    
    
    /**
     * 
     */
    String mCurrentNetCDFFileName = null;
    
    /**
     * 
     */
    String mCurrentOutputDataFileName = null;

    /**
     * Returns the current file directory.
     * 
     */
    String getCurrentFileDirectory() {
        Preferences pref = Preferences.userNodeForPackage(this.getClass());
        String currentDir = pref.get(PREF_KEY_CURRENT_DIRECTORY, null);

        boolean b = false;
        if (currentDir == null) {
            b = true;
        } else {
            File f = new File(currentDir);
            if (f.exists() == false) {
                b = true;
            }
        }

        if (b) {
            StringBuffer sb = new StringBuffer();
            sb.append(USER_HOME);
            if (SGUtility.identifyOS(OS_NAME_WINDOWS)) {
                sb.append(FILE_SEPARATOR);
                sb.append(MY_DOCUMENTS);
            }
            String path = sb.toString();
            this.setCurrentFileDirectory(path);
            currentDir = path;
        }

        return currentDir;
    }

    /**
     * 
     * @param path
     */
    void setCurrentFileDirectory(final String path) {
        Preferences pref = Preferences.userNodeForPackage(this.getClass());
        pref.put(PREF_KEY_CURRENT_DIRECTORY, path);
        // this.mCurrentFileDirectory = path;
        // System.out.println(path);
    }

    /**
     * Show the about dialog.
     * 
     * @param wnd
     */
    public boolean showAboutDialog(SGDrawingWindow wnd) {
        if (wnd == null)
            wnd = this.mWindowManager.getCurrentWindow();
        if (wnd == null)
            return false;

        SGAboutDialog dg = new SGAboutDialog(wnd, true, this.mAppProp
                .getVersionString());

        final int width = dg.getWidth();
        final int height = dg.getHeight();

        final int x = wnd.getX() + wnd.getWidth() / 2 - width / 2;
        final int y = wnd.getY() + wnd.getHeight() / 2 - height / 2;

        dg.setLocation(x, y);

        // show a modal dialog
        dg.setVisible(true);

        // dispose
        dg.dispose();
        return true;
    }

    /**
     * Duplicate the focused data object.
     * 
     * @return true if succeeded
     */
    boolean duplicateFocusedData(SGDrawingWindow wnd) {
        List<SGFigure> fList = wnd.getVisibleFigureList();
        for (int ii = 0; ii < fList.size(); ii++) {
            SGFigure f = (SGFigure) fList.get(ii);
            
            // get figure elements
            SGIFigureElement[] elArray = f.getIFigureElementArray();
            SGIFigureElementGraph gElement = f.getGraphElement();
            
            // get focused data objects
            List<SGData> dataList = gElement.getFocusedDataList();
            
            List<SGData> dListOriginal = new ArrayList<SGData>();
            List<SGData> dListDuplicated = new ArrayList<SGData>();
            for (int jj = 0; jj < dataList.size(); jj++) {
                SGData data = (SGData) dataList.get(jj);

                // get properties of data before duplication
                String dataType = data.getDataType();
                String name = gElement.getDataName(data);
                SGDataColumnInfo[] cols = gElement.getDataColumnInfoArray(data);
                Map<String, Object> infoMap = gElement.getInfoMap(data);
                SGDataColumnInfoSet colInfoSet = new SGDataColumnInfoSet(cols);
                
                // create a new name
                String nameNew = f.getNewDataName(name);
                
                // create a dialog and set data
                SGDataDuplicationDialog dg = null;
                if (data instanceof SGArrayData) {
                    SGArrayDataDuplicationDialog adg = new SGArrayDataDuplicationDialog(
                            wnd, true);
                    if (adg.setData(nameNew, dataType, colInfoSet, infoMap) == false) {
                        return false;
                    }
                    dg = adg;
                } else if (data instanceof SGNetCDFData) {
                    SGNetCDFData nData = (SGNetCDFData) data;
                    SGNetCDFDataDuplicationDialog ndg = new SGNetCDFDataDuplicationDialog(
                            wnd, true);
                    if (ndg.setData(nameNew, nData, colInfoSet, infoMap) == false) {
                        return false;
                    }
                    dg = ndg;
                } else {
                    throw new Error("Invalid data type: " + data.getDataType());
                }
                
                // show the dialog
                dg.setLocation(wnd.getX() + 20, wnd.getY() + 20);
                dg.setVisible(true);

                // after the dialog is closed
                SGData dCopy = null;
                if (dg.getCloseOption() == SGDialog.OK_OPTION) {
                    // if OK button is pressed
                    
                    // get the results
                    SGDataColumnInfo[] result = dg.getDataColumnTypes();
                    
                    // copy the data and set columns
                    dCopy = (SGData) data.copy();
                    if (dCopy.setColumnInfo(result) == false) {
                    	return false;
                    }
                    
                } else {
                    continue;
                }
                
                // get the input name                
                String nameInput = dg.getDataName();
                
                // get data properties
                SGProperties dp = dCopy.getProperties();
                
                Map<Class<? extends SGIFigureElement>, SGProperties> propertiesMap =
                    new HashMap<Class<? extends SGIFigureElement>, SGProperties>();
                for (int kk = 0; kk < elArray.length; kk++) {
                    // get properties of the original data from figure elements
                    SGProperties p = elArray[kk].getDataProperties(data);
                    
                    // synchronize the properties for duplicated data object
                    SGProperties sp = elArray[kk].synchronizeDataProperties(p, dp);
                    if (sp == null) {
                        continue;
                    }
                    
                    // put into the map
                    propertiesMap.put(elArray[kk].getClass(), sp);
                }
                
                // add data to the figure
                if (f.addData(dCopy, nameInput, propertiesMap) == false) {
                    return false;
                }
                
                dListOriginal.add(data);
                dListDuplicated.add(dCopy);
            }
            
            for (SGData data : dListOriginal) {
                // set unfocused the selected data
                gElement.setDataFocused(data, false);                
            }
            for (SGData data : dListDuplicated) {
                // set focused the new data
                gElement.setDataFocused(data, true);                
            }
        }
        return true;
    }
    
    /**
     * 
     * @return
     */
    boolean duplicateFocusedFigures(SGDrawingWindow wnd) {

        DOMImplementation domImpl = SGApplicationUtility.getDOMImplementation();
        if (domImpl == null) {
            return false;
        }

        // create a Document object
        Document document = domImpl.createDocument("",
                TAG_NAME_FOCUSED_FIGURES, null);

        // create a DOM tree
        if (wnd.createDOMTree(document,
                SGIRootObjectConstants.FOCUSED_FIGURES_FOR_DUPLICATION,
                SAVE_PROPERTIES_FOR_DUPLICATION) == false) {
            return false;
        }

        // create an array of wrapped data objects
        List<WrappedData> wdList = new ArrayList<WrappedData>();
        List<SGFigure> fList = wnd.getFocusedFigureList();
        for (int ii = 0; ii < fList.size(); ii++) {
            SGFigure figure = fList.get(ii);
            List<SGData> dList = new ArrayList<SGData>(figure.getVisibleDataList());
            for (int jj = 0; jj < dList.size(); jj++) {
                SGData data = (SGData) dList.get(jj);
                FigureData fd = new FigureData(data, ii);
                WrappedData wd = new WrappedData(fd);
                wdList.add(wd);
            }
        }
        WrappedData[] wDataArray = new WrappedData[wdList.size()];
        wdList.toArray(wDataArray);

        // get root element - property
        Element root = document.getDocumentElement();
        
        // get version number
        String versionNumber = root.getAttribute(KEY_VERSION_NUMBER);

        // get the node of window
        NodeList wList = root
                .getElementsByTagName(SGIRootObjectConstants.TAG_NAME_WINDOW);
        if (wList.getLength() == 0) {
            return false;
        }
        Element elWnd = (Element) wList.item(0);

        int before = wnd.getFigureList().size();

        final int ret = this.createFiguresFromPropertyFile(elWnd, wnd, wDataArray, 
                true, versionNumber, LOAD_PROPERTIES_IN_DUPLICATION);
        if (ret != SGIConstants.SUCCESSFUL_COMPLETION) {
            return false;
        }

        int after = wnd.getFigureList().size();

        wnd.setChanged(before != after);

        return true;
    }
    
    // call before discarded a window
    int beforeDiscard(final SGDrawingWindow wnd) {
        final int ret = this.confirmBeforeDiscard(wnd);
        if (ret == JOptionPane.YES_OPTION) {

        } else if (ret == JOptionPane.NO_OPTION
                || ret == JOptionPane.CLOSED_OPTION) {
            // canceled and there is nothing to do
            return CANCEL_OPTION;
        } else if (ret == JOptionPane.CANCEL_OPTION) {
            // save the properties
            final int retSave = this.mPropertyFileManager
                    .savePropertiesByDialog(wnd);
            return retSave;
        }

        return OK_OPTION;
    }

    // show the confirmation dialog for saving properties of the window
    int confirmBeforeClosing(final SGDrawingWindow wnd) {
        final Object[] options = { MSG_CLOSE_WITHOUT_SAVING,
                SGDialog.CANCEL_BUTTON_TEXT, MSG_SAVE };
        return this.showConfirmationDialog(wnd, options,
                new SGCloseWindowConfirmPanel());
    }

    // show the confirmation dialog for saving properties of the window
    int confirmBeforeDiscard(final SGDrawingWindow wnd) {
        final Object[] options = {
        // MSG_DISCARD_WITHOUT_SAVING,
                MSG_CLOSE_WITHOUT_SAVING, SGDialog.CANCEL_BUTTON_TEXT, MSG_SAVE };
        return this.showConfirmationDialog(wnd, options,
                new SGCloseWindowConfirmPanel());
    }

    /**
     * Shows the confirmatin dialog.
     */
    private int showConfirmationDialog(Component component, Object[] options,
            Object message) {
        // beep
        Toolkit.getDefaultToolkit().beep();

        // show a dialog
        final int ret = JOptionPane.showOptionDialog(component, message,
                TITLE_CONFIRMATION, JOptionPane.YES_NO_CANCEL_OPTION,
                JOptionPane.WARNING_MESSAGE, null, options, options[2]);

        return ret;
    }

    /**
     * 
     * @return
     */
    boolean showChangeLogDialog(Frame owner) {
    	
        // create and set a dialog object
        SGChangeLogDialog dg = new SGChangeLogDialog(owner, true);
        dg.addActionListener(this);
        dg.setCenter(owner);

        // set a message
        String msg = "Change Log of Samurai Graph.";
        dg.setMessage(msg);

        // get the file of change log
        Class inClass = getClass();
        URL url = inClass.getResource(RESOURCES_DIRNAME + "ChangeLog.html");
        if (url == null) {
            JOptionPane.showMessageDialog(owner,
                    "Failed to get log information.");
            return false;
        }

        // set the html file
        dg.setPage(url);
        dg.pack();

        // show
        dg.setVisible(true);

        // dispose
        dg.dispose();

        return true;
    }

    private boolean addDataByDragAndDrop(final ActionEvent e) {
        Object source = e.getSource();
        SGWizardDialog dg = (SGWizardDialog) source;
        String command = e.getActionCommand();

        // cancel or previous
        if (command.equals(SGDialog.CANCEL_BUTTON_TEXT)) {
            dg.setVisible(false);
            this.clearTemporaryData();
        } else if (command.equals(SGDialog.PREVIOUS_BUTTON_TEXT)) {
            dg.showPrevious();
        } else if (command.equals(SGDialog.NEXT_BUTTON_TEXT)) {

            if (dg.equals(this.mArrayDataTypeSelectionWizardDialog)) {
                if (this.makeTransition(
                        this.mArrayDataTypeSelectionWizardDialog,
                        this.mDataColumnSelectionWizardDialog,
                        this.mDroppedFile.file) == false) {
                    return false;
                }
            } else if (dg.equals(this.mNetCDFDataTypeSelectionWizardDialog)) {
                if (this.makeTransition(
                        this.mNetCDFDataTypeSelectionWizardDialog, 
                        this.mNetCDFSetupWizardDialog,
                        this.mDroppedFile.file.getPath()) == false) {
                    return false;
                }
            } else if (dg.equals(this.mDataColumnSelectionWizardDialog)) {
                
                String dataType = this.mArrayDataTypeSelectionWizardDialog.getSelectedDataType();
                setupPlotTypeSelectionWizardDialogConnection(FILETYPE_DATA, dataType);
                dg.showNext();
                
            } else if (dg.equals(this.mNetCDFSetupWizardDialog)) {
                
                String dataType = this.mNetCDFDataTypeSelectionWizardDialog.getSelectedDataType();
                setupPlotTypeSelectionWizardDialogConnection(FILETYPE_NETCDF, dataType);
                dg.showNext();
                
            }
            
        } else if (command.equals(SGDialog.OK_BUTTON_TEXT)) {
            try {
                if (this.addDataByDragAndDropOK(dg) == false) {
                    return false;
                }
            } finally {
                this.clearTemporaryData();
            }
        }

        return true;
    }

    /**
     * Add data from a dropped file.
     * 
     * @param dg
     *            an event source
     * @return true if succeeded
     */
    private boolean addDataByDragAndDropOK(SGWizardDialog dg) {

        SGDrawingWindow wnd = dg.getOwnerWindow();
        String path = this.mDroppedFile.file.getAbsolutePath();

        // the location of a figure if it is created
        Point figureLocation;

        Object com;
        if (this.mDroppedFile.pos != null) {
            com = wnd.getComponent(this.mDroppedFile.pos.x, this.mDroppedFile.pos.y);
        } else {
            com = wnd;
        }
        
        // the ID of 
        int figureID;
        if (com instanceof SGDrawingWindow) {
            // get current figure ID
            figureID = wnd.assignFigureId();
            
            // set the dropped point to the new figure location
            figureLocation = this.mDroppedFile.pos;
        } else {
            SGFigure figure = (SGFigure) com;
            figureID = figure.getID();
            figureLocation = null;
        }
        
        if (dg.equals(this.mArrayDataTypeSelectionWizardDialog)
                || dg.equals(this.mDataColumnSelectionWizardDialog)) {
            // text data
            
            if (this.drawNewGraphOfArrayDataByDadOrToolbar(wnd, dg, path, figureID, figureLocation) == false) {
                return false;
            }
            
        } else if (dg.equals(this.mNetCDFSetupWizardDialog)
                || dg.equals(this.mNetCDFDataTypeSelectionWizardDialog)) {
            // netCDF data
            
            if (this.drawNewGraphOfNetcdfDataByDadOrToolbar(wnd, dg, path, figureID, figureLocation) == false) {
                return false;
            }
            
        } else if (dg.equals(this.mPlotTypeSelectionWizardDialog)) {
            // plot type selection (text or netCDF)
            
            if (dg.getPrevious().equals(this.mDataColumnSelectionWizardDialog)) {
                if (this.drawNewGraphOfArrayDataByDadOrToolbar(wnd, dg, path, figureID, figureLocation) == false) {
                    return false;
                }
            } else {
                if (this.drawNewGraphOfNetcdfDataByDadOrToolbar(wnd, dg, path, figureID, figureLocation) == false) {
                    return false;
                }
            }
        }
        
        wnd.notifyToRoot();
        
        return true;
    }
    
    private boolean drawNewGraphOfArrayDataByDadOrToolbar(
            SGDrawingWindow wnd, SGWizardDialog dg, String path, int figureID, Point figureLocation) {
        
        // get data type
        String dataType = this.mArrayDataTypeSelectionWizardDialog.getSelectedDataType();
        if (dataType == null) {
            SGUtility.showErrorMessageDialog(wnd, ERRMSG_TO_GET_DATA_TYPE, TITLE_ERROR);
            return false;
        }

        // get information
        Map<String, Object> infoMap =
            createInfoMap(dataType, this.mArrayDataTypeSelectionWizardDialog);

        // get selected column types
        SGDataColumnInfoSet colInfoSet = null;
        if (dg.equals(this.mArrayDataTypeSelectionWizardDialog)) {
            colInfoSet = this.getDefaultDataColumnInfo(path, dataType, infoMap, false, null);
        } else if (dg.equals(this.mDataColumnSelectionWizardDialog)) {
            colInfoSet = this.mDataColumnSelectionWizardDialog.getDataColumnInfoSet();
            infoMap.put(KEY_DATA_NAME, this.mDataColumnSelectionWizardDialog.getDataName());
        } else if (dg.equals(this.mPlotTypeSelectionWizardDialog)) {
            colInfoSet = this.mDataColumnSelectionWizardDialog.getDataColumnInfoSet();
            infoMap.put(KEY_DATA_NAME, this.mDataColumnSelectionWizardDialog.getDataName());
            addPlotTypeSelectionValuesToInfoMap(infoMap, this.mPlotTypeSelectionWizardDialog);
        }
        if (colInfoSet == null) {
            SGUtility.showErrorMessageDialog(wnd, ERRMSG_TO_DRAW_GRAPH, TITLE_ERROR);
            return false;
        }

        // draw the graph
        if (this.drawGraph(
                wnd, figureID, colInfoSet, infoMap, path, 
                null, true, figureLocation) == false) {
            return false;
        }
        
        return true;
    }
    
    private boolean drawNewGraphOfNetcdfDataByDadOrToolbar(
            SGDrawingWindow wnd, SGWizardDialog dg, String path, int figureID, Point figureLocation) {
        
        // get data type
        String dataType = this.mNetCDFDataTypeSelectionWizardDialog.getSelectedDataType();
        if (dataType == null) {
            SGUtility.showErrorMessageDialog(wnd, ERRMSG_TO_GET_DATA_TYPE, TITLE_ERROR);
            return false;
        }
        
        // create a information map
        Map<String, Object> infoMap =
            createInfoMap(dataType, this.mNetCDFDataTypeSelectionWizardDialog);

        // open the file
        NetcdfFile ncFile = null;
        SGDataColumnInfoSet colInfoSet = null;
        try {
            try {
                ncFile = NetcdfFile.open(path);
                SGNetCDFFile nc = new SGNetCDFFile(ncFile);
                
                // put the netCDF file
                infoMap.put(KEY_DATA_SOURCE, nc);

                // get selected column types
                if (dg.equals(this.mNetCDFDataTypeSelectionWizardDialog)) {
                    colInfoSet = this.getNetCDFDefaultDataColumnInfo(nc, dataType, infoMap);
                    String dataName = SGUtility.createDataNameBase(path);
                    infoMap.put(KEY_DATA_NAME, dataName);
                } else if (dg.equals(this.mNetCDFSetupWizardDialog)) {
                    colInfoSet = this.mNetCDFSetupWizardDialog.getDataColumnInfoSet();
                    infoMap.put(KEY_DATA_NAME, this.mNetCDFSetupWizardDialog.getDataName());
                    addDimensionValuesToInfoMap(infoMap, this.mNetCDFSetupWizardDialog);
                } else if (dg.equals(this.mPlotTypeSelectionWizardDialog)) {
                    colInfoSet = this.mNetCDFSetupWizardDialog.getDataColumnInfoSet();
                    infoMap.put(KEY_DATA_NAME, this.mNetCDFSetupWizardDialog.getDataName());
                    addDimensionValuesToInfoMap(infoMap, this.mNetCDFSetupWizardDialog);
                    addPlotTypeSelectionValuesToInfoMap(infoMap, this.mPlotTypeSelectionWizardDialog);
                }
            } finally {
                ncFile.close();
            }
        } catch (IOException ex) {
            return false;
        }
        
        if (colInfoSet == null) {
            SGUtility.showErrorMessageDialog(wnd, MSG_INCOMPLETE_NETCDF_DATA_FILE, TITLE_ERROR);
            return false;
        }

        if (dg.equals(this.mNetCDFDataTypeSelectionWizardDialog)) {
            // set default value of dimension origin and step
            if (this.setupDefaultDimensionValues(dataType, infoMap) == false) {
                return false;
            }
        } else if (dg.equals(this.mNetCDFSetupWizardDialog) ||
                dg.equals(this.mPlotTypeSelectionWizardDialog)) {
            // set dimension origin and step
            if (addDimensionValuesToInfoMap(infoMap, this.mNetCDFSetupWizardDialog) == false) {
                return false;
            }
        }

        // draw the graph
        if (this.drawGraph(
                wnd, figureID, colInfoSet, infoMap, path,
                null, false, figureLocation) == false) {
            SGUtility.showErrorMessageDialog(wnd, ERRMSG_NETCDF_DATA_ADDITION, TITLE_ERROR);
            return false;
        }
        
        return true;
    }

    /** error message that it is failed to get the data type. */
    public static final String ERRMSG_TO_GET_DATA_TYPE = "Failed to get the data type.";
    
    public static final String ERRMSG_TO_DRAW_GRAPH = "Failed to draw the graph.";
    
    public static final String ERRMSG_TO_LOAD_DATASET = "Failed to load dataset file.";
    
    private static final String ERRMSG_SCRIPT_START =
        "To use Samurai Graph script, the application must be started in the command mode.";
    
    private static final String ERRMSG_NETCDF_DATA_ADDITION =
        "Failed to add netCDF data.\n"
        + "Valid data was not obtained with input values.";
    
    private static final String ERRMSG_URL_OF_NETCDF = "Failed to add netCDF file from inputted URL.";

    private boolean addDataByToolBar(final ActionEvent e) {

        Object source = e.getSource();
        SGWizardDialog dg = (SGWizardDialog) source;
        String command = e.getActionCommand();

        // cancel or previous
        if (command.equals(SGDialog.CANCEL_BUTTON_TEXT)) {
            dg.setVisible(false);
            this.clearTemporaryData();
        } else if (command.equals(SGDialog.PREVIOUS_BUTTON_TEXT)) {
            dg.showPrevious();
        } else if (command.equals(SGDialog.NEXT_BUTTON_TEXT)) {
            
            if (dg.equals(this.mFigureIDSelectionWizardDialog)) {
                dg.showNext();
            } else if (dg.equals(this.mSingleDataFileChooserWizardDilaog)) {
                
                String filename = this.mSingleDataFileChooserWizardDilaog.getFileName();
                
                if (this.mSingleDataFileChooserWizardDilaog.isLocalFileSelected()) {
                    File file = new File(filename);

                    // select the next dialog
                    String filePath = file.getAbsolutePath();
                    if (SGApplicationUtility.isNetCDFFile(filePath)) {
                        this.setupDataTypeSelectionWizardDialogConnection(
                                FILETYPE_NETCDF, DATA_ADDITION_TOOL_BAR);
                        
                        // setup the next dialog
                        if (this.toDataTypeDialog(this.mSingleDataFileChooserWizardDilaog,
                                this.mNetCDFDataTypeSelectionWizardDialog, file) == false) {
                            return false;
                        }
                        
                    } else {
                        this.setupDataTypeSelectionWizardDialogConnection(
                                FILETYPE_DATA, DATA_ADDITION_TOOL_BAR);
                        
                        // setup the next dialog
                        if (this.toDataTypeDialog(this.mSingleDataFileChooserWizardDilaog,
                                this.mArrayDataTypeSelectionWizardDialog, file) == false) {
                            return false;
                        }
                    }

                    // set to the file chooser
                    this.mSingleDataFileChooserWizardDilaog.setSelectedFile(file);
                } else {
                    // setup the next dialog
                    this.setupDataTypeSelectionWizardDialogConnection(
                            FILETYPE_NETCDF, DATA_ADDITION_TOOL_BAR);
                }

                // show the next dialog
                dg.showNext();

            } else if (dg.equals(this.mArrayDataTypeSelectionWizardDialog)) {
                
                File f = this.mSingleDataFileChooserWizardDilaog
                        .getSelectedFile();
                if (this.makeTransition(
                        this.mArrayDataTypeSelectionWizardDialog,
                        this.mDataColumnSelectionWizardDialog, f) == false) {
                    return false;
                }
                
            } else if (dg.equals(this.mNetCDFDataTypeSelectionWizardDialog)) {

                String filename = this.mSingleDataFileChooserWizardDilaog.getFileName();
                
                if (this.mSingleDataFileChooserWizardDilaog.isLocalFileSelected()) {
                    if (this.makeTransition(
                            this.mNetCDFDataTypeSelectionWizardDialog, 
                            this.mNetCDFSetupWizardDialog,
                            filename) == false) {
                        return false;
                    }
                } else {
                    if (SGApplicationUtility.isNetCDFFile(filename)) {
                        if (this.makeTransition(
                                this.mNetCDFDataTypeSelectionWizardDialog, 
                                this.mNetCDFSetupWizardDialog,
                                filename) == false) {
                            return false;
                        }
                    } else {
                        if (SGUtility.showErrorMessageDialog(dg, ERRMSG_URL_OF_NETCDF, TITLE_ERROR)) {
                            this.mNetCDFDataTypeSelectionWizardDialog.setVisible(false);
                            this.mSingleDataFileChooserWizardDilaog.setVisible(true);
                        }
                    }
                }
                
            } else if (dg.equals(this.mDataColumnSelectionWizardDialog)) {
                
                String dataType = this.mArrayDataTypeSelectionWizardDialog.getSelectedDataType();
                setupPlotTypeSelectionWizardDialogConnection(FILETYPE_DATA, dataType);
                dg.showNext();
                
            } else if (dg.equals(this.mNetCDFSetupWizardDialog)) {
                
                String dataType = this.mNetCDFDataTypeSelectionWizardDialog.getSelectedDataType();
                setupPlotTypeSelectionWizardDialogConnection(FILETYPE_NETCDF, dataType);
                dg.showNext();
                
            }
            
        } else if (command.equals(SGDialog.OK_BUTTON_TEXT)) {
            
            try {
                if (this.addDataByToolBar(dg) == false) {
                    return false;
                }
            } finally {
                this.clearTemporaryData();
            }
        }
        
        return true;
    }
    
    /**
     * Do add data to figure.
     * 
     * @param dg wizard dialog which is source on doing addition.
     * @return
     */
    private boolean addDataByToolBar(SGWizardDialog dg) {

        // set invisible the dialog
        dg.setVisible(false);

        SGDrawingWindow wnd = dg.getOwnerWindow();

        // figure id
        final int figureID = this.mFigureIDSelectionWizardDialog.getFigureID();
    	
        // data path
        String filename = this.mSingleDataFileChooserWizardDilaog.getFileName();
        String path = filename;
        if (this.mSingleDataFileChooserWizardDilaog.isLocalFileSelected()) {
            // path of the local data file
            File f = this.mSingleDataFileChooserWizardDilaog.getSelectedFile();
            path = f.getPath();
        }
        
        if (dg.equals(this.mArrayDataTypeSelectionWizardDialog)
                || dg.equals(this.mDataColumnSelectionWizardDialog)) {
            // text data

            if (this.drawNewGraphOfArrayDataByDadOrToolbar(wnd, dg, path, figureID, null) == false) {
                return false;
            }
	        
        } else if (dg.equals(this.mNetCDFDataTypeSelectionWizardDialog)
                || dg.equals(this.mNetCDFSetupWizardDialog)) {
            // netCDF data
            
            if (this.drawNewGraphOfNetcdfDataByDadOrToolbar(wnd, dg, path, figureID, null) == false) {
                return false;
            }
            
        } else if (dg.equals(this.mPlotTypeSelectionWizardDialog)) {
            
            if (dg.getPrevious().equals(this.mDataColumnSelectionWizardDialog)) {
                if (this.drawNewGraphOfArrayDataByDadOrToolbar(wnd, dg, path, figureID, null) == false) {
                    return false;
                }
            } else {
                if (this.drawNewGraphOfNetcdfDataByDadOrToolbar(wnd, dg, path, figureID, null) == false) {
                    return false;
                }
            }
            
        }
        
        wnd.notifyToRoot();

        return true;
    }
    
    // Sets the default value of dimension origin and step.
    boolean setupDefaultDimensionValues(
            final String dataType, final Map<String, Object> infoMap) {
        final boolean multipleVariable = true;
        infoMap.put(KEY_MULTIPLE_VARIABLE, Boolean.valueOf(multipleVariable));

        SGNetCDFFile ncSource = (SGNetCDFFile) infoMap.get(KEY_DATA_SOURCE);
        NetcdfFile ncFile = null;
        try {
            ncFile = NetcdfFile.open(ncSource.getNetcdfFile().getLocation());
            SGNetCDFFile nc = new SGNetCDFFile(ncFile);
            if (this.setupNetCDFDataSetupDialog(nc, dataType, infoMap, this.mNetCDFSetupWizardDialog) == false) {
                return false;
            }
        } catch (IOException e1) {
            return false;
        } finally {
            try {
                ncFile.close();
            } catch (IOException e1) {
            }
        }
        
        infoMap.put(KEY_MULTIPLE_DIMENSION_START, this.mNetCDFSetupWizardDialog.getMultipleDimensionStart());
        infoMap.put(KEY_MULTIPLE_DIMENSION_END, this.mNetCDFSetupWizardDialog.getMultipleDimensionEnd());
        infoMap.put(KEY_MULTIPLE_DIMENSION_STEP, this.mNetCDFSetupWizardDialog.getMultipleDimensionStep());
        return true;
    }
    
    /**
     * Process to data type selection dialog.
     * 
     * @param owner
     *            the owner of dialogs
     * @param next
     *            a dialog for data type selection
     * @param f
     *            a file
     * @return true if succeeded
     */
    private boolean toDataTypeDialog(Window owner,
            SGArrayDataTypeSelectionWizardDialog next, File f) {

        if (f.exists() == false) {
            SGUtility.showFileNotFoundMessageDialog(owner);
            return false;
        }
        
        String pathName = f.getPath();
        
        List<String> cList = new ArrayList<String>();
        try {
            if (this.mDataCreator.getDataTypeCandidateList(pathName, cList) == false) {
                SGApplicationUtility.showDataFileInvalidMessageDialog(owner);
                return false;
            }
        } catch (FileNotFoundException ex) {
            SGUtility.showFileNotFoundMessageDialog(owner);
            return false;
        }
        if (cList.size() == 0) {
            SGApplicationUtility.showDataFileInvalidMessageDialog(owner);
            return false;
        }
        
        List<String> typeList = new ArrayList<String>();
        for (int i = 0; i < cList.size(); i++) {
            typeList.add(SGApplicationUtility.getArrayDataType(cList.get(i)));
        }

        // set candidate data-type to the dialog
        if (next.setAvailableDataType(typeList) == false) {
            SGApplicationUtility.showDataFileInvalidMessageDialog(owner);
            return false;
        }
        
        return true;
    }

    /**
     * Process to data type selection dialog.
     * 
     * @param owner
     *            the owner of dialogs
     * @param next
     *            a dialog for data type selection
     * @param f
     *            a file
     * @return true if succeeded
     */
    private boolean toDataTypeDialog(Window owner,
            SGNetCDFDataTypeSelectionWizardDialog next, File f) {

        if (f.exists() == false) {
            SGUtility.showFileNotFoundMessageDialog(owner);
            return false;
        }
/*        
        // path of the data file
        String pathName = f.getPath();

        // ArrayList infoList = new ArrayList();
        List<String> cList = new ArrayList<String>();
        try {
            if (this.mDataCreator.getDataTypeCandidateList(pathName, cList) == false) {
                SGApplicationUtility.showDataFileInvalidMessageDialog(owner);
                return false;
            }
        } catch (FileNotFoundException ex) {
            SGUtility.showFileNotFoundMessageDialog(owner);
            return false;
        }

        if (cList.size() == 0) {
            SGApplicationUtility.showDataFileInvalidMessageDialog(owner);
            return false;
        }

        // set candidate data-type to the dialog
        if (next.setAvailableDataType(cList) == false) {
            SGApplicationUtility.showDataFileInvalidMessageDialog(owner);
            return false;
        }
*/
        return true;
    }
    
    /**
     * Makes the transition between wizard dialogs.
     * 
     * @param prev
     *            previous dialog to select data type
     * @param next
     *            next dialog to select column
     * @param f
     *            a data file
     * @return true if succeeded
     */
    private boolean makeTransition(
            SGArrayDataTypeSelectionWizardDialog prev,
            SGArrayDataColumnSelectionWizardDialog next, File f) {

        SGDrawingWindow wnd = prev.getOwnerWindow();

        // set invisible the dialog
        prev.setVisible(false);

        // get selected file type
        String dataType = prev.getSelectedDataType();
        if (dataType == null) {
            SGUtility.showErrorMessageDialog(wnd, ERRMSG_TO_GET_DATA_TYPE, TITLE_ERROR);
            return false;
        }

        // create a map of information
        Map<String, Object> infoMap = createInfoMap(dataType, prev);
        
        // path of the data file
        String pathName = f.getPath();

        SGDataColumnInfoSet colInfoSet = this.getDefaultDataColumnInfo(pathName, 
        		dataType, infoMap, false, null);
        if (colInfoSet == null) {
            SGApplicationUtility.showDataFileInvalidMessageDialog(wnd);
            return false;
        }
        
        // create default data name
        String dataName = SGUtility.createDataNameBase(pathName);
        
        // put into the infoMap
        infoMap.put(KEY_DATA_NAME, dataName);
        
        // set information to the dialog for data column selection
        next.setData(dataType, colInfoSet, infoMap, false);

        // show the dialog to select data columns
        next.setCenter(wnd);
        next.setVisible(true);

        return true;
    }

    /**
     * Makes the transition between wizard dialogs of netcdf data.
     * 
     * @param prev
     *            previous dialog to select data type
     * @param next
     *            next dialog to select column
     * @param path
     *            a data file path
     * @return true if succeeded
     */
    private boolean makeTransition(
            SGNetCDFDataTypeSelectionWizardDialog prev,
            SGNetCDFDataSetupWizardDialog next, String path) {

        Window owner = prev.getOwner();

        // set invisible the dialog
        prev.setVisible(false);

        // get selected file type
        String dataType = prev.getSelectedDataType();
        if (dataType == null) {
            SGUtility.showErrorMessageDialog(owner, ERRMSG_TO_GET_DATA_TYPE, TITLE_ERROR);
            return false;
        }
        
        // create a map of information
        Map<String, Object> infoMap = createInfoMap(dataType, prev);
        
        // create default data name
        String dataName = SGUtility.createDataNameBase(path);
//        if (SGNetCDFDataFileChooserPanel.isNetcdfUrl(path)) {
//            dataName = SGNetCDFDataFileChooserPanel.getNetcdfUrlDataName(path);
//        }

        // put into the infoMap
        infoMap.put(KEY_DATA_NAME, dataName);
        
        // open the file
        NetcdfFile ncFile = null;
        try {
            ncFile = NetcdfFile.open(path);
        } catch (IOException e1) {
            return false;
        }
        SGNetCDFFile nc = new SGNetCDFFile(ncFile);

        if (this.setupNetCDFDataSetupDialog(nc, dataType, infoMap, next) == false) {
            return false;
        }
        
        try {
            ncFile.close();
        } catch (IOException e1) {
        }
        
        // show the dialog to select data columns
        next.setCenter(owner);
        next.setVisible(true);

        return true;
    }
    
    
    /**
     * Returns default data column information.
     * 
     * @param pathName
     *                the path of data file
     * @param dataType
     *                the data type
     * @param infoMap
     *                the information map
     * @param isPropertyFileData
     *                true if the data is for the property file
     * @param versionNumber
     *                the version number of the property file
     * @return default the data column information
     */
    SGDataColumnInfoSet getDefaultDataColumnInfo(String pathName, String dataType, 
    		Map<String, Object> infoMap,
            final boolean isPropertyFileData, final String versionNumber) {
        
        // parse the data file
        FileColumn[] fileColInfo = null;
        try {
            fileColInfo = this.mDataCreator.parseFileComlumnType(pathName, dataType, 
            		isPropertyFileData, versionNumber);
        } catch (FileNotFoundException e1) {
            return null;
        }
        
        if (fileColInfo == null) {
            return null;
        }
        
        // get column information: title and value type
        SGDataColumnInfo[] colInfoArray = new SGDataColumnInfo[fileColInfo.length];
        for (int ii = 0; ii < fileColInfo.length; ii++) {
            colInfoArray[ii] = new SGDataColumnInfo(fileColInfo[ii].title, 
            		fileColInfo[ii].valueType);
        }
        
        return this.createColumnInfoSet(dataType, infoMap, colInfoArray);
    }

    /**
     * Returns default data column information.
     * 
     * @param file
     *                the data source
     * @param dataType
     *                the data type
     * @param infoMap
     *                the information map
     * @param isPropertyFileData
     *                true if the data is for the property file
     * @param versionNumber
     *                the version number of the property file
     * @return default the data column information
     */
    SGDataColumnInfoSet getDefaultDataColumnInfo(SGTextDataFile file, String dataType, 
    		Map<String, Object> infoMap) {
        
    	SGDataColumn[] columns = file.getDataColumns();
        SGDataColumnInfo[] colInfoArray = new SGDataColumnInfo[columns.length];
        for (int ii = 0; ii < colInfoArray.length; ii++) {
            colInfoArray[ii] = new SGDataColumnInfo(columns[ii].getTitle(),
            		columns[ii].getValueType());
        }
        return this.createColumnInfoSet(dataType, infoMap, colInfoArray);
    }

    /**
     * Returns an array of the data column information.
     * @param ncFile
     *                netCDF file
     * @param dataType
     *                the data type
     * @param infoMap
     *                the information map
     * @return an array of the data column information
     */
    SGDataColumnInfoSet getNetCDFDefaultDataColumnInfo(SGNetCDFFile ncFile, String dataType, 
            Map<String, Object> infoMap) {
        
        List<SGVariable> varList = ncFile.getVariables();
        
        /*
        final int varNum = variables.size();

        // get the origin map
        NamedNodeMap nodeMap = (NamedNodeMap) infoMap.get(SGIFigureElementGraph.KEY_NODE_MAP);
        Map<String, Integer> originMap = null;
        if (nodeMap != null) {
        	originMap = SGDefaultColumnTypeUtility.getOriginMap(nodeMap);
        }
        
        // get column information: title and value type
        SGNetCDFDataColumnInfo[] colArray = new SGNetCDFDataColumnInfo[varNum];
        for (int ii = 0; ii < varNum; ii++) {
            SGVariable var = variables.get(ii);
            
            // get the origin
            int origin = 0;
			if (originMap != null) {
				if (var.isCoordinateVariable()) {
					Integer num = originMap.get(var.getName());
					if (num != null) {
						Dimension dim = var.getDimension(0);
						origin = num.intValue();
						if (origin < 0 || origin >= dim.getLength()) {
							origin = 0;
						}
					}
				}
			}
			
			colArray[ii] = new SGNetCDFDataColumnInfo(var,
					var.getNameInPriorityOrder(), var.getValueType(), origin);
        }
        */

        SGNetCDFDataColumnInfo[] colArray = SGDataUtility.getNetCDFDataColumnInfo(varList, infoMap);
        return this.createColumnInfoSet(dataType, infoMap, colArray);
    }

    /**
     * Create a map of information for each data type.
     * 
     * @param dataType
     *            the type of data
     * @param dg
     *            a wizard dialog to select data type
     * @return a map of information
     */
    public static Map<String, Object> createInfoMap(String dataType,
            SGDataTypeSelectionWizardDialog dg) {

        Map<String, Object> infoMap = new HashMap<String, Object>();
        infoMap.put(KEY_DATA_TYPE, dataType);

        if (SGDataUtility.isVXYTypeData(dataType)) {
            boolean b = dg.isPolarSelected();
            infoMap.put(KEY_POLAR_SELECTED, Boolean.valueOf(b));
        }
        
        final Boolean multiple = dg.isMultipleSelected();
        infoMap.put(KEY_MULTIPLE, multiple);
        if (SGDataUtility.isNetCDFData(dataType)) {
            final boolean multipleVariable = true;
            infoMap.put(KEY_MULTIPLE_VARIABLE, Boolean.valueOf(multipleVariable));
        } else if (SGDataUtility.isArrayData(dataType) && SGDataUtility.isSXYTypeData(dataType)) {
            // add the sampling rate to the infoList
            SGArrayDataTypeSelectionWizardDialog adg = (SGArrayDataTypeSelectionWizardDialog) dg;
            Double d = adg.getSamplingRate();
            if (d != null) {
                putSamplingRate(infoMap, d);
            }
        }

        return infoMap;
    }

    /**
     * Create a map of information for each data type.
     * <p>
     * from property file.
     * 
     * @param dataType
     *            the type of data
     * @param el
     *            an Element object
     * @return a map of information
     */
    Map<String, Object> createInfoMap(String dataType, Element el) {
        Map<String, Object> infoMap = new HashMap<String, Object>();
        infoMap.put(KEY_DATA_TYPE, dataType);

        if (SGDataUtility.isVXYTypeData(dataType)) {

            String str = el.getAttribute(SGIFigureElementGraph.KEY_POLAR);
            if (str.length() == 0) {
                return null;
            }
            Boolean b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return null;
            }
            infoMap.put(KEY_POLAR_SELECTED, b);
        }

        final boolean multiple = SGDataUtility.isMultipleData(dataType);
        infoMap.put(KEY_MULTIPLE, Boolean.valueOf(multiple));
        if (SGDataUtility.isNetCDFData(dataType)) {
            if (SGDataUtility.isNetCDFDimensionData(dataType)) {
                infoMap.put(KEY_MULTIPLE_VARIABLE, Boolean.FALSE);
            } else {
                infoMap.put(KEY_MULTIPLE_VARIABLE, Boolean.TRUE);
            }
            String str = el.getAttribute(SGIDataColumnNameConstants.KEY_PICKUP_DIMENSION_NAME);
            if (null!=str && str.length()!=0) {
                infoMap.put(KEY_MULTIPLE_VARIABLE, Boolean.FALSE);
            }
            str = el.getAttribute(SGIDataColumnNameConstants.KEY_PICKUP_START);
            if (null!=str && str.length()!=0) {
                Integer num = SGUtilityText.getInteger(str.trim());
                infoMap.put(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_START, num);
            }
            str = el.getAttribute(SGIDataColumnNameConstants.KEY_PICKUP_END);
            if (null!=str && str.length()!=0) {
                Integer num = SGUtilityText.getInteger(str.trim());
                infoMap.put(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_END, num);
            }
            str = el.getAttribute(SGIDataColumnNameConstants.KEY_PICKUP_STEP);
            if (null!=str && str.length()!=0) {
                Integer num = SGUtilityText.getInteger(str.trim());
                infoMap.put(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_STEP, num);
            }
        }

        // add the sampling rate to the infoList
        String str = el.getAttribute(SGIFigureElementGraph.KEY_SAMPLING_RATE);
        if (str.length() != 0) {
            putSamplingRate(infoMap, str);
        }
        
        // add the node map
        NamedNodeMap nodeMap = el.getAttributes();
        infoMap.put(SGIFigureElementGraph.KEY_NODE_MAP, nodeMap);

        return infoMap;
    }

    /**
     * Create a map of information for each data type.
     * <p>
     * from Data command.
     * @param dataType
     *            the type of data
     * @param map
     *            a map of properties
     * @return a map of information
     */
    Map<String, Object> createInfoMap(String dataType, SGPropertyMap map) {
        Map<String, Object> infoMap = new HashMap<String, Object>();
        infoMap.put(KEY_DATA_TYPE, dataType);

        if (SGDataUtility.isVXYTypeData(dataType)) {
            String str = null;
            Iterator<String> itr = map.getKeyIterator();
            while (itr.hasNext()) {
                String key = itr.next();
                if (COM_DATA_POLAR.equalsIgnoreCase(key)) {
                    str = map.getValueString(key);
                    break;
                }
            }
            if (str == null) {
                return null;
            }
            Boolean b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return null;
            }
            infoMap.put(KEY_POLAR_SELECTED, b);
        }

        final boolean multiple = SGDataUtility.isMultipleData(dataType);
        infoMap.put(KEY_MULTIPLE, Boolean.valueOf(multiple));
        if (SGDataUtility.isNetCDFData(dataType)) {
            if (SGDataUtility.isNetCDFDimensionData(dataType)) {
                infoMap.put(KEY_MULTIPLE_VARIABLE, Boolean.FALSE);
            } else {
                infoMap.put(KEY_MULTIPLE_VARIABLE, Boolean.TRUE);
            }
        }

        // add the sampling rate to the infoList
        if (dataType.equalsIgnoreCase(SGDataTypeConstants.SXY_SAMPLING_DATA) ||
                dataType.equalsIgnoreCase(SGDataTypeConstants.SXY_MULTIPLE_DATA)) {
            String str = null;
            Iterator<String> itr = map.getKeyIterator();
            while (itr.hasNext()) {
                String key = itr.next();
                if (COM_DATA_SAMPLING_RATE.equalsIgnoreCase(key)) {
                    str = map.getValueString(key);
                    break;
                }
            }
            if (str != null) {
                if (putSamplingRate(infoMap, str) == false) {
                    return null;
                }
                if (dataType.equalsIgnoreCase(SGDataTypeConstants.SXY_SAMPLING_DATA)) {
                    infoMap.put(KEY_DATA_TYPE, SGDataTypeConstants.SXY_MULTIPLE_DATA);
                    infoMap.put(KEY_MULTIPLE, false);
                }
            }
        }

        return infoMap;
    }
    
    /** Put sampling rate to the information map.
     * @return false if str is not double or less than 0. Otherwise true.
     */
    private static boolean putSamplingRate(Map<String, Object> infoMap, String str) {
        Double d = SGUtilityText.getDouble(str);
        if (d == null) {
            return false;
        }
    	return putSamplingRate(infoMap, d);
    }

    /** Put sampling rate value to the information map.
     * @return false if d <= 0.0. Otherwise true.
     */
    private static boolean putSamplingRate(Map<String, Object> infoMap, Double d) {
        if (d.doubleValue() <= 0.0) {
            return false;
        }
        infoMap.put(KEY_SAMPLING_RATE, d);
    	return true;
    }

    boolean drawGraphFromDialog(final SGDrawingWindow wnd) {
        // create wizard dialogs
        this.createDataAdditionWizardDialogs(wnd);
        
        // set figure ID numbers
        final int[] idArray = wnd.getVisibleFigureIDArray();
        this.mFigureIDSelectionWizardDialog.setIDNumbers(idArray);

        // set location
        this.mFigureIDSelectionWizardDialog.setCenter(wnd);

        // show the first wizard dialog
        this.mFigureIDSelectionWizardDialog.setVisible(true);

        // update the selected file name
        File f = this.mSingleDataFileChooserWizardDilaog.getSelectedFile();
        if (f == null)
            return false;
        this.updateCurrentFile(f, FILETYPE_DATA);
        return true;
    }

    /**
     * 
     * @param wnd
     * @param figureID
     * @param colInfoSet
     * @param infoMap
     * @param pathName
     * @param dataIdArray
     * @param showDialog
     * @param figureLocation
     * @return
     */
    private Boolean drawGraphSub(
            final SGDrawingWindow wnd, final int figureID,
            final SGDataColumnInfoSet colInfoSet,
            final Map<String, Object> infoMap,
            final String pathName, final Integer[] dataIdArray,
            final boolean showDialog, 
            final Point figureLocation) {

        if (wnd == null || infoMap == null || pathName == null) {
            return Boolean.FALSE;
        }

        if (infoMap.size() == 0) {
            return Boolean.FALSE;
        }

        // get or create data name base
        String dataNameBase = null;
        Object obj = infoMap.get(KEY_DATA_NAME);
        if (obj != null) {
            dataNameBase = (String) obj;
        } else {
            dataNameBase = SGUtility.createDataNameBase(pathName);
            if (dataNameBase == null) {
                return Boolean.FALSE;
            }
        }

        final SGFigure fig = wnd.getFigure(figureID);

        // create a data object
        SGDataCreator.CreatedDataSet cdSet = null;
        try {
            cdSet = this.mDataCreator.create(pathName, colInfoSet, infoMap, wnd, null, -1);
        } catch (FileNotFoundException ex) {
            if (showDialog) {
                SGUtility.showFileNotFoundMessageDialog(wnd);
            }
            return Boolean.FALSE;
        }
        if (cdSet == null) {
            return Boolean.FALSE;
        }
        
        return this.addDataToDrawGraph(
                wnd, figureID, dataIdArray, showDialog, infoMap,
                figureLocation, fig, cdSet, dataNameBase, false);
    }
    
    /**
     * 
     * @param wnd
     * @param figureID
     * @param dataIdArray
     * @param showDialog
     * @param infoMap
     * @param figureLocation
     * @param fig
     * @param cdSet
     * @param dataNameBase
     * @param fitAxisToFocused
     * @return true if succeeds
     */
    boolean addDataToDrawGraph(
            final SGDrawingWindow wnd,
            final int figureID,
            final Integer[] dataIdArray,
            final boolean showDialog, 
            final Map<String, Object> infoMap,
            final Point figureLocation,
            final SGFigure fig,
            final SGDataCreator.CreatedDataSet cdSet,
            final String dataNameBase,
            final boolean fitAxisToFocused) {
        
        final int len = cdSet.getDataLength();
        if (len==0) {
            if (showDialog) {
                SGApplicationUtility.showDataFileInvalidMessageDialog(wnd);
            }
            return Boolean.FALSE;
        }
        String[] nameArray = new String[len];
        SGData[] dataArray = new SGData[len];
        for (int ii = 0; ii < len; ii++) {
            SGDataCreator.CreatedData cd = cdSet.getData(ii);
            dataArray[ii] = cd.getData();
            StringBuffer sb = new StringBuffer();
            sb.append(dataNameBase);
            String title = cd.getTitle();
            if (title != null) {
                sb.append(cdSet.getMiddleString());
                sb.append(title);
            }
            nameArray[ii] = sb.toString();
        }
        
        // add data
        if (this.addData(
                wnd, figureID, dataIdArray, showDialog,
                figureLocation, fig, dataArray, nameArray, infoMap) == false) {
            return Boolean.FALSE;
        }

        if (fitAxisToFocused) {
            // fit axes
            SGIFigureElementGraph gElement = fig.getGraphElement();
            for (int ii = 0; ii < dataArray.length; ii++) {
                gElement.setDataFocused(dataArray[ii], true);
            }
            fig.fitAxisRangeToFocusedData();
        }
        
        return Boolean.TRUE;
    }
    
    private boolean addData(final SGDrawingWindow wnd, final int figureID,
            final Integer[] dataIdArray, final boolean showDialog, 
            final Point figureLocation, SGFigure fig, SGData[] dataArray, 
            String[] nameArray, final Map<String, Object> infoMap) {
    	
        boolean createFigureFlag = (fig == null);
        if (fig != null) {
            if (fig.isVisible() == false) {
                createFigureFlag = true;
            }
        }

        if (createFigureFlag) {
            // create new figure and add data.
            if (!SGMainFunctions.this.mFigureCreator.createNewFigure(wnd,
                    figureID, figureLocation, dataIdArray, dataArray, nameArray, infoMap)) {
                return Boolean.FALSE;
            }
            Object obj = infoMap.get(SGPlotTypeConstants.KEY_PLOT_TYPE_BAR);
            if (obj instanceof Boolean) {
                if (((Boolean)obj).booleanValue()) {
                    SGFigure figure = wnd.getFigure(figureID);
                    figure.fitAxisRangeToVisibleData();
                }
            }
        } else {
            // add data to figure already exists.
            SGIProgressControl progress = (SGIProgressControl) wnd;
            progress.endProgress();
            progress.startProgress();
            progress.setProgressMessage("Add Data");
            if (!fig.addData(dataArray, dataIdArray, nameArray, progress, 0.0f, 1.0f, infoMap)) {
                progress.endProgress();
                if (showDialog) {
                    SGApplicationUtility.showDataFileInvalidMessageDialog(wnd);
                }
            }
            if (fig.getClassType()==null || fig.getClassType().equals("")) {
                String type = "";
                SGData dataFirst = dataArray[0];
                if (dataFirst instanceof SGISXYTypeData || dataFirst instanceof SGISXYTypeMultipleData) {
                    type = SGIFigureTypeConstants.FIGURE_TYPE_SXY;
                } else if (dataFirst instanceof SGIVXYTypeData) {
                    type = SGIFigureTypeConstants.FIGURE_TYPE_VXY;
                } else if (dataFirst instanceof SGISXYZTypeData) {
                    type = SGIFigureTypeConstants.FIGURE_TYPE_SXY;
                }
                fig.setClassType(type);
            }
            progress.endProgress();
        }

        return true;
    }

    /**
     * Draw a new graph.
     * @param wnd
     *            a window that the graph belongs
     * @param figureID
     *            the ID of figure to draw the graph
     * @param colInfo
     *            an array of data column information
     * @param infoMap
     *            a map for data information
     * @param pathName
     *            path name of data
     * @param dataId
     *            the ID of data object to add
     * @param showDialog
     *            true if error and warning dialog is shown
     * @param figureLocation
     *            The location of new figure if it is created.
     *            If data is added to a figure that already exists,
     *            this value is neglected.
     * @return
     *            true if succeeded
     */
    boolean drawGraph(final SGDrawingWindow wnd, final int figureID,
            final SGDataColumnInfoSet colInfoSet, final Map<String, Object> infoMap,
            final String pathName, final Integer[] dataIdArray, final boolean showDialog, 
            final Point figureLocation) {
        Boolean result = null;
        wnd.setWaitCursor(true);

        if (!USE_FOXTROT) {
            result = drawGraphSub(wnd, figureID, colInfoSet, infoMap,
                    pathName, dataIdArray, showDialog, figureLocation);
        } else {
            try {
                result = (Boolean) Worker.post(new Task() {
                    public Object run() throws Exception {
                        return drawGraphSub(wnd, figureID, colInfoSet,
                                infoMap, pathName, dataIdArray, showDialog, figureLocation);
                    }
                });
            } catch (Exception ex) {
                result = Boolean.FALSE;
                ex.printStackTrace();
            }
        }

        wnd.setWaitCursor(false);
        return result.booleanValue();
    }

    /**
     * 
     * @param f
     * @param type
     */
    void updateCurrentFile(File f, int type) {
        String parent = f.getParent();
        if (parent != null) {
            this.setCurrentFileDirectory(parent);
            String path = f.getPath();
            String name = path.substring(parent.length() + 1);
            this.updateCurrentFile(name, type);
        }
    }
    
    void updateCurrentFile(String name, int type) {
        if (type == FILETYPE_DATA) {
            this.mCurrentDataFileName = name;
        } else if (type == FILETYPE_PROPERTY) {
            this.mCurrentPropertyFileName = name;
        } else if (type == FILETYPE_IMAGE) {
            this.mCurrentImageFileName = name;
        } else if (type == FILETYPE_DATASET) {
            this.mCurrentArchiveFileName = name;
        } else if (type == FILETYPE_NETCDF) {
            this.mCurrentNetCDFFileName = name;
        } else if (type == FILETYPE_OUTPUT_DATA) {
            this.mCurrentOutputDataFileName = name;
        }
    }
    
    /**
     * Return true if obj is a wizard dialog for data-addition.
     * @param obj
     * @return
     */
    private boolean isDataAdditionDialog(Object obj) {
		return obj.equals(this.mFigureIDSelectionWizardDialog)
				|| obj.equals(this.mSingleDataFileChooserWizardDilaog)
				|| obj.equals(this.mArrayDataTypeSelectionWizardDialog)
				|| obj.equals(this.mDataColumnSelectionWizardDialog)
				|| obj.equals(this.mNetCDFDataTypeSelectionWizardDialog)
				|| obj.equals(this.mNetCDFSetupWizardDialog)
				|| obj.equals(this.mPlotTypeSelectionWizardDialog);
    }

    /**
     * Invoked when an action event is thrown.
     */
    public void actionPerformed(final ActionEvent e) {
        Object source = e.getSource();
        if (this.isDataAdditionDialog(source)) {
            if (this.mDroppedFile != null) {
                // drag and drop
                this.addDataByDragAndDrop(e);
            } else if (this.mTransformedData != null) {
                // transformation
                this.addDataByDataTransformation(e);
            } else {
                // tool bar
                this.addDataByToolBar(e);
            }
        }
    }

    /**
     * Exit the application normally with confirmation.
     */
    public void exit() {
        this.mWindowManager.closeAllWindow();
    }

    // Exit the application.
    void exitApplication(final int status) {
        if (status != 0) {
            String msg = "A fatal error has occured.\n"
                    + "The current application will be terminated.";
            String title = " Forced termination";
            SGUtility.showMessageDialog(null, msg, title,
                    JOptionPane.ERROR_MESSAGE);
        }
        
        // dispose objects
        this.mClipBoardManager.dispose();
        this.mPluginManager.dispose();
        
        // exit
        System.exit(status);
    }

    /**
     * Opens given file for MacOSX.
     * 
     * @param wnd
     *          the window
     * @param fname
     *          the file name
     * @return true if succeeded
     */
    public boolean openFile(SGDrawingWindow wnd, final String fname) {
        if (wnd == null) {
            wnd = this.mWindowManager.getCurrentWindow();
        }
        if (wnd == null) {
            return false;
        }

        // set unabled the window
        wnd.setWaitCursor(true);
        try {
            // analyze the file list
            File propertyFile = null;
            File archiveFile = null;
            File scriptFile = null;
            File file = new File(fname);
            String path = file.getAbsolutePath();

            // property file?
            final boolean propertyFlag = SGApplicationUtility.hasExtension(path, PROPERTY_FILE_EXTENSION);

            // archive file?
            final boolean archiveFlag = SGApplicationUtility.hasExtension(path, ARCHIVE_FILE_EXTENSION);
            
            // script file?
            final boolean scriptFlag = SGApplicationUtility.hasExtension(path, SCRIPT_FILE_EXTENSION);

            // as a property file
            if (propertyFlag) {
                propertyFile = file;
            } else if (archiveFlag) {
                archiveFile = file;
            } else if (scriptFlag) {
            	scriptFile = file;
            }
            // as a data file
            else {
                return false;
            }
            // initialize progress value

            // use property file
            if (propertyFile != null) {
                if (wnd.needsConfirmationBeforeDiscard()) {
                    final int ret = this.beforeDiscard(wnd);
                    if (ret == CANCEL_OPTION) {
                        return true;
                    }
                }
                if (this.mPropertyFileManager.showMultiDataFileChooserDialog(
                        propertyFile, wnd) == false) {
                    return false;
                }
            } else if (archiveFile != null) {
                if (wnd.needsConfirmationBeforeDiscard()) {
                    final int ret = this.beforeDiscard(wnd);
                    if (ret == CANCEL_OPTION) {
                        return true;
                    }
                }
                final boolean result = this.mDataSetManager
                        .loadDataSetFromEventDispatchThread(wnd, archiveFile);
                wnd.setSaved(result);
            } else if (scriptFile != null) {
                // setup standard output stream
                mStdoutWriter = new BufferedWriter(new OutputStreamWriter(
                        System.out));
                
                // read the input script file
                readCommand(scriptFile.getPath());
                
                // setup standard input stream
                mStdinReader = new BufferedReader(new InputStreamReader(
                        System.in));
                try {
                    // read input recursively
                	startReadingInput();
                } catch (IOException ex) {
                    return false;
                }
            } else {
                return false; // unreachable
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            // set enabled the window
            wnd.setWaitCursor(false);
        }

        return true;
    }
    
    // start to read the input
    private void startReadingInput() throws IOException {
    	this.readRecursively(mStdinReader, mStdoutWriter, 0, true, false);
    }
    
    /**
     * 
     */
    boolean fromDragAndDrop(final int x, final int y,
            final SGDrawingWindow wnd, final List<File> fileList) {

        wnd.setWaitCursor(true);

        try {
            // analyze the file list
            List<File> textDataFileList = new ArrayList<File>();
            List<File> netCDFDataFileList = new ArrayList<File>();
            File propertyFile = null;
            File archiveFile = null;
            File netcdfArchiveFile = null;
            File scriptFile = null;
            File imageFile = null;
            for (int ii = 0; ii < fileList.size(); ii++) {
                File file = (File) fileList.get(ii);
                String path = file.getAbsolutePath();

                // property file?
                final boolean propertyFlag = SGApplicationUtility.hasExtension(path,
                		PROPERTY_FILE_EXTENSION);

                // archive file?
                final boolean archiveFlag = SGApplicationUtility.hasExtension(path,
                        ARCHIVE_FILE_EXTENSION);
                
                // script file?
                final boolean scriptFlag = SGApplicationUtility.hasExtension(path,
                        SCRIPT_FILE_EXTENSION);
                
                // image file?
                boolean imageFlag = false;
                String[] imgArray = SGIImageConstants.DRAWABLE_IMAGE_EXTENSIONS;
                for (int jj = 0; jj < imgArray.length; jj++) {
                    if (SGApplicationUtility.hasExtension(path, imgArray[jj])) {
                        imageFlag = true;
                        break;
                    }
                }
                
                // netCDF file? and it is dateset?
                boolean netcdfArchiveFlag = false;
                boolean netCDFFlag = false;
                if (SGApplicationUtility.isNetCDFFile(path)) {
                    if (SGNetCDFDataSetManager.isNetCDFDatasetFile(path)) {
                        netcdfArchiveFlag = true;
                    } else {
                        netCDFFlag = true;
                    }
                }

                if (propertyFlag) {
                    // as a property file
                    propertyFile = file;
                } else if (archiveFlag) {
                    // as an archive file
                    archiveFile = file;
                } else if (scriptFlag) {
                	// as a script file
                	scriptFile = file;
                } else if (imageFlag) {
                    // as an image file
                    imageFile = file;
                } else if (netcdfArchiveFlag) {
                    // as an netCDF archive file
                    netcdfArchiveFile = file;
                } else if (netCDFFlag) {
                    // as a netCDF data file
                    netCDFDataFileList.add(file);
                } else {
                    // as a data file
                    textDataFileList.add(file);
                }
            }

            if (propertyFile != null) {
                // use an property file
                if (wnd.needsConfirmationBeforeDiscard()) {
                    final int ret = this.beforeDiscard(wnd);
                    if (ret == CANCEL_OPTION) {
                        return true;
                    }
                }
                if (this.mPropertyFileManager.showMultiDataFileChooserDialog(
                        propertyFile, wnd) == false) {
                    return false;
                }
            } else if (archiveFile != null) {
                // use an archive file
                if (wnd.needsConfirmationBeforeDiscard()) {
                    final int ret = this.beforeDiscard(wnd);
                    if (ret == CANCEL_OPTION) {
                        return true;
                    }
                }
                SGUtility.clearMessageDialogVisible();
                final boolean result = this.mDataSetManager
						.loadDataSetFromEventDispatchThread(wnd, archiveFile);
                if (result==false && SGUtility.wasMessageDialogVisible()==false) {
                    SGUtility.showErrorMessageDialog(wnd, ERRMSG_TO_LOAD_DATASET, TITLE_ERROR);
                }
                SGUtility.clearMessageDialogVisible();
                wnd.setSaved(result);
            } else if (netcdfArchiveFile != null) {
                // user an netCDF archive file
                if (wnd.needsConfirmationBeforeDiscard()) {
                    final int ret = this.beforeDiscard(wnd);
                    if (ret == CANCEL_OPTION) {
                        return true;
                    }
                }
                SGUtility.clearMessageDialogVisible();
                final boolean result = this.mDataSetManager
                        .loadDataSetFromEventDispatchThread(wnd, netcdfArchiveFile);
                if (result==false && SGUtility.wasMessageDialogVisible()==false) {
                    SGUtility.showErrorMessageDialog(wnd, ERRMSG_TO_LOAD_DATASET, TITLE_ERROR);
                }
                SGUtility.clearMessageDialogVisible();
                wnd.setSaved(result);
            } else if (scriptFile != null) {
            	if (this.mStdoutWriter != null && this.mStdinReader != null) {
                    this.readCommand(scriptFile.getPath());
            	} else {
            	    SGUtility.showErrorMessageDialog(wnd, ERRMSG_SCRIPT_START, TITLE_ERROR);
            	}
            } else if (imageFile != null) {
                // use a image file
            	byte[] imageByteArray = SGApplicationUtility.toByteArray(imageFile);
            	String ext = SGApplicationUtility.getExtension(imageFile);
            	if (ext == null) {
            		return false;
            	}
                if (wnd.setImage(imageByteArray, ext, true) == false) {
                    return false;
                }
            } else if (netCDFDataFileList.size() != 0) {
                // add netCDF data
                if (this.onNetCDFDataFilesDropped(new Point(x, y), wnd, netCDFDataFileList) == false) {
                    return false;
                }
            } else {
                // add text data
                if (this.onTextDataFilesDropped(new Point(x, y), wnd, textDataFileList) == false) {
                    return false;
                }
            }

        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        } finally {
            // set enabled the window
            wnd.setWaitCursor(false);
        }

        return true;
    }
    
    // Update the pattern of the tool bar in the preferences.
    void updateToolBarPatternInPreferences(final String[] array) {
        Preferences pref = Preferences.userNodeForPackage(this.getClass());

        // update the tool bar pattern
        StringBuffer sb = new StringBuffer();
        for (int ii = 0; ii < array.length; ii++) {
            sb.append(array[ii]);
            if (ii != array.length - 1) {
                sb.append(',');
            }
        }
        pref.put(PREF_KEY_TOOL_BAR_PATTERN, sb.toString());
    }

    /**
     * 
     * @param elFigure
     * @param figure
     * @return
     */
    private int createSingleFigureFromPropertyFile(final Element elFigure,
            final SGDrawingWindow wnd, final ArrayList<WrappedData> dataList, 
            final boolean readDataProperty, final String versionNumber, final int mode) {

        final int ic = SGIConstants.PROPERTY_FILE_INCORRECT;

        String str = null;
        Number num = null;

        str = elFigure.getAttribute( SGFigure.KEY_FIGURE_TYPE );
        if (str.length() == 0) {
            return ic;
        }

        SGFigure figure = null;
        if (SGIFigureTypeConstants.FIGURE_TYPE_SXY.equals(str)
                || SGIFigureTypeConstants.FIGURE_TYPE_VXY.equals(str)
                || SGIFigureTypeConstants.FIGURE_TYPE_XY.equals(str)) {
            figure = new SGXYFigure(wnd);
            figure.setClassType(str);
        } else {
            return ic;
        }

        //
        // create a figure
        //

        final int figureID = wnd.assignFigureId();
        figure.setID(figureID);

        //
        // create SGIFigureElement objects
        //

        if (figure.readProperty(elFigure) == false) {
            return ic;
        }

        if (this.mFigureCreator.createFigureElementFromPropertyFile(figure,
                elFigure) == ic) {
            return ic;
        }

        //
        // called after created SGIAxisElement object
        //

        // space axis line and numbers
        str = elFigure.getAttribute(SGFigure.KEY_SPACE_AXIS_LINE_AND_NUMBER);
        if (str.length() != 0) {
            StringBuffer uSpaceLN = new StringBuffer();
            num = SGUtilityText.getNumber(str, uSpaceLN);
            if (num == null) {
                return ic;
            }
            final float spaceLN = num.floatValue();
            SGIFigureElementAxis aElement = figure.getAxisElement();
            aElement.setSpaceAxisLineAndNumber(spaceLN, uSpaceLN.toString());
            if (figure.setSpaceAxisLineAndNumber(spaceLN, uSpaceLN.toString()) == false) {
                return ic;
            }
        }

        // space numbers and title
        str = elFigure.getAttribute(SGFigure.KEY_SPACE_NUMBER_AND_TITLE);
        if (str.length() != 0) {
            StringBuffer uSpaceNT = new StringBuffer();
            num = SGUtilityText.getNumber(str, uSpaceNT);
            if (num == null) {
                return ic;
            }
            final float spaceNT = num.floatValue();
            SGIFigureElementAxis aElement = figure.getAxisElement();
            aElement.setSpaceNumberAndTitle(spaceNT, uSpaceNT.toString());
            if (figure.setSpaceNumberAndTitle(spaceNT, uSpaceNT.toString()) == false) {
                return ic;
            }
        }

        // create data objects
        final int ret = this.createDataObjectsFromPropertyFile(elFigure,
                figure, dataList, wnd, readDataProperty, versionNumber, mode);
        if (ret != SGIConstants.SUCCESSFUL_COMPLETION) {
            return ret;
        }

        // add the figure to the window
        wnd.addFigure(figure);

        // initialize properties of the figure and figure elements
        figure.initPropertiesHistory();

        return SGIConstants.SUCCESSFUL_COMPLETION;
    }

    // create the data object
    private int createDataObjectsFromPropertyFile(final Element elFigure,
            final SGFigure figure, final ArrayList<WrappedData> dataList,
            final SGIProgressControl progress, final boolean readDataProperty,
            final String versionNumber, final int mode) {

        final int ic = SGIConstants.PROPERTY_FILE_INCORRECT;
        final int di = SGIConstants.DATA_FILE_INVALID;

        NodeList nList = elFigure
                .getElementsByTagName(SGIFigureElementGraph.TAG_NAME_DATA);
        final int len = nList.getLength();
        if (len != dataList.size()) {
            return ic;                
        }

        int[] indexArray = new int[len];
        SGData[] dataArray = new SGData[len];
        boolean indexValid = true;
        for (int ii = 0; ii < len; ii++) {

            //
            // get information from the Element object
            //

            Node node = nList.item(ii);
            if ((node instanceof Element) == false) {
                continue;
            }
            Element elData = (Element) node;

            // data type
            String dataType = elData.getAttribute(SGIFigureElement.KEY_DATA_TYPE);
            if (dataType == null) {
                return ic;
            }
            
            // replaces the data type
            // because sampling SXY-data and date SXY-date is to be eliminated at some stage.
//            if (SGDataTypeConstants.SXY_SAMPLING_DATA.equals(dataType)) {
//            	dataType = SGDataTypeConstants.SXY_MULTIPLE_DATA;
//            } else if (SGDataTypeConstants.SXY_DATE_DATA.equals(dataType)) {
//            	dataType = SGDataTypeConstants.SXY_DATA;
//            }
            dataType = SGApplicationUtility.getArrayDataType(dataType);

            Class<?> dataClass = null;
            if (SGDataUtility.isArrayData(dataType)) {
            	if (SGDataUtility.isSXYTypeData(dataType)) {
            		dataClass = SGSXYMultipleData.class;
                } else if (SGDataTypeConstants.VXY_DATA.equals(dataType)) {
                	dataClass = SGVXYData.class;
                } else if (SGDataTypeConstants.SXYZ_DATA.equals(dataType)) {
                	dataClass = SGSXYZData.class;
            	} else {
            		return ic;
            	}
            } else if (SGDataUtility.isNetCDFData(dataType)) {
            	if (SGDataUtility.isSXYTypeData(dataType)) {
            	    if (SGDataUtility.isNetCDFDimensionData(dataType)) {
            	        dataClass = SGSXYMultipleDimensionNetCDFData.class;
            	    } else {
            	        dataClass = SGSXYMultipleVariableNetCDFData.class;
            	    }
                } else if (SGDataTypeConstants.VXY_NETCDF_DATA.equals(dataType)) {
                	dataClass = SGVXYNetCDFData.class;
                } else if (SGDataTypeConstants.SXYZ_NETCDF_DATA.equals(dataType)) {
                	dataClass = SGSXYZNetCDFData.class;
            	} else {
            		return ic;
            	}
            } else {
            	return ic;
            }
            String className = dataClass.getName();
            
            // set the class type for backward compatibility between 1.0.7
            if (SGIFigureTypeConstants.FIGURE_TYPE_XY.equals(figure.getClassType())) {
                String type = null;
                if (SGDataUtility.isVXYTypeData(dataType)) {
                    type = SGIFigureTypeConstants.FIGURE_TYPE_VXY;
                } else {
                    type = SGIFigureTypeConstants.FIGURE_TYPE_SXY;
                }
                figure.setClassType(type);
            }

            // create information map
            Map<String, Object> infoMap = this.createInfoMap(dataType, elData);

            // get the Class object
            Class<?> cl = null;
            try {
                cl = Class.forName(className);
            } catch (ClassNotFoundException ex) {
                return ic;
            }
            if (cl == null) {
                return ic;
            }

            // create a new SGData object
            SGData data = null;
            try {
                Object obj = cl.newInstance();
                if (obj instanceof SGData) {
                    data = (SGData)obj;
                }
            } catch (Exception ex) {
                return ic;
            }
            if (data == null) {
                return ic;
            }

            // get the data file
            if (dataList.size() == 0) {
                return SGIConstants.DATA_NUMBER_SHORTAGE;
            }
            
            // get an object from dataList
            WrappedData wData = (WrappedData) dataList.get(ii);
            if (!wData.hasFigureData()) {
                SGPropertyFileData pfData = wData.getPropertyFileData();
                String fileName = pfData.getFileName();

//                // check the type of data
//                List<String> cList = new ArrayList<String>();

                try {
/*                    
                    // check whether the given data type is contained in the candidates
                    if (this.mDataCreator.getDataTypeCandidateList(fileName, cList) == false) {
                        return di;
                    }
                    if (!cList.contains(dataType)) {
                        return di;
                    }
*/
                    // get the data column information
//                    SGDataColumnInfo[] colInfo = pfData.getDataColumnInfoArray();
//                    SGDataColumnInfoSet colInfoSet = null;
//                    if (colInfo == null) {
//                        if (SGDataUtility.isArrayData(dataType)) {
//                        	// get default column information
//                        	colInfoSet = this.getDefaultDataColumnInfo(fileName, dataType, 
//                        			infoMap, true, versionNumber);
//                        } else if (SGDataUtility.isNetCDFData(dataType)) {
//                            NetcdfFile ncFile = null;
//                            try {
//                                ncFile = NetcdfFile.open(fileName);
//                                SGNetCDFFile nc = new SGNetCDFFile(ncFile);
//                                infoMap.put(KEY_NETCDF_FILE, nc);
//                                colInfoSet = this.getNetCDFDefaultDataColumnInfo(nc, 
//                                		dataType, infoMap);
//                                ncFile.close();
//                            } catch (IOException e) {
//                                return di;
//                            }
//                        } else {
//                            return di;
//                        }
//                        if (colInfoSet == null) {
//                            return di;
//                        }
//                    } else {
//    					colInfoSet = new SGDataColumnInfoSet(colInfo, null);
//                    }
                    
                    SGDataColumnInfoSet colInfoSet = pfData.getColumnInfoSet();
                    if (colInfoSet == null) {
                        if (SGDataUtility.isArrayData(dataType)) {
                        	// get default column information
                        	colInfoSet = this.getDefaultDataColumnInfo(fileName, dataType, 
                        			infoMap, true, versionNumber);
                        } else if (SGDataUtility.isNetCDFData(dataType)) {
                            NetcdfFile ncFile = null;
                            try {
                                if (NetcdfFile.canOpen(fileName)==false) {
                                    return di;
                                }
                                ncFile = NetcdfFile.open(fileName);
                                try {
                                    SGNetCDFFile nc = new SGNetCDFFile(ncFile);
                                    infoMap.put(KEY_DATA_SOURCE, nc);
                                    colInfoSet = this.getNetCDFDefaultDataColumnInfo(nc, dataType, infoMap);
                                } finally {
                                    ncFile.close();
                                }
                            } catch (Exception e) {
                                return di;
                            }
                        } else {
                            return di;
                        }
                        if (colInfoSet == null) {
                            return di;
                        }
                    }
                    if (SGDataUtility.isNetCDFDimensionData(dataType)) {
                        Map pfInfoMap = pfData.getInfoMap();
                        Object startObj = pfInfoMap.get(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_START);
                        Object endObj = pfInfoMap.get(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_END);
                        Object stepObj = pfInfoMap.get(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_STEP);
                        if (startObj==null || endObj==null || stepObj==null) {
                            SGDataColumnInfo[] columns = colInfoSet.getDataColumnInfoArray();
                            SGNetCDFDataColumnInfo[] nCols = new SGNetCDFDataColumnInfo[columns.length];
                            for (int i = 0; i < nCols.length; i++) {
                                nCols[i] = (SGNetCDFDataColumnInfo)columns[i];
                            }
                            SGDataUtility.updatePickupParameters(infoMap, nCols);
                        }
                        if (startObj!=null) {
                            infoMap.put(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_START, startObj);
                        }
                        if (endObj!=null) {
                            infoMap.put(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_END, endObj);
                        }
                        if (stepObj!=null) {
                            infoMap.put(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_STEP, stepObj);
                        }
                    }

                    // create data object
                    SGDataCreator.CreatedDataSet cdSet = this.mDataCreator.create(fileName,
                            colInfoSet, infoMap, progress, versionNumber, mode);
                    if (cdSet == null) {
                        return di;
                    }
                    if (cdSet.getDataLength() == 0) {
                    	return di;
                    }
                    SGDataCreator.CreatedData cd = cdSet.getData(0);
                    SGData firstData = cd.getData();
                    try {
                        if (data.setData(firstData) == false) {
                            return di;
                        }
                    } finally {
                    	// disposes data
                    	firstData.dispose();
                    }
                    
                } catch (FileNotFoundException ex) {
                    return SGIConstants.FILE_OPEN_FAILURE;
                }

            } else if (wData.hasFigureData()) {
                if (data.setData(wData.getFigureData().getData()) == false) {
                    return ic;
                }
            } else {
                return ic;
            }

            // get the index in the legend
            String str = elData
                    .getAttribute(SGIFigureElement.KEY_INDEX_IN_LEGEND);
            if (str.length() != 0) {
                Number num = SGUtilityText.getInteger(str);
                if (num == null) {
                    return ic;
                }
                indexArray[ii] = num.intValue();
                dataArray[ii] = data;
            } else {
                indexValid = false;
            }

            // create data objects
            if (figure.createDataObjectFromPropertyFile(elData, data, readDataProperty) == false) {
                return ic;
            }
        }

        // sort the order of data objects in legend
        if (indexValid) {
            SGIFigureElementLegend lElement = figure.getLegendElement();
            lElement.sortLegend(dataArray, indexArray);
        }

        SGIFigureElement[] array = figure.getIFigureElementArray();
        for (int ii = 0; ii < array.length; ii++) {
            array[ii].initPropertiesHistory();
        }

        return SGIConstants.SUCCESSFUL_COMPLETION;

    }

    /**
     * 
     * @param elWnd
     * @param wnd
     * @return
     */
    int createFiguresFromPropertyFile(final Element elWnd,
            final SGDrawingWindow wnd, final WrappedData[] wDataArray,
            final boolean readDataProperty, final String versionNumber, final int mode) {
        
        NodeList nList = elWnd.getElementsByTagName(SGFigure.TAG_NAME_FIGURE);
        final int len = nList.getLength();

        for (int ii = 0; ii < len; ii++) {
            Node node = nList.item(ii);
            if (node instanceof Element) {
                Element el = (Element) node;
                ArrayList<WrappedData> dataList = new ArrayList<WrappedData>();
                for (int jj = 0; jj < wDataArray.length; jj++) {
                    if (wDataArray[jj].hasFigureData()) {
                        FigureData fd = wDataArray[jj].getFigureData();
                        if (fd.getSerialFigureNo() == ii) {
                            dataList.add(wDataArray[jj]);
                        }
                    } else {
                        SGPropertyFileData pfData = wDataArray[jj].getPropertyFileData();
                        if (pfData.getFigureId() == ii + 1) {
                            String fileName = pfData.getFileName();
                            if (fileName != null && !SGPropertyDataFileChooserWizardDialog.NO_DATA.equals(fileName)) {
                                dataList.add(wDataArray[jj]);
                            }
                        }
                    }
                }
                final int ret = this.createSingleFigureFromPropertyFile(el,
                        wnd, dataList, readDataProperty, versionNumber, mode);
                if (ret != SGIConstants.SUCCESSFUL_COMPLETION) {
                    return ret;
                }
            }
        }
        
        return SGIConstants.SUCCESSFUL_COMPLETION;
    }

    /**
     * The class for dropped data file.
     */
    private static class DroppedData {

    	Point pos = null;
    	
        File file = null;

        DroppedData(Point pos, File file) {
            super();
            this.pos = pos;
            this.file = file;
        }
    }

    /**
     * A file drag and dropped into a window.
     */
    private DroppedData mDroppedFile = null;

    /**
     * Called when text data files are dropped onto the window.
     * 
     * @param x
     *            x coordinate of the dropped point
     * @param y
     *            y coordinate of the dropped point
     * @param wnd
     *            a window to which files are dropped
     * @param fileList
     *            a list of dropped files
     */
    private boolean onTextDataFilesDropped(final Point pos,
            final SGDrawingWindow wnd, final List fileList) {

        // sets up wizard dialogs
    	this.createDataAdditionWizardDialogs(wnd);
    	this.setupDataTypeSelectionWizardDialogConnection(FILETYPE_DATA, DATA_ADDITION_DRAG_AND_DROP);
    	
        // get only the first file
        File file = (File) fileList.get(0);

        // set to an attribute
        this.mDroppedFile = new DroppedData(pos, file);

        if (this.toDataTypeDialog(wnd,
                this.mArrayDataTypeSelectionWizardDialog,
                this.mDroppedFile.file) == false) {
            return false;
        }

        // set the location of wizard dialog
        this.mArrayDataTypeSelectionWizardDialog.setCenter(wnd);

        // show a modal dialog to choose data-type from candidates
        this.mArrayDataTypeSelectionWizardDialog.setVisible(true);

        return true;
    }

    /**
     * Called when netCDF data files are dropped onto the window.
     * 
     * @param x
     *            x coordinate of the dropped point
     * @param y
     *            y coordinate of the dropped point
     * @param wnd
     *            a window to which files are dropped
     * @param fileList
     *            a list of dropped files
     */
    private boolean onNetCDFDataFilesDropped(final Point pos,
            final SGDrawingWindow wnd, final List fileList) {

        // sets up wizard dialogs
    	this.createDataAdditionWizardDialogs(wnd);
    	this.setupDataTypeSelectionWizardDialogConnection(FILETYPE_NETCDF, DATA_ADDITION_DRAG_AND_DROP);
    	
        // get only the first file
        File file = (File) fileList.get(0);
        
        // get commands from global attributes
        NetcdfFile ncFile = null;
        try {
			ncFile = NetcdfFile.open(file.getPath());
		} catch (IOException e) {
			return false;
		}
		
		final Attribute attr = ncFile.findGlobalAttribute("samurai_graph_command");
		if (attr != null) {
        	// Confirms whether to read commands.
            int ret = JOptionPane.showConfirmDialog(wnd, "This netCDF data has commands. Execute them?", 
            		"Confirmation", JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
        	if (ret == JOptionPane.OK_OPTION) {
        		final String dotPath = "\".\"";
        		StringBuffer pathString = new StringBuffer();
        		pathString.append('"');
        		pathString.append(file.getPath());
        		pathString.append('"');
        		this.mPropertyManager.addAlias(COM_DATA, COM_DATA_FILE_PATH, dotPath, pathString.toString());
            	SwingUtilities.invokeLater(new Runnable() {
            		public void run() {
            			// Creates and starts the thread to execute commands.
            			Thread th = new Thread() {
            				public void run() {
                            	String commands = attr.getStringValue();
                                // setup standard output stream
                                mStdoutWriter = new BufferedWriter(new OutputStreamWriter(
                                        System.out));
                                // setup standard input stream
                                mStdinReader = new BufferedReader(new StringReader(commands));
                                try {
                                    // read input recursively
                                	startReadingInput();
                                	mPropertyManager.removeAlias(dotPath);
                    			} catch (IOException e1) {
                    				return;
                    			} finally {
                    	        	try {
                    	        		// Replaces the input stream.
                    	        		mStdinReader.close();
                    	                mStdinReader = new BufferedReader(new InputStreamReader(
                    	                        System.in));
                                    	startReadingInput();
                    				} catch (IOException e) {
                    				}
                    			}
            				}
            			};
            			th.start();        			
            		}
            	});
            	
            	return true;
        	}
		}
        
        // set to an attribute
        this.mDroppedFile = new DroppedData(pos, file);
        
        if (this.toDataTypeDialog(wnd,
                this.mNetCDFDataTypeSelectionWizardDialog,
                this.mDroppedFile.file) == false) {
            return false;
        }
        
        // set the location of wizard dialog
        this.mNetCDFDataTypeSelectionWizardDialog.setCenter(wnd);

        // show a modal dialog to choose data-type from candidates
        this.mNetCDFDataTypeSelectionWizardDialog.setVisible(true);
        
        return true;
    }
    
    /**
     * Sets up the the dialog to setup the netCDF data.
     * 
     * @param ncFile
     *           a netCDF file
     * @param dataType
     *           the type of data
     * @param dg
     *           the dialog to setup netCDF data
     * @return true if succeeded
     */
    private boolean setupNetCDFDataSetupDialog(SGNetCDFFile ncFile, 
            String dataType, Map<String, Object> infoMap, SGNetCDFDataSetupWizardDialog dg) {
        
        Window wnd = dg.getOwner();

        // create column info
        List<SGVariable> varList = ncFile.getVariables();
        final int size = varList.size();
        SGNetCDFDataColumnInfo[] cols = new SGNetCDFDataColumnInfo[size];
        int cnt = 0;
        for (SGVariable var : varList) {
            String name = var.getNameInPriorityOrder();
            cols[cnt] = new SGNetCDFDataColumnInfo(var, name, var.getValueType());
            cnt++;
        }
        SGDataColumnInfoSet colInfoSet = new SGDataColumnInfoSet(cols);

        // set the data
        if (dg.setData(ncFile, dataType, colInfoSet, infoMap, true) == false) {
            SGUtility.showErrorMessageDialog(wnd, MSG_INCOMPLETE_NETCDF_DATA_FILE,
                    TITLE_ERROR);
            return false;
        }
        
        // pack the dialog
        dg.pack();
        
        return true;
    }
        
    /**
     * A wrapper class for data object.
     * This class object has one of two atttibutes: <code>SGPropertyFileData</code> 
     * or <code>FigureData</code>.
     * The former is used when a property file is read, and the latter is used 
     * in the duplication and copy/paste of figures.
     */
    static class WrappedData {
        FigureData figureData;
        SGPropertyFileData pfData;
        WrappedData(FigureData data) {
            this.figureData = data;
            this.pfData = null;
        }
        WrappedData(SGPropertyFileData pfData) {
            this.pfData = pfData;
            this.figureData = null;
        }
        boolean hasFigureData() {
            return (this.figureData != null);
        }
        FigureData getFigureData() {
            return this.figureData;
        }
        SGPropertyFileData getPropertyFileData() {
            return this.pfData;
        }
    }
    
    /**
     * A wrapped data in a figure.
     * This class object is used in paste and duplication of fiugres.
     * The attribute <code>serialFigureNo</code> is not the figure ID
     * but the serial number of copied figures.
     */
    static class FigureData {
        SGData data;
        int serialFigureNo;
        FigureData(SGData data, int no) {
            this.data = data;
            this.serialFigureNo = no;
        }
        SGData getData() {
            return this.data;
        }
        int getSerialFigureNo() {
            return this.serialFigureNo;
        }
    }
    

    // the prompt
    private static final String PROMPT = "$ ";
    
    // a symbol for the file input stream
    private static final String FILE_INPUT = "<<";

    // the maximum number for the recursion of file input
    private static final int FILE_RECURSION_DEPTH_MAX = 10;

    // the header for a comment line
    private static final String COMMENT_HEADER = "#";

    // the header for a comment line
    private static final String COMMENT_HEADER_2 = "//";

    // a text string that represents the start of a block comment
    private static final String BLOCK_COMMENT_START = "/*";

    // a text string that represents the end of a block comment
    private static final String BLOCK_COMMENT_END = "*/";
    
    // a flag whether the current position is in a block comment
    private boolean mBlockComment = false;

    private void readRecursively(BufferedReader br, BufferedWriter bw,
            int depth, final boolean showPrompt, final boolean inFile
            ) throws IOException {

        // check whether the recursion depth is within range
        depth++;
        if (depth > FILE_RECURSION_DEPTH_MAX) {
            bw.write("Recursion is too deep.\n");
            bw.flush();
            return;
        }

        // infinite loop as a server process
        while (true) {
        	
            if (showPrompt) {
                // display the prompt
                bw.write(PROMPT);
                bw.flush();
            }

            // get a line
            String line = br.readLine();
            if (line == null) {
                break;
            }

            // closes the text field
            this.closeTextField();
            
            // checks whether a dialog is open
        	if (this.isDialogOpen()) {
                bw.write("Dialog is open.\n");
                bw.flush();
        		continue;
        	}
        	
            // trim the line
            String tLine = line.trim();

            // skip an empty line
            if (tLine.length() == 0) {
                continue;
            }

            // skip the comment line
            if (tLine.startsWith(COMMENT_HEADER) || tLine.startsWith(COMMENT_HEADER_2)) {
                continue;
            }

            // start or end a block comment only when the file input sream is opened
            if (inFile) {
//              if (tLine.startsWith(BLOCK_COMMENT_START)) {
//              this.mBlockComment = true;
//              continue;
//            } else if (tLine.startsWith(BLOCK_COMMENT_END)) {
//              this.mBlockComment = false;
//              continue;
//            }
//
//            // skip the current line if it is in a block comment
//            if (this.mBlockComment) {
//              continue;
//            }
                
                // remove block comment.
                int spos = tLine.indexOf(BLOCK_COMMENT_START);
                int epos = tLine.indexOf(BLOCK_COMMENT_END);
                if (this.mBlockComment && ((spos<0 && epos>=0) || (epos>=0 && spos>epos+1))) {
                    // "... */" in block comment
                    tLine = tLine.substring(epos+2);
                    this.mBlockComment = false;
                    spos = tLine.indexOf(BLOCK_COMMENT_START);
                    epos = tLine.indexOf(BLOCK_COMMENT_END);
                }
                
                if (this.mBlockComment) {
                    continue;
                }
                
                while (spos>=0 && epos>=1 && spos+1<epos) {
                    //   "/* ... */"
                    tLine = tLine.substring(0, spos) + tLine.substring(epos+2);
                    spos = tLine.indexOf(BLOCK_COMMENT_START);
                    epos = tLine.indexOf(BLOCK_COMMENT_END);
                }
                
                spos = tLine.indexOf(BLOCK_COMMENT_START);
                if (spos>=0) {
                    // "/* ..."
                    this.mBlockComment = true;
                    tLine = tLine.substring(0, spos);
                }
                tLine = tLine.trim();
                if (tLine.length() == 0) {
                    continue;
                }
            }
            
            // interpret as a file path
            if (tLine.startsWith(FILE_INPUT)) {
                String path = tLine.substring(FILE_INPUT.length());
                path = path.trim();
                if (SGUtilityText.isDoubleQuoted(path)) {
                	path = path.substring(1, path.length() - 1);
                }
                File file = new File(path);
                if (!file.exists()) {
                    String errmsg = getFileNotFoundString(file);
                    bw.write(errmsg);
                    bw.flush();
                    continue;
                }
                SGBufferedFileReader reader = new SGBufferedFileReader(path);
                BufferedReader br2 = reader.getBufferedReader();
                readRecursively(br2, bw, depth, false, true);
                reader.close();
                continue;
            }

            // parse the line and execute the command
            final int ret = this.exec(tLine);
            
            // output the status
            StringBuffer status = new StringBuffer();
            if (ret == SGMainFunctions.STATUS_FAILED) {
                status.append("failed: ");
            } else if (ret == SGMainFunctions.STATUS_NOT_FOUND) {
                status.append("not found: ");
            } else if (ret == SGMainFunctions.STATUS_SUCCEEDED) {
                status.append("succeeded: ");
            } else if (ret == SGMainFunctions.STATUS_PARTIALLY_FAILED) {
            	status.append("partially failed: ");
            }
            status.append(tLine);
            status.append('\n');
            bw.write(status.toString());
            bw.flush();
        }
    }

    private void readCommand(final String scriptFileName) {
    	new CommandThread(scriptFileName);
    }
    
    class CommandThread extends Thread {
    	
    	private String mScriptFileName = null;
    	
    	CommandThread(final String script) {
    		super();
    		this.mScriptFileName = script;
    		this.start();
    	}
    	
        /**
         * Runs the thread.
         */
        public void run() {

            File sf = new File(this.mScriptFileName);
            SGBufferedFileReader reader = null;
            try {
            	reader = new SGBufferedFileReader(this.mScriptFileName);
                BufferedReader brs = reader.getBufferedReader();
                try {
                    // read the input script file
                    readRecursively(brs, mStdoutWriter, 0, false, true);
                    
                    // show a prompt
                    mStdoutWriter.write(PROMPT);
                    mStdoutWriter.flush();
                } catch (IOException ex) {
                }
            } catch (FileNotFoundException e) {
                String errmsg = getFileNotFoundString(sf);
                try {
                    mStdoutWriter.write(errmsg);
                    mStdoutWriter.flush();
                } catch (IOException e1) {
                }
			} catch (IOException e) {
			} finally {
				if (reader != null) {
					reader.close();
				}
            }
        }
    }

    private String getFileNotFoundString(File file) {
        StringBuffer sb = new StringBuffer();
        sb.append("file not found: ");
        sb.append(file.getPath());
        sb.append('\n');
        return sb.toString();
    }

    
    /**
     * Splits focused SXY type data into multiple data.
     * 
     * @param wnd
     *           a window
     * @return true if succeeded
     */
    boolean splitSXYData(final SGDrawingWindow wnd) {
        Boolean result = null;
        wnd.setWaitCursor(true);
        if (!USE_FOXTROT) {
            result = SGMainFunctionsSplitMerge.splitSXYDataSub(wnd);
        } else {
            try {
                result = (Boolean) Worker.post(new Task() {
                    public Object run() throws Exception {
                        return SGMainFunctionsSplitMerge.splitSXYDataSub(wnd);
                    }
                });
            } catch (Exception ex) {
                result = Boolean.FALSE;
                ex.printStackTrace();
            }
        }
        wnd.setWaitCursor(false);
        return result.booleanValue();
    }
    
    /**
     * Merge focused SXY type data
     * which was splitted from multiple graph data
     * into one multiple graph data.
     * 
     * @param wnd
     *           a window
     * @return true if succeeded
     */
    boolean mergeSXYData(final SGDrawingWindow wnd) {
        Boolean result = null;
        wnd.setWaitCursor(true);
        if (!USE_FOXTROT) {
            result = SGMainFunctionsSplitMerge.mergeSXYDataSub(wnd);
        } else {
            try {
                result = (Boolean) Worker.post(new Task() {
                    public Object run() throws Exception {
                        return SGMainFunctionsSplitMerge.mergeSXYDataSub(wnd);
                    }
                });
            } catch (Exception ex) {
                result = Boolean.FALSE;
                ex.printStackTrace();
            }
        }
        wnd.setWaitCursor(false);
        return result.booleanValue();
    }
    
    /**
     * Shows the memory information.
     * 
     * @param wnd
     *           the window
     */
    public void showMemoryInfo(SGDrawingWindow wnd) {
        DecimalFormat fByte = new DecimalFormat("#,###KB");
        DecimalFormat fRatio = new DecimalFormat("##.#");
        final long free = Runtime.getRuntime().freeMemory() / 1024;
        final long total = Runtime.getRuntime().totalMemory() / 1024;
        final long max = Runtime.getRuntime().maxMemory() / 1024;
        final long used = total - free;
        final double ratio = (used * 100 / (double)total);
        StringBuffer sb = new StringBuffer();
        sb.append("total=");
        sb.append(fByte.format(total));
        sb.append(", used=");
        sb.append(fByte.format(used));
        sb.append(" (");
        sb.append(fRatio.format(ratio));
        sb.append("%), max=");
        sb.append(fByte.format(max));
        SGUtility.showMessageDialog(wnd, sb.toString(), "Memory Info", 
        		JOptionPane.INFORMATION_MESSAGE);
    }
    
    static class TransformedData {
        SGData data = null;
        String name = null;
        int figureId = -1;
        TransformedData(SGData data, String name, int figureId) {
            super();
            this.data = data;
            this.name = name;
            this.figureId = figureId;
        }
    }
    
    private TransformedData mTransformedData = null;
    
    /**
     * Transforms data object to the other type data.
     * 
     * @param wnd
     *           the window
     * @return true if succeeded
     */
    boolean transformData(SGDrawingWindow wnd) {

        this.createDataAdditionWizardDialogs(wnd);

        List<SGFigure> fList = wnd.getVisibleFigureList();
        for (SGFigure f : fList) {

            // get focused data objects
            SGIFigureElementGraph gElement = f.getGraphElement();
            List<SGData> dataList = gElement.getFocusedDataList();

            if (dataList.size() > 0) {
                SGData data = dataList.get(0);
                String name = gElement.getDataName(data);
                this.mTransformedData = new TransformedData(data, name,
                        f.getID());

                if (SGDataUtility.isArrayData(data)) {

                    // set the location of wizard dialog
                    this.mArrayDataTypeSelectionWizardDialog.setCenter(wnd);

                    // show a modal dialog to choose data-type from candidates
                    this.mArrayDataTypeSelectionWizardDialog.setVisible(true);

                } else if (SGDataUtility.isNetCDFData(data)) {

                    // set the location of wizard dialog
                    this.mNetCDFDataTypeSelectionWizardDialog.setCenter(wnd);

                    // show a modal dialog to choose data-type from candidates
                    this.mNetCDFDataTypeSelectionWizardDialog.setVisible(true);

                } else {
                    throw new Error("Unsupported data type: " + data.getDataType());
                }

                break;
            }
        }

        return true;
    }
    
    private boolean addDataByDataTransformation(final ActionEvent e) {
        TransformedData transformedData =
            new TransformedData(
                    this.mTransformedData.data,
                    this.mTransformedData.name,
                    this.mTransformedData.figureId);
        
        SGMainFunctionsTransform transform =
            new SGMainFunctionsTransform(this,
                    this.mArrayDataTypeSelectionWizardDialog,
                    this.mDataColumnSelectionWizardDialog,
                    this.mNetCDFDataTypeSelectionWizardDialog,
                    this.mNetCDFSetupWizardDialog,
                    this.mPlotTypeSelectionWizardDialog,
                    transformedData);
        
        return transform.addDataByDataTransformation(e);
    }

    public void windowOpened(WindowEvent e) {
    }

    /**
     * Invoded when a wizard dialog for data addition is closed.
     * 
     */
    public void windowClosing(WindowEvent e) {
        Object source = e.getSource();
        if (this.isDataAdditionDialog(source)) {
            this.clearTemporaryData();
        }
    }
	
    void clearTemporaryData() {
        // clear dropped file
        this.mDroppedFile = null;

        // clear transformed data
        this.mTransformedData = null;
    }

    public void windowClosed(WindowEvent e) {
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
    }

    public void windowActivated(WindowEvent e) {
    }

    public void windowDeactivated(WindowEvent e) {
    }
    
    /**
     * 
     * @param dataType
     * @param infoMap
     * @param colInfoArray
     * @return Set of column information. Return null if failed to get default column types.
     */
    SGDataColumnInfoSet createColumnInfoSet(
            String dataType, Map<String, Object> infoMap,
            SGDataColumnInfo[] colInfoArray) {

        SGDataColumnInfo[] aColInfoArray =
            SGApplicationUtility.getAdditionalInfoArray(dataType, infoMap, colInfoArray);

        SGDataColumnInfo[] allInfoArray = new SGDataColumnInfo[colInfoArray.length + aColInfoArray.length];
        for (int ii = 0; ii < colInfoArray.length; ii++) {
            allInfoArray[ii] = colInfoArray[ii];
        }
        for (int ii = 0; ii < aColInfoArray.length; ii++) {
            allInfoArray[ii + colInfoArray.length] = aColInfoArray[ii];
        }

        List<SGDataColumnInfo> columnInfoList = new ArrayList<SGDataColumnInfo>(Arrays.asList(allInfoArray));

        // get default column types and set the each column information
        SGDefaultColumnTypeUtility.DefaultColumnTypeResult result =
            SGDefaultColumnTypeUtility.getDefaultColumnTypes(dataType, columnInfoList, infoMap);
        if (result.isSucceeded() == false) {
            return null;
        }
        
        String[] columnTypes = result.getDefaultColumnTypes();
        for (int ii = 0; ii < columnTypes.length; ii++) {
            allInfoArray[ii].setColumnType(columnTypes[ii]);
        }
        
        SGDataColumnInfoSet colInfoSet = new SGDataColumnInfoSet(allInfoArray);
        return colInfoSet;
    }

    /**
     * Sets the dimension origin and step.
     * 
     * @param infoMap map
     * @param dg data setup dialog
     * @return
     */
    static boolean addDimensionValuesToInfoMap(
            final Map<String, Object> infoMap,
            final SGNetCDFDataSetupWizardDialog dg) {
        infoMap.put(KEY_MULTIPLE_VARIABLE,
                dg.isVariableDataType());
        infoMap.put(KEY_MULTIPLE_DIMENSION_START,
                dg.getMultipleDimensionStart());
        infoMap.put(KEY_MULTIPLE_DIMENSION_END,
                dg.getMultipleDimensionEnd());
        infoMap.put(KEY_MULTIPLE_DIMENSION_STEP,
                dg.getMultipleDimensionStep());
        return true;
    }
    
    /**
     * Add values which are selected in plot type selection dialog into infoMap.
     * 
     * @param infoMap
     */
    static void addPlotTypeSelectionValuesToInfoMap(
            final Map<String, Object> infoMap,
            final SGPlotTypeSelectionWizardDialog dialog) {
        infoMap.put(SGPlotTypeConstants.KEY_PLOT_TYPE_SELECTED,
                Boolean.TRUE);
        infoMap.put(SGPlotTypeConstants.KEY_PLOT_TYPE_LINE,
                Boolean.valueOf(dialog.isLineSelected()));
        infoMap.put(SGPlotTypeConstants.KEY_PLOT_TYPE_SYMBOL,
                Boolean.valueOf(dialog.isSymbolSelected()));
        infoMap.put(SGPlotTypeConstants.KEY_PLOT_TYPE_BAR,
                Boolean.valueOf(dialog.isBarSelected()));
        infoMap.put(SGPlotTypeConstants.KEY_PLOT_TYPE_ERRORBAR_ONLINE,
                Boolean.valueOf(dialog.isErrorBarPlaceLineSymbolSelected()));
    }
    
    /**
     * Proceeds plugin query message.
     * 
     * @param message has message string and object of content.
     */
    void updatePluginsMessage(final SGPluginsQueryMessage message) {
        if (SGPluginsQueryMessage.MENUCMD_OUTPUT_TO_FILE_IS_ENABLED.equals(message.getMessage())) {
            message.set(Boolean.valueOf(this.mPluginManager.hasOutputPlugins()));
        } else if (SGPluginsQueryMessage.MENUCMD_EXEC_OUTPUT_TO_FILE.equals(message.getMessage())) {
            SGDrawingWindow wnd = this.mWindowManager.getCurrentWindow();
            Object obj = message.get();
            if (obj instanceof SGData) {
                try {
                    this.mPluginManager.doOutputToFile(wnd, (SGData)obj);
                } catch (Exception e) {
                    SGUtility.showErrorMessageDialog(wnd, "Failed to output data to file", TITLE_ERROR);
                }
            }
        }
    }

}
