package jp.riken.brain.ni.samuraigraph.application;

import java.awt.event.ActionEvent;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import jp.riken.brain.ni.samuraigraph.application.SGMainFunctions.TransformedData;
import jp.riken.brain.ni.samuraigraph.base.SGData;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfoSet;
import jp.riken.brain.ni.samuraigraph.base.SGDialog;
import jp.riken.brain.ni.samuraigraph.base.SGDrawingWindow;
import jp.riken.brain.ni.samuraigraph.base.SGFigure;
import jp.riken.brain.ni.samuraigraph.base.SGIDataSource;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityText;
import jp.riken.brain.ni.samuraigraph.data.SGArrayData;
import jp.riken.brain.ni.samuraigraph.data.SGDataColumn;
import jp.riken.brain.ni.samuraigraph.data.SGDataUtility;
import jp.riken.brain.ni.samuraigraph.data.SGIDataColumnTypeConstants;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFData;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFFile;
import jp.riken.brain.ni.samuraigraph.data.SGSamplingDataColumn;
import jp.riken.brain.ni.samuraigraph.data.SGTextDataFile;
import jp.riken.brain.ni.samuraigraph.data.SGVariable;

class SGMainFunctionsTransform {
    
    private final SGMainFunctions mMain;
    
    /**
     * A wizard dialog to select the data type.
     */
    private final SGArrayDataTypeSelectionWizardDialog mArrayDataTypeSelectionWizardDialog;

    /**
     * A wizard dialog to select data columns.
     */
    private final SGArrayDataColumnSelectionWizardDialog mDataColumnSelectionWizardDialog;

    /**
     * A wizard dialog to select the data type.
     */
    private final SGNetCDFDataTypeSelectionWizardDialog mNetCDFDataTypeSelectionWizardDialog;

    /**
     * A wizard dialog to setup the netCDF data.
     */
    private final SGNetCDFDataSetupWizardDialog mNetCDFSetupWizardDialog;
    
    /**
     * A wizard dialog to select the plot type if the data type is scalar-xy.
     */
    private final SGPlotTypeSelectionWizardDialog mPlotTypeSelectionWizardDialog;
    
    private final TransformedData mTransformedData;
    
    SGMainFunctionsTransform(final SGMainFunctions mainFunctions,
            final SGArrayDataTypeSelectionWizardDialog arrayDataTypeSelectionWizardDialog,
            final SGArrayDataColumnSelectionWizardDialog dataColumnSelectionWizardDialog,
            final SGNetCDFDataTypeSelectionWizardDialog netCDFDataTypeSelectionWizardDialog,
            final SGNetCDFDataSetupWizardDialog netCDFSetupWizardDialog,
            final SGPlotTypeSelectionWizardDialog plotTypeSelectionWizardDialog,
            final TransformedData transformedData) {
        
        this.mMain = mainFunctions;
        this.mArrayDataTypeSelectionWizardDialog = arrayDataTypeSelectionWizardDialog;
        this.mDataColumnSelectionWizardDialog = dataColumnSelectionWizardDialog;
        this.mNetCDFDataTypeSelectionWizardDialog = netCDFDataTypeSelectionWizardDialog;
        this.mNetCDFSetupWizardDialog = netCDFSetupWizardDialog;
        this.mPlotTypeSelectionWizardDialog = plotTypeSelectionWizardDialog;
        
        this.mTransformedData = transformedData;
    }

    /**
     *  
     * @param e
     * @return
     */
    boolean addDataByDataTransformation(final ActionEvent e) {
        Object source = e.getSource();
        SGWizardDialog dg = (SGWizardDialog) source;
        String command = e.getActionCommand();

        // cancel or previous
        if (command.equals(SGDialog.CANCEL_BUTTON_TEXT)) {
            dg.setVisible(false);
            this.mMain.clearTemporaryData();
        } else if (command.equals(SGDialog.PREVIOUS_BUTTON_TEXT)) {
            dg.showPrevious();
        } else if (command.equals(SGDialog.NEXT_BUTTON_TEXT)) {

            if (dg instanceof SGDataTypeSelectionWizardDialog) {
                // current is column selection dialog.
                SGDataTypeSelectionWizardDialog prev = (SGDataTypeSelectionWizardDialog) dg;

                // set invisible the dialog
                prev.setVisible(false);

                // get selected file type
                String dataType = prev.getSelectedDataType();
                if (dataType == null) {
                    SGDrawingWindow wnd = prev.getOwnerWindow();
                    SGUtility.showErrorMessageDialog(wnd,
                            SGMainFunctions.ERRMSG_TO_GET_DATA_TYPE,
                            SGIApplicationTextConstants.TITLE_ERROR);
                    return false;
                }

                // create a map of information
                Map<String, Object> infoMap = SGMainFunctions.createInfoMap(dataType, prev);
                
                // create default data name
                String dataName = this.mTransformedData.name;

                // put into the infoMap
                infoMap.put(SGIDataColumnTypeConstants.KEY_DATA_NAME, dataName);

                if (dg.equals(this.mArrayDataTypeSelectionWizardDialog)) {
                    this.mMain.setupPlotTypeSelectionWizardDialogConnection(
                            SGIApplicationConstants.FILETYPE_DATA, dataType);
                    
                    SGArrayData aData = (SGArrayData) this.mTransformedData.data;
                    infoMap.put(SGIDataColumnTypeConstants.KEY_DATA_SOURCE, aData.getDataSource());
                    if (this.makeTransition(
                            this.mArrayDataTypeSelectionWizardDialog,
                            this.mDataColumnSelectionWizardDialog,
                            (SGArrayData) this.mTransformedData.data, dataType,
                            infoMap) == false) {
                        return false;
                    }
                } else if (dg.equals(this.mNetCDFDataTypeSelectionWizardDialog)) {
                    this.mMain.setupPlotTypeSelectionWizardDialogConnection(
                            SGIApplicationConstants.FILETYPE_NETCDF, dataType);
                    
                    SGNetCDFData nData = (SGNetCDFData) this.mTransformedData.data;
                    infoMap.put(SGIDataColumnTypeConstants.KEY_DATA_SOURCE, nData.getDataSource());
                    if (this.makeTransition(
                            this.mNetCDFDataTypeSelectionWizardDialog,
                            this.mNetCDFSetupWizardDialog,
                            (SGNetCDFData) this.mTransformedData.data, dataType,
                            infoMap) == false) {
                        return false;
                    }
                }
                
            } else if (dg.equals(this.mDataColumnSelectionWizardDialog)) {
                dg.showNext();
            } else if (dg.equals(this.mNetCDFSetupWizardDialog)) {
                dg.showNext();
            }
            
        } else if (command.equals(SGDialog.OK_BUTTON_TEXT)) {
            
            try {
                if (this.addDataByTransformation(dg) == false) {
                    return false;
                }
            } finally {
                this.mMain.clearTemporaryData();
            }
        }

        return true;
    }

    /**
     * Makes the transition between wizard dialogs.
     * 
     * @param prev
     *            previous dialog to select data type
     * @param next
     *            next dialog to select column
     * @param data
     *            an array data
     * @param dataType
     *            the type of data
     * @param infoMap
     *            the information map
     * @return true if succeeded
     */
    private boolean makeTransition(
            SGArrayDataTypeSelectionWizardDialog prev,
            SGArrayDataColumnSelectionWizardDialog next, SGArrayData data,
            String dataType, Map<String, Object> infoMap) {

        // creates column information set
        SGDataColumnInfoSet colInfoSet = this.createColumnInfoList(data.getDataFile(), 
                        dataType, infoMap);
        if (null==colInfoSet) {
            return false;
        }
        
        // set information to the dialog for data column selection
        if (next.setData(dataType, colInfoSet, infoMap, false)==false) {
            return false;
        }

        // show the dialog to select data columns
        next.setCenter(prev.getOwner());
        next.setVisible(true);

        return true;
    }

    /**
     * Makes the transition between wizard dialogs.
     * 
     * @param prev
     *            previous dialog to select data type
     * @param next
     *            next dialog to select column
     * @param data
     *            an array data
     * @param dataType
     *            the type of data
     * @param infoMap
     *            the information map
     * @return true if succeeded
     */
    private boolean makeTransition(
            SGNetCDFDataTypeSelectionWizardDialog prev,
            SGNetCDFDataSetupWizardDialog next, SGNetCDFData data,
            String dataType, Map<String, Object> infoMap) {

        // creates column information set
        SGDataColumnInfoSet colInfoSet = this.createColumnInfoList(data, 
                        dataType, infoMap);
        if (null==colInfoSet) {
            return false;
        }
                
        // set information to the dialog for data column selection
        SGNetCDFFile ncfile = (SGNetCDFFile) infoMap.get(SGIDataColumnTypeConstants.KEY_DATA_SOURCE);
        if (next.setData(ncfile, dataType, colInfoSet, infoMap, false)==false) {
            return false;
        }
        next.pack();

        // show the dialog to select data columns
        next.setCenter(prev.getOwner());
        next.setVisible(true);

        return true;
    }
    
    private SGDataColumnInfo[] removeAdditionalColumnInfo(SGDataColumnInfo[] colInfo) {
        List<SGDataColumnInfo> newColInfo = new ArrayList<SGDataColumnInfo>();
        
        for (int i = 0; i < colInfo.length; i++) {
            if (SGIDataColumnTypeConstants.VALUE_TYPE_SAMPLING_RATE.equals(colInfo[i].getValueType())==false) {
                newColInfo.add(colInfo[i]);
            }
        }
        
        return newColInfo.toArray(new SGDataColumnInfo[newColInfo.size()]);
    }
    
    private boolean isContainedAdditionalColumnInfo(SGDataColumnInfo[] colInfo) {
        for (int i = 0; i < colInfo.length; i++) {
            if (SGIDataColumnTypeConstants.VALUE_TYPE_SAMPLING_RATE.equals(colInfo[i].getValueType())) {
                return true;
            }
        }
        return false;
    }

    private SGDataColumnInfoSet createColumnInfoList(SGTextDataFile f,
                String dataType, Map<String, Object> infoMap) {
        
        SGDataColumn[] columns = f.getDataColumns();
        
        // get column information: title and value type
        SGDataColumnInfo[] colArray = new SGDataColumnInfo[columns.length];
        for (int ii = 0; ii < colArray.length; ii++) {
            colArray[ii] =
                new SGDataColumnInfo(columns[ii].getTitle(), columns[ii].getValueType());
        }
        
        colArray = removeAdditionalColumnInfo(colArray);
        
        return this.mMain.createColumnInfoSet(dataType, infoMap, colArray);
    }

    private SGDataColumnInfoSet createColumnInfoList(SGNetCDFData data,
                String dataType, Map<String, Object> infoMap) {
        
        SGNetCDFFile ncfile = data.getNetcdfFile();
        List<SGVariable> varList = ncfile.getVariables();
        
        // get column information: title and value type
        SGNetCDFDataColumnInfo[] colArray = new SGNetCDFDataColumnInfo[varList.size()];
        for (int ii = 0; ii < colArray.length; ii++) {
            SGVariable var = varList.get(ii);
            int origin = 0;
            if (var.isCoordinateVariable()) {
                origin = data.getOrigin(var.getName());
            }
            colArray[ii] = new SGNetCDFDataColumnInfo(var, var.getNameInPriorityOrder(), var.getValueType(), origin);
        }
        
        return this.mMain.createColumnInfoSet(dataType, infoMap, colArray);
    }

    /**
     * Adds data by transforming an existing data.
     * 
     * @param dg
     *            an event source
     * @return true if succeeded
     */
    private boolean addDataByTransformation(SGWizardDialog dg) {

        SGDrawingWindow wnd = dg.getOwnerWindow();
        final int figureID = this.mTransformedData.figureId;
        SGFigure fig = wnd.getFigure(figureID);
        SGData data = this.mTransformedData.data;
        
        if (dg.equals(this.mArrayDataTypeSelectionWizardDialog)
                || dg.equals(this.mDataColumnSelectionWizardDialog)) {
            // text data
                
            if (addArrayDataByTransformation(wnd, figureID, fig, data, dg) == false) {
                return false;
            }

        } else if (dg.equals(this.mNetCDFSetupWizardDialog)
                || dg.equals(this.mNetCDFDataTypeSelectionWizardDialog)) {
            // netCDF data
            
            if (addNetCDFDataByTransformation(wnd, figureID, fig, data, dg) == false) {
                return false;
            }
            
        } else if (dg.equals(this.mPlotTypeSelectionWizardDialog)) {
            
            if (dg.getPrevious().equals(this.mDataColumnSelectionWizardDialog)) {
                if (addArrayDataByTransformation(wnd, figureID, fig, data, dg) == false) {
                    return false;
                }
            } else if (dg.getPrevious().equals(this.mNetCDFSetupWizardDialog)) {
                if (addNetCDFDataByTransformation(wnd, figureID, fig, data, dg) == false) {
                    return false;
                }
            } else {
                throw new InternalError("dg.getPrevious() equals null.");
            }
            
        } else {
            SGUtility.showErrorMessageDialog(wnd,
                    SGMainFunctions.ERRMSG_TO_DRAW_GRAPH,
                    SGIApplicationTextConstants.TITLE_ERROR);
                return false;
        }
        
        wnd.notifyToRoot();

        return true;
    }
    
    /**
     * Adds data by transforming an existing text data.
     * 
     * @param wnd
     * @param figureID
     * @param fig
     * @param data
     * @param dg
     * @return true if succeeds
     */
    private boolean addArrayDataByTransformation(
            final SGDrawingWindow wnd, final int figureID, final SGFigure fig,
            SGData data, SGWizardDialog dg) {
        
        final SGDataCreator.CreatedDataSet cdSet;
        Map<String, Object> infoMap = null;
        
        SGArrayData aData = (SGArrayData) data;
        SGTextDataFile file = aData.getDataFile();

        // get data type
        String dataType = this.mArrayDataTypeSelectionWizardDialog.getSelectedDataType();
        if (dataType == null) {
            SGUtility.showErrorMessageDialog(wnd,
                    SGMainFunctions.ERRMSG_TO_GET_DATA_TYPE,
                    SGIApplicationTextConstants.TITLE_ERROR);
            return false;
        }

        // get information
        infoMap = SGMainFunctions.createInfoMap(dataType, this.mArrayDataTypeSelectionWizardDialog);
        infoMap.put(SGIDataColumnTypeConstants.KEY_DATA_SOURCE, data.getDataSource());

        // get selected column types
        SGDataColumnInfoSet colInfoSet = null;
        String dataName = this.mTransformedData.name;
        if (dg.equals(this.mArrayDataTypeSelectionWizardDialog)) {
            colInfoSet = this.createColumnInfoList(file, dataType, infoMap);
            dataName = this.mTransformedData.name;
            infoMap.put(SGIDataColumnTypeConstants.KEY_DATA_NAME, dataName);
        } else if (dg.equals(this.mDataColumnSelectionWizardDialog)) {
            colInfoSet = this.mDataColumnSelectionWizardDialog.getDataColumnInfoSet();
            dataName = this.mDataColumnSelectionWizardDialog.getDataName();
            infoMap.put(SGIDataColumnTypeConstants.KEY_DATA_NAME, dataName);
        } else if (dg.equals(this.mPlotTypeSelectionWizardDialog)) {
            colInfoSet = this.mDataColumnSelectionWizardDialog.getDataColumnInfoSet();
            dataName = this.mDataColumnSelectionWizardDialog.getDataName();
            infoMap.put(SGIDataColumnTypeConstants.KEY_DATA_NAME, dataName);
            SGMainFunctions.addPlotTypeSelectionValuesToInfoMap(infoMap, this.mPlotTypeSelectionWizardDialog);
        }
        if (colInfoSet == null) {
            SGUtility.showErrorMessageDialog(wnd,
                    SGMainFunctions.ERRMSG_TO_DRAW_GRAPH,
                    SGIApplicationTextConstants.TITLE_ERROR);
            return false;
        }
        
        final SGDataColumn[] columns = file.getDataColumns();
        final int dataLength = columns[0].getLength();
        SGDataColumnInfo[] colInfo = colInfoSet.getDataColumnInfoArray();
        if (colInfo.length!=columns.length) {
            SGDataColumn[] newColumns = new SGDataColumn[colInfo.length];
            for (int i = 0; i < colInfo.length; i++) {
                String valueType = colInfo[i].getValueType();
                if (SGUtilityText.isEqualString(SGIDataColumnTypeConstants.VALUE_TYPE_SAMPLING_RATE, valueType)==false) {
                    newColumns[i] = columns[i];
                } else {
                    Object obj = infoMap.get(SGIDataColumnTypeConstants.KEY_SAMPLING_RATE);
                    if (null!=obj && (obj instanceof Number)) {
                        double samplingRate = ((Number)obj).doubleValue();
                        newColumns[i] = new SGSamplingDataColumn(samplingRate, dataLength);
                    } else {
                        return false;
                    }
                }
            }
            file.setDataColumns(newColumns);
        }
        
        // creates a data object
        cdSet = this.mMain.mDataCreator.create(file, colInfoSet, infoMap, wnd);
        if (cdSet == null) {
            SGUtility.showErrorMessageDialog(wnd,
                    SGMainFunctions.ERRMSG_TO_DRAW_GRAPH,
                    SGIApplicationTextConstants.TITLE_ERROR);
            return false;
        }
        
        // hide the original data
        if (wnd.hideSelectedObjects() == false) {
            return false;
        }

        return this.mMain.addDataToDrawGraph(
                wnd, figureID, null, true, infoMap,
                null, fig, cdSet, dataName, true);
    }
    
    /**
     * Adds data by transforming an existing netCDF data.
     * 
     * @param wnd
     * @param figureID
     * @param fig
     * @param data
     * @param dg
     * @return true if succeeds
     */
    private boolean addNetCDFDataByTransformation(
            final SGDrawingWindow wnd, final int figureID, final SGFigure fig,
            SGData data, SGWizardDialog dg) {
        
        final SGDataCreator.CreatedDataSet cdSet;
        Map<String, Object> infoMap = null;
        
        // get data type
        String dataType = this.mNetCDFDataTypeSelectionWizardDialog.getSelectedDataType();
        if (dataType == null) {
            SGUtility.showErrorMessageDialog(wnd,
                    SGMainFunctions.ERRMSG_TO_GET_DATA_TYPE,
                    SGIApplicationTextConstants.TITLE_ERROR);
            return false;
        }

        // create a information map
        infoMap = SGMainFunctions.createInfoMap(dataType, this.mNetCDFDataTypeSelectionWizardDialog);
        infoMap.put(SGIDataColumnTypeConstants.KEY_DATA_SOURCE, data.getDataSource());

        // get selected column types
        SGDataColumnInfoSet colInfoSet = null;
        String dataName = "";
        if (dg.equals(this.mNetCDFDataTypeSelectionWizardDialog)) {
            SGNetCDFFile nc = (SGNetCDFFile) infoMap.get(SGIDataColumnTypeConstants.KEY_DATA_SOURCE);
            colInfoSet = this.mMain.getNetCDFDefaultDataColumnInfo(nc, dataType, infoMap);
            dataName = this.mTransformedData.name;
            infoMap.put(SGIDataColumnTypeConstants.KEY_DATA_NAME, dataName);
        } else if (dg.equals(this.mNetCDFSetupWizardDialog)) {
            colInfoSet = this.mNetCDFSetupWizardDialog.getDataColumnInfoSet();
            dataName = this.mNetCDFSetupWizardDialog.getDataName();
            infoMap.put(SGIDataColumnTypeConstants.KEY_DATA_NAME, dataName);
        } else if (dg.equals(this.mPlotTypeSelectionWizardDialog)) {
            colInfoSet = this.mNetCDFSetupWizardDialog.getDataColumnInfoSet();
            dataName = this.mNetCDFSetupWizardDialog.getDataName();
            infoMap.put(SGIDataColumnTypeConstants.KEY_DATA_NAME, dataName);
            SGMainFunctions.addPlotTypeSelectionValuesToInfoMap(infoMap, this.mPlotTypeSelectionWizardDialog);
        }

        if (colInfoSet == null) {
            SGUtility.showErrorMessageDialog(wnd,
                    SGMainFunctions.MSG_INCOMPLETE_NETCDF_DATA_FILE,
                    SGIApplicationTextConstants.TITLE_ERROR);
            return false;
        }

        if (dg.equals(this.mNetCDFDataTypeSelectionWizardDialog)) {
            // set default value of dimension origin and step
            if (this.mMain.setupDefaultDimensionValues(dataType, infoMap) == false) {
                return false;
            }
        } else if (dg.equals(this.mNetCDFSetupWizardDialog)) {
            // set dimension origin and step
            if (SGMainFunctions.addDimensionValuesToInfoMap(infoMap, this.mNetCDFSetupWizardDialog) == false) {
                return false;
            }
        }

        // creates data
        cdSet = this.mMain.mDataCreator.create(data.getDataSource(), colInfoSet, infoMap, wnd);
        if (cdSet == null) {
            SGUtility.showErrorMessageDialog(wnd,
                    SGMainFunctions.ERRMSG_TO_DRAW_GRAPH,
                    SGIApplicationTextConstants.TITLE_ERROR);
            return false;
        }
        
        // hide the original data
        if (wnd.hideSelectedObjects() == false) {
            return false;
        }

        return this.mMain.addDataToDrawGraph(
                wnd, figureID, null, true, infoMap,
                null, fig, cdSet, dataName, true);
    }
}
