/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * SGPropertyFileDataDialog.java
 *
 * Created on 2009/07/28, 15:15:38
 */

package jp.riken.brain.ni.samuraigraph.application;

import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

import jp.riken.brain.ni.samuraigraph.application.SGDataCreator.FileColumn;
import jp.riken.brain.ni.samuraigraph.base.SGButton;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfoSet;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.data.SGArrayDataColumnSelectionPanel;
import jp.riken.brain.ni.samuraigraph.data.SGIDataColumnTypeConstants;
import jp.riken.brain.ni.samuraigraph.data.SGIDataSetupTableHolder;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFDataSetupPanel;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFFile;
import jp.riken.brain.ni.samuraigraph.data.SGVariable;
import ucar.nc2.NetcdfFile;

/**
 *
 * @author kuromaru
 */
public abstract class SGPropertyFileDataDialog extends SGSingleFileChooserWizardDialog
        implements PropertyChangeListener, SGIDataColumnTypeConstants {

    /**
     * The title of this dialog.
     */
    public static final String TITLE = "Setup the Data";

    /** Creates new form SGPropertyFileDataDialog */
    public SGPropertyFileDataDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        this.initProperty();
    }

    /** Creates new form SGPropertyFileDataDialog */
    public SGPropertyFileDataDialog(java.awt.Dialog parent, boolean modal) {
        super(parent, modal);
        this.initProperty();
    }

    protected abstract SGIDataSetupTableHolder getDataSetupTableHolder();
        
    // dummy Previous button
    private SGButton mPreviousButton;
    
    // dummy Next button
    private SGButton mNextButton;

    /**
     * The data type.
     */
    protected String mDataType = null;
    
    /**
     * An information map.
     */
    protected Map<String, Object> mInfoMap = null;

//    /**
//     * An array of selected data column information.
//     */
//    protected SGDataColumnInfo[] mDataColumnInfo = null;

    protected SGDataColumnInfoSet mDataColumnInfoSet = null;
    
    /**
     * Version number of the property file.
     */
    protected String mVersionNumber = null;
    
    /**
     * Initialize this dialog.
     */
    private void initProperty() {
        
        // set the title
        this.setTitle(TITLE);

        // create dummy buttons
        this.mPreviousButton = new SGButton(PREVIOUS_BUTTON_TEXT);
        this.mNextButton = new SGButton(NEXT_BUTTON_TEXT);

        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
//                // disable the OK button by default
//                acceptFile(false);

                // add this dialog as an action event listener of the buttons
                getDataSetupTableHolder().getRestoreButton().addActionListener(SGPropertyFileDataDialog.this);
                getDataSetupTableHolder().getClearButton().addActionListener(SGPropertyFileDataDialog.this);
                getDataSetupTableHolder().getComplementButton().addActionListener(SGPropertyFileDataDialog.this);

                // add this dialog as a property change event listener of the table
                getDataSetupTableHolder().getTable().addPropertyChangeListener(SGPropertyFileDataDialog.this);
            }
        });
    }

    /**
     * Called when an action event is thrown.
     * @param e
     *          an action event
     */
    public void actionPerformed(final ActionEvent e) {
        Object source = e.getSource();
        if (source.equals(this.getFileChooserButton())) {
            String fileName = this.getFileName();
            if (this.isAcceptable(fileName)) {
                if (this.getDataSetupTableHolder().checkSelectedItems()) {
                    // enable the OK button
                    acceptFile(true);
                }
            }
        } else if (source.equals(this.getOKButton())) {
            // get selected items and set them to the attribute
            this.mDataColumnInfoSet = this.getDataSetupTableHolder().getDataColumnInfoSet();
        } else if (source.equals(this.getDataSetupTableHolder().getClearButton())) {
            this.getOKButton().setEnabled(false);
        } else if (source.equals(this.getDataSetupTableHolder().getRestoreButton())) {
            this.getOKButton().setEnabled(true);
        } else if (source.equals(this.getDataSetupTableHolder().getComplementButton())) {
            this.getOKButton().setEnabled(true);
        }
        
        // calls the method of the super class
        super.actionPerformed(e);
    }

    protected JButton getPreviousButton() {
        return this.mPreviousButton;
    }

    protected JButton getNextButton() {
        return this.mNextButton;
    }

    /**
     * Clear all data.
     */
    public void clear() {
        
        // clear the table
        this.getDataSetupTableHolder().clear();
        
        // clear the file name
        this.setFileName("");
    }    
    
    /**
     * Sets data.
     * 
     * @param dataType
     *            data type
     * @param infoMap
     *            a map of information
     */
    public void setDataType(final String dataType, final Map<String, Object> infoMap) {
        if (dataType == null || infoMap == null) {
            throw new IllegalArgumentException("dataType == null || infoMap == null");
        }
        this.mDataType = dataType;
        this.mInfoMap = new HashMap<String, Object>(infoMap);
        this.getDataSetupTableHolder().setDataType(dataType);
    }
    
//    /**
//     * Returns selected column types.
//     * 
//     * @return selected column types
//     */
//    public SGDataColumnInfo[] getDataColumnTypes() {
//        return this.getDataSetupTableHolder().getDataColumnTypes();
//    }
    
    /**
     * Overrode to enable or disable the OK button and the table instead of the Next button.
     */
    protected void acceptFile(final boolean b) {
        
        // set enable or disable the OK button
        this.getOKButton().setEnabled(b);
    }

//    /**
//     * Returns an array of data column information.
//     * @return
//     *         an array of data column information
//     */
//    public SGDataColumnInfo[] getDataColumnInfo() {
//        if (this.mDataColumnInfo == null) {
//            return null;
//        } else {
//            return (SGDataColumnInfo[]) mDataColumnInfo.clone();
//        }
//    }

    /**
     * Returns an array of data column information.
     * @return
     *         an array of data column information
     */
    public SGDataColumnInfoSet getDataColumnInfoSet() {
        if (this.mDataColumnInfoSet == null) {
            return null;
        } else {
            return (SGDataColumnInfoSet) this.mDataColumnInfoSet.clone();
        }
    }

    /**
     * Overrode to setup the table.
     * @param e
     *          a document event
     */
    protected void onEdited(DocumentEvent e) {
        super.onEdited(e);
        Document doc = e.getDocument();
        String text = "";
        try {
            text = doc.getText(0, doc.getLength());
        } catch (BadLocationException e1) {
            return;
        }
        
        boolean enable = true;
        
        // check whether the file path is acceptable
        if (this.isAcceptable(text)) {

            SGIDataSetupTableHolder holder = this.getDataSetupTableHolder();

            // parse the data file
            boolean valid = true;
            FileColumn[] colInfo = null;
            NetcdfFile ncFile = null;
            
            if (holder instanceof SGArrayDataColumnSelectionPanel) {
                SGDataCreator creator = new SGDataCreator();
                try {
                    colInfo = creator
                            .parseFileComlumnType(text, this.mDataType, 
                                    true, this.mVersionNumber);
                } catch (FileNotFoundException e1) {
                    SGApplicationUtility.showDataFileInvalidMessageDialog(this);
                    valid = false;
                }
                if (valid && colInfo == null) {
                    SGUtility.showMessageDialog(this,
                            "Data File is invalid.",
                            SGIApplicationTextConstants.TITLE_ERROR,
                            JOptionPane.ERROR_MESSAGE);
                    valid = false;
                }
            } else {
                try {
                    ncFile = NetcdfFile.open(text);
                } catch (IOException e1) {
                    SGUtility.showMessageDialog(this,
                            "Failed to open the data file.",
                            SGIApplicationTextConstants.TITLE_ERROR,
                            JOptionPane.WARNING_MESSAGE);
                    valid = false;
                }
            }
            
            if (valid) {

                // set the new data
                if (holder instanceof SGArrayDataColumnSelectionPanel) {
                	
//                    final int colNum = colInfo.length;
//                    
//                    // set information to the dialog for data columns
//                    SGDataColumnInfo[] infoArray = new SGDataColumnInfo[colNum];
//                    for (int jj = 0; jj < colNum; jj++) {
//                        infoArray[jj] = new SGDataColumnInfo(colInfo[jj].title,
//                                colInfo[jj].valueType);
//                    }
//
//                    SGDataColumnInfo[] addInfoArray = null;
//                    
//                    //
//                    // for older data type
//                    //
//                    if (SGDataTypeConstants.SXY_SAMPLING_DATA.equals(dataType)) {
//                    	
//                    	// replace the data type
//                    	dataType = SGDataTypeConstants.SXY_MULTIPLE_DATA;
//                    	
//                    	// get sampling rate
//                    	Double samplingRate = (Double) infoMap.get(KEY_SAMPLING_RATE);
//                    	if (samplingRate == null) {
//                    		return;
//                    	}
//                    	
//        				// create column info for sampling
//        				SGDataColumnInfo samplingRateColumnInfo = new SGDataColumnInfo(
//        						SGDataUtility.createSamplingRateTitle(samplingRate.doubleValue()), 
//        						SGIDataColumnTypeConstants.VALUE_TYPE_SAMPLING_RATE);
//
//        				// set X_VALUE to the column type
//        				samplingRateColumnInfo.setColumnType(X_VALUE);
//
//        				// create an array of columns
//        				addInfoArray = new SGDataColumnInfo[] { samplingRateColumnInfo };
//        				
//                    } else if (SGDataTypeConstants.SXY_DATE_DATA.equals(dataType)) {
//                    	
//                    	// replace the data type
//                    	dataType = SGDataTypeConstants.SXY_MULTIPLE_DATA;
//                    }
//                    
//                    SGDataColumnInfoSet colInfoSet = new SGDataColumnInfoSet(infoArray, addInfoArray);
                    
                    SGDataColumnInfoSet colInfoSet = SGApplicationUtility
							.getArrayDataColInfoSet(colInfo, this.mDataType, this.mInfoMap);
					String dataTypeNew = SGApplicationUtility
							.getArrayDataType(this.mDataType);

                    SGArrayDataColumnSelectionPanel p = (SGArrayDataColumnSelectionPanel) holder;
                    if (p.setData(dataTypeNew, colInfoSet,
                            this.mInfoMap, true) == false) {
                        enable = false;
                    }
                    
                } else if (holder instanceof SGNetCDFDataSetupPanel) {
                	SGNetCDFFile nc = new SGNetCDFFile(ncFile);
                    List<SGVariable> varList = nc.getVariables();
                    final int colNum = varList.size();
                    
                    // set information to the dialog for data columns
                    SGDataColumnInfo[] colArray = new SGNetCDFDataColumnInfo[colNum];
                    for (int jj = 0; jj < colNum; jj++) {
                    	SGVariable var = varList.get(jj);
                        colArray[jj] = new SGNetCDFDataColumnInfo(var, var.getNameInPriorityOrder(),
                                var.getValueType());
                    }
                    SGDataColumnInfoSet colInfoSet = new SGDataColumnInfoSet(colArray);
                    SGNetCDFDataSetupPanel p = (SGNetCDFDataSetupPanel) holder;
                    if (p.setData(nc, this.mDataType, colInfoSet,
                            this.mInfoMap, true) == false) {
                        enable = false;
                    }                    
                }
                
            } else {
                enable = false;
                this.getDataSetupTableHolder().clear();
            }
            
        } else {
            enable = false;
            this.getDataSetupTableHolder().clear();
        }
        
        if (!enable) {
            // only set the data type
            this.getDataSetupTableHolder().setDataType(this.mDataType);
        }
        
        // enable or disable the components
        this.getOKButton().setEnabled(enable);
        this.getDataSetupTableHolder().getClearButton().setEnabled(enable);
        this.getDataSetupTableHolder().getRestoreButton().setEnabled(enable);
        this.getDataSetupTableHolder().getComplementButton().setEnabled(enable);
    }
    
    /**
     * Called when the component is resized.
     *
     * @param e
     *            a component event
     */
    public void componentResized(ComponentEvent e) {
        super.componentResized(e);
        this.updateTable();
    }

    /**
     * Called when the component is shown.
     *
     * @param e
     *            a component event
     */
    public void componentShown(ComponentEvent e) {
        super.componentShown(e);
        this.updateTable();
    }
    
    /**
     * Update the table.
     */
    private void updateTable() {
        this.getDataSetupTableHolder().updateTable();
    }

    /**
     * Called when the table is changed.
     * 
     * @param evt
     *            an event that the property changes
     */
    public void propertyChange(PropertyChangeEvent evt) {
        Object source = evt.getSource();
        if (source.equals(this.getDataSetupTableHolder().getTable())) {
            String name = evt.getPropertyName();
            if ("tableCellEditor".equals(name)) {
                SwingUtilities.invokeLater(new Runnable(){
                    public void run() {
                        final boolean b = getDataSetupTableHolder().checkSelectedItems();
                        getOKButton().setEnabled(b);
                    }
                });
            }
        }
    }
    
}
