package jp.riken.brain.ni.samuraigraph.base;


/**
 * The set of data columns.
 * 
 */
public class SGDataColumnInfoSet implements Cloneable {
	
	/**
	 * The array of data column information.
	 */
	private SGDataColumnInfo[] mDataColumnInfoArray = null;

	/**
	 * Builds a data column set.
	 * 
	 * @param infoArray
	 *                   the array of data column information
	 */
	public SGDataColumnInfoSet(final SGDataColumnInfo[] infoArray) {
		super();
		if (infoArray == null) {
			throw new IllegalArgumentException("infoArray == null");
		}
		this.mDataColumnInfoArray = (SGDataColumnInfo[]) infoArray.clone();
	}

	/**
	 * Returns the array of data column information.
	 * 
	 * @return the array of data column information
	 */
	public SGDataColumnInfo[] getDataColumnInfoArray() {
		if (this.mDataColumnInfoArray == null) {
			return new SGDataColumnInfo[] {};
		} else {
			return (SGDataColumnInfo[]) this.mDataColumnInfoArray.clone();
		}
	}

    /**
     * Clones this data column.
     * 
     * @return copy of this data column
     */
    public Object clone() {
    	SGDataColumnInfoSet set = null;
        try {
        	set = (SGDataColumnInfoSet) super.clone();
        } catch (CloneNotSupportedException ex) {
            // this shouldn't happen, since we are Cloneable
            throw new InternalError();
        }
        return set;
    }

    @Override
    public String toString() {
        if (null==this.mDataColumnInfoArray) {
            return "[SGDataColumnInfoSet]";
        }
        StringBuffer sb = new StringBuffer("[\n");
        for (int i = 0; i < this.mDataColumnInfoArray.length; i++) {
            sb.append(" ").append(this.mDataColumnInfoArray[i].toString()).append("\n");
        }
        sb.append("]");
        return sb.toString();
    }
}
