/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * SGCoordinateVariablePanel.java
 *
 * Created on 2009/08/18, 13:35:30
 */

package jp.riken.brain.ni.samuraigraph.data;

import java.awt.MediaTracker;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import javax.swing.BoundedRangeModel;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.ImageIcon;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import jp.riken.brain.ni.samuraigraph.base.SGIConstants;


/**
 * A panel for single coordinate variable.
 *
 */
public class SGCoordinateVariablePanel extends javax.swing.JPanel
    implements ActionListener, ChangeListener {

    /**
     * Serial Version UID
     */
    private static final long serialVersionUID = 5417060635224002801L;

    /**
     * The variable.
     */
    private SGVariable mVariable = null;

    /**
     * The range of values.
     */
    private double[] mValues = null;
    
    /**
     * The default constaructor.
     *
     */
    public SGCoordinateVariablePanel() {
        super();
        initComponents();
    }

    /**
     * Builds this component with given parameters.
     * 
     * @param var
     *           the variable
     * @param values
     *           the values
     * @param initIndex
     *           initial index of values
     * @param unit
     *           a text string for the unit
     */
    public SGCoordinateVariablePanel(final SGVariable var,
    		final double[] values, final int initIndex, final String unit) {
        super();
        initComponents();
        
        if (initIndex < 0 || initIndex >= values.length) {
            throw new IllegalArgumentException("Index out of bounds: " + initIndex);
        }
        
        if (var.isCoordinateVariable() == false) {
        	throw new IllegalArgumentException("Given variable is not a coordinate variable.");
        }

        // set to an attribute
        this.mVariable = var;

        // set the name
        final String name = var.getDimension(0).getName();
        this.mLabel.setText(name);
        this.mLabel.setToolTipText(name);
        
        // set the values
        this.mValues = (double[]) values.clone();

        // set the text
        this.setValueText(initIndex);
        
        // set the unit
        String unitString = null;
        if (unit == null) {
            unitString = "";
        } else {
            unitString = unit;
        }
        this.mUnitLabel.setText(unitString);
        this.mUnitLabel.setToolTipText(unitString);

        // setup the slider model
        BoundedRangeModel model = new DefaultBoundedRangeModel();
        model.setMinimum(0);
        model.setMaximum(values.length - 1);
        model.setExtent(0);
        model.setValue(initIndex);
        this.mSlider.setModel(model);
        this.setSliderToolTipText(initIndex);

        // add action event listener
        this.mLeftButton.addActionListener(this);
        this.mRightButton.addActionListener(this);

        // add change event listener
        this.mSlider.addChangeListener(this);

        // create icons
        this.mLeftButton.setIcon(this.createIcon("Minus.png"));
        this.mRightButton.setIcon(this.createIcon("Plus.png"));
        this.mLeftButton.setDisabledIcon(this.createIcon("MinusDisabled.png"));
        this.mRightButton.setDisabledIcon(this.createIcon("PlusDisabled.png"));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        mPanel = new javax.swing.JPanel();
        mLabel = new javax.swing.JLabel();
        mSlider = new javax.swing.JSlider();
        mTextField = new jp.riken.brain.ni.samuraigraph.base.SGTextField();
        mLeftButton = new jp.riken.brain.ni.samuraigraph.base.SGButton();
        mRightButton = new jp.riken.brain.ni.samuraigraph.base.SGButton();
        mUnitLabel = new javax.swing.JLabel();

        mPanel.setPreferredSize(new java.awt.Dimension(400, 36));
        mPanel.setLayout(new java.awt.GridBagLayout());

        setLayout(new java.awt.GridBagLayout());

        mLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        mLabel.setText("Name");
        mLabel.setPreferredSize(new java.awt.Dimension(40, 16));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        add(mLabel, gridBagConstraints);

        mSlider.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(4, 2, 4, 2);
        add(mSlider, gridBagConstraints);

        mTextField.setColumns(6);
        mTextField.setEditable(false);
        mTextField.setText("Value");
        mTextField.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        add(mTextField, gridBagConstraints);

        mLeftButton.setPreferredSize(new java.awt.Dimension(18, 18));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        add(mLeftButton, gridBagConstraints);

        mRightButton.setPreferredSize(new java.awt.Dimension(18, 18));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        add(mRightButton, gridBagConstraints);

        mUnitLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        mUnitLabel.setText("Unit");
        mUnitLabel.setPreferredSize(new java.awt.Dimension(40, 13));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        add(mUnitLabel, gridBagConstraints);
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel mLabel;
    private jp.riken.brain.ni.samuraigraph.base.SGButton mLeftButton;
    private javax.swing.JPanel mPanel;
    private jp.riken.brain.ni.samuraigraph.base.SGButton mRightButton;
    private javax.swing.JSlider mSlider;
    private jp.riken.brain.ni.samuraigraph.base.SGTextField mTextField;
    private javax.swing.JLabel mUnitLabel;
    // End of variables declaration//GEN-END:variables

    /**
     * Called when the slider moves.
     */
    public void stateChanged(ChangeEvent e) {
        Object source = e.getSource();
        if (source.equals(this.mSlider)) {
            final int index = this.getCurrentIndex();
            this.setValueText(index);
            this.setSliderToolTipText(index);
        }
    }

    /**
     * Set the tooltip text to the slider.
     * 
     * @param index
     *           the index of the slider
     */
    private void setSliderToolTipText(final int index) {
        final int len = this.mValues.length - 1;
        StringBuffer sb = new StringBuffer();
        sb.append(index);
        sb.append('/');
        sb.append(len);
        final String text = sb.toString();
        this.mSlider.setToolTipText(text);
        this.mLeftButton.setToolTipText(text);
        this.mRightButton.setToolTipText(text);
    }

    /**
     * Sets the text of value.
     * 
     * @param index
     *           index of values
     */
    private void setValueText(final int index) {
        final double value = this.mValues[index];
        final String text = Double.toString(value);
        this.mTextField.setText(text);
        this.mTextField.setToolTipText(text);
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                mTextField.setCaretPosition(0);
            }
        });
    }
    
    /**
     * Returns the current index.
     * 
     * @return the current index
     */
    public int getCurrentIndex() {
        return this.mSlider.getValue();
    }
    
    /**
     * Returns the current value.
     * 
     * @return the current value
     */
    public double getCurrentValue() {
        return this.mValues[this.getCurrentIndex()];
    }

    /**
     * Sets the components enabled.
     * 
     * @param enabled
     *           true to enable
     */
    public void setComponentsEnabled(final boolean enabled) {
        this.mLabel.setEnabled(enabled);
        this.mSlider.setEnabled(enabled);
        this.mLeftButton.setEnabled(enabled);
        this.mRightButton.setEnabled(enabled);
        this.mTextField.setEnabled(enabled);
    }

    /**
     * Returns the name of variable.
     * 
     * @return the name of variable
     */
    public String getVariableName() {
    	return this.mVariable.getName();
    }

    /**
     * Create an icon object.
     *
     * @param name
     *            name of resource file
     */
    private ImageIcon createIcon(final String name) {
        URL url = this.getClass().getResource(SGIConstants.RESOURCES_DIRNAME + name);
        ImageIcon icon = new ImageIcon(url);
                // set the media tracker
        final MediaTracker mt = new MediaTracker(this);
        mt.addImage(icon.getImage(), 0);
        try {
            mt.waitForAll();
        } catch (InterruptedException ex) {
        }
        return icon;
    }

    /**
     * Called when an action event is invoked.
     *
     * @param e
     *          an action event
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source.equals(this.mLeftButton)) {
            this.mSlider.setValue(this.mSlider.getValue() - 1);
        } else if (source.equals(this.mRightButton)) {
            this.mSlider.setValue(this.mSlider.getValue() + 1);
        }
    }

}
