/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * SGDataDuplicationDialog.java
 *
 * Created on 2009/08/31, 16:16:08
 */

package jp.riken.brain.ni.samuraigraph.data;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.base.SGDialog;

/**
 *
 * @author kuromaru
 */
public abstract class SGDataDuplicationDialog extends SGDialog implements PropertyChangeListener {

    /** Creates new form SGDataDuplicationDialog */
    public SGDataDuplicationDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
    }

    /**
     * Returns the OK button.
     *
     * @return the OK button
     */
    protected abstract JButton getOKButton();

    /**
     * Returns the Cancel button.
     *
     * @return the Cancel button
     */
    protected abstract JButton getCancelButton();
    
    /**
     * Returns an object that has a table to setup data.
     * 
     * @return an object that has a table to setup data
     */
    protected abstract SGIDataSetupTableHolder getTableHolder();
    
    /**
     * Returns the text field to set data name.
     * 
     * @return a text field to set the data name
     */
    public abstract JTextField getDataNameTextField();

    /**
     * Close this dialog.
     */
    protected void onEscKeyTyped() {
        this.onCanceled();
    }

    /**
     * Called when the OK button is pressed.
     *
     * @return true if succeeded
     */
    protected boolean onOK() {
        this.setCloseOption(OK_OPTION);
        this.setVisible(false);
        return true;
    }

    /**
     * Called when the cancel button is pressed.
     *
     * @return true if succeeded
     */
    protected boolean onCanceled() {
        this.setCloseOption(CANCEL_OPTION);
        this.setVisible(false);
        return true;
    }

    /**
     * Called when an action event is invoked.
     *
     * @param e
     *          an action event
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source.equals(this.getOKButton())) {
            this.onOK();
        } else if (source.equals(this.getCancelButton())) {
            this.onCanceled();
        } else if (source.equals(this.getTableHolder().getClearButton())) {
            this.getOKButton().setEnabled(false);
        } else if (source.equals(this.getTableHolder().getRestoreButton())) {
            this.getOKButton().setEnabled(true);
        } else if (source.equals(this.getTableHolder().getComplementButton())) {
            this.getOKButton().setEnabled(true);
        }
    }

    /**
     * Returns selected column types.
     * 
     * @return
     *        selected column types
     */
    public abstract SGDataColumnInfo[] getDataColumnTypes();
    
    public String getDataName() {
        return this.getDataNameTextField().getText();
    }

    /**
     * Called when the table is changed.
     * 
     * @param evt
     *            an event that the property changes
     */
    public void propertyChange(PropertyChangeEvent evt) {
        Object source = evt.getSource();
        if (source.equals(this.getTableHolder().getTable())) {
            String name = evt.getPropertyName();
            if ("tableCellEditor".equals(name)) {
                SwingUtilities.invokeLater(new Runnable(){
                    public void run() {
                        final boolean b = getTableHolder().checkSelectedItems();
                        getOKButton().setEnabled(b);
                    }
                });
            }
        }
    }
    
}
