package jp.riken.brain.ni.samuraigraph.data;



/**
 * The class of data column of the date type.
 *
 */
public class SGDateDataColumn extends SGDataColumn implements SGINumberDataColumn {
    
    /**
     * An array of text strings.
     */
    protected SGDate[] mArray = null;

    /**
     * Builds this object.
     * @param title
     *              a text string for the title
     * @param array
     *              an array of text strings
     */
    public SGDateDataColumn(final String title, final SGDate[] array) {
        
        super(title);
        
        // check
        if (array == null) {
            throw new IllegalArgumentException("array == null");
        }
        
        // set to an attribute
        final SGDate[] dest = new SGDate[array.length];
        System.arraycopy(array, 0, dest, 0, array.length);
        this.mArray = dest;
    }
    
    /**
     * Returns the length of data column.
     * @return
     *         the length of data column
     */
    public int getLength() {
        return this.mArray.length;
    }

    /**
     * Returns the value type of this column.
     * @return
     *         the value type of this column
     */
    public String getValueType() {
        return SGIDataColumnTypeConstants.VALUE_TYPE_DATE;
    }

    /**
     * Dispose this object.
     */
    public void dispose() {
        super.dispose();
        this.mArray = null;
        this.mStringArray = null;
    }

//    /**
//     * Copy the object.
//     * @return
//     *       A copied object.
//     */
//    public Object copy() {
//        return new SGDateDataColumn(this.mTitle, this.mArray);
//    }

    SGDate[] getDateArray() {
        return this.mArray;
    }
    
    /**
     * Returns a value in this data column at given row index.
     * 
     * @param rowIndex
     *            the row index
     * @return a value at given row index
     */
    public Object getValue(final int rowIndex) {
        if (rowIndex > this.mArray.length) {
            throw new IllegalArgumentException("Index out of bounds: "
                    + rowIndex + " > " + this.mArray.length);
        }
        return this.mArray[rowIndex];
    }
    
    /**
     * An array of text string for date objects.
     */
    private String[] mStringArray = null;

    /**
     * Returns an array of text strings.
     * 
     * @return an array of text strings
     */
    public String[] getStringArray() {
        if (this.mStringArray == null) {
            String[] array = new String[this.mArray.length];
            for (int ii = 0; ii < array.length; ii++) {
                array[ii] = this.mArray[ii].toString();
            }
            this.mStringArray = array;
        }
        return (String[]) this.mStringArray.clone();
    }

    /**
     * Clones this data column.
     * 
     * @return
     *         shallow copy of this data column
     */
    public Object clone() {
        SGDateDataColumn col = (SGDateDataColumn) super.clone();
        col.mStringArray = null;
        return col;
    }

    /**
     * Returns a value to export.
     * 
     * @param rowIndex
     *          the row index
     * @return
     *          a value to export
     */
    protected Object getExportedValue(int rowIndex) {
        Object value = this.getValue(rowIndex);
        StringBuffer sb = new StringBuffer();
        sb.append('\"');
        sb.append(value.toString());
        sb.append('\"');
        return sb.toString();
    }

    /**
     * Returns an array of numbers.
     * 
     * @return an array of numbers
     */
    public double[] getNumberArray() {
        final SGDate[] dArray = this.getDateArray();
//        final int num = dArray.length;
//        final double[] values = new double[num];
//        final Calendar cal = Calendar.getInstance();
//        final int ratio = 1000 * 3600 * 24;
//        for (int ii = 0; ii < num; ii++) {
//            final SGDate d = dArray[ii];
//            cal.set(d.year, d.month, d.date);
//            values[ii] = cal.getTimeInMillis() / ratio;
//        }
//        return values;
        return SGDataUtility.getDateValueArray(dArray);
    }

}
