package jp.riken.brain.ni.samuraigraph.data;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

import ucar.ma2.Array;
import ucar.ma2.InvalidRangeException;
import ucar.nc2.Dimension;
import ucar.nc2.Variable;

public class SGDateVariable extends SGVariable {
    
	/**
	 * A array of date objects
	 */
    private SGDate[] mDate = null;

    /**
     * Builds an object with a given variable.
     *
     * @param var
     *           the variable
	 * @param dim
	 *           the dimension
	 * @param stringArray
	 *           an array of text strings
     */
    public SGDateVariable(final Variable var, final String[] stringArray) {
        super(var);

		// check
		if (stringArray == null) {
			throw new IllegalArgumentException("stringArray == null");
		}
        
        try {
        	this.mDate = new SGDate[stringArray.length];
            for (int ii = 0; ii < stringArray.length; ii++) {
            	this.mDate[ii] = new SGDate(stringArray[ii].trim());
            }
        } catch (ParseException e) {
        	this.mDate = null;
        }
    }

	/**
	 * Returns the array of date objects.
	 * 
	 * @return the array of date objects
	 */
    public SGDate[] getDate() {
        if (this.mDate == null) {
            return null;
        }
    	return (SGDate[]) this.mDate.clone();
    }
    
	/**
	 * Returns the array of numbers.
	 * 
	 * @return the array of numbers
	 */
    public double[] getNumberArray() {
        if (this.mDate == null) {
            return null;
        }
        return SGDataUtility.getDateValueArray(this.mDate);
    }
    
	/**
	 * Returns the array of text strings.
	 * 
	 * @return the array of text strings
	 */
    public String[] getStringArray() {
        if (this.mDate==null) {
            return null;
        }
        String[] values = new String[this.mDate.length];
        for (int i = 0; i < values.length; i++) {
            values[i] = this.mDate[i].toString();
        }
        return values;
    }
    
	/**
	 * Returns the value type.
	 * 
	 * @return the value type
	 */
	public String getValueType() {
		return SGIDataColumnTypeConstants.VALUE_TYPE_DATE;
	}

	/**
	 * Returns the list of dimension.
	 * 
	 * @return the list of dimensions
	 */
	public List<Dimension> getDimensions() {
		// returns a list that contains only the first dimension
		List<Dimension> dimList = super.getDimensions();
		return new ArrayList<Dimension>(dimList.subList(0, 1));
	}
	
	/**
	 * Returns the dimension of given index.
	 * 
	 * @param index
	 *           the index of dimension
     * @return the dimension of given index
	 */
	public Dimension getDimension(final int index) {
		if (index > 0) {
			return null;
		}
		return this.mVariable.getDimension(index);
	}

	public Array read(int[] a, int[] b) throws InvalidRangeException, IOException {
		// always returns null
		return null;
	}

}
