package jp.riken.brain.ni.samuraigraph.data;

/**
 * An interface for Scalar-type XY data.
 */
public interface SGISXYTypeData extends SGISXYData, SGITwoDimensionalData {

    /**
     * Returns the x value with the given index.
     * 
     * @param n
     *            index
     * @return x-value with the given index
     */
    public Double getXValue(final int n);

    /**
     * Returns the y value with the given index.
     * 
     * @param n
     *            index
     * @return y-value with the given index
     */
    public Double getYValue(final int n);

    /**
     * Returns the lower error value with the given index if it exists.
     * 
     * @param n
     *            index
     * @return lower error value with the given index
     */
    public Double getLowerErrorValue(final int n);

    /**
     * Returns the upper error value with the given index if it exists.
     * 
     * @param n
     *            index
     * @return upper error value with the given index
     */
    public Double getUpperErrorValue(final int n);

    /**
     * Returns the string with the given index if it exists.
     * 
     * @param n
     *            index
     * @return a string with the given index
     */
    public String getString(final int n);

    /**
     * Returns a copy of x-value array.
     * 
     * @return x-value array
     */
    public double[] getXValueArray();

    /**
     * Returns a copy of y-value array.
     * 
     * @return y-value array
     */
    public double[] getYValueArray();

    /**
     * Returns a copy of lower-error-value array if they exist.
     * 
     * @return lower-error-value array
     */
    public double[] getLowerErrorValueArray();

    /**
     * Returns a copy of upper-error-value array if they exist.
     * 
     * @return upper-error-value array
     */
    public double[] getUpperErrorValueArray();

    /**
     * Returns a copy of string array if they exist.
     * 
     * @return string array
     */
    public String[] getStringArray();
    
    /**
     * Returns whether a given data object has the same tick label resources.
     * 
     * @param data
     *           a data to compare
     * @return true if a given data object has the same tick label resources
     */
    public boolean hasEqualTickLabelResource(SGISXYTypeData data);
    
}
