package jp.riken.brain.ni.samuraigraph.data;

/**
 * An interface for Scalar-type multiple XY data.
 */
public interface SGISXYTypeMultipleData extends SGISXYData, SGITwoDimensionalData {

    /**
	 * Returns the m-th x value at the n-th index.
	 * 
	 * @param m
	 *            the index of data
	 * @param n
	 *            the index of the data point
	 * @return the n-th x-value of the m-th data
	 */
    public Double getXValue(final int m, final int n);

	/**
	 * Returns the m-th y value at the n-th index.
	 * 
	 * @param m
	 *            the index of data
	 * @param n
	 *            the index of the data point
	 * @return the n-th y-value of the m-th data
	 */
    public Double getYValue(final int m, final int n);

    /**
	 * Returns an array of SXYData objects.
	 * 
	 * @return an array of SXYData objects
	 */
    public SGISXYTypeData[] getSXYDataArray();

    /**
	 * Returns an array of multiple SXYData objects.
	 * 
	 * @return an array of multiple SXYData objects
	 */
    public SGISXYTypeMultipleData[] getSXYTypeMultipleDataArray();

    /**
     * Returns a copy of x-value arrays.
     * 
     * @return arrays of x-values
     */
    public double[][] getXValueArray();

    /**
     * Returns a copy of y-value arrays.
     * 
     * @return arrays of y-values
     */
    public double[][] getYValueArray();

    /**
     * Returns the maximum number of SXY data objects that can be obtained from
     * this data object.
     * 
     * @return the maximum number of SXY data objects that can be obtained from
     *         this data object
     */
    public int getSXYDataNumMax();
    
    /**
     * Returns true if this data enables to be split.
     * 
     * @return true if this data enables to be split
     */
    public boolean isSplitEnabled();
    
}
