package jp.riken.brain.ni.samuraigraph.data;

import java.util.HashMap;

import org.w3c.dom.Element;

import ucar.nc2.Dimension;

public class SGSXYNetCDFDateData extends SGSXYNetCDFData {
    
    private boolean mIsTickLabelSubstitue = false;

    public SGSXYNetCDFDateData(final SGNetCDFFile ncfile, SGDataSourceObserver obs,
            final String xName, final String yName, final String lName, final String uName,
            final String ehName, final String tName, final String thName,
            final String timeName, final String serialNumberName) {
        
        super(ncfile, obs, xName, yName, lName, uName, ehName, tName, thName, timeName, serialNumberName);
        
        if (this.mTickLabelVariable==null) {
            if (this.mXVariable instanceof SGDateVariable) {
                this.mTickLabelVariable = this.mXVariable;
                this.mTickLabelHolderVariable = this.mYVariable;
            } else if (this.mYVariable instanceof SGDateVariable) {
                this.mTickLabelVariable = this.mYVariable;
                this.mTickLabelHolderVariable = this.mXVariable;
            }
            this.mIsTickLabelSubstitue = true;
        }
    }
    
    @Override
    public boolean isTickLabelAvailable() {
        return (this.mTickLabelVariable != null);
    }
    
    @Override
    public Boolean isTickLabelHorizontal() {
        if (this.isTickLabelAvailable()) {
            return (this.mTickLabelHolderVariable.equals(this.mYVariable));
        } else {
            return null;
        }
    }

    @Override
    public SGVariable[] getVariables() {
        final boolean be = this.isErrorBarAvailable();
        final boolean bt = this.isTickLabelAvailable() && this.mIsTickLabelSubstitue==false;
        if (be && bt) {
            return new SGVariable[]{ this.mXVariable, this.mYVariable, 
                    this.mLowerErrorVariable, this.mUpperErrorVariable,
                    this.mTickLabelVariable};
        } else if (be && !bt) {
            return new SGVariable[]{ this.mXVariable, this.mYVariable, 
                    this.mLowerErrorVariable, this.mUpperErrorVariable };
        } else if (!be && bt) {
            return new SGVariable[]{ this.mXVariable, this.mYVariable, 
                    this.mTickLabelVariable};
        } else {
            return new SGVariable[]{ this.mXVariable, this.mYVariable };
        }
    }
    
    @Override
    public boolean writeProperty(Element el, final int type) {
        if (super.writeProperty(el, type) == false) {
            return false;
        }
        el.setAttribute(KEY_X_VALUE_NAMES, this.mXVariable.getName());
        el.setAttribute(KEY_Y_VALUE_NAMES, this.mYVariable.getName());
        if (this.isErrorBarAvailable()) {
            el.setAttribute(KEY_LOWER_ERROR_VALUE_NAMES, this.mLowerErrorVariable.getName());
            el.setAttribute(KEY_UPPER_ERROR_VALUE_NAMES, this.mUpperErrorVariable.getName());
            el.setAttribute(KEY_ERROR_BAR_HOLDER_NAMES, this.mErrorBarHolderVariable.getName());
        }
        if (this.isTickLabelAvailable() && this.mIsTickLabelSubstitue==false) {
            el.setAttribute(KEY_TICK_LABEL_NAMES, this.mTickLabelVariable.getOriginalName());
            el.setAttribute(KEY_TICK_LABEL_HOLDER_NAMES, this.mTickLabelHolderVariable.getName());
        }
        return true;
    }
    
    @Override
    public SGISXYTypeMultipleData toMultiple() {
        // Convert this class (SGSXYNetCDFData) to class (SGSXYMultipleVariableNetCDFData).
        final boolean be = this.isErrorBarAvailable();
        final boolean bt = this.mTickLabelVariable!=null && this.mIsTickLabelSubstitue==false;
        String[] x = new String[] { this.mXVariable.getName() };
        String[] y = new String[] { this.mYVariable.getName() };
        String[] le = be ? new String[] { this.mLowerErrorVariable.getName() } : null;
        String[] ue = be ? new String[] { this.mUpperErrorVariable.getName() } : null;
        String[] eh = be ? new String[] { this.mErrorBarHolderVariable.getName() } : null;
        String[] t = bt ? new String[] { this.mTickLabelVariable.getName() } : null;
        String[] th = bt ? new String[] { this.mTickLabelHolderVariable.getName() } : null;
        String time = (this.mTimeVariable != null) ? this.mTimeVariable.getName() : null;
        String index = this.isSerialNumberAvailable() ? this.mSerialNumberVariable.getName() : null;
        SGSXYMultipleVariableNetCDFData data = new SGSXYMultipleVariableNetCDFData(
                this.mNCFile, this.mObserver, x, y, le, ue, eh, t, th, time, index);
        data.mOriginMap = new HashMap<String, Integer>(this.mOriginMap);
        data.setDecimalPlaces(this.mDecimalPlaces);
        data.setExponent(this.mExponent);
        data.mVariableCache = new HashMap<String, Object>(this.mVariableCache);
        return data;
    }

    @Override
    public SGISXYTypeMultipleData toMultiple(final Dimension dim, final int index) {
        final boolean be = this.isErrorBarAvailable();
        final boolean bt = this.mTickLabelVariable!=null && this.mIsTickLabelSubstitue==false;
        String le = be ? this.mLowerErrorVariable.getName() : null;
        String ue = be ? this.mUpperErrorVariable.getName() : null;
        String t = bt ? this.mTickLabelVariable.getName() : null;
        String time = (this.mTimeVariable != null) ? this.mTimeVariable.getName() : null;
        String id = this.isSerialNumberAvailable() ? this.mSerialNumberVariable.getName() : null;
        SGSXYMultipleDimensionNetCDFData data = new SGSXYMultipleDimensionNetCDFData(
                this.mNCFile, this.mObserver, this.mXVariable.getName(),
                this.mYVariable.getName(), le, ue, t, dim.getName(),
                index, index, 0, time, id);
        data.mVariableCache = new HashMap<String, Object>(this.mVariableCache);
        return data;
    }

}
