package jp.riken.brain.ni.samuraigraph.data;

import jp.riken.brain.ni.samuraigraph.base.SGUtility;

/**
 * The class of data column of the text type.
 *
 */
public class SGTextDataColumn extends SGDataColumn {

    /**
     * An array of text strings.
     */
    protected String[] mArray = null;
    
    /**
     * Builds this object.
     * @param title
     *              a text string for the title
     * @param array
     *              an array of text strings
     */
    public SGTextDataColumn(final String title, final String[] array) {
        
        super(title);
        
        // check
        if (array == null) {
            throw new IllegalArgumentException("array == null");
        }
        
        // set to an attribute
        this.mArray = SGUtility.copyStringArray(array);
    }
    
    /**
     * Returns the length of data column.
     * @return
     *         the length of data column
     */
    public int getLength() {
        return this.mArray.length;
    }

    /**
     * Returns the value type of this column.
     * @return
     *         the value type of this column
     */
    public String getValueType() {
        return SGIDataColumnTypeConstants.VALUE_TYPE_TEXT;
    }
    
    /**
     * Dispose this object.
     */
    public void dispose() {
        super.dispose();
        this.mArray = null;
    }
    
//    /**
//     * Copy the object.
//     * @return
//     *       A copied object.
//     */
//    public Object copy() {
//        return new SGTextDataColumn(this.mTitle, this.mArray);
//    }

    String[] getTextArray() {
        return this.mArray;
    }
    
    /**
     * Returns a value in this data column at given row index.
     * 
     * @param rowIndex
     *            the row index
     * @return a value at given row index
     */
    public Object getValue(final int rowIndex) {
        if (rowIndex > this.mArray.length) {
            throw new IllegalArgumentException("Index out of bounds: "
                    + rowIndex + " > " + this.mArray.length);
        }
        return this.mArray[rowIndex];
    }

    /**
     * Returns an array of text strings.
     * 
     * @return an array of text strings
     */
    public String[] getStringArray() {
        return (String[]) this.mArray.clone();
    }

    /**
     * Returns a value to export.
     * 
     * @param rowIndex
     *          the row index
     * @return
     *          a value to export
     */
    protected Object getExportedValue(int rowIndex) {
        Object value = this.getValue(rowIndex);
        StringBuffer sb = new StringBuffer();
        sb.append('\"');
        sb.append(value.toString());
        sb.append('\"');
        return sb.toString();
    }
    
}
