package jp.riken.brain.ni.samuraigraph.figure;

import java.awt.Color;

import jp.riken.brain.ni.samuraigraph.base.SGIPropertyDialogObserver;
import jp.riken.brain.ni.samuraigraph.base.SGITwoAxesHolder;

/**
 * An interface for the observer of color bar dialog.
 * 
 * @author kuromaru
 *
 */
public interface SGIColorBarDialogObserver extends SGIPropertyDialogObserver,
        SGIAxisPanelObserver, SGITwoAxesHolder {

    //
    // Layout
    //
    
    public double getXValue();

    public double getYValue();

    public float getBarWidth(final String unit);

    public float getBarLength(final String unit);

    public float getSpaceToScale(final String unit);

    public float getSpaceToTitle(final String unit);

    public String getDirection();
    
    public boolean isReversedOrder();

    public boolean setXValue(final double xValue);
    
    public boolean setYValue(final double yValue);

    public boolean setBarWidth(final float w, final String unit);
    
    public boolean setBarLength(final float len, final String unit);
    
    public boolean setSpaceToScale(final float space, final String unit);
    
    public boolean setSpaceToTitle(final float space, final String unit);
    
    public boolean setDirection(final String dir);

    public boolean setReversedOrder(final boolean b);
    
    //
    // Style
    //
    
    public String getColorBarStyle();
    
    public boolean setColorBarStyle(final String style);
    

    //
    // Line
    //
    
    public float getFrameLineWidth(final String unit);

    public Color getLineColor();
    
    public float getTickMarkWidth(final String unit);
    
    public float getTickMarkLength(final String unit);
    
    public boolean setFrameLineWidth(final float w, final String unit);
    
    public boolean setLineColor(final Color cl);
    
    public boolean setTickMarkWidth(final float w, final String unit);
    
    public boolean setTickMarkLength(final float len, final String unit);

    
    //
    // Font
    //
    
    public String getFontName();
    
    public float getFontSize(final String unit);
    
    public int getFontStyle();
    
    public Color getFontColor();
    
    public boolean setFontName(final String name);
    
    public boolean setFontSize(final float size, final String unit);
    
    public boolean setFontStyle(final int style);
    
    public boolean setFontColor(final Color cl);
    
    /**
     * Returns whether the observer has valid x axis value.
     * 
     * @param config
     *           configuration of the x-axis
     * @param value
     *           axis value
     * @return true for valid axis value
     */
    public boolean hasValidXAxisValue(final int config, final Number value);

    /**
     * Returns whether the observer has valid y axis value.
     * 
     * @param config
     *           configuration of the y-axis
     * @param value
     *           axis value
     * @return true for valid axis value
     */
    public boolean hasValidYAxisValue(final int config, final Number value);
    
}
