package jp.riken.brain.ni.samuraigraph.figure;

import java.awt.Color;
import java.util.Iterator;

import javax.swing.SwingUtilities;

import jp.riken.brain.ni.samuraigraph.base.SGDrawingWindow;
import jp.riken.brain.ni.samuraigraph.base.SGFigure;
import jp.riken.brain.ni.samuraigraph.base.SGIDrawingElementConstants;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElement;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementAxisBreak;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementGrid;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementShape;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementSignificantDifference;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementString;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementTimingLine;
import jp.riken.brain.ni.samuraigraph.base.SGProperties;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyDialog;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyMap;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyResults;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityText;

/**
 * An XY-type figure.
 * 
 */
public class SGXYFigure extends SGFigure implements SGIXYFigureDialogObserver,
        SGIFigureTypeConstants {

    /**
     * Builds a figure.
     * 
     * @param wnd
     *           the parent window
     */
    public SGXYFigure(SGDrawingWindow wnd) {
        super(wnd);
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                mPropertyDialog = new SGXYFigureDialog(mWnd, true);
            }
        });
    }

    /**
     * Returns the type of this class.
     * 
     * @return The type of class.
     */
    public String getType() {
        return FIGURE_TYPE_XY;
    }

    /**
     * Returns a property dialog.
     * @return
     *        a property dialog
     */
    public SGPropertyDialog getPropertyDialog() {
        SGPropertyDialog dg = null;
        if (this.mPropertyDialog != null) {
            dg = this.mPropertyDialog;
        } else {
            dg = new SGXYFigureDialog(this.mWnd, true);
            this.mPropertyDialog = dg;
        }
        return dg;
    }

    /**
     * 
     * @return
     */
    public SGIFigureElementGrid getGridElement() {
        return (SGIFigureElementGrid) this
                .getIFigureElement(SGIFigureElementGrid.class);
    }

    public SGIFigureElement getSymbolElement(final int symbolElementType) {
        Class cl = null;
        switch (symbolElementType) {
        case SGIDrawingElementConstants.SYMBOL_ELEMENT_TYPE_STRING:
            cl = SGIFigureElementString.class;
            break;
        case SGIDrawingElementConstants.SYMBOL_ELEMENT_TYPE_AXISBREAK:
            cl = SGIFigureElementAxisBreak.class;
            break;
        case SGIDrawingElementConstants.SYMBOL_ELEMENT_TYPE_SIGDIFF:
            cl = SGIFigureElementSignificantDifference.class;
            break;
        case SGIDrawingElementConstants.SYMBOL_ELEMENT_TYPE_TIMINGLINE:
            cl = SGIFigureElementTimingLine.class;
            break;
        case SGIDrawingElementConstants.SYMBOL_ELEMENT_TYPE_SHAPE:
            cl = SGIFigureElementShape.class;
            break;
        }
        if (cl == null)
            return null;
        return this.getIFigureElement(cl);
    }

    /**
     * 
     */
    public int getXAxisLocation() {
        return this.getGridElement().getXAxisLocation();
    }

    /**
     * 
     */
    public int getYAxisLocation() {
        return this.getGridElement().getYAxisLocation();
    }

    public boolean setXAxisLocation(final int location) {
        return this.getGridElement().setXAxisLocation(location);
    }

    public boolean setYAxisLocation(final int location) {
        return this.getGridElement().setYAxisLocation(location);
    }

    public boolean setGridVisible(final boolean b) {
        return this.getGridElement().setGridVisible(b);
    }

    public boolean setAutoCalculateRange(final boolean b) {
        return this.getGridElement().setAutoRangeFlag(b);
    }

    public boolean setGridStepValueX(final double value) {
        return this.getGridElement().setStepValueX(value);
    }

    public boolean setGridStepValueY(final double value) {
        return this.getGridElement().setStepValueY(value);
    }

    public boolean setGridBaselineValueX(final double value) {
        return this.getGridElement().setBaselineValueX(value);
    }

    public boolean setGridBaselineValueY(final double value) {
        return this.getGridElement().setBaselineValueY(value);
    }

    public boolean setGridLineWidth(final float width, final String unit) {
        return this.getGridElement().setLineWidth(width, unit);
    }

    public boolean setGridLineType(final int type) {
        return this.getGridElement().setLineType(type);
    }

    public boolean setGridLineColor(final Color cl) {
        return this.getGridElement().setColor(cl);
    }

    public boolean isGridVisible() {
        return this.getGridElement().isGridVisible();
    }

    public boolean isAutoCalculateRange() {
        return this.getGridElement().isAutoRange();
    }

    public double getGridStepValueX() {
        return this.getGridElement().getStepValueX();
    }

    public double getGridStepValueY() {
        return this.getGridElement().getStepValueY();
    }

    public double getGridBaselineValueX() {
        return this.getGridElement().getBaselineValueX();
    }

    public double getGridBaselineValueY() {
        return this.getGridElement().getBaselineValueY();
    }

    public float getGridLineWidth(final String unit) {
        return this.getGridElement().getLineWidth(unit);
    }

    public int getGridLineType() {
        return this.getGridElement().getLineType();
    }

    public Color getGridLineColor() {
        return this.getGridElement().getColor();
    }

    public boolean hasValidStepXValue(final Number step) {
        return this.getGridElement().hasValidStepXValue(step);
    }

    public boolean hasValidStepYValue(final Number step) {
        return this.getGridElement().hasValidStepYValue(step);
    }

    /**
     * 
     */
    public SGProperties getProperties() {
        XYFigureProperties p = new XYFigureProperties();
        if (this.getProperties(p) == false)
            return null;
        return p;
    }

    /**
     * 
     * @param p
     * @return
     */
    public boolean getProperties(SGProperties p) {
        if ((p instanceof XYFigureProperties) == false)
            return false;

        if (super.getProperties(p) == false)
            return false;

        XYFigureProperties fp = (XYFigureProperties) p;

        SGIFigureElementGrid el = this.getGridElement();
        fp.mXAxisConfiguration = el.getXAxisLocation();
        fp.mYAxisConfiguration = el.getYAxisLocation();
        fp.mBaselineValueX = el.getBaselineValueX();
        fp.mBaselineValueY = el.getBaselineValueY();
        fp.mStepValueX = el.getStepValueX();
        fp.mStepValueY = el.getStepValueY();
        fp.mVisibleFlag = el.isGridVisible();
        fp.mAutoRangeFlag = el.isAutoRange();
        fp.mLineWidth = el.getLineWidth();
        fp.mLineType = el.getLineType();
        fp.mColor = el.getColor();

        return true;
    }

    /**
     * 
     */
    public boolean setProperties(SGProperties p) {
        if ((p instanceof XYFigureProperties) == false)
            return false;

        if (super.setProperties(p) == false)
            return false;

        XYFigureProperties fp = (XYFigureProperties) p;

        SGIFigureElementGrid el = this.getGridElement();
        el.setXAxisLocation(fp.mXAxisConfiguration);
        el.setYAxisLocation(fp.mYAxisConfiguration);
        el.setBaselineValueX(fp.mBaselineValueX);
        el.setBaselineValueY(fp.mBaselineValueY);
        el.setStepValueX(fp.mStepValueX);
        el.setStepValueY(fp.mStepValueY);
        el.setGridVisible(fp.mVisibleFlag);
        el.setAutoRangeFlag(fp.mAutoRangeFlag);
        el.setLineWidth(fp.mLineWidth);
        el.setLineType(fp.mLineType);
        el.setColor(fp.mColor);

        return true;
    }

    /**
     * 
     * @author kuromaru
     * 
     */
    public static class XYFigureProperties extends FigureProperties {
        private int mXAxisConfiguration;

        private int mYAxisConfiguration;

        private double mStepValueX;

        private double mBaselineValueX;

        private double mStepValueY;

        private double mBaselineValueY;

        private boolean mVisibleFlag;

        private boolean mAutoRangeFlag;

        private float mLineWidth;

        private int mLineType;

        private Color mColor;

        /**
         * 
         * 
         */
        public XYFigureProperties() {
            super();
        }

        /**
         * 
         */
        public boolean equals(Object obj) {
            if ((obj instanceof XYFigureProperties) == false)
                return false;

            if (super.equals(obj) == false)
                return false;

            XYFigureProperties p = (XYFigureProperties) obj;

            if (this.mXAxisConfiguration != p.mXAxisConfiguration)
                return false;
            if (this.mYAxisConfiguration != p.mYAxisConfiguration)
                return false;
            if (this.mBaselineValueX != p.mBaselineValueX)
                return false;
            if (this.mStepValueX != p.mStepValueX)
                return false;
            if (this.mBaselineValueY != p.mBaselineValueY)
                return false;
            if (this.mStepValueY != p.mStepValueY)
                return false;
            if (this.mVisibleFlag != p.mVisibleFlag)
                return false;
            if (this.mAutoRangeFlag != p.mAutoRangeFlag)
                return false;
            if (this.mLineWidth != p.mLineWidth)
                return false;
            if (this.mLineType != p.mLineType)
                return false;
            if (this.mColor.equals(p.mColor) == false)
                return false;

            return true;
        }

    }

    
    protected SGPropertyResults setProperties(SGPropertyMap map,
            SGPropertyResults iResult) {
        
        SGPropertyResults result = (SGPropertyResults) iResult.clone();
        result = super.setProperties(map, result);
        if (result == null) {
            return null;
        }
        
        Iterator<String> itr = map.getKeyIterator();
        while (itr.hasNext()) {
            String key = itr.next();
            String value = map.getValueString(key);

            if (COM_FIGURE_GRID_VISIBLE.equalsIgnoreCase(key)) {
                Boolean b = SGUtilityText.getBoolean(value);
                if (b == null) {
                    result.putResult(COM_FIGURE_GRID_VISIBLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setGridVisible(b.booleanValue()) == false) {
                    result.putResult(COM_FIGURE_GRID_VISIBLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_FIGURE_GRID_VISIBLE, SGPropertyResults.SUCCEEDED);
            } else if (COM_FIGURE_GRID_AXIS_X.equalsIgnoreCase(key)) {
                final int loc = SGUtility.getAxisLocation(value);
                if (loc == -1) {
                    result.putResult(COM_FIGURE_GRID_AXIS_X, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setXAxisLocation(loc) == false) {
                    result.putResult(COM_FIGURE_GRID_AXIS_X, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_FIGURE_GRID_AXIS_X, SGPropertyResults.SUCCEEDED);
            } else if (COM_FIGURE_GRID_AXIS_Y.equalsIgnoreCase(key)) {
                final int loc = SGUtility.getAxisLocation(value);
                if (loc == -1) {
                    result.putResult(COM_FIGURE_GRID_AXIS_Y, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setYAxisLocation(loc) == false) {
                    result.putResult(COM_FIGURE_GRID_AXIS_Y, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_FIGURE_GRID_AXIS_Y, SGPropertyResults.SUCCEEDED);
            } else if (COM_FIGURE_GRID_AUTO.equalsIgnoreCase(key)) {
                Boolean b = SGUtilityText.getBoolean(value);
                if (b == null) {
                    result.putResult(COM_FIGURE_GRID_AUTO, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setAutoCalculateRange(b.booleanValue()) == false) {
                    result.putResult(COM_FIGURE_GRID_AUTO, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_FIGURE_GRID_AUTO, SGPropertyResults.SUCCEEDED);
            } else if (COM_FIGURE_GRID_STEP_X.equalsIgnoreCase(key)) {
                Number num = SGUtilityText.getDouble(value);
                if (num == null) {
                    result.putResult(COM_FIGURE_GRID_STEP_X, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
            	if (SGUtility.isValidPropertyValue(num.doubleValue()) == false) {
                    result.putResult(COM_FIGURE_GRID_STEP_X, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
                if (this.setGridStepValueX(num.doubleValue()) == false) {
                    result.putResult(COM_FIGURE_GRID_STEP_X, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_FIGURE_GRID_STEP_X, SGPropertyResults.SUCCEEDED);
            } else if (COM_FIGURE_GRID_STEP_Y.equalsIgnoreCase(key)) {
                Number num = SGUtilityText.getDouble(value);
                if (num == null) {
                    result.putResult(COM_FIGURE_GRID_STEP_Y, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
            	if (SGUtility.isValidPropertyValue(num.doubleValue()) == false) {
                    result.putResult(COM_FIGURE_GRID_STEP_Y, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
                if (this.setGridStepValueY(num.doubleValue()) == false) {
                    result.putResult(COM_FIGURE_GRID_STEP_Y, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_FIGURE_GRID_STEP_Y, SGPropertyResults.SUCCEEDED);
            } else if (COM_FIGURE_GRID_BASE_X.equalsIgnoreCase(key)) {
                Number num = SGUtilityText.getDouble(value);
                if (num == null) {
                    result.putResult(COM_FIGURE_GRID_BASE_X, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
            	if (SGUtility.isValidPropertyValue(num.doubleValue()) == false) {
                    result.putResult(COM_FIGURE_GRID_BASE_X, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
                if (this.setGridBaselineValueX(num.doubleValue()) == false) {
                    result.putResult(COM_FIGURE_GRID_BASE_X, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_FIGURE_GRID_BASE_X, SGPropertyResults.SUCCEEDED);
            } else if (COM_FIGURE_GRID_BASE_Y.equalsIgnoreCase(key)) {
                Number num = SGUtilityText.getDouble(value);
                if (num == null) {
                    result.putResult(COM_FIGURE_GRID_BASE_Y, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
            	if (SGUtility.isValidPropertyValue(num.doubleValue()) == false) {
                    result.putResult(COM_FIGURE_GRID_BASE_Y, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
                if (this.setGridBaselineValueY(num.doubleValue()) == false) {
                    result.putResult(COM_FIGURE_GRID_BASE_Y, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_FIGURE_GRID_BASE_Y, SGPropertyResults.SUCCEEDED);
            } else if (COM_FIGURE_GRID_LINE_WIDTH.equalsIgnoreCase(key)) {
                StringBuffer unit = new StringBuffer();
                Number num = SGUtilityText.getNumber(value, unit);
                if (num == null) {
                    result.putResult(COM_FIGURE_GRID_LINE_WIDTH, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setGridLineWidth(num.floatValue(), unit
                        .toString()) == false) {
                    result.putResult(COM_FIGURE_GRID_LINE_WIDTH, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_FIGURE_GRID_LINE_WIDTH, SGPropertyResults.SUCCEEDED);
            } else if (COM_FIGURE_GRID_LINE_TYPE.equalsIgnoreCase(key)) {
                final Integer type = SGDrawingElementLine.getLineTypeFromName(value);
                if (type == null) {
                    result.putResult(COM_FIGURE_GRID_LINE_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setGridLineType(type) == false) {
                    result.putResult(COM_FIGURE_GRID_LINE_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_FIGURE_GRID_LINE_TYPE, SGPropertyResults.SUCCEEDED);
            } else if (COM_FIGURE_GRID_LINE_COLOR.equalsIgnoreCase(key)) {
                Color cl = SGUtilityText.getColor(value);
                if (cl != null) {
                    if (this.setGridLineColor(cl) == false) {
                        result.putResult(COM_FIGURE_GRID_LINE_COLOR, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                } else {
                	cl = SGUtilityText.parseColor(value);
					if (cl == null) {
						result.putResult(COM_FIGURE_GRID_LINE_COLOR,
								SGPropertyResults.INVALID_INPUT_VALUE);
						continue;
					}
					if (this.setGridLineColor(cl) == false) {
						result.putResult(COM_FIGURE_GRID_LINE_COLOR,
								SGPropertyResults.INVALID_INPUT_VALUE);
						continue;
					}
                }
                result.putResult(COM_FIGURE_GRID_LINE_COLOR, SGPropertyResults.SUCCEEDED);
            }
        }
            
        return result;
    }

}
