package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

import jp.riken.brain.ni.samuraigraph.base.SGAxis;
import jp.riken.brain.ni.samuraigraph.base.SGDrawingElement;
import jp.riken.brain.ni.samuraigraph.base.SGTuple2f;
import jp.riken.brain.ni.samuraigraph.data.SGISXYData;
import jp.riken.brain.ni.samuraigraph.figure.SGISXYDataConstants;

import org.w3c.dom.Element;

/**
 * @author okumura
 */
public class SGElementGroupBarInGraph extends SGElementGroupBarForData implements
        SGIElementGroupInGraph, SGISXYDataConstants {

    /**
     * Maximum number of anchors.
     */
    public static final int MAX_NUMBER_OF_ANCHORS = 8;

    private SGFigureElementGraph mGraph = null;

    protected SGElementGroupSetInGraph mGroupSet = null;

    public boolean setElementGroupSet(SGElementGroupSetInGraph gs) {
        this.mGroupSet = gs;
        return true;
    }

    private boolean mFocusedFlag = false;

    public void setFocused(final boolean b) {
        this.mFocusedFlag = b;
    }

    public boolean isFocused() {
        return this.mFocusedFlag;
    }

    /**
     * 
     */
    public SGElementGroupBarInGraph(SGISXYData data,
    		SGFigureElementGraph graph) {
        super(data);
        this.mGraph = graph;
    }

    /**
     * 
     * @return
     */
    public boolean initDrawingElement(final int num) {
        super.initDrawingElement(num);
        return true;
    }

    /**
     * 
     * @return
     */
    protected boolean initDrawingElement(final SGTuple2f[] array) {
        super.initDrawingElement(array);
        return true;
    }

    /**
     * 
     */
    public boolean writeProperty(final Element el) {
        if (super.writeProperty(el) == false) {
            return false;
        }
        el.setAttribute(KEY_VISIBLE, Boolean.toString(this.mVisibleFlag));
        return true;
    }

    /**
     * 
     * @param el
     * @return
     */
    public boolean readProperty(final Element el) {
        if (super.readProperty(el) == false) {
            return false;
        }

        return this.mGraph.readProperty(this, el);
    }

    /**
     * 
     */
    private float getBaselineLocation(final double baselineValue) {
    	SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
    	final boolean vertical = this.isVertical();
        final SGAxis axis = vertical ? gs.getYAxis() : gs.getXAxis();
        return this.mGraph.calcLocation(baselineValue, axis, !vertical);
    }

    /**
     * 
     */
    public boolean setLocation(final SGTuple2f[] pointArray) {
        if (this.mDrawingElementArray == null) {
            return true;
        }
        
//        final int numberPoints = pointArray.length;
        
        SGTuple2f[] shiftedPointArray = pointArray;
//        final float shiftx = getShiftXInGraph();
//        final float shifty = getShiftYInGraph();
//        if (shiftx!=0.0f || shifty!=0.0f) {
//            shiftedPointArray = new SGTuple2f[numberPoints];
//            for (int i = 0; i < numberPoints; i++) {
//                shiftedPointArray[i] = new SGTuple2f(pointArray[i].x+shiftx, pointArray[i].y+shifty);
//            }
//        }

        if (pointArray.length != this.mDrawingElementArray.length) {
            this.initDrawingElement(shiftedPointArray);
        }

        // get the location of the baseline
        final float baseline = this.getBaselineLocation(this.getBaselineValue());

        boolean baselineAvailable = true;
        if (Float.isInfinite(baseline) || Float.isNaN(baseline)) {
            baselineAvailable = false;
        }

        // set the bounds of drawing elements
        final SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
        final SGAxis axis = this.mVerticalFlag ? gs.getXAxis() : gs.getYAxis();
        final double minValue = axis.getMinValue();
        final double maxValue = axis.getMaxValue();
        final float ratio = (float) (Math.abs(this.getWidthValueInGraph()) / (maxValue - minValue));
        float w = 0.0f;
        float h = 0.0f;
        if (this.mVerticalFlag) {
            w = this.mGraph.getGraphRectWidth() * ratio;
            this.setRectangleWidth(w);
        } else {
            h = this.mGraph.getGraphRectHeight() * ratio;
            this.setRectangleHeight(h);
        }

        SGDrawingElement[] array = this.mDrawingElementArray;
        for (int ii = 0; ii < array.length; ii++) {
            final SGDrawingElementBar2D bar = (SGDrawingElementBar2D) array[ii];
            final float x = shiftedPointArray[ii].x;
            final float y = shiftedPointArray[ii].y;
            if (!baselineAvailable || Float.isInfinite(x) || Float.isNaN(x) 
            		|| Float.isInfinite(y) || Float.isNaN(y)) {
                bar.setVisible(false);
                continue;
            } else {
                bar.setVisible(true);
            }

            if (this.mVerticalFlag) {
                h = baseline - y;
            } else {
                w = baseline - x;
            }
            bar.setBounds(x, y, w, h);
        }

        return true;
    }

    /**
     * 
     */
    public boolean paintElement(final Graphics2D g2d, final Rectangle2D rect) {
        super.paintElement(g2d, rect);

        // ElementGroupSetInGraph gs = (ElementGroupSetInGraph)this.mGroupSet;
        SGDrawingElement[] array = this.mDrawingElementArray;

        if (this.isFocused()
                && this.mGraph.isSymbolsVisibleAroundFocusedObjects()) {
            final int num = array.length;
            if (num <= MAX_NUMBER_OF_ANCHORS) {
                for (int ii = 0; ii < num; ii++) {
                    SGDrawingElementBar2D bar = (SGDrawingElementBar2D) array[ii];
                    emphasisBar(bar, g2d);
                }
            } else {
                int div = num / MAX_NUMBER_OF_ANCHORS;
                int cnt = 0;
                while (true) {
                    SGDrawingElementBar2D bar = (SGDrawingElementBar2D) array[cnt];
                    emphasisBar(bar, g2d);
                    cnt += div;
                    if (cnt >= num) {
                        break;
                    }
                }
            }

        }

        return true;
    }

    /**
     * 
     * @param g2d
     * @param symbol
     * @return
     */
    private boolean emphasisBar(final SGDrawingElementBar2D bar,
            final Graphics2D g2d) {
        SGUtilityForFigureElementJava2D.drawAnchorsOnRectangle(bar
                .getElementBounds(), g2d);
        return true;
    }

    /**
     * Returns whether this group set contains the given point.
     * 
     * @param x
     *          the x coordinate
     * @param y
     *          the y coordinate
     * @return
     *          true if this element group contains the given point
     */
    public boolean contains(final int x, final int y) {
	// if a given point is out of the graph rectangle, returns false;
	Rectangle2D gRect = this.mGraph.getGraphRect();
	if (!gRect.contains(x, y)) {
	    return false;
	}
	return super.contains(x, y);
    }

    @Override
    public float getX(int index) {
        return this.getLocation(index).x;
    }

    @Override
    public float getY(final int index) {
        return this.getLocation(index).y;
    }
    
    public SGTuple2f getLocation(final int index) {
        if (this.mGroupSet instanceof SGElementGroupSetInGraphSXY) {
            SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
            return gs.getBarLocation(index);
        } else {
            throw new Error("Not supported.");
        }
    }
    
    /**
     * Sets the baseline value.
     * 
     * @param value
     *           axis value to set to the baseline value
     * @return true if succeeded
     */
    @Override
    public boolean setBaselineValue(final double value) {
        if (this.isVisible()) {
            SGAxis axis = this.isVertical() ? this.mGroupSet.getYAxis() : this.mGroupSet.getXAxis();
    	    if (axis.isValidValue(value) == false) {
    	        return false;
    	    }
    	}
        if (super.setBaselineValue(value) == false) {
        	return false;
        }
        return true;
    }

    /**
     * Sets the width value.
     * 
     * @param value
     *          axis value to set to the width value
     * @return true if succeeded
     */
    public boolean setWidthValue(final double value) {
    	SGAxis axis = this.isVertical() ? this.mGroupSet.getXAxis() : this.mGroupSet.getYAxis();
    	if (axis.isValidValue(value) == false) {
    		return false;
    	}
        if (super.setWidthValue(value) == false) {
        	return false;
        }
        return true;
    }
    
    protected double getIntervalInGraph() {
        return this.mInterval;
    }
    
    protected double getWidthValueInGraph() {
        return super.getWidthValue();
    }
    
//    private int getSeriesIndex() {
//        final SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
//        return gs.getSeriesIndex();
//    }
//    
//    private int getNumberOfSeries() {
//        final SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
//        return gs.getNumberOfSeries();
//    }
//    
//    @Override
//    protected float getShiftXInGraph() {
//        final double shift = this.getShiftX();
//        final double interval = this.getIntervalInGraph();
//        if (shift==0.0f && interval==0.0f) {
//            return 0.0f;
//        }
//
//        if (this.mGroupSet instanceof SGElementGroupSetInGraphSXY) {
//            final SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
//            float shiftx = gs.getShiftXLength(shift);
//            if (this.isVertical()) {
//                float intervalLength = gs.getShiftXLength(interval);
//                float intervalShift = intervalLength * this.getSeriesIndex();
//                float offsetShift = 0.0f;
//                offsetShift = intervalLength * (this.getNumberOfSeries()-1) * 0.5f;
//                return shiftx+intervalShift - offsetShift;
//            } else {
//                return shiftx;
//            }
//        } else {
//            return 0.0f;
//        }
//    }
//    
//    @Override
//    protected float getShiftYInGraph() {
//        final double shift = this.getShiftY();
//        final double interval = this.getIntervalInGraph();
//        if (shift==0.0f && interval==0.0f) {
//            return 0.0f;
//        }
//        
//        if (this.mGroupSet instanceof SGElementGroupSetInGraphSXY) {
//            final SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
//            float shifty = gs.getShiftYLength(shift);
//            if (this.isVertical()==false) {
//                float intervalLength = gs.getShiftYLength(interval);
//                float intervalShift = intervalLength * this.getSeriesIndex();
//                float offsetShift = 0.0f;
//                offsetShift = intervalLength * (this.getNumberOfSeries()-1) * 0.5f;
//                return - shifty - intervalShift + offsetShift;
//            } else {
//                return - shifty;
//            }
//        } else {
//            return 0.0f;
//        }
//    }

    /**
     * 
     * @return
     */
    protected SGDrawingElement createDrawingElementInstance(final int index) {
        return new BarInGraph(this, index);
    }

    protected static class BarInGraph extends BarInGroup {
    	
        /**
         * Builds a rectangle in a group of rectangles.
         * 
         * @param group
         *           a group of rectangles
         */
        public BarInGraph(SGElementGroupBarInGraph group, final int index) {
            super(group, index);
        }
        
//        @Override
//        public float getX() {
//            return super.getX()+this.mGroup.getShiftXInGraph();
//        }
//        
//        @Override
//        public float getY() {
//            return super.getY()+this.mGroup.getShiftYInGraph();
//        }

        /**
         * Returns the bounding box of this bar.
         * 
         * @return a bounding box of this bar
         */
        public Rectangle2D getElementBounds() {
            final float x = this.getX();
            final float y = this.getY();
            final float w = this.getWidth();
            final float h = this.getHeight();

            final float ww = Math.abs(w);
            final float hh = Math.abs(h);

            float xx;
            float yy;
            if (this.isVertical()) {
                xx = x - 0.50f * ww;
                yy = h > 0.0f ? y : y - hh;
            } else {
                xx = w > 0.0f ? x : x - ww;
                yy = y - 0.50f * hh;
            }

            Rectangle2D rect = new Rectangle2D.Float(xx, yy, ww, hh);
            return rect;
        }

        @Override
        public float getWidth() {
            return this.isVertical() ? this.mGroup.getRectangleWidth() : this.mWidth;
        }

        @Override
        public float getHeight() {
            return this.isVertical() ? this.mHeight : this.mGroup.getRectangleHeight();
        }

        @Override
        public boolean setWidth(float w) {
            this.mWidth = w;
            return true;
        }

        @Override
        public boolean setHeight(float h) {
            this.mHeight = h;
            return true;
        }
    }

}
