package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.geom.Rectangle2D;

import jp.riken.brain.ni.samuraigraph.base.SGAxis;
import jp.riken.brain.ni.samuraigraph.base.SGTuple2d;
import jp.riken.brain.ni.samuraigraph.base.SGTuple2f;
import jp.riken.brain.ni.samuraigraph.data.SGISXYTypeData;
import jp.riken.brain.ni.samuraigraph.data.SGISXYTypeMultipleData;
import jp.riken.brain.ni.samuraigraph.figure.SGDrawingElementErrorBar;
import jp.riken.brain.ni.samuraigraph.figure.SGISXYDataConstants;

import org.w3c.dom.Element;

/**
 * A group of error bars in the graph object.
 * 
 * @author okumura
 */
public class SGElementGroupErrorBarInGraph extends SGElementGroupErrorBarForData
        implements SGIElementGroupInGraph, SGISXYDataConstants {

    /**
     * Maximum number of anchors.
     */
    public static final int MAX_NUMBER_OF_ANCHORS = 8;

    private SGFigureElementGraph mGraph = null;

    protected SGElementGroupSetInGraph mGroupSet = null;

    public boolean setElementGroupSet(SGElementGroupSetInGraph gs) {
        this.mGroupSet = gs;
        return true;
    }

    /**
     * location of start points and end points
     * 
     * @uml.property name="mStartArray"
     * @uml.associationEnd multiplicity="(0 -1)"
     */
    private SGTuple2f[] mLowerArray = null;

    /**
     * @uml.property name="mEndArray"
     * @uml.associationEnd multiplicity="(0 -1)"
     */
    private SGTuple2f[] mUpperArray = null;
    
    private SGTuple2f[] mCenterArray = null;

    private boolean mFocusedFlag = false;

    public void setFocused(final boolean b) {
        this.mFocusedFlag = b;
    }

    public boolean isFocused() {
        return this.mFocusedFlag;
    }

    public SGElementGroupErrorBarInGraph(SGISXYTypeData data,
            SGFigureElementGraph graph) {
        super(data);
        this.mGraph = graph;
        final int num = (data != null) ? data.getPointsNumber() : 0;
        this.initDrawingElement(num);
    }

    public SGElementGroupErrorBarInGraph(SGISXYTypeMultipleData data,
            SGFigureElementGraph graph) {
        super(data);
        this.mGraph = graph;
        final int num = (data != null) ? data.getPointsNumber() : 0;
        this.initDrawingElement(num);
    }

    /**
     * 
     */
    public void dispose() {
        super.dispose();
        this.mCenterArray = null;
        this.mLowerArray = null;
        this.mUpperArray = null;
    }

    /**
     * 
     */
    public boolean writeProperty(final Element el) {
        if (super.writeProperty(el) == false) {
            return false;
        }
        el.setAttribute(KEY_VISIBLE, Boolean.toString(this.mVisibleFlag));
        return true;
    }

    /**
     * 
     * @param el
     * @return
     */
    public boolean readProperty(final Element el) {
        if (super.readProperty(el) == false) {
            return false;
        }

        return this.mGraph.readProperty(this, el);
    }

    boolean calcErrorBarLocation(final SGISXYTypeData dataSXY,
            final SGAxis axisX, final SGAxis axisY, final SGTuple2d[] centerArray,
            final double shift, final SGTuple2f[] pointsArray) {

        final int num = dataSXY.getPointsNumber();
//        final double[] xArray = dataSXY.getXValueArray();
//        final double[] yArray = dataSXY.getYValueArray();

//        final SGTuple2d[] centerArray = new SGTuple2d[num];
        final SGTuple2d[] lowerArray = new SGTuple2d[num];
        final SGTuple2d[] upperArray = new SGTuple2d[num];
        for (int ii = 0; ii < num; ii++) {
//            centerArray[ii] = new SGTuple2d();
//            lowerArray[ii] = new SGTuple2d();
//            upperArray[ii] = new SGTuple2d();
//            centerArray[ii].x = xArray[ii];
//            centerArray[ii].y = yArray[ii];
//            lowerArray[ii].x = xArray[ii];
//            lowerArray[ii].y = yArray[ii];
//            upperArray[ii].x = xArray[ii];
//            upperArray[ii].y = yArray[ii];
            lowerArray[ii] = new SGTuple2d(centerArray[ii]);
            upperArray[ii] = new SGTuple2d(centerArray[ii]);
        }

        final double[] lArray = dataSXY.getLowerErrorValueArray();
        final double[] uArray = dataSXY.getUpperErrorValueArray();
        
        final Boolean vertical = this.isVertical();
        if (vertical == null) {
        	return false;
        }
        
        // initialize the drawing elements
        if (this.mDrawingElementArray.length != num) {
        	this.initDrawingElement(num);
        }

        if (this.mErrorBarStyle == ERROR_BAR_BOTHSIDES) {
            for (int ii = 0; ii < num; ii++) {
                SGDrawingElementErrorBar eBar = (SGDrawingElementErrorBar) this.mDrawingElementArray[ii];
                final boolean lEnable = !Double.isNaN(lArray[ii]);
                final boolean uEnable = !Double.isNaN(uArray[ii]);
                eBar.setLowerVisible(lEnable);
                eBar.setUpperVisible(uEnable);
                final double l = lEnable ? lArray[ii] : 0.0;
                final double u = uEnable ? uArray[ii] : 0.0;
                if (Boolean.TRUE.equals(vertical)) {
                    lowerArray[ii].y += l;
                    upperArray[ii].y += u;
                } else {
                    lowerArray[ii].x += l;
                    upperArray[ii].x += u;
                }
            }
        } else if (this.mErrorBarStyle == ERROR_BAR_UPSIDE) {
            for (int ii = 0; ii < num; ii++) {
                SGDrawingElementErrorBar eBar = (SGDrawingElementErrorBar) this.mDrawingElementArray[ii];
                final boolean uEnable = !Double.isNaN(uArray[ii]);
                eBar.setLowerVisible(false);
                eBar.setUpperVisible(uEnable);
                final double u = uEnable ? uArray[ii] : 0.0;
                if (Boolean.TRUE.equals(vertical)) {
                    upperArray[ii].y += u;
                } else {
                    upperArray[ii].x += u;
                }
            }
        } else if (this.mErrorBarStyle == ERROR_BAR_DOWNSIDE) {
            for (int ii = 0; ii < num; ii++) {
                SGDrawingElementErrorBar eBar = (SGDrawingElementErrorBar) this.mDrawingElementArray[ii];
                final boolean lEnable = !Double.isNaN(lArray[ii]);
                eBar.setLowerVisible(lEnable);
                eBar.setUpperVisible(false);
                final double l = lEnable ? lArray[ii] : 0.0;
                if (Boolean.TRUE.equals(vertical)) {
                    lowerArray[ii].y += l;
                } else {
                    lowerArray[ii].x += l;
                }
            }
        } else {
            throw new Error();
        }
        
        // set visibility with log scale axis
        if (axisY.getScaleType() == SGAxis.LOG_SCALE) {
            for (int ii = 0; ii < num; ii++) {
                SGDrawingElementErrorBar eBar = (SGDrawingElementErrorBar) this.mDrawingElementArray[ii];
                if (centerArray[ii].y <= 0.0) {
                    eBar.setLowerVisible(false);
                    eBar.setUpperVisible(false);
                } else if (lowerArray[ii].y <= 0.0) {
                    eBar.setLowerVisible(false);
                }
            }
        }

        // calculate the location
//        this.mGraph.calcLocationOfPoints(centerArray, axisX, axisY, this.mCenterArray);
        this.mCenterArray = pointsArray;
        this.mGraph.calcLocationOfPoints(lowerArray, axisX, axisY, this.mLowerArray);
        this.mGraph.calcLocationOfPoints(upperArray, axisX, axisY, this.mUpperArray);

        return true;
    }
    
    /**
     * Overrode to update the attributes.
     * 
     * @param num
     *           the number of drawing elements
     */
    public boolean initDrawingElement(final int num) {
    	if (super.initDrawingElement(num) == false) {
    		return false;
    	}
        SGTuple2f[] centerArray = new SGTuple2f[num];
        SGTuple2f[] lowerArray = new SGTuple2f[num];
        SGTuple2f[] upperArray = new SGTuple2f[num];
        for (int ii = 0; ii < num; ii++) {
            centerArray[ii] = new SGTuple2f();
            lowerArray[ii] = new SGTuple2f();
            upperArray[ii] = new SGTuple2f();
        }
        this.mCenterArray = centerArray;
        this.mLowerArray = lowerArray;
        this.mUpperArray = upperArray;
    	return true;
    }

    /**
     * Update the location of error bars.
     * @return true if succeeded
     */
    public boolean updateLocation() {
//        this.shiftWithLineOrBarLocation(this.isPositionOnLine());
        return this.setLocation(this.mCenterArray, this.mLowerArray, this.mUpperArray);
    }
    
//    private void shiftWithLineOrBarLocation(final boolean isPositionOnLine) {
//        ArrayList<SGElementGroup> list = this.mGroupSet.getElementGroupList();
//        float shiftXInGraph = 0.0f;
//        float shiftYInGraph = 0.0f;
//        SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
//        for (int i = 0; i < list.size(); i++) {
//            SGElementGroup eg = list.get(i);
//            if (isPositionOnLine) {
//                if (eg.getTagName().equals(SGILineConstants.TAG_NAME_LINE)) {
//                    if (eg instanceof SGElementGroupLine) {
//                        SGElementGroupLine line = (SGElementGroupLine)eg;
//                        shiftXInGraph = ((SGElementGroupLineInGraph)line).getShiftXInGraph();
//                        shiftYInGraph = ((SGElementGroupLineInGraph)line).getShiftYInGraph();
//                        break;
//                    }
//                }
//            } else {
//                if (eg.getTagName().equals(SGIBarConstants.TAG_NAME_BAR)) {
//                    if (eg instanceof SGElementGroupBar) {
//                        SGElementGroupBar bar = (SGElementGroupBar)eg;
//                        shiftXInGraph = ((SGElementGroupBarInGraph)bar).getShiftXInGraph();
//                        shiftYInGraph = ((SGElementGroupBarInGraph)bar).getShiftYInGraph();
//                        break;
//                    }
//                }
//            }
//        }
//        
//        for (int i = 0; i < this.mCenterArray.length; i++) {
//            this.mCenterArray[i].x = this.mCenterArray[i].x + shiftXInGraph;
//            this.mCenterArray[i].y = this.mCenterArray[i].y + shiftYInGraph;
//            this.mLowerArray[i].x = this.mLowerArray[i].x + shiftXInGraph;
//            this.mLowerArray[i].y = this.mLowerArray[i].y + shiftYInGraph;
//            this.mUpperArray[i].x = this.mUpperArray[i].x + shiftXInGraph;
//            this.mUpperArray[i].y = this.mUpperArray[i].y + shiftYInGraph;
//        }
//    }
    
    
    /**
     * Returns whether this group set contains the given point.
     * 
     * @param x
     *          the x coordinate
     * @param y
     *          the y coordinate
     * @return
     *          true if this element group contains the given point
     */
    public boolean contains(final int x, final int y) {
    	// if a given point is out of the graph rectangle, returns false;
    	Rectangle2D gRect = this.mGraph.getGraphRect();
    	if (!gRect.contains(x, y)) {
    	    return false;
    	}
        
        // if the data object do not have error bars, return false
        SGISXYTypeData dataSXY = (SGISXYTypeData) this.mGraph.getData(this.mGroupSet);
        if (dataSXY == null) {
            return false;
        }
        if (dataSXY.isErrorBarAvailable() == false) {
            return false;
        }
        
    	return super.contains(x, y);
    }

    @Override
    public SGTuple2f getStartLocation(int index) {
        return this.mCenterArray[index];
    }

    @Override
    public SGTuple2f getLowerEndLocation(int index) {
        return this.mLowerArray[index];
    }

    @Override
    public SGTuple2f getUpperEndLocation(int index) {
        return this.mUpperArray[index];
    }
    
}
