package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JLabel;
import javax.swing.JPopupMenu;

import jp.riken.brain.ni.samuraigraph.base.SGAxis;
import jp.riken.brain.ni.samuraigraph.base.SGData;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.base.SGDrawingElement;
import jp.riken.brain.ni.samuraigraph.base.SGIConstants;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElement;
import jp.riken.brain.ni.samuraigraph.base.SGIUndoable;
import jp.riken.brain.ni.samuraigraph.base.SGPluginsQueryMessage;
import jp.riken.brain.ni.samuraigraph.base.SGProperties;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyMap;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyResults;
import jp.riken.brain.ni.samuraigraph.base.SGUndoManager;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityText;
import jp.riken.brain.ni.samuraigraph.base.SGData.DataProperties;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementForData.DataLabel;
import jp.riken.brain.ni.samuraigraph.data.SGArrayData;
import jp.riken.brain.ni.samuraigraph.data.SGDataUtility;
import jp.riken.brain.ni.samuraigraph.data.SGIDataAnimation;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFData;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFFile;
import jp.riken.brain.ni.samuraigraph.data.SGSXYMultipleData;
import jp.riken.brain.ni.samuraigraph.data.SGSXYMultipleDimensionNetCDFData;
import jp.riken.brain.ni.samuraigraph.data.SGSXYMultipleNetCDFData;
import jp.riken.brain.ni.samuraigraph.data.SGVariable;
import jp.riken.brain.ni.samuraigraph.figure.SGElementGroup;
import jp.riken.brain.ni.samuraigraph.figure.SGElementGroupSet;

import org.w3c.dom.Element;

/**
 * The class for the set of drawing elements group used in this package.
 */
public abstract class SGElementGroupSetInFigureElement extends
        SGElementGroupSet implements SGIElementGroupSetInFigureElement,
        SGIUndoable, SGIDataPropertyDialogObserver, SGIConstants,
        SGIDataAnimation, ActionListener {

    /**
     * The related data object.
     */
    protected SGData mData = null;

    /**
     * The name of this group set.
     */
    protected String mName = null;

    /**
     * A flag whether this group set is visible in legend.
     */
    protected boolean mVisibleInLegendFlag = true;

    /**
     * Flag whether this object is focused.
     */
    protected boolean mSelectedFlag = false;
    
    /**
     * The X-axis for this group set.
     */
    protected SGAxis mXAxis = null;

    /**
     * The Y-axis for this group set.
     */
    protected SGAxis mYAxis = null;

    /**
     * The Z-axis for this group set.
     */
    protected SGAxis mZAxis = null;
    
    /**
     * The label list of this group set.
     * 
     * This list has relation of label id with label text.
     */
    protected Map<Integer, String> mLabelStringList = new HashMap<Integer, String>();

    /**
     * The ID number.
     */
    private int mID;

    /**
     * The pop-up menu.
     */
    protected JPopupMenu mPopupMenu = null;
    
    /**
     * Undo manager object.
     */
    private SGUndoManager mUndoManager = new SGUndoManager(this);

    /**
     * The default constructor.
     */
    public SGElementGroupSetInFigureElement(SGData data) {
        super();
        this.mData = data;
    }

    /**
     * Returns the related data object.
     * @return
     *         the related data object.
     */
    public SGData getData() {
    	return this.mData;
    }
    
    /**
     * Returns the ID of this object.
     */
    public int getID() {
        return this.mID;
    }

    /**
     * Sets the ID to this object.
     */
    public boolean setID(final int id) {
        this.mID = id;
        return true;
    }
    
    /**
     * Dispose this object.
     */
    public void dispose() {
        super.dispose();
        
        this.mPopupMenu = null;
        
        this.mName = null;

        this.mUndoManager.dispose();
        this.mUndoManager = null;

        this.mXAxis = null;
        this.mYAxis = null;
        this.mZAxis = null;
    }

    /**
     * Sets the name to this group set.
     * 
     * @param name
     *            the name to set to this group set
     * @return true if succeeded
     */
    public boolean setName(final String name) {
        this.mName = name;
        return true;
    }

    /**
     * Sets visibility of this group set in legend.
     * 
     * @param b
     *            a value to set to visibility in legend
     * @return true if succeeded
     */
    public boolean setVisibleInLegend(final boolean b) {
        this.mVisibleInLegendFlag = b;
        return true;
    }

    /**
     * Returns the name.
     * 
     * @return the name
     */
    public String getName() {
        return this.mName;
    }

    /**
     * Returns the visibility in legend.
     * 
     * @return the visibility in legend
     */
    public boolean isVisibleInLegend() {
        return this.mVisibleInLegendFlag;
    }

    /**
     * Get the flag as a focused object.
     * 
     * @return whether this object is focused.
     */
    public boolean isSelected() {
        return this.mSelectedFlag;
    }

    /**
     * Set the flag as a focused object.
     * 
     * @param b
     *            focused
     */
    public void setSelected(final boolean b) {
        this.mSelectedFlag = b;
    }
    
    /**
     * Set the X-axis.
     * 
     * @param axis
     *            an axis to set to the X-axis
     * @return true if succeeded
     */
    public boolean setXAxis(final SGAxis axis) {
        this.mXAxis = axis;
        return true;
    }

    /**
     * Set the Y-axis.
     * 
     * @param axis
     *            an axis to set to the Y-axis
     * @return true if succeeded
     */
    public boolean setYAxis(final SGAxis axis) {
        this.mYAxis = axis;
        return true;
    }

    /**
     * Sets the Z-axis.
     * 
     * @param axis
     *            an axis to set to the Z-axis
     * @return true if succeeded
     */
    public boolean setZAxis(SGAxis axis) {
        this.mZAxis = axis;
        return true;
    }

    /**
     * Returns the X-axis.
     * 
     * @return the X-axis
     */
    public SGAxis getXAxis() {
        return this.mXAxis;
    }

    /**
     * Returns the Y-axis.
     * 
     * @return the Y-axis
     */
    public SGAxis getYAxis() {
        return this.mYAxis;
    }

    /**
     * Returns the Z-axis.
     * 
     * @return the Z-axis
     */
    public SGAxis getZAxis() {
        return this.mZAxis;
    }

    /**
     * Sets the location of x-axis.
     * 
     * @param location
     *           the location of the x-axis
     * @return true if succeeded
     */
    public abstract boolean setXAxisLocation(final int location);

    /**
     * Sets the location of y-axis.
     * 
     * @param location
     *           the location of the y-axis
     * @return true if succeeded
     */
    public abstract boolean setYAxisLocation(final int location);

    /**
     * Class of displayed string and its enabled for popup menu item.
     */
    static class PopupMenuItem {
        final String mItemString;
        boolean mItemEnabled;
        
        PopupMenuItem(String itemString, boolean itemEnabled) {
            this.mItemString = itemString;
            this.mItemEnabled = itemEnabled;
        }
        public void setEnabled(final boolean enabled) {
            this.mItemEnabled = enabled;
        }
    }
    
    /**
     * Returns a pop-up menu.
     * @return
     *         a pop-up menu
     */
    public JPopupMenu getPopupMenu() {
        JPopupMenu p = null;
        if (this.mPopupMenu != null) {
            p = this.mPopupMenu;
        } else {
            p = this.createGroupSetPopupMenu();
            this.mPopupMenu = p;
        }
        return p;
    }

    /**
     * Create a pop-up menu for groupSet object.
     * 
     * @return a pop-up menu
     */
    protected JPopupMenu createGroupSetPopupMenu() {
	
        JPopupMenu p = new JPopupMenu();
        p.setBounds(0, 0, 100, 100);

        p.add(new JLabel(this.getPopupMenuTitle()));
        p.addSeparator();

        List<List<PopupMenuItem>> commandList = this.getCommands();
        for (int ii = 0; ii < commandList.size(); ii++) {
            List<PopupMenuItem> list = commandList.get(ii);
            this.addCommands(p, list);
            if (list.size() != 0 && ii != commandList.size() - 1) {
                p.addSeparator();
            }
        }
        
        return p;
    }
    
    /**
     * Returns all commands.
     * 
     * @return the list of all commands
     */
    protected List<List<PopupMenuItem>> getCommands() {
    	List<List<PopupMenuItem>> list = new ArrayList<List<PopupMenuItem>>();
    	list.add(convertCommandToPopupMenuItem(this.getMoveCommandList()));
    	list.add(convertCommandToPopupMenuItem(this.getEditCommandList()));
        if (SGDataUtility.isNetCDFData(this.getData())) {
            // add an item only for netCDF data
            list.add(convertCommandToPopupMenuItem(this.getNetCDFCommandList()));
        }
    	list.add(this.getOptionalCommandList());
    	//list.add(this.getPluginsCommandList());
    	list.add(convertCommandToPopupMenuItem(this.getPropertyCommandList()));
    	return list;
    }
    
    private List<PopupMenuItem> convertCommandToPopupMenuItem(List<String> commands) {
        List<PopupMenuItem> list = new ArrayList<PopupMenuItem>();
        for (int i = 0; i < commands.size(); i++) {
            list.add(new PopupMenuItem(commands.get(i), true));
        }
        return list;
    }
    
    private void addCommands(JPopupMenu p, List<PopupMenuItem> commands) {
        for (int ii = 0; ii < commands.size(); ii++) {
            String command = commands.get(ii).mItemString;
            boolean enabled = commands.get(ii).mItemEnabled;
            SGUtility.addItem(p, this, command, enabled);
        }
    }
    
    /**
     * Returns the commands to move data objects.
     * 
     * @return the commands
     */
    protected abstract List<String> getMoveCommandList();

    /**
     * Returns the commands to edit data objects.
     * 
     * @return the commands
     */
    protected List<String> getEditCommandList() {
    	List<String> list = new ArrayList<String>();
    	list.add(MENUCMD_CUT);
    	list.add(MENUCMD_COPY);
    	list.add(MENUCMD_DUPLICATE);
    	list.add(MENUCMD_DELETE);
    	return list;
    }

    /**
     * Returns the commands for netCDF data objects.
     * 
     * @return the commands
     */
    protected List<String> getNetCDFCommandList() {
    	List<String> list = new ArrayList<String>();
    	list.add(MENUCMD_INSERT_NETCDF_LABEL);
    	list.add(MENUCMD_NETCDF_ANIMATION);
    	return list;
    }
    
    /**
     * Returns the optional commands for data objects.
     * 
     * @return the commands
     */
    protected List<PopupMenuItem> getOptionalCommandList() {
    	List<PopupMenuItem> list = new ArrayList<PopupMenuItem>();
    	list.add(new PopupMenuItem(MENUCMD_FIT_AXES_TO_DATA, true));
    	list.add(new PopupMenuItem(MENUCMD_TRANSFORM_DATA, true));
    	return list;
    }
    
    /**
     * Returns the commands for the plugins about data objects.
     * 
     * @return the commands
     */
    protected List<PopupMenuItem> getPluginsCommandList() {
        List<PopupMenuItem> list = new ArrayList<PopupMenuItem>();
        
        SGPluginsQueryMessage messageObject = new SGPluginsQueryMessage(SGPluginsQueryMessage.MENUCMD_OUTPUT_TO_FILE_IS_ENABLED);
        this.notifyToListener(SGPluginsQueryMessage.MENUCMD_OUTPUT_TO_FILE_IS_ENABLED, messageObject);
        
        boolean isOutputToFileEnabled = false;
        Object result = messageObject.get();
        if (result instanceof Boolean) {
            isOutputToFileEnabled = ((Boolean)result).booleanValue();
        }
        
        list.add(new PopupMenuItem(MENUCMD_OUTPUT_TO_FILE, isOutputToFileEnabled));
        return list;
    }

    /**
     * Returns the commands for the properties of data objects.
     * 
     * @return the commands
     */
    protected List<String> getPropertyCommandList() {
    	List<String> list = new ArrayList<String>();
    	list.add(MENUCMD_PROPERTY);
    	return list;
    }
    
    /**
     * Returns a text string for the title of popup menu.
     * 
     * @return a text string for the title of popup menu
     */
    protected String getPopupMenuTitle() {
        StringBuffer sb = new StringBuffer();
        sb.append("  -- Data: ");
        sb.append(this.getID());
        sb.append(" ");
        sb.append(SGUtility.removeEscapeChar(this.mName));
        sb.append(" --");
        return sb.toString();
    }
    
    /**
     * 
     */
    public List<Boolean> getVisibleFlagList() {
        final List<Boolean> list = new ArrayList<Boolean>();
        for (int ii = 0; ii < this.mDrawingElementGroupList.size(); ii++) {
            final SGElementGroup group = (SGElementGroup) this.mDrawingElementGroupList.get(ii);
            final boolean flag = group.isVisible();
            list.add(Boolean.valueOf(flag));
        }
        return list;
    }
    
    /**
	 * Check whether this group set contains a given point.
	 * 
	 * @param x
	 *            the x coordinate
	 * @param y
	 *            the y coordinate
	 * @return true if this group set contains a given point
	 */
    public boolean onDrawingElement(final int x, final int y) {
        for (int ii = 0; ii < this.mDrawingElementGroupList.size(); ii++) {
            SGElementGroup group = (SGElementGroup) this.mDrawingElementGroupList
                    .get(ii);
            SGDrawingElement[] array = group.getDrawingElementArray();
            for (int jj = 0; jj < array.length; jj++) {
                if (array[jj].contains(x, y)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Write properties of this object to the Element.
     * 
     * @param el
     *            the Element object
     * @param type
     *            type of the method to save properties
     * @return true if succeeded
     */
    public boolean writeProperty(final Element el, final int type) {
        el.setAttribute(SGIFigureElement.KEY_DATA_NAME, this.mName);
        el.setAttribute(SGIFigureElement.KEY_VISIBLE_IN_LEGEND, Boolean
                .toString(this.mVisibleInLegendFlag));
        
        switch (type) {
        case SAVE_PROPERTIES_TO_PROPERTY_FILE:
            SGData sdata = this.getData();
            if (sdata instanceof SGSXYMultipleData) {
                // write information of sampling rate.
                ((SGSXYMultipleData)sdata).writeSamplingRateToProperty(el);
            }
        case SAVE_PROPERTIES_TO_DATA_SET:
        case SAVE_PROPERTIES_FOR_DUPLICATION:
        case SAVE_PROPERTIES_FOR_COPYING:
        case SAVE_PROPERTIES_FOR_CUTTING:
        case SAVE_PROPERTIES_TO_DATA_SET_OLDER:
        case SAVE_PROPERTIES_TO_DATA_SET_NETCDF:
            SGData data = this.getData();
            if (data.writeProperty(el, type) == false) {
                return false;
            }
            break;
        default:
        }
                
        return true;
    }

    /**
     * Returns properties of this group set.
     * 
     * @return properties of this group set
     */
    public SGProperties getProperties() {
        ElementGroupSetPropertiesInFigureElement ep = new ElementGroupSetPropertiesInFigureElement();
        if (this.getProperties(ep) == false) {
            return null;
        }
        return ep;
    }

    /**
     * Returns properties of this group set.
     * 
     * @return properties of this group set
     */
    public boolean getProperties(final SGProperties p) {
        if ((p instanceof ElementGroupSetPropertiesInFigureElement) == false) {
            return false;
        }
        if (super.getProperties(p) == false) {
            return false;
        }
        ElementGroupSetPropertiesInFigureElement ep = (ElementGroupSetPropertiesInFigureElement) p;
        ep.name = this.mName;
        ep.visibleInLegend = this.mVisibleInLegendFlag;
        ep.dataProperties = (DataProperties) this.getDataProperties();
        ep.labelStringList.clear();
        ep.labelStringList.putAll(this.mLabelStringList);
        return true;
    }

    /**
     * Set properties to this group set.
     * 
     * @param p
     *            properties to set to this group set
     * @return true if succeeded
     */
    public boolean setProperties(final SGProperties p) {
        if ((p instanceof ElementGroupSetPropertiesInFigureElement) == false) {
            return false;
        }
        if (super.setProperties(p) == false) {
            return false;
        }
        ElementGroupSetPropertiesInFigureElement ep = (ElementGroupSetPropertiesInFigureElement) p;
        this.setName(ep.name);
        this.setVisibleInLegend(ep.visibleInLegend);
        this.setDataProperties(ep.dataProperties);
        this.mLabelStringList.clear();
        this.mLabelStringList.putAll(ep.labelStringList);
        
        return true;
    }
    
//    /**
//     * Sets the properties of element groups.
//     * 
//     * @param elementGroupPropertiesList
//     * @return true if succeeded
//     * 
//     */
//    protected boolean setElementGroupProperties(List elementGroupPropertiesList) {
//        
//        for (int ii = 0; ii < elementGroupPropertiesList.size(); ii++) {
//            SGProperties gp = (SGProperties) elementGroupPropertiesList.get(ii);
//            SGElementGroup group = null;
//            if (gp instanceof SGElementGroupLine.LineProperties) {
//                group = this.getLineGroup();
//            } else if (gp instanceof SGElementGroupSymbol.SymbolProperties) {
//                group = this.getSymbolGroup();
//            } else if (gp instanceof SGElementGroupBar.BarProperties) {
//                group = this.getBarGroup();
//            } else if (gp instanceof SGElementGroupErrorBar.ErrorBarProperties) {
//                group = this.getErrorBarGroup();
//            } else if (gp instanceof SGElementGroupString.StringProperties) {
//                group = this.getTickLabelGroup();
//            } else if (gp instanceof SGElementGroupArrow.ArrowProperties) {
//                group = this.getArrowGroup();
//            } else if (gp instanceof SGElementGroupColorMap.ColorMapProperties) {
//                group = this.getColorMapGroup();
//            } else if (gp instanceof SGElementGroupGridColorMap.GridColorMapProperties) {
//                group = this.getGridColorMapGroup();
//            } else {
//                throw new Error("Illegal group property: " + gp);
//            }
//            if (group == null) {
//                continue;
//            }
//            if (group.setProperties(gp) == false) {
//                return false;
//            }
//        }
//        return true;
//    }

    /**
     * Sets the properties of data.
     * @param dp
     *           properties of data
     *           
     */
    public boolean setDataProperties(final SGProperties dp) {
    	SGData data = this.getData();
    	if (data != null) {
            return data.setProperties(dp);
    	}
    	return false;
    }
    
    /**
     * Returns the properties of data.
     * @return
     *         properties of data
     */
    public SGProperties getDataProperties() {
    	SGData data = this.getData();
    	if (data != null) {
            return data.getProperties();
    	} else {
    		return null;
    	}
    }

    
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("[name=");
        sb.append(this.mName);
        sb.append(", ");
        sb.append(super.toString());
        sb.append("]");
        return sb.toString();
    }
    
    
    public boolean isChanged() {
        return this.mUndoManager.isChanged();
    }

    public void setChanged(final boolean b) {
        this.mUndoManager.setChanged(b);
    }

    public boolean isChangedRoot() {
        return this.isChanged();
    }

    public SGProperties getMemento() {
        return this.getProperties();
    }

    public boolean setMemento(SGProperties p) {
        return this.setProperties(p);
    }

    public boolean isUndoable() {
        return this.mUndoManager.isUndoable();
    }

    public boolean isRedoable() {
        return this.mUndoManager.isRedoable();
    }

    public boolean setMementoBackward() {
        if (this.mUndoManager.setMementoBackward() == false) {
            return false;
        }

        // set previous properties to the data object
        if (this.synchronizeDataOnUndo() == false) {
            return false;
        }
        
        return true;
    }

    public boolean setMementoForward() {
        if (this.mUndoManager.setMementoForward() == false) {
            return false;
        }
        
        // set next properties to the data object
        if (this.synchronizeDataOnUndo() == false) {
            return false;
        }
        
        return true;
    }

    /**
     * Synchronize the data object to the properties of this group set.
     * @return
     *         true if succeeded
     */
    private boolean synchronizeDataOnUndo() {
        SGData data = this.getData();
        ElementGroupSetPropertiesInFigureElement p = (ElementGroupSetPropertiesInFigureElement) this.getProperties();
        if (data.setProperties(p.dataProperties) == false) {
            return false;
        }
        return true;
    }

    public boolean undo() {
        return this.setMementoBackward();
    }

    public boolean redo() {
        return this.setMementoForward();
    }

    public boolean updateHistory() {
        return this.mUndoManager.updateHistory();
    }

    public void initUndoBuffer() {
        this.mUndoManager.initUndoBuffer();
    }

    public boolean initPropertiesHistory() {
        return this.mUndoManager.initPropertiesHistory();
    }
    
    /**
     * Delete all forward histories.
     *
     * @return
     *         true if succeeded
     */
    public boolean deleteForwardHistory() {
	return this.mUndoManager.deleteForwardHistory();
    }

    /**
     * Clear changed flag of this undoable object and all child objects.
     *
     */
    public void clearChanged() {
        this.setChanged(false);
    }
    
    /**
     * A class of properties of a group set in figure element.
     */
    public static class ElementGroupSetPropertiesInFigureElement extends
            ElementGroupSetProperties {

        // name of data
        String name;

        // visible in legend
        boolean visibleInLegend;

        // x-axis
        int xAxis;

        // y-axis
        int yAxis;

        // properties of data
        DataProperties dataProperties;
        
        Map<Integer, String> labelStringList = new HashMap<Integer, String>();

        /**
         * The default constructor.
         */
        public ElementGroupSetPropertiesInFigureElement() {
            super();
        }

        /**
         * Returns whether this object is equal to given object.
         * 
         * @param obj
         *            an object to be compared
         * @return true if two objects are equal
         */
        public boolean equals(final Object obj) {

            if ((obj instanceof ElementGroupSetPropertiesInFigureElement) == false) {
                return false;
            }

            if (super.equals(obj) == false) {
                return false;
            }

            ElementGroupSetPropertiesInFigureElement p = (ElementGroupSetPropertiesInFigureElement) obj;

            if (p.name.equals(this.name) == false) {
                return false;
            }
            if (p.visibleInLegend != this.visibleInLegend) {
                return false;
            }
            if (p.xAxis != this.xAxis) {
                return false;
            }
            if (p.yAxis != this.yAxis) {
                return false;
            }
            if (SGUtility.equals(p.dataProperties, this.dataProperties) == false) {
                return false;
            }
            if (p.labelStringList.equals(this.labelStringList) == false) {
                return false;
            }

            return true;
        }

        /**
         * Returns string representation of this object.
         * 
         * @return string representation of this object
         */
        public String toString() {
            String str = super.toString();
            StringBuffer sb = new StringBuffer();
            sb.append("[");
            sb.append(str);
            sb.append(", name=");
            sb.append(this.name);
            sb.append(", visibleInLegend=");
            sb.append(this.visibleInLegend);
            sb.append(", data=");
            sb.append(this.dataProperties);
            sb.append(this.toStringLabelList());
            sb.append("]");
            return sb.toString();
        }
        
        /**
         * Returns string representation of label list in this object.
         * 
         * @return string representation of label list in this object
         */
        public String toStringLabelList() {
            StringBuffer sb = new StringBuffer();
            if (this.labelStringList.size()>0) {
                sb.append(", labelList=");
                Iterator<Integer> it = this.labelStringList.keySet().iterator();
                while (it.hasNext()) {
                    Integer v = it.next();
                    sb.append("("+v.intValue()+","+this.labelStringList.get(v)+")");
                }
            }
            return sb.toString();
        }
        
        /**
         * Copy this object.
         * @return a copied object
         */
        public Object copy() {
            Object obj = super.copy();
            ElementGroupSetPropertiesInFigureElement p = (ElementGroupSetPropertiesInFigureElement) obj;
            if (this.dataProperties != null) {
                p.dataProperties = (DataProperties) this.dataProperties.copy();
            }
            p.labelStringList.clear();
            p.labelStringList.putAll(this.labelStringList);
            return p;
        }

        /**
         * Dispose this object.
         */
        public void dispose() {
            super.dispose();
            if (this.dataProperties != null) {
                this.dataProperties.dispose();
                this.dataProperties = null;
            }
            this.labelStringList.clear();
            this.labelStringList = null;
            this.name = null;
        }
    }

    
    /**
     * A class of properties for multiple data.
     * 
     */
    public static class MultipleElementGroupSetProperties extends ElementGroupSetPropertiesInFigureElement {
        
//        private ElementGroupSetPropertiesInFigureElement mFirstGroupSetProperties = null;
        protected List<SGProperties> mChildPropertyList = new ArrayList<SGProperties>();

        /**
         * Default constructors.
         */
        public MultipleElementGroupSetProperties() {
            super();
        }

        /**
         * Copy this object.
         * @return a copied object
         */
        public Object copy() {
            Object obj = super.copy();
            MultipleElementGroupSetProperties p = (MultipleElementGroupSetProperties) obj;
//            if (this.mFirstGroupSetProperties != null) {
//                p.mFirstGroupSetProperties = (ElementGroupSetPropertiesInFigureElement) this.mFirstGroupSetProperties.copy();
//            }
            List<SGProperties> list = new ArrayList<SGProperties>();
            for (int ii = 0; ii < this.mChildPropertyList.size(); ii++) {
            	list.add(this.mChildPropertyList.get(ii));
            }
            p.mChildPropertyList = list;
            return p;
        }

        /**
         * Dispose this object.
         */
        public void dispose() {
            super.dispose();
//            if (this.mFirstGroupSetProperties != null) {
//                this.mFirstGroupSetProperties.dispose();
//                this.mFirstGroupSetProperties = null;
//            }
            this.mChildPropertyList.clear();
        }

        /**
         * Returns whether this object is equal to given object.
         * 
         * @param obj
         *            an object to be compared
         * @return true if two objects are equal
         */
        public boolean equals(final Object obj) {

            if ((obj instanceof MultipleElementGroupSetProperties) == false) {
                return false;
            }
            
            if (super.equals(obj) == false) {
                return false;
            }

            MultipleElementGroupSetProperties p = (MultipleElementGroupSetProperties) obj;
//            if (!p.mFirstGroupSetProperties.equals(this.mFirstGroupSetProperties)) {
//                return false;
//            }
            if (!p.mChildPropertyList.equals(this.mChildPropertyList)) {
            	return false;
            }

            return true;
        }

        /**
         * Returns string representation of this object.
         * 
         * @return string representation of this object
         */
        public String toString() {
            String str = super.toString();
            StringBuffer sb = new StringBuffer();
            sb.append("[");
            sb.append(str);
//            sb.append(", First=");
//            sb.append(this.mFirstGroupSetProperties);
            sb.append(", Child=");
            sb.append(this.mChildPropertyList);
            sb.append("]");
            return sb.toString();
        }

//        public ElementGroupSetPropertiesInFigureElement getFirstGroupSetProperties() {
//            return (ElementGroupSetPropertiesInFigureElement) mFirstGroupSetProperties.copy();
//        }

//        public void setFirstGroupSetProperties(ElementGroupSetProperties firstGroupSetProperties) {
//            mFirstGroupSetProperties = (ElementGroupSetPropertiesInFigureElement) firstGroupSetProperties.copy();
//        }
    }
    
    
//    /**
//     * Returns an array of data column indices for numbers.
//     * @return
//     *        an array of data column indices for numbers
//     */
//    public int[] getNumbersIndexArray() {
//        SGData data = this.getData();
//        if (!(data instanceof SGArrayData)) {
//            throw new Error("Data is not of array type.");
//        }
//        SGArrayData aData = (SGArrayData) data;
//        return aData.getIndexArray(SGNumberDataColumn.class);
//    }
//    
//    /**
//     * Returns an array of data column indices for text strings.
//     * @return
//     *        an array of data column indices for text strings
//     */
//    public int[] getStringsIndexArray() {
//        SGData data = this.getData();
//        if (!(data instanceof SGArrayData)) {
//            throw new Error("Data is not of array type.");
//        }
//        SGArrayData aData = (SGArrayData) data;
//        return aData.getIndexArray(SGTextDataColumn.class);
//    }
//    
//    /**
//     * Returns an array of data column indices for date text strings.
//     * @return
//     *        an array of data column indices for date date strings
//     */
//    public int[] getDateIndexArray() {
//        SGData data = this.getData();
//        if (!(data instanceof SGArrayData)) {
//            throw new Error("Data is not of array type.");
//        }
//        SGArrayData aData = (SGArrayData) data;
//        return aData.getIndexArray(SGDateDataColumn.class);
//    }
    
    /**
     * Returns an array of titles.
     * @return
     *        an array of titles
     */
    public String[] getTitleArray() {
        SGData data = this.getData();
        if (!(data instanceof SGArrayData)) {
            throw new Error("Data is not of array type.");
        }
        SGArrayData aData = (SGArrayData) data;
        return aData.getTitles();
    }

    /**
     * Returns a map which has data information.
     * @return
     *        a map which has data information
     */
    public Map<String, Object> getInfoMap() {
        return this.getData().getInfoMap();
    }
    
    /**
     * Returns an array of value types.
     * @return
     *        an array of value types
     */
    public String[] getValueTypeArray() {
        SGData data = this.getData();
        if (!(data instanceof SGArrayData)) {
            throw new Error("Data is not of array type.");
        }
        SGArrayData aData = (SGArrayData) data;
        String[] valueTypes = aData.getValueTypes();
        return valueTypes;
    }

    /**
     * Sets the information of data columns.
     * @param columns
     *                information of data columns
     */
    public boolean setColumnInfo(SGDataColumnInfo[] columns, String message) {
        SGData data = this.getData();
        return data.setColumnInfo(columns);
    }

    /**
     * Returns the type of data.
     * @return
     *         the type of data
     */
    public String getDataType() {
        return this.getData().getDataType();
    }

    /**
     * Returns an array of information of data columns.
     * @return
     *        an array of information of data columns
     */
    public SGDataColumnInfo[] getDataColumnInfoArray() {
        
        SGData data = this.getData();
        SGDataColumnInfo[] colArray = null;
        if (data instanceof SGArrayData) {
            SGArrayData aData = (SGArrayData) data;
            final int colNum = aData.getColNum();

            String[] titles = aData.getTitles();
            String[] valueTypes = this.getValueTypeArray();
            String[] colTypes = aData.getCurrentColumnType();
     
            colArray = new SGDataColumnInfo[colNum];
            for (int ii = 0; ii < colNum; ii++) {
                colArray[ii] = new SGDataColumnInfo(titles[ii], valueTypes[ii]);
                colArray[ii].setColumnType(colTypes[ii]);
            }
            
        } else if (data instanceof SGNetCDFData) {
            SGNetCDFData nData = (SGNetCDFData) data;
            SGNetCDFFile ncFile = nData.getNetcdfFile();
            List<SGVariable> varList = ncFile.getVariables();
            final int varNum = varList.size();
            colArray = new SGNetCDFDataColumnInfo[varNum];
            String[] colTypes = nData.getCurrentColumnType();
            for (int ii = 0; ii < varNum; ii++) {
            	SGVariable var = varList.get(ii);
                final String name = var.getName();
                final int origin;
                if (var.isCoordinateVariable()) {
                    origin = nData.getOrigin(name);
                } else {
                    origin = 0;
                }
                colArray[ii] = new SGNetCDFDataColumnInfo(var, var.getNameInPriorityOrder(), 
                		var.getValueType(), origin);
                colArray[ii].setColumnType(colTypes[ii]);
            }
        } else {
            throw new Error("Data type is not supported: " + data.getDataType());
        }
        
        return colArray;
    }
    

    /**
     * Returns a label for a data object.
     * 
     * @return a label for a data object 
     */
    public DataLabel getDataLabel() {
        SGData data = this.getData();
        StringBuffer sb = new StringBuffer();
        int x = 0;
        int y = 0;
        boolean valid = false;
        if (data instanceof SGNetCDFData) {
            SGNetCDFData nData = (SGNetCDFData) data;
            SGNetCDFFile ncFile = nData.getNetcdfFile();
            List<SGVariable> varList = ncFile.getVariables();
            final int size = varList.size();
            boolean first = true;
            for (int ii = 0; ii < size; ii++) {
            	SGVariable var = varList.get(ii);
                
                // only for coordinate variables
                if (var.isCoordinateVariable() == false) {
                    continue;
                }
                
                // get the name
                String name = var.getName();
                
                // only fixed coordinate variable
                if (nData.isFixedCoordinateVariable(name) == false) {
                    continue;
                }
                
                // get value
                int origin = -1;
                if ((data instanceof SGSXYMultipleDimensionNetCDFData)==false) {
                    origin = nData.getOrigin(name);
                } else {
                    SGSXYMultipleDimensionNetCDFData dData = (SGSXYMultipleDimensionNetCDFData)nData;
                    if (SGDataUtility.isSameNetCDFGroup(name, dData.getDimensionName())) {
                        if (name.equals(dData.getDimensionName())) {
                            origin = dData.getDimensionStart();
                        } else {
                            origin = nData.getOrigin(name);
                        }
                    } else {
                        continue;
                    }
                }
                if (origin == -1) {
                    continue;
                }
                double value;
                try {
                    value = nData.getCoordinateValue(name, origin);
                } catch (IOException e) {
                    return null;
                }
                
                valid = true;
                
                // get unit string
                String unit = var.getUnitsString();
                
                // add escape characters
                String escName = SGUtility.addEscapeChar(var.getIdentifiableNameInPriorityOrder());
                String escUnit = null;
                if (unit != null) {
                    escUnit = SGUtility.addEscapeChar(unit);
                }
                
                // append values
                if (!first) {
                    sb.append(", ");
                }
                sb.append(escName);
                sb.append('=');
                sb.append(value);
                if (escUnit != null) {
                    sb.append('[');
                    sb.append(escUnit);
                    sb.append(']');
                }
                
                first = false;
            }
        } else {
            // not supported
            return null;
        }
        if (!valid) {
        	return null;
        }
        DataLabel dl = new DataLabel(sb.toString(), x, y);
        return dl;
    }
    
    /**
     * Add ID and string of label into list stored.
     * @param index
     * @param initialLabelText
     * @return
     */
    public boolean addLabelStringId(final int index, final String initialLabelText) {
        Integer key = Integer.valueOf(index);
        String s = this.mLabelStringList.get(key);
        if (s==null || s.equals(initialLabelText)==false) {
            this.mLabelStringList.put(Integer.valueOf(index), initialLabelText);
            return true;
        }
        return false;
    }
    
    /**
     * Temporary data properties.
     */
    protected SGProperties mTempDataProperties = null;
    
    /**
     * Returns the number of frames.
     * 
     * @return the number of frames
     */
    public int getFrameNumber() {
        if (this.mData instanceof SGNetCDFData) {
            SGNetCDFData data = (SGNetCDFData) this.mData;
            double[] array = data.getTimeValueArray();
            if (array == null) {
                return -1;
            }
            return array.length;
        } else {
            throw new Error("Animation is not supported.");
        }
    }

    /**
     * Returns the current frame index.
     * 
     * @return the current frame index
     */
    public int getCurrentFrameIndex() {
        if (this.mData instanceof SGNetCDFData) {
            SGNetCDFData data = (SGNetCDFData) this.mData;
            return data.getCurrentTimeValueIndex();
        } else {
            throw new Error("Animation is not supported.");
        }
    }

    /**
     * Sets the current frame index.
     * 
     * @param index the frame index to be set
     */
    public void setCurrentFrameIndex(int index) {
        if (this.mData instanceof SGNetCDFData) {
            SGNetCDFData data = (SGNetCDFData) this.mData;
            data.setCurrentTimeValueIndex(index);
        } else {
            throw new Error("Animation is not supported.");
        }
    }    
    
    /**
     * Saves all changes of this data source.
     *
     */
    public void saveChanges() {
        if (this.mData instanceof SGNetCDFData) {
            SGNetCDFData data = (SGNetCDFData) this.mData;
            SGProperties p = data.getProperties();
            if (p.equals(this.mTempDataProperties) == false) {
                this.onSaveChanges();
            }
            this.mTempDataProperties = null;
        } else {
            throw new Error("Animation is not supported.");
        }
    }
    
    protected abstract void onSaveChanges();
    
    /**
     * Cancels all changes of this data source.
     *
     */
    public void cancelChanges() {
        if (this.mData instanceof SGNetCDFData) {
            SGNetCDFData data = (SGNetCDFData) this.mData;
            data.setProperties(this.mTempDataProperties);
            
            // update drawing elements
            this.updateWithData();
            this.mTempDataProperties = null;
        } else {
            throw new Error("Animation is not supported.");
        }
    }

    /**
     * Returns the name of data source.
     * 
     * @return the name of this animation data source
     */
    public String getDataSourceName() {
        SGNetCDFData data = (SGNetCDFData) this.mData;
        SGVariable var = data.getTimeVariable();
        if (var != null) {
            return var.getIdentifiableNameInPriorityOrder();
        } else {
            return null;
        }
    }

    /**
     * Returns the unit string of data source.
     * 
     * @return the unit string of data source
     */
    public String getDataSourceUnitString() {
        SGNetCDFData data = (SGNetCDFData) this.mData;
        SGVariable var = data.getTimeVariable();
        if (var != null) {
            return var.getUnitsString();
        } else {
            return null;
        }
    }

    /**
     * Returns the current value of data object.
     * 
     * @return the current value of data object
     */
    public Number getCurrentValue() {
        SGNetCDFData data = (SGNetCDFData) this.mData;
        return data.getCurrentTimeValue();
    }

    /**
     * Sets the properties.
     * 
     * @param map
     *           a map of properties
     * @return the result of setting properties
     */
    public SGPropertyResults setProperties(SGPropertyMap map) {
        SGPropertyResults result = new SGPropertyResults();
        
        // prepare
        if (this.prepare() == false) {
            return null;
        }
        
        result = this.setProperties(map, result);
        if (result == null) {
            return null;
        }

        // commit the changes
        if (this.commit() == false) {
            return null;
        }
        this.notifyToRoot();
        
        return result;
    }

    protected SGPropertyResults setProperties(SGPropertyMap map,
            SGPropertyResults iResult) {
        
        SGPropertyResults result = (SGPropertyResults) iResult.clone();

        Iterator<String> itr = map.getKeyIterator();
        while (itr.hasNext()) {
            String key = itr.next();
            String value = map.getValueString(key);
            
            if (COM_DATA_AXIS_X.equalsIgnoreCase(key)) {
                final int loc = SGUtility.getAxisLocation(value);
                if (loc == -1) {
                    result.putResult(COM_DATA_AXIS_X, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setXAxisLocation(loc) == false) {
                    result.putResult(COM_DATA_AXIS_X, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_AXIS_X, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_AXIS_Y.equalsIgnoreCase(key)) {
                final int loc = SGUtility.getAxisLocation(value);
                if (loc == -1) {
                    result.putResult(COM_DATA_AXIS_Y, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setYAxisLocation(loc) == false) {
                    result.putResult(COM_DATA_AXIS_Y, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_AXIS_Y, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_VISIBLE_IN_LEGEND.equalsIgnoreCase(key)) {
                Boolean b = SGUtilityText.getBoolean(value);
                if (b == null) {
                    result.putResult(COM_DATA_VISIBLE_IN_LEGEND, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setVisibleInLegend(b.booleanValue()) == false) {
                    result.putResult(COM_DATA_VISIBLE_IN_LEGEND, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_VISIBLE_IN_LEGEND, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_NAME.equalsIgnoreCase(key)) {
            	if (map.isDoubleQuoted(key) == false) {
                    result.putResult(COM_DATA_NAME, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
                String name = SGUtility.createDataNameBase(value);
                if (name == null) {
                    result.putResult(COM_DATA_NAME, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setName(name) == false) {
                    result.putResult(COM_DATA_NAME, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_NAME, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_COLUMN_TYPE.equalsIgnoreCase(key)) {
            	String[] values = SGUtilityText.getStringsInBracket(value);
            	if (values == null) {
                    result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
            	if (values.length == 0) {
                    result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
            	
            	// get arrays of column names or indices and values for each column
            	boolean succeeded = true;
            	String[] colNames = new String[values.length];
            	String[] colValues = new String[values.length];
            	for (int ii = 0; ii < values.length; ii++) {
            		final int index = values[ii].indexOf(':');
            		if (index == -1) {
            			succeeded = false;
            			break;
            		}
            		String colName = values[ii].substring(0, index);
            		String colValue = values[ii].substring(index + 1);
            		colNames[ii] = colName.trim();
            		colValues[ii] = colValue.trim();
            	}
            	if (!succeeded) {
                    result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
            	
        		// check duplication of column names
            	for (int ii = 0; ii < colNames.length - 1; ii++) {
            		for (int jj = ii + 1; jj < colNames.length; jj++) {
            			if (colNames[ii].equals(colNames[jj])) {
            				succeeded = false;
            				break;
            			}
            		}
            	}
            	if (!succeeded) {
                    result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
            	
            	if (this.mData instanceof SGArrayData) {
            		SGArrayData aData = (SGArrayData) this.mData;
            		
            		// get column indices
            		int[] colIndices = new int[colNames.length];
            		for (int ii = 0; ii < colNames.length; ii++) {
            			Integer colIndex = SGUtilityText.getInteger(colNames[ii]);
            			if (colIndex == null) {
            				succeeded = false;
            				break;
            			}
            			colIndices[ii] = colIndex.intValue();
            		}
                	if (!succeeded) {
                        result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                	}

                	// create an array of column types
                	final int dataColNum = aData.getColNum();
                	String[] columnTypes = new String[dataColNum];
                	for (int ii = 0; ii < columnTypes.length; ii++) {
                		columnTypes[ii] = "";
                	}
                	for (int ii = 0; ii < colIndices.length; ii++) {
                		final int index = colIndices[ii] - 1;
                		if (index < 0 || index >= columnTypes.length) {
            				succeeded = false;
                			break;
                		}
                		columnTypes[index] = colValues[ii];
                	}
                	if (!succeeded) {
                        result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                	}
                	
                	// set to the data
                	if (aData.setColumnType(columnTypes) == false) {
                        result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                	}
                	
                    if (this.columnTypeChanged(aData)==false) {
                        result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                	
            	} else if (this.mData instanceof SGNetCDFData) {
            		SGNetCDFData nData = (SGNetCDFData) this.mData;
            		SGNetCDFFile ncfile = nData.getNetcdfFile();
            		List<SGVariable> varList = ncfile.getVariables();
            		
                	// create an array of column types
            		String[] columnTypes = new String[varList.size()];
                	for (int ii = 0; ii < columnTypes.length; ii++) {
                		columnTypes[ii] = "";
                	}
                	for (int ii = 0; ii < colNames.length; ii++) {
                    	final int index = ncfile.getVariableIndex(colNames[ii]);
                    	if (index == -1) {
            				succeeded = false;
                			break;
                    	}
                    	columnTypes[index] = colValues[ii];
                	}
                	if (!succeeded) {
                        result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                	}
                	
                	// set to the data
                	if (nData.setColumnType(columnTypes) == false) {
                        result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                	}
                	
                    if (this.columnTypeChanged(nData)==false) {
                        result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
            	}
                result.putResult(COM_DATA_COLUMN_TYPE, SGPropertyResults.SUCCEEDED);
                
            } else if (COM_DATA_COORDINATE_VARIABLES_INDEX.equalsIgnoreCase(key)) {
            	if (!(this.mData instanceof SGNetCDFData)) {
                    result.putResult(COM_DATA_COORDINATE_VARIABLES_INDEX, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
        		SGNetCDFData nData = (SGNetCDFData) this.mData;
            	boolean succeeded = true;

            	// parse the input string
            	String[] values = SGUtilityText.getStringsInBracket(value);
            	if (values == null) {
                    result.putResult(COM_DATA_COORDINATE_VARIABLES_INDEX, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
            	if (values.length == 0) {
                    result.putResult(COM_DATA_COORDINATE_VARIABLES_INDEX, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
            	String[] dimArray = new String[values.length];
            	int[] originArray = new int[values.length];
            	for (int ii = 0; ii < values.length; ii++) {
            		final int openIndex = values[ii].indexOf('(');
            		if (openIndex == -1) {
            			succeeded = false;
            			break;
            		}
            		final int closedIndex = values[ii].lastIndexOf(')');
            		if (closedIndex == -1) {
            			succeeded = false;
            			break;
            		}
            		String dim = values[ii].substring(0, openIndex);
            		String origin = values[ii].substring(openIndex + 1, closedIndex);
            		Integer num = SGUtilityText.getInteger(origin.trim());
            		if (num == null) {
            			succeeded = false;
            			break;
            		}
            		dimArray[ii] = dim.trim();
            		originArray[ii] = num.intValue();
            	}
            	if (!succeeded) {
                    result.putResult(COM_DATA_COORDINATE_VARIABLES_INDEX, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
            	
            	// set to the data
            	for (int ii = 0; ii < values.length; ii++) {
            		if (nData.setOrigin(dimArray[ii], originArray[ii]) == false) {
            			succeeded = false;
            			break;
            		}
            	}
            	if (!succeeded) {
                    result.putResult(COM_DATA_COORDINATE_VARIABLES_INDEX, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}

                result.putResult(COM_DATA_COORDINATE_VARIABLES_INDEX, SGPropertyResults.SUCCEEDED);
                
            } else if (COM_DATA_PICKUP_START.equalsIgnoreCase(key)) {
                if (!(this.mData instanceof SGSXYMultipleNetCDFData)) {
                    result.putResult(COM_DATA_PICKUP_START, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                SGSXYMultipleNetCDFData nData = (SGSXYMultipleNetCDFData) this.mData;
                Number num = SGUtilityText.getInteger(value);
                if (num == null) {
                    result.putResult(COM_DATA_PICKUP_START, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (nData.setDimensionStart(num.intValue()) == false) {
                    result.putResult(COM_DATA_PICKUP_START, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.columnTypeChanged(nData)==false) {
                    result.putResult(COM_DATA_PICKUP_START, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_PICKUP_START, SGPropertyResults.SUCCEEDED);
                
            } else if (COM_DATA_PICKUP_END.equalsIgnoreCase(key)) {
                if (!(this.mData instanceof SGSXYMultipleNetCDFData)) {
                    result.putResult(COM_DATA_PICKUP_END, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                SGSXYMultipleNetCDFData nData = (SGSXYMultipleNetCDFData) this.mData;
                Number num = SGUtilityText.getInteger(value);
                if (num == null) {
                    result.putResult(COM_DATA_PICKUP_END, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (nData.setDimensionEnd(num.intValue()) == false) {
                    result.putResult(COM_DATA_PICKUP_END, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.columnTypeChanged(nData)==false) {
                    result.putResult(COM_DATA_PICKUP_END, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_PICKUP_END, SGPropertyResults.SUCCEEDED);
                
            } else if (COM_DATA_PICKUP_STEP.equalsIgnoreCase(key)) {
                if (!(this.mData instanceof SGSXYMultipleNetCDFData)) {
                    result.putResult(COM_DATA_PICKUP_STEP, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                SGSXYMultipleNetCDFData nData = (SGSXYMultipleNetCDFData) this.mData;
                Number num = SGUtilityText.getInteger(value);
                if (num == null) {
                    result.putResult(COM_DATA_PICKUP_STEP, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (nData.setDimensionStep(num.intValue()) == false) {
                    result.putResult(COM_DATA_PICKUP_STEP, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.columnTypeChanged(nData)==false) {
                    result.putResult(COM_DATA_PICKUP_STEP, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_PICKUP_STEP, SGPropertyResults.SUCCEEDED);
                
            }
        }
        
        return result;
    }

    
    /**
     * Compares current data columns and temporary data columns.
     *
     * @param msg
     *          a message to send to listeners
     */
    protected void compareDataColumns(final String msg, final SGProperties pTemp) {
        // if data column changes, notify to listeners
        SGProperties pPresent = this.getProperties();
        ElementGroupSetPropertiesInFigureElement egTemp = (ElementGroupSetPropertiesInFigureElement) pTemp;
        ElementGroupSetPropertiesInFigureElement egPresent = (ElementGroupSetPropertiesInFigureElement) pPresent;
        if (egTemp.dataProperties.equals(egPresent.dataProperties) == false) {
            final boolean b = this.isSelected();
            this.setSelected(true);
            this.notifyToListener(msg);
            this.setSelected(b);
        }
    }
    
    protected abstract void notifyToListener(final String msg);
    
    protected abstract void notifyToListener(final String msg, Object source);

//    /**
//     * Returns whether all given data columns are valid.
//     * 
//     * @param columns
//     *           data columns
//     * @return true if all given data columns are valid
//     */
//    public boolean isValidColumn(SGDataColumnInfo[] columns) {
//    	return this.getData().isValidColumn(columns);
//    }

    /**
     * Sets the data.
     * 
     * @param data
     *            a data object
     * @return true if succeeded
     */
    public boolean setData(SGData data) {
    	this.mData = data;
    	return true;
    }

    /**
     * Returns whether this group set "contains" the given group set.
     * By default, this method returns the equality obtained by "equals" method.
     * 
     * @param gs
     *           the group set
     * @return true if this group set "contains" the given group set
     */
    public boolean contains(SGElementGroupSetInFigureElement gs) {
    	return this.equals(gs);
    }
    
    /**
     * Column type of a given data is changed.
     * 
     * @param data
     * @return
     */
    protected abstract boolean columnTypeChanged(final SGData data);
}
