package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import jp.riken.brain.ni.samuraigraph.base.SGData;
import jp.riken.brain.ni.samuraigraph.base.SGIChildObject;
import jp.riken.brain.ni.samuraigraph.base.SGIConstants;
import jp.riken.brain.ni.samuraigraph.base.SGIData;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElement;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementAxis;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementGraph;
import jp.riken.brain.ni.samuraigraph.base.SGINode;
import jp.riken.brain.ni.samuraigraph.base.SGIPropertyDialogObserver;
import jp.riken.brain.ni.samuraigraph.base.SGISelectable;
import jp.riken.brain.ni.samuraigraph.base.SGProperties;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyDialog;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.data.SGDataTypeConstants;
import jp.riken.brain.ni.samuraigraph.data.SGISXYData;

import org.w3c.dom.Element;

/**
 * A set of element groups in the graph object.
 * 
 * @author kuromaru
 * 
 */
public abstract class SGElementGroupSetInGraph extends
        SGElementGroupSetInFigureElement implements 
        SGIPropertyDialogObserver, SGISelectable, SGINode, SGIChildObject,
        SGIConstants {

    /**
     * The graph object.
     */
    protected SGFigureElementGraph mGraph = null;

    /**
     * Flag whether this object is clipped.
     */
    private boolean mClipFlag = true;

    /**
     * Temporary properties of this object.
     */
    protected SGProperties mTemporaryProperties = null;
    
    /**
     * Sets the location of x-axis.
     * 
     * @param location
     *           the location of the x-axis
     */
    public boolean setXAxisLocation(final int location) {
        if (location != SGIFigureElementAxis.AXIS_HORIZONTAL_1
                && location != SGIFigureElementAxis.AXIS_HORIZONTAL_2) {
            return false;
        }
        this.mXAxis = this.mGraph.getAxisElement().getAxisInPlane(location);
        return true;
    }

    /**
     * Sets the location of y-axis.
     * 
     * @param location
     *           the location of the y-axis
     */
    public boolean setYAxisLocation(final int location) {
        if (location != SGIFigureElementAxis.AXIS_PERPENDICULAR_1
                && location != SGIFigureElementAxis.AXIS_PERPENDICULAR_2) {
            return false;
        }
        this.mYAxis = this.mGraph.getAxisElement().getAxisInPlane(location);
        return true;
    }

    /**
     * Disposes this object.
     * 
     */
    public void dispose() {
        super.dispose();
        this.mPopupMenu = null;
        this.mTemporaryProperties = null;
        this.mGraph = null;
    }

    /**
     * 
     * @param b
     */
    protected void setClipFlag(final boolean b) {
        this.mClipFlag = b;
    }

    /**
     * 
     * @return
     */
    protected boolean getClipFlag() {
        return this.mClipFlag;
    }

    /**
     * Set the flag as a focused object.
     * 
     * @param b
     *            focused
     */
    public void setSelected(final boolean b) {
        super.setSelected(b);
        ArrayList list = new ArrayList(this.mDrawingElementGroupList);
        for (int ii = 0; ii < list.size(); ii++) {
            SGIElementGroupInGraph group = (SGIElementGroupInGraph) list
                    .get(ii);
            group.setFocused(b);
        }
    }

    /**
     * 
     */
    public SGElementGroupSetInGraph(SGData data, SGFigureElementGraph graph) {
        super(data);
        this.mGraph = graph;
    }

    /**
     * 
     */
    public void actionPerformed(final ActionEvent e) {
        final String command = e.getActionCommand();
        if (command.equals(SGIConstants.MENUCMD_PROPERTY)) {
            this.mGraph.setPropertiesOfSelectedObjects();
        } else if (command.equals(SGIConstants.MENUCMD_NETCDF_ANIMATION)) {
            this.mTempDataProperties = this.mData.getProperties();
            this.mGraph.notifyToListener(command);
            this.mGraph.doAnimation(this);
        } else {
            this.mGraph.notifyToListener(command);
        }
    }

    /**
     * 
     * @param e
     * @param groupSet
     * @return
     */
    protected boolean onMouseClicked(final MouseEvent e) {

        final int x = e.getX();
        final int y = e.getY();
        final int cnt = e.getClickCount();

        // clicked on the line elements
        if (this.contains(x, y)) {
            
            // update the data selection
            this.mGraph.updateFocusedObjectsList(this, e);

            // notify that the data selection is updated
            this.mGraph.notifyDataSelection();
            
            if (SwingUtilities.isLeftMouseButton(e) && cnt == 2) {
                // show the property dialog
                this.mGraph.setPropertiesOfSelectedObjects();
            } else if (SwingUtilities.isRightMouseButton(e) && cnt == 1) {
	        	// show the pop-up menu
	        	this.mGraph.showDataPopupMenu(this, x, y);
            }

            return true;
        }

        return false;
    }

    /**
     * 
     * @param e
     * @param groupSet
     * @return
     */
    protected boolean onMousePressed(final MouseEvent e) {
        return this.contains(e.getX(), e.getY());
    }
    
    /**
     * Update drawing elements with related data object.
     * @return
     *         true if succeeded
     */
    public boolean updateWithData() {
        if (this.mGraph.updateAllDrawingElementsLocation() == false) {
            return false;
        }
        return true;
    }

    /**
     * 
     * @param dg
     * @return
     */
    public boolean commit() {
        
        // compare current data columns and temporary data columns
        this.compareDataColumns(SGIFigureElement.NOTIFY_DATA_COLUMN_CHANGE_ON_COMMIT,
        		this.mTemporaryProperties);

        // compare properties
        SGProperties pTemp = this.mTemporaryProperties;
        SGProperties pPresent = this.getProperties();
        if (pTemp.equals(pPresent) == false) {
            this.setChanged(true);
        }
        
        // clear temporary properties
        this.mTemporaryProperties = null;

        // update drawing elements
        if (this.updateWithData() == false) {
            return false;
        }
        this.mGraph.repaint();
        this.mGraph.notifyChangeOnCommit();

        return true;
    }

    /**
     * 
     */
    public boolean cancel() {
        
        // compare current data columns and temporary data columns
        this.compareDataColumns(SGIFigureElement.NOTIFY_DATA_COLUMN_CHANGE_ON_CANCEL,
        		this.mTemporaryProperties);

        // restore properties
        if (this.setProperties(this.mTemporaryProperties) == false) {
            return false;
        }
        
        // clear temporary properties
        this.mTemporaryProperties = null;
        
        // restore data properties
        SGData data = this.getData();
        ElementGroupSetPropertiesInFigureElement eg = (ElementGroupSetPropertiesInFigureElement) this
                .getProperties();
        data.setProperties(eg.dataProperties);

        // update drawing elements
        if (this.updateWithData() == false) {
            return false;
        }
        this.mGraph.repaint();
        this.mGraph.notifyChange();
        
        return true;
    }

    /**
     * 
     * @param dg
     * @return
     */
    public boolean preview() {

        // compare current data columns and temporary data columns
        this.compareDataColumns(SGIFigureElement.NOTIFY_DATA_COLUMN_CHANGE_ON_PREVIEW,
        		this.mTemporaryProperties);

        // update drawing elements
        if (this.updateWithData() == false) {
            return false;
        }
        this.mGraph.repaint();
        this.mGraph.notifyChange();
        
        return true;
    }
    
    @Override
    protected void notifyToListener(final String msg) {
    	this.mGraph.notifyToListener(msg);
    }
    
    @Override
    protected void notifyToListener(final String msg, final Object source) {
        this.mGraph.notifyToListener(msg, source);
    }

    /**
     * Returns a list of child nodes.
     * 
     * @return a list of chid nodes
     */
    public ArrayList getChildNodes() {
        return new ArrayList();
    }

    /**
     * 
     */
    public boolean getProperties(final SGProperties p) {
        if ((p instanceof ElementGroupSetPropertiesInFigureElement) == false) {
            return false;
        }
        if (super.getProperties(p) == false) {
            return false;
        }
        ElementGroupSetPropertiesInFigureElement ep = (ElementGroupSetPropertiesInFigureElement) p;
        SGIFigureElementAxis aElement = this.mGraph.getAxisElement();
        ep.xAxis = aElement.getLocationInPlane(this.getXAxis());
        ep.yAxis = aElement.getLocationInPlane(this.getYAxis());
        return true;
    }

    /**
     * 
     */
    public boolean setProperties(final SGProperties p) {
        if ((p instanceof ElementGroupSetPropertiesInFigureElement) == false) {
            return false;
        }
        if (super.setProperties(p) == false) {
            return false;
        }
        ElementGroupSetPropertiesInFigureElement ep = (ElementGroupSetPropertiesInFigureElement) p;
        SGIFigureElementAxis aElement = this.mGraph.getAxisElement();
        this.setXAxis(aElement.getAxisInPlane(ep.xAxis));
        this.setYAxis(aElement.getAxisInPlane(ep.yAxis));
        return true;
    }

    /**
     * Returns a text string that represents the description of this instance.
     * 
     * @return a text string that represents the description of this instance
     */
    public String getInstanceDescription() {
        String dataType = this.getDataType();
		String dataTypeLong = SGDataTypeConstants
				.getLongDataTypeConstant(dataType);
		StringBuffer sb = new StringBuffer();
		sb.append(this.getID());
		sb.append(": ");
		sb.append(dataTypeLong);
		sb.append(": ");
		sb.append(SGUtility.removeEscapeChar(this.mName));
		return sb.toString();
    }

    /**
     * Write properties of this object to the Element.
     * 
     * @param el
     *            the Element object
     * @param type
     *            type of the method to save properties
     * @return true if succeeded
     */
    public boolean writeProperty(final Element el, final int type) {
        if (super.writeProperty(el, type) == false) {
            return false;
        }
        SGData data = this.getData();
        el.setAttribute(SGIFigureElement.KEY_DATA_TYPE, data.getDataType(type));
        return true;
    }

    /**
     * 
     * @return
     */
    public String getTagName() {
        return SGIFigureElementGraph.TAG_NAME_DATA;
    }

    /**
     * Update the location of drawing elements.
     */
    protected abstract boolean updateDrawingElementsLocation(final SGIData data);

    /**
     * Undo operation.
     */
    public boolean setMementoBackward() {
        if (super.setMementoBackward() == false) {
            return false;
        }
        this.mGraph.updateAllDrawingElementsLocation();
        this.mGraph.notifyChangeOnUndo();
        return true;
    }

    /**
     * Redo operation.
     */
    public boolean setMementoForward() {
        if (super.setMementoForward() == false) {
            return false;
        }
        this.mGraph.updateAllDrawingElementsLocation();
        this.mGraph.notifyChangeOnUndo();
        return true;
    }

    /**
     * 
     */
    public void notifyToRoot() {
        this.mGraph.notifyToRoot();
    }

    /**
     * Returns a property dialog.
     * 
     * @return property dialog
     */
    public SGPropertyDialog getPropertyDialog() {
        SGData data = this.mGraph.getData(this);
        SGPropertyDialog dg = this.mGraph.getDataDialog(data);
        return dg;
    }

    /**
     * Set name to this group set.
     * 
     * @param name
     *            the name to set to this group set
     * @return true if succeeded
     */
    public boolean setName(final String name) {

        if (super.setName(name) == false) {
                return false;
        }
        
        // update the popup menu
        JPopupMenu menu = this.getPopupMenu();
        if (menu != null) {
            Component[] cArray = menu.getComponents();
            JLabel item = (JLabel) cArray[0];
            item.setText(this.getPopupMenuTitle());
        }

        return true;
    }
    
    /**
     * Sets the current frame index.
     * 
     * @param index the frame index to be set
     */
    public void setCurrentFrameIndex(int index) {
        super.setCurrentFrameIndex(index);
        this.updateDrawingElementsLocation(this.mData);
        this.mGraph.repaint();
    }    
    
    protected void onSaveChanges() {
        // notify the change
        this.setChanged(true);
        this.mGraph.notifyChangeOnCommit();
        this.notifyToRoot();
        
        // update drawing elements
        this.updateWithData();
        
        this.mGraph.repaint();
    }

    /**
     * Cancels all changes of this data source.
     *
     */
    public void cancelChanges() {
        super.cancelChanges();
        this.mGraph.repaint();
    }

    /**
     * Returns the commands to move data objects.
     * 
     * @return the commands
     */
    protected List<String> getMoveCommandList() {
    	List<String> list = new ArrayList<String>();
    	list.add(MENUCMD_BRING_TO_FRONT);
    	list.add(MENUCMD_BRING_FORWARD);
    	list.add(MENUCMD_SEND_BACKWARD);
    	list.add(MENUCMD_SEND_TO_BACK);
    	return list;
    }
    
    
    @Override
    protected boolean columnTypeChanged(final SGData data) {
        if (data instanceof SGISXYData) {
            final Boolean vertical = ((SGISXYData)data).isErrorBarVertical();
            if (vertical != null) {
                SGElementGroupSetInFigureElement groupSet = this.mGraph.getElementGroupSet(data);
                if (groupSet instanceof SGISXYDataDialogObserver) {
                    ((SGISXYDataDialogObserver)groupSet).setErrorBarVertical(vertical.booleanValue());
                }
            }
        }
        if (this instanceof SGIElementGroupSetMultipleSXY) {
            if (SGMultipleSXYUtility.setColumnInfo((SGIElementGroupSetMultipleSXY)this, this.mGraph) == false) {
                return false;
            }
        }
        return true;
    }
}
