package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;

import jp.riken.brain.ni.samuraigraph.base.SGAxis;
import jp.riken.brain.ni.samuraigraph.base.SGData;
import jp.riken.brain.ni.samuraigraph.base.SGDrawingElement;
import jp.riken.brain.ni.samuraigraph.base.SGIData;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElement;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementAxis;
import jp.riken.brain.ni.samuraigraph.base.SGIPaint;
import jp.riken.brain.ni.samuraigraph.base.SGProperties;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyMap;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyResults;
import jp.riken.brain.ni.samuraigraph.base.SGTuple2d;
import jp.riken.brain.ni.samuraigraph.base.SGTuple2f;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.data.SGISXYData;
import jp.riken.brain.ni.samuraigraph.data.SGISXYTypeData;
import jp.riken.brain.ni.samuraigraph.figure.SGDrawingElementString;
import jp.riken.brain.ni.samuraigraph.figure.SGElementGroup;
import jp.riken.brain.ni.samuraigraph.figure.SGIFigureDrawingElementConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGUtilityForFigureElement;

import org.w3c.dom.Element;

/**
 * @author okumura
 */
public class SGElementGroupSetInGraphSXY extends SGElementGroupSetInGraph
        implements SGIElementGroupSetInGraphSXY, SGIElementGroupSetSXY,
        SGIFigureDrawingElementConstants, SGISXYDataDialogObserver,
        SGIStrokeEditDialogObserver, SGISXYAxisShiftEnabled {

    private SGTuple2f[] mLinePointsArray = null;

    private SGTuple2f[] mBarPointsArray = null;

    /**
     * The series index in multiple graph
     * when this group set is created from the group set of multiple graph. 
     */
    private int mSeriesIndex = -1;
    
    /**
     * The total number of series in multiple graph
     * which creates this group set. 
     */
    private int mNumberOfSeries = -1;
    
    /**
     * 
     */
    protected SGElementGroupSetInGraphSXY(SGData data, SGFigureElementGraph graph) {
        super(data, graph);
    }

    /**
     * 
     */
    public void dispose() {
        super.dispose();
        this.mLinePointsArray = null;
        this.mBarPointsArray = null;
        if (this.mData != null) {
            this.mData.dispose();
            this.mData = null;
        }
    }

    /**
     * 
     * @return
     */
    public String getClassDescription() {
        return "";
    }

    // /**
    // *
    // * @param groupList
    // * @param p
    // * @return
    // */
    // private boolean setProperties( ArrayList groupList, SGProperties p )
    // {
    // for( int ii=0; ii<groupList.size(); ii++ )
    // {
    // SGElementGroup group = (SGElementGroup)groupList.get(ii);
    // if( group.setProperties(p) == false )
    // {
    // return false;
    // }
    // }
    //
    // return true;
    // }

    /**
     * 
     */
    public boolean getLegendVisibleFlag() {
        return this.isVisibleInLegend();
    }

    /**
     * 
     */
    public int getXAxisLocation() {
        return this.mGraph.getAxisElement().getLocationInPlane(this.getXAxis());
    }

    /**
     * 
     */
    public int getYAxisLocation() {
        return this.mGraph.getAxisElement().getLocationInPlane(this.getYAxis());
    }

//    public List<SGElementGroupLine> getLineGroupList() {
//        return this.getLineGroups();
//    }
//
//    public List<SGElementGroupSymbol> getSymbolGroupList() {
//        return this.getSymbolGroups();
//    }
//
//    public List<SGElementGroupBar> getBarGroupList() {
//        return this.getBarGroups();
//    }
//
//    public List<SGElementGroupErrorBar> getErrorBarGroupList() {
//        return this.getErrorBarGroups();
//    }
//
//    public List<SGElementGroupTickLabel> getTickLabelGroupList() {
//        return this.getTickLabelGroups();
//    }

    public int getSelectedTabIndex() {
        return this.mSelectedTabIndex;
    }

    private int mSelectedTabIndex = -1;

    //
    // Line
    //

    public boolean isLineVisible() {
        return this.getLineGroup().isVisible();
    }

    public float getLineWidth() {
        return this.getLineGroup().getLineWidth();
    }

    public float getLineWidth(final String unit) {
        return this.getLineGroup().getLineWidth(unit);
    }

    public int getLineType() {
        return this.getLineGroup().getLineType();
    }

    public Color getLineColor() {
        return this.getLineGroup().getColor();
    }
    
    /**
     * Returns whether the lines connect all effective points.
     * 
     * @return true if connecting all effective points
     */
    public boolean isLineConnectingAll() {
        return this.getLineGroup().isLineConnectingAll();
    }

    public boolean setLineVisible(final boolean b) {
        this.getLineGroup().setVisible(b);
        return true;
    }

    public boolean setLineWidth(final float width, final String unit) {
        return this.getLineGroup().setLineWidth(width, unit);
    }

    public boolean setLineType(final int type) {
        return this.getLineGroup().setLineType(type);
    }

    public boolean setLineColor(final Color cl) {
        return this.getLineGroup().setColor(cl);
    }

    /**
     * Sets whether the lines connect all effective points.
     * 
     * @return true if succeeded
     */
    public boolean setLineConnectingAll(final boolean b) {
        return this.getLineGroup().setLineConnectingAll(b);
    }
    
    public void setLineNumber1(int num) {
        this.getLineGroup().setLineNumber1(num);
    }

    public void setLineNumber2(int num) {
        this.getLineGroup().setLineNumber2(num);
    }

    public void setLineLength1(float len) {
        this.getLineGroup().setLineLength1(len);
    }

    public void setLineLength2(float len) {
        this.getLineGroup().setLineLength2(len);
    }

    public void setSpace(float space) {
        this.getLineGroup().setSpace(space);
    }

    public int getLineNum1() {
        return this.getLineGroup().getLineNum1();
    }

    public int getLineNum2() {
        return this.getLineGroup().getLineNum2();
    }

    public float getLineLength1() {
        return this.getLineGroup().getLineLength1();
    }

    public float getLineLength2() {
        return this.getLineGroup().getLineLength2();
    }

    public float getSpace() {
        return this.getLineGroup().getSpace();
    }

    //
    // Symbol
    //

    public boolean isSymbolVisible() {
        return this.getSymbolGroup().isVisible();
    }

    public int getSymbolType() {
        return this.getSymbolGroup().getType();
    }

    public float getSymbolSize() {
        return this.getSymbolGroup().getSize();
    }

    public float getSymbolSize(final String unit) {
        return this.getSymbolGroup().getSize(unit);
    }

    public float getSymbolLineWidth() {
        return this.getSymbolGroup().getLineWidth();
    }

    public float getSymbolLineWidth(final String unit) {
        return this.getSymbolGroup().getLineWidth(unit);
    }

    @Override
    public SGIPaint getSymbolInnerPaint() {
        return this.getSymbolGroup().getInnerPaint();
    }

    public Color getSymbolLineColor() {
        return this.getSymbolGroup().getLineColor();
    }

    @Override
    public boolean isSymbolLineVisible() {
        return this.getSymbolGroup().isLineVisible();
    }

    public boolean setSymbolVisible(final boolean b) {
        this.getSymbolGroup().setVisible(b);
        return true;
    }

    public boolean setSymbolType(final int type) {
        return this.getSymbolGroup().setType(type);
    }

    public boolean setSymbolSize(final float size, final String unit) {
        return this.getSymbolGroup().setSize(size, unit);
    }

    public boolean setSymbolLineWidth(final float lw, final String unit) {
        return this.getSymbolGroup().setLineWidth(lw, unit);
    }

    @Override
    public boolean setSymbolInnerPaint(final SGIPaint paint) {
        return this.getSymbolGroup().setInnerPaint(paint);
    }

    public boolean setSymbolLineColor(final Color cl) {
        return this.getSymbolGroup().setLineColor(cl);
    }

    @Override
    public boolean setSymbolLineVisible(boolean b) {
        return this.getSymbolGroup().setLineVisible(b);
    }

    //
    // Bar
    //

    public boolean isBarVisible() {
        return this.getBarGroup().isVisible();
    }

    public double getBarBaselineValue() {
        return this.getBarGroup().getBaselineValue();
    }

    public float getBarWidth() {
        return this.getBarGroup().getRectangleWidth();
    }

    public double getBarWidthValue() {
        return this.getBarGroup().getWidthValue();
    }

    public float getBarEdgeLineWidth() {
        return this.getBarGroup().getEdgeLineWidth();
    }

    public float getBarEdgeLineWidth(final String unit) {
        return this.getBarGroup().getEdgeLineWidth(unit);
    }

    @Override
    public SGIPaint getBarInnerPaint() {
        return this.getBarGroup().getInnerPaint();
    }

    @Override
    public Color getBarEdgeLineColor() {
        return this.getBarGroup().getEdgeLineColor();
    }

    @Override
    public boolean isBarEdgeLineVisible() {
        return this.getBarGroup().isEdgeLineVisible();
    }
    
    @Override
    public boolean isBarVertical() {
        return this.getBarGroup().isVertical();
    }

    public boolean setBarVisible(final boolean b) {
        this.getBarGroup().setVisible(b);
        return true;
    }

    public boolean setBarBaselineValue(final double value) {
        return this.getBarGroup().setBaselineValue(value);
    }

//    public boolean setBarWidth(final float width) {
//        return this.getBarGroup().setRectangleWidth(width);
//    }

    public boolean setBarWidthValue(final double value) {
        return this.getBarGroup().setWidthValue(value);
    }

    public boolean setBarEdgeLineWidth(final float lw, final String unit) {
        return this.getBarGroup().setEdgeLineWidth(lw, unit);
    }

    @Override
    public boolean setBarInnerPaint(final SGIPaint paint) {
        return this.getBarGroup().setInnerPaint(paint);
    }

    @Override
    public boolean setBarEdgeLineColor(final Color cl) {
        return this.getBarGroup().setEdgeLineColor(cl);
    }

    @Override
    public boolean setBarEdgeLineVisible(boolean b) {
        return this.getBarGroup().setEdgeLineVisible(b);
    }
    
    @Override
    public boolean setBarVertical(boolean b) {
        return this.getBarGroup().setVertical(b);
    }

    public boolean hasValidBaselineValue(final int config, final Number value) {
        final SGAxis axis = (config == -1) ? this.mYAxis : this.mGraph
                .getAxisElement().getAxisInPlane(config);
        final double v = (value != null) ? value.doubleValue() : this
                .getBarBaselineValue();
        return axis.isValidValue(v);
    }

    // Error Bar
    public boolean isErrorBarAvailable() {
        SGISXYTypeData data = (SGISXYTypeData) this.mData;
        return data.isErrorBarAvailable();
    }
    
    public boolean isErrorBarVisible() {
        return this.getErrorBarGroup().isVisible();
    }

    public int getErrorBarHeadType() {
        return this.getErrorBarGroup().getHeadType();
    }

    public float getErrorBarHeadSize() {
        return this.getErrorBarGroup().getHeadSize();
    }

    public float getErrorBarHeadSize(final String unit) {
        return this.getErrorBarGroup().getHeadSize(unit);
    }

    public Color getErrorBarColor() {
        return this.getErrorBarGroup().getColor();
    }

    public float getErrorBarLineWidth() {
        return this.getErrorBarGroup().getLineWidth();
    }

    public float getErrorBarLineWidth(final String unit) {
        return this.getErrorBarGroup().getLineWidth(unit);
    }

    public int getErrorBarStyle() {
        return this.getErrorBarGroup().getErrorBarStyle();
    }
    
    public boolean isErrorBarVertical() {
    	return this.getErrorBarGroup().isVertical();
    }

    @Override
    public boolean isErrorBarOnLinePosition() {
        return this.getErrorBarGroup().isPositionOnLine();
    }

    public boolean setErrorBarVisible(final boolean b) {
        this.getErrorBarGroup().setVisible(b);
        return true;
    }

    public boolean setErrorBarHeadType(final int type) {
        return this.getErrorBarGroup().setHeadType(type);
    }

    public boolean setErrorBarHeadSize(final float size, final String unit) {
        return this.getErrorBarGroup().setHeadSize(size, unit);
    }

    public boolean setErrorBarColor(final Color cl) {
        return this.getErrorBarGroup().setColor(cl);
    }

    public boolean setErrorBarLineWidth(final float lw, final String unit) {
        return this.getErrorBarGroup().setLineWidth(lw, unit);
    }

    public boolean setErrorBarStyle(final int style) {
        return this.getErrorBarGroup().setErrorBarStyle(style);
    }
    
    public boolean setErrorBarVertical(final boolean b) {
    	return this.getErrorBarGroup().setVertical(b);
    }

    @Override
    public boolean setErrorBarOnLinePosition(boolean b) {
        this.getErrorBarGroup().setPositionOnLine(b);
        return true;
    }

    // Tick Label
    public boolean isTickLabelAvailable() {
        SGISXYTypeData data = (SGISXYTypeData) this.mData;
        return data.isTickLabelAvailable();
    }
    
    public boolean isTickLabelVisible() {
        return this.getTickLabelGroup().isVisible();
    }

    public String getTickLabelFontName() {
        return this.getTickLabelGroup().getFontName();
    }

    public int getTickLabelFontStyle() {
        return this.getTickLabelGroup().getFontStyle();
    }

    public float getTickLabelFontSize() {
        return this.getTickLabelGroup().getFontSize();
    }

    public float getTickLabelFontSize(final String unit) {
        return this.getTickLabelGroup().getFontSize(unit);
    }

    public Color getTickLabelColor() {
        return this.getTickLabelGroup().getColor();
    }

    public float getTickLabelAngle() {
        return this.getTickLabelGroup().getAngle();
    }
    
    public boolean hasTickLabelHorizontalAlignment() {
        return this.getTickLabelGroup().hasHorizontalAlignment();
    }

    public int getTickLabelDecimalPlaces() {
        return this.getTickLabelGroup().getDecimalPlaces();
    }
    
    public int getTickLabelExponent() {
        return this.getTickLabelGroup().getExponent();
    }
    
    public boolean setTickLabelVisible(final boolean b) {
        this.getTickLabelGroup().setVisible(b);
        return true;
    }

    public boolean setTickLabelFontName(final String name) {
        return this.getTickLabelGroup().setFontName(name);
    }

    public boolean setTickLabelFontStyle(final int style) {
        return this.getTickLabelGroup().setFontStyle(style);
    }

    public boolean setTickLabelFontSize(final float size, final String unit) {
        return this.getTickLabelGroup().setFontSize(size, unit);
    }

    public boolean setTickLabelColor(final Color cl) {
        return this.getTickLabelGroup().setColor(cl);
    }

    public boolean setTickLabelAngle(final float angle) {
        return this.getTickLabelGroup().setAngle(angle);
    }

    public boolean setTickLabelDecimalPlaces(final int dp) {
        this.getTickLabelGroup().setDecimalPlaces(dp);
        return true;
    }
    
    public boolean setTickLabelExponent(final int exp) {
        this.getTickLabelGroup().setExponent(exp);
        return true;
    }

	public boolean setTickLabelHorizontalAlignment(boolean b) {
		return this.getTickLabelGroup().setHorizontalAlignment(b);
	}


    /**
     * 
     */
    public boolean addDrawingElementGroup(final int type) {

	SGISXYData data = (SGISXYData) this.mData;
        SGElementGroupSXY group = null;
        if (type == SGIElementGroupConstants.POLYLINE_GROUP) {
            group = new SGElementGroupLineInGraph(data, this.mGraph);
        } else if (type == SGIElementGroupConstants.RECTANGLE_GROUP) {
            group = new SGElementGroupBarInGraph(data, this.mGraph);
        } else if (type == SGIElementGroupConstants.SYMBOL_GROUP) {
            group = new SGElementGroupSymbolInGraph(data, this.mGraph);
        } else {
            return false;
        }

        // set the group set
        SGIElementGroupInGraph iGroup = (SGIElementGroupInGraph) group;
        iGroup.setElementGroupSet(this);

        // create drawing elements
        SGTuple2f[] array = null;
        if (type == SGIElementGroupConstants.POLYLINE_GROUP 
        		|| type == SGIElementGroupConstants.SYMBOL_GROUP) {
        	array = this.mLinePointsArray;
        } else if (type == SGIElementGroupConstants.RECTANGLE_GROUP) {
        	array = this.mBarPointsArray;
        }
        if (group.initDrawingElement(array) == false) {
            return false;
        }

        // add a group to the list
        this.mDrawingElementGroupList.add(group);

        return true;
    }

    /**
     * Creates drawing elements of error bars.
     * 
     * @param dataXY
     *             XY type data
     * @return the group of error bars
     */
    public SGElementGroupErrorBar createErrorBars(
            SGISXYTypeData dataSXY) {
        final SGElementGroupErrorBarInGraph group = new SGElementGroupErrorBarInGraph(
                dataSXY, this.mGraph);
//        group.setVisible(dataSXY.isErrorBarAvailable());
        this.mDrawingElementGroupList.add(group);
        group.mGroupSet = this;
        return group;
    }

    /**
     * Creates drawing elements of tick labels.
     * 
     * @param dataXY
     *             XY type data
     * @return the group of tick labels
     */
    public SGElementGroupTickLabel createTickLabels(
            SGISXYTypeData dataXY) {
        final SGElementGroupTickLabelInGraphSXY group = new SGElementGroupTickLabelInGraphSXY(
                dataXY, this.mGraph);
        this.mDrawingElementGroupList.add(group);
        group.mGroupSet = this;
        return group;
    }

    protected void initPointsArray(final int num) {
        this.mLinePointsArray = this.createPoints(num);
        this.mBarPointsArray = this.createPoints(num);
    }
    
    private SGTuple2f[] createPoints(final int num) {
        SGTuple2f[] pointsArray = new SGTuple2f[num];
        for (int ii = 0; ii < num; ii++) {
            pointsArray[ii] = new SGTuple2f();
        }
        return pointsArray;
    }

    /**
     * Called when the location of data points is changed.
     * 
     * @param data
     *           a data object
     * @return true if succeeded
     */
    public boolean updateDrawingElementsLocation(final SGIData data) {

        if ((data instanceof SGISXYTypeData) == false) {
            return false;
        }
        SGISXYTypeData dataSXY = (SGISXYTypeData) data;

        // set the location of the data points
        final double[] xArray = dataSXY.getXValueArray();
        final double[] yArray = dataSXY.getYValueArray();
        if (xArray == null || yArray == null) {
        	return true;
        }
        if (xArray.length != yArray.length) {
        	return false;
        }
//        if (this.mPointsArray.length != xArray.length) {
//        	SGTuple2f[] pArray = new SGTuple2f[xArray.length];
//        	for (int ii = 0; ii < pArray.length; ii++) {
//        		pArray[ii] = new SGTuple2f();
//        	}
//        	this.mPointsArray = pArray;
//        }
        if (this.mLinePointsArray.length != xArray.length) {
            this.mLinePointsArray = this.createPoints(xArray.length);
        }
        if (this.mBarPointsArray.length != xArray.length) {
            this.mBarPointsArray = this.createPoints(xArray.length);
        }
//        if (this.mGraph.calcLocationOfPoints(xArray, yArray, this.getXAxis(), 
//        		this.getYAxis(), this.mPointsArray) == false) {
//            return false;
//        }
        

        // calculate the location of points
        final int num = xArray.length;
		final SGAxis xAxis = this.getXAxis();
		final SGAxis yAxis = this.getYAxis();
		final double lineShiftX = this.getLineShiftX();
		final double lineShiftY = this.getLineShiftY();
		SGTuple2d[] lineValues = new SGTuple2d[num];
		for (int ii = 0; ii < lineValues.length; ii++) {
			lineValues[ii] = new SGTuple2d(xArray[ii] + lineShiftX, yArray[ii] + lineShiftY);
		}
		if (!this.mGraph.calcLocationOfPoints(lineValues, xAxis, yAxis, this.mLinePointsArray)) {
			return false;
		}
//		if (this.mGraph.calcLocationOfPoints(xArray, yArray, xAxis, yAxis,
//				lineShiftX, lineShiftY, this.mLinePointsArray) == false) {
//			return false;
//		}
		final double barShiftX = this.getBarShiftX();
		final double barShiftY = this.getBarShiftY();
		SGTuple2d[] barValues = new SGTuple2d[num];
		for (int ii = 0; ii < lineValues.length; ii++) {
			barValues[ii] = new SGTuple2d(xArray[ii] + barShiftX, yArray[ii] + barShiftY);
		}
		if (!this.mGraph.calcLocationOfPoints(barValues, xAxis, yAxis, this.mBarPointsArray)) {
			return false;
		}
//		if (this.mGraph.calcLocationOfPoints(xArray, yArray, xAxis, yAxis,
//				barShiftX, barShiftY, this.mBarPointsArray) == false) {
//			return false;
//		}
        
        List<SGElementGroup> gList = this.mDrawingElementGroupList;
        for (int ii = 0; ii < gList.size(); ii++) {
            SGElementGroup group = (SGElementGroup) gList.get(ii);
            if (group.isVisible()) {
                if (group instanceof SGElementGroupErrorBar) {
                    
                    // if the data does not have error values, go next
                    if (!dataSXY.isErrorBarAvailable()) {
                        continue;
                    }
                    
                    // set the location of error bars
                    SGElementGroupErrorBarInGraph eGroup = (SGElementGroupErrorBarInGraph) group;
                    final SGTuple2d[] valueArray;
                    final SGTuple2f[] pointsArray;
                    final double shiftX, shiftY;
                    if (eGroup.isPositionOnLine()) {
                        valueArray = lineValues;
                        pointsArray = this.mLinePointsArray;
                        shiftX = lineShiftX;
                        shiftY = lineShiftY;
                    } else {
                    	valueArray = barValues;
                    	pointsArray = this.mBarPointsArray;
                    	shiftX = barShiftX;
                    	shiftY = barShiftY;
                    }
                    final double shift = eGroup.isVertical() ? shiftY : shiftX;
                    if (eGroup.calcErrorBarLocation(dataSXY, xAxis, yAxis, valueArray, shift, pointsArray) == false) {
                        return false;
                    }

                    if (eGroup.updateLocation() == false) {
                        return false;
                    }
                } else if (group instanceof SGElementGroupTickLabel) {

                    // if the data does not have tick label, go next
                    if (!dataSXY.isTickLabelAvailable()) {
                        continue;
                    }
                    
                    // set the location of tick labels
                    SGElementGroupTickLabelInGraph tl = (SGElementGroupTickLabelInGraph) group;
                    Boolean horizontal = dataSXY.isTickLabelHorizontal();
                    if (null!=horizontal) {
                        tl.setHorizontalAlignment(horizontal.booleanValue());
                    }

                    if (tl.updateLocation(dataSXY, xAxis, yAxis) == false) {
                        return false;
                    }

                    if (tl.updateLocation() == false) {
                        return false;
                    }
//                } else if ((group instanceof SGElementGroupLine)
//                        || (group instanceof SGElementGroupSymbol)
//                        || (group instanceof SGElementGroupBar)) {
//                    SGElementGroupSXY groupSXY = (SGElementGroupSXY) group;
//                    if (groupSXY.setLocation(this.mPointsArray) == false) {
//                        return false;
//                    }
                } else if ((group instanceof SGElementGroupLine)
                        || (group instanceof SGElementGroupSymbol)) {
                    SGElementGroupSXY groupSXY = (SGElementGroupSXY) group;
                    if (groupSXY.setLocation(this.mLinePointsArray) == false) {
                        return false;
                    }
                } else if (group instanceof SGElementGroupBar) {
                    SGElementGroupSXY groupSXY = (SGElementGroupSXY) group;
                    if (groupSXY.setLocation(this.mBarPointsArray) == false) {
                        return false;
                    }
                }
            }
        }

        return true;
    }

    /**
     * Returns the bounding box of tick labels for a given data.
     * @param data
     *             a data object
     */
    public Rectangle2D getTickLabelsBoundingBox(final SGIData data) {

        if ((data instanceof SGISXYTypeData) == false) {
            throw new IllegalArgumentException("(data instanceof SGISXYTypeData) == false");
        }

        ArrayList rectList = new ArrayList();
        List<SGElementGroupTickLabel> tickLabelGroups = this.getTickLabelGroups();
        if (tickLabelGroups.size() == 0) {
            return null;
        }
        final SGElementGroupTickLabel group = tickLabelGroups
                .get(0);
        if (!group.isVisible()) {
            return null;
        }

        ArrayList strList = new ArrayList();
        SGDrawingElement[] elArray = group.getDrawingElementArray();
        for (int ii = 0; ii < elArray.length; ii++) {
            if (!elArray[ii].isVisible()) {
                continue;
            }
            if (elArray[ii] instanceof SGDrawingElementString) {
                SGDrawingElementString elStr = (SGDrawingElementString) elArray[ii];
                String str = elStr.getString();
                
                // skip an empty string
                if ("".equals(str)) {
                    continue;
                }
            }
            strList.add(elArray[ii]);
        }

        Rectangle2D rectAll = SGUtilityJava2D.getBoundingBox(strList);
        if (rectAll == null) {
            return null;
        }
        rectList.add(rectAll);

        // join the rectangles
        Rectangle2D uniRect = SGUtility.createUnion(rectList);
        if (uniRect == null) {
            return null;
        }

        return uniRect;
    }

    /**
     * Paint bars, error bars, lines and symbols.
     */
    @Override
    public void paintGraphics2D(final Graphics2D g2d) {

        Rectangle2D gRect = null;
        if (!this.getClipFlag()) {
            gRect = this.mGraph.getGraphRect();
        }
        
        // bar
        final SGElementGroupBarInGraph groupBar = this.mGraph.getGroupBar(this);
        if (groupBar != null) {
            if (groupBar.isVisible()) {
                groupBar.paintElement(g2d, gRect);
            }
        }

        // error bar
        SGData data = this.getData();
        if (data instanceof SGISXYTypeData) {
            SGISXYTypeData dataSXY = (SGISXYTypeData) data;
            if (dataSXY.isErrorBarAvailable()) {
                final SGElementGroupErrorBar eGroup = this.getErrorBarGroup();
                if (eGroup != null) {
                    if (eGroup.isVisible()) {
                        eGroup.paintElement(g2d, gRect);
                    }
                }
            }
        }

        // line
        final SGElementGroupLineInGraph groupLine = this.mGraph
                .getGroupLine(this);
        if (groupLine != null) {
            if (groupLine.isVisible()) {
                groupLine.paintElement(g2d, gRect);
            }
        }

        // symbol
        final SGElementGroupSymbolInGraph groupSymbol = this.mGraph
                .getGroupSymbol(this);
        if (groupSymbol != null) {
            if (groupSymbol.isVisible()) {
                groupSymbol.paintElement(g2d, gRect);
            }
        }

    }

    /**
     * Paint tick labels.
     * @param g2d
     *            the graphics context
     */
    public boolean paintDataString(final Graphics2D g2d) {
        
        SGData data = this.getData();
        if (data instanceof SGISXYTypeData) {
            SGISXYTypeData dataSXY = (SGISXYTypeData) data;
            if (dataSXY.isTickLabelAvailable()) {
                List<SGElementGroupTickLabel> tList = this.getTickLabelGroups();
                if (tList.size() != 0) {
                    final SGElementGroupTickLabel groupString = tList.get(0);
                    if (groupString != null) {
                        if (groupString.isVisible()) {
                            groupString.paintElement(g2d);
                        }
                    }
                }
            }
        }

        return true;
    }

    /**
     * 
     */
    public boolean setXAxisLocation(final int config) {
        if (config != SGIFigureElementAxis.AXIS_HORIZONTAL_1
                && config != SGIFigureElementAxis.AXIS_HORIZONTAL_2) {
            return false;
        }
        return this.setXAxis(this.getAxis(config));
    }

    /**
     * 
     */
    public boolean setYAxisLocation(final int config) {
        if (config != SGIFigureElementAxis.AXIS_PERPENDICULAR_1
                && config != SGIFigureElementAxis.AXIS_PERPENDICULAR_2) {
            return false;
        }
        return this.setYAxis(this.getAxis(config));
    }

    private SGAxis getAxis(final int config) {
        return this.mGraph.mAxisElement.getAxisInPlane(config);
    }

    /**
     * 
     */
    public boolean prepare() {
//        this.mTemporaryProperties = this.getWholeProperties();
        this.mTemporaryProperties = this.getProperties();
        return true;
    }

    /**
     * Write properties of this object to the Element.
     * 
     * @param el
     *            the Element object
     * @param type
     *            type of the method to save properties
     * @return true if succeeded
     */
    public boolean writeProperty(final Element el, final int type) {
        if (super.writeProperty(el, type) == false) {
            return false;
        }

        final String strX = this.mGraph.getAxisElement().getLocationName(
                this.getXAxis());
        final String strY = this.mGraph.getAxisElement().getLocationName(
                this.getYAxis());

        el.setAttribute(SGIFigureElement.KEY_X_AXIS_POSITION, strX);
        el.setAttribute(SGIFigureElement.KEY_Y_AXIS_POSITION, strY);

        return true;
    }

    /**
     * 
     * @param e
     * @param groupSet
     * @return
     */
    protected boolean onMouseClicked(final MouseEvent e) {
        if (super.onMouseClicked(e) == false) {
            return false;
        }

        SGElementGroup group = this.getElementGroupAt(e.getX(), e.getY());

        int index = -1;
        if (group instanceof SGElementGroupLineInGraph) {
            index = 0;
        } else if (group instanceof SGElementGroupSymbolInGraph) {
            index = 1;
        } else if (group instanceof SGElementGroupBarInGraph) {
            index = 2;
        } else if (group instanceof SGElementGroupErrorBarInGraph) {
            index = 3;
        } else if (group instanceof SGElementGroupTickLabelInGraph) {
            index = 4;
        }
        this.mSelectedTabIndex = index;

        return true;
    }

    /**
     * Returns an element group at a given point.
     * 
     * @param x
     *            the x coordinate
     * @param y
     *            the y coordinate
     * @return an element group if it contains the given point
     */
    public SGElementGroup getElementGroupAt(final int x, final int y) {
    	List<SGElementGroup> list = this.mDrawingElementGroupList;
        for (int ii = list.size() - 1; ii >= 0; ii--) {
            SGElementGroup group = (SGElementGroup) list.get(ii);
            if (group.isVisible() == false) {
                continue;
            }
            if (group.contains(x, y)) {
                return group;
            }
        }
        return null;
    }
    
    /**
     * Update the text strings of tick labels.
     */
    public void updateTickLabelStrings() {
        SGData data = this.getData();
        if (data != null) {
            SGISXYTypeData dataSXY = (SGISXYTypeData) data;
            if (dataSXY.isTickLabelAvailable()) {
                SGElementGroupTickLabel tg = this.getTickLabelGroup();
                boolean eq = true;

                // set decimal places and exponent
                final int dp = tg.getDecimalPlaces();
                final int exp = tg.getExponent();
                if (dp != dataSXY.getDecimalPlaces() || exp != dataSXY.getExponent()) {
                    eq = false;
                }
                
            	// set the decimal places and the exponent to the data object
                dataSXY.setDecimalPlaces(dp);
                dataSXY.setExponent(exp);

                // compare all text strings
                String[] strArray = dataSXY.getStringArray();
                SGDrawingElement[] tlArray = tg.getDrawingElementArray();
                if (strArray.length >= tlArray.length) {
                    for (int ii = 0; ii < tlArray.length; ii++) {
                        SGDrawingElementString el = (SGDrawingElementString) tlArray[ii];
                        if (!strArray[ii].equals(el.getString())) {
                            eq = false;
                            break;
                        }
                    }
                }

                // set the text strings to the tick labels when tick labels, decimal places
                // or the exponent are not equal
                if (!eq) {
                    tg.setStrings(strArray);
                }
            } else {
                SGElementGroupTickLabel tg = this.getTickLabelGroup();
                SGDrawingElement[] tlArray = tg.getDrawingElementArray();
                tg.initDrawingElement(tlArray.length);
            }
        }
    }

    /**
     * Update drawing elements with related data object.
     * @return
     *         true if succeeded
     */
    public boolean updateWithData() {
        this.updateTickLabelStrings();
        return super.updateWithData();
    }
 
    /**
     * Sets the properties of data.
     * @param dp
     *           properties of data
     */
    @Override
    public boolean setDataProperties(final SGProperties dp) {
        super.setDataProperties(dp);
        this.updateTickLabelStrings();
        return true;
    }    
    
//    /**
//     * Returns the location at a given index.
//     * 
//     * @param index
//     *           the index
//     * @return the location
//     */
//    public SGTuple2f getLocation(final int index) {
//        if (this.mPointsArray == null) {
//            return null;
//        }
//        if (index < 0 || index >= this.mPointsArray.length) {
//            throw new IllegalArgumentException("Index out of bounds: " + index);
//        }
//        return this.mPointsArray[index];
//    }

    /**
     * Returns the location of a line and symbols at a given index.
     * 
     * @param index
     *           the index
     * @return the location
     */
    public SGTuple2f getLineLocation(final int index) {
    	return this.getLocation(this.mLinePointsArray, index);
    }

    /**
     * Returns the location of a symbol at a given index.
     * 
     * @param index
     *           the index
     * @return the location
     */
    public SGTuple2f getSymbolLocation(final int index) {
    	return this.getLocation(this.mLinePointsArray, index);
    }

    /**
     * Returns the location of a bar at a given index.
     * 
     * @param index
     *           the index
     * @return the location
     */
    public SGTuple2f getBarLocation(final int index) {
    	return this.getLocation(this.mBarPointsArray, index);
    }

    private SGTuple2f getLocation(final SGTuple2f[] pointsArray, final int index) {
        if (pointsArray == null) {
            return null;
        }
        if (index < 0 || index >= pointsArray.length) {
            throw new IllegalArgumentException("Index out of bounds: " + index);
        }
        return pointsArray[index];
    }

    protected SGPropertyResults setProperties(SGPropertyMap map,
            SGPropertyResults iResult) {

        SGPropertyResults result = (SGPropertyResults) iResult.clone();
        result = super.setProperties(map, result);
        if (result == null) {
            return null;
        }
        
        result = SGUtilityForFigureElementJava2D
				.checkDataElementGroupVisibility(this, map, result);
		if (result == null) {
			return null;
		}
        
        List<SGElementGroupLine> lineGroups = this.getLineGroups();
        for (int ii = 0; ii < lineGroups.size(); ii++) {
            SGElementGroupLineForData lines = (SGElementGroupLineForData) lineGroups.get(ii);
            result = lines.setProperties(map, result);
            if (result == null) {
                return null;
            }
        }
        
        List<SGElementGroupSymbol> symbolGroups = this.getSymbolGroups();
        for (int ii = 0; ii < symbolGroups.size(); ii++) {
            SGElementGroupSymbolForData symbols = (SGElementGroupSymbolForData) symbolGroups.get(ii);
            result = symbols.setProperties(map, result);
            if (result == null) {
                return null;
            }
        }

        List<SGElementGroupBar> barGroups = this.getBarGroups();
        for (int ii = 0; ii < barGroups.size(); ii++) {
        	SGElementGroupBarForData bar = (SGElementGroupBarForData) barGroups.get(ii);
            result = bar.setProperties(map, result);
            if (result == null) {
                return null;
            }
        }
        
        List<SGElementGroupErrorBar> errorBarGroups = this.getErrorBarGroups();
        for (int ii = 0; ii < errorBarGroups.size(); ii++) {
            SGElementGroupErrorBarForData errorBar = (SGElementGroupErrorBarForData) errorBarGroups.get(ii);
            result = errorBar.setProperties(map, result);
            if (result == null) {
                return null;
            }
        }
        
        List<SGElementGroupTickLabel> tickLabelGroups = this.getTickLabelGroups();
        for (int ii = 0; ii < tickLabelGroups.size(); ii++) {
            SGElementGroupTickLabelForData tickLabel = (SGElementGroupTickLabelForData) tickLabelGroups.get(ii);
            result = tickLabel.setProperties(map, result);
            if (result == null) {
                return null;
            }
        }

        return result;
    }

    /**
     * Sets the data.
     * 
     * @param data
     *            a data object
     * @return true if succeeded
     */
    public boolean setData(SGData data) {
    	if (!super.setData(data)) {
    		return false;
    	}
    	if (data == null) {
        	this.initPointsArray(0);
    		return true;
    	} else {
        	if (!(data instanceof SGISXYTypeData)) {
        		return false;
        	}
        	SGISXYTypeData dataSXY = (SGISXYTypeData) data;
        	this.initPointsArray(dataSXY.getPointsNumber());
        	if (!this.updateDrawingElementsLocation(dataSXY)) {
        		return false;
        	}
    	}
    	return true;
    }

    /**
	 * Returns a list of line groups.
	 * 
	 * @return a list of line groups
	 */
	public List<SGElementGroupLine> getLineGroups() {
		List<SGElementGroupLine> retList = new ArrayList<SGElementGroupLine>();
		List<SGElementGroup> list = SGUtilityForFigureElement.getGroups(
				SGElementGroupLine.class, this.mDrawingElementGroupList);
		for (int ii = 0; ii < list.size(); ii++) {
			retList.add((SGElementGroupLine) list.get(ii));
		}
		return retList;
	}

	/**
	 * Returns a list of symbol groups.
	 * 
	 * @return a list of symbol groups
	 */
	public List<SGElementGroupSymbol> getSymbolGroups() {
		List<SGElementGroupSymbol> retList = new ArrayList<SGElementGroupSymbol>();
		List<SGElementGroup> list = SGUtilityForFigureElement.getGroups(
				SGElementGroupSymbol.class, this.mDrawingElementGroupList);
		for (int ii = 0; ii < list.size(); ii++) {
			retList.add((SGElementGroupSymbol) list.get(ii));
		}
		return retList;
	}

	/**
	 * Returns a list of bar groups.
	 * 
	 * @return a list of bar groups
	 */
	public List<SGElementGroupBar> getBarGroups() {
		List<SGElementGroupBar> retList = new ArrayList<SGElementGroupBar>();
		List<SGElementGroup> list = SGUtilityForFigureElement.getGroups(
				SGElementGroupBar.class, this.mDrawingElementGroupList);
		for (int ii = 0; ii < list.size(); ii++) {
			retList.add((SGElementGroupBar) list.get(ii));
		}
		return retList;
	}

	/**
	 * Returns a list of error bar groups.
	 * 
	 * @return a list of error bar groups
	 */
	public List<SGElementGroupErrorBar> getErrorBarGroups() {
		List<SGElementGroupErrorBar> retList = new ArrayList<SGElementGroupErrorBar>();
		List<SGElementGroup> list = SGUtilityForFigureElement.getGroups(
				SGElementGroupErrorBar.class, this.mDrawingElementGroupList);
		for (int ii = 0; ii < list.size(); ii++) {
			retList.add((SGElementGroupErrorBar) list.get(ii));
		}
		return retList;
	}

	/**
	 * Returns a list of tick label groups.
	 * 
	 * @return a list of tick label groups
	 */
	public List<SGElementGroupTickLabel> getTickLabelGroups() {
		List<SGElementGroupTickLabel> retList = new ArrayList<SGElementGroupTickLabel>();
		List<SGElementGroup> list = SGUtilityForFigureElement.getGroups(
				SGElementGroupTickLabel.class, this.mDrawingElementGroupList);
		for (int ii = 0; ii < list.size(); ii++) {
			retList.add((SGElementGroupTickLabel) list.get(ii));
		}
		return retList;
	}

	/**
	 * Returns a line group which is the first element of an array.
	 * 
	 * @return the first element of an array of line groups, or null when this
	 *         group set does not have any line groups
	 */
	public SGElementGroupLine getLineGroup() {
		return (SGElementGroupLine) SGUtilityForFigureElement.getGroup(
				SGElementGroupLine.class, this.mDrawingElementGroupList);
	}

	/**
	 * Returns a symbol group which is the first element of an array.
	 * 
	 * @return the first element of an array of symbol groups, or null when this
	 *         group set does not have any symbol groups
	 */
	public SGElementGroupSymbol getSymbolGroup() {
		return (SGElementGroupSymbol) SGUtilityForFigureElement.getGroup(
				SGElementGroupSymbol.class, this.mDrawingElementGroupList);
	}

	/**
	 * Returns a bar group which is the first element of an array.
	 * 
	 * @return the first element of an array of bar groups, or null when this
	 *         group set does not have any bar groups
	 */
	public SGElementGroupBar getBarGroup() {
		return (SGElementGroupBar) SGUtilityForFigureElement.getGroup(
				SGElementGroupBar.class, this.mDrawingElementGroupList);
	}

	/**
	 * Returns an error bar group which is the first element of an array.
	 * 
	 * @return the first element of an array of error bar groups, or null when
	 *         this group set does not have any error bar groups
	 */
	public SGElementGroupErrorBar getErrorBarGroup() {
		return (SGElementGroupErrorBar) SGUtilityForFigureElement
				.getGroup(SGElementGroupErrorBar.class,
						this.mDrawingElementGroupList);
	}

	/**
	 * Returns a tick label group which is the first element of an array.
	 * 
	 * @return the first element of an array of tick label groups, or null when
	 *         this group set does not have any tick label groups
	 */
	public SGElementGroupTickLabel getTickLabelGroup() {
		return (SGElementGroupTickLabel) SGUtilityForFigureElement
				.getGroup(SGElementGroupTickLabel.class,
						this.mDrawingElementGroupList);
	}

    /**
     * Sets the properties of element groups.
     * 
     * @param elementGroupPropertiesList
     * @return true if succeeded
     * 
     */
    protected boolean setElementGroupProperties(List elementGroupPropertiesList) {
        
        for (int ii = 0; ii < elementGroupPropertiesList.size(); ii++) {
            SGProperties gp = (SGProperties) elementGroupPropertiesList.get(ii);
            SGElementGroup group = null;
            if (gp instanceof SGElementGroupLine.LineProperties) {
                group = this.getLineGroup();
            } else if (gp instanceof SGElementGroupSymbol.SymbolProperties) {
                group = this.getSymbolGroup();
            } else if (gp instanceof SGElementGroupBar.BarProperties) {
                group = this.getBarGroup();
            } else if (gp instanceof SGElementGroupErrorBar.ErrorBarProperties) {
                group = this.getErrorBarGroup();
            } else if (gp instanceof SGElementGroupString.StringProperties) {
                group = this.getTickLabelGroup();
            } else {
                throw new Error("Illegal group property: " + gp);
            }
            if (group == null) {
                continue;
            }
            if (group.setProperties(gp) == false) {
                return false;
            }
        }
        return true;
    }

    @Override
    public int getMultipleDimensionStart() {
    	return -1;
    }

    @Override
    public boolean setMultipleDimensionStart(final int start) {
    	return false;
    }

    @Override
    public int getMultipleDimensionEnd() {
        return -1;
    }

    @Override
    public boolean setMultipleDimensionEnd(final int end) {
        return false;
    }
    
    /**
     * Returns the step of multiple dimensions.
     * 
     * @return the step of multiple dimensions
     */
    public int getMultipleDimensionStep() {
    	return -1;
    }
    
    /**
     * Sets the step of multiple dimensions.
     * 
     * @param step
     *           the step of multiple dimensions
     * @return true if succeeded
     */
    public boolean setMultipleDimensionStep(final int step) {
    	return false;
    }
    
    public int getSeriesIndex() {
        return this.mSeriesIndex;
    }
    
    public int getNumberOfSeries() {
        return this.mNumberOfSeries;
    }
    
    public void setSeriesIndexAndNumber(final int index, final int number) {
        this.mSeriesIndex = index;
        this.mNumberOfSeries = number;
    }
    
    @Override
    public double getLineShiftX() {
        return this.getLineGroup().getLineShiftX();
    }

    @Override
    public double getLineShiftY() {
        return this.getLineGroup().getLineShiftY();
    }

    @Override
    public boolean setLineShiftX(final double shift) {
        return this.getLineGroup().setLineShiftX(shift);
    }

    @Override
    public boolean setLineShiftY(final double shift) {
        return this.getLineGroup().setLineShiftY(shift);
    }

    @Override
    public double getSymbolShiftX() {
        return this.getSymbolGroup().getShiftX();
    }

    @Override
    public double getSymbolShiftY() {
        return this.getSymbolGroup().getShiftY();
    }

    @Override
    public boolean setSymbolShiftX(final double shift) {
        return this.getSymbolGroup().setShiftX(shift);
    }

    @Override
    public boolean setSymbolShiftY(final double shift) {
        return this.getSymbolGroup().setShiftY(shift);
    }

    @Override
    public double getBarShiftX() {
        return this.getBarGroup().getShiftX();
    }

    @Override
    public double getBarShiftY() {
        return this.getBarGroup().getShiftY();
    }

    @Override
    public boolean setBarShiftX(final double shift) {
        return this.getBarGroup().setShiftX(shift);
    }

    @Override
    public boolean setBarShiftY(final double shift) {
        return this.getBarGroup().setShiftY(shift);
    }

    @Override
    public double getBarInterval() {
        return this.getBarGroup().getInterval();
    }

    @Override
    public boolean setBarInterval(final double interval) {
        return this.getBarGroup().setInterval(interval);
    }

//    /**
//     * 
//     * @param shift
//     * @return
//     */
//    public float getShiftXLength(final double shift) {
//        if (shift==0.0f) {
//            return 0.0f;
//        }
//        
//        final SGAxis axis = this.getXAxis();
//        final double minValue = axis.getMinValue();
//        final double maxValue = axis.getMaxValue();
//        final double diff = maxValue - minValue;
//        if (diff==0.0) {
//            return 0.0f;
//        }
//        return this.mGraph.getGraphRectWidth() * (float) (shift / diff);
//    }
//    
//    /**
//     * 
//     * @param shift
//     * @return
//     */
//    public float getShiftYLength(final double shift) {
//        if (shift==0.0f) {
//            return 0.0f;
//        }
//        
//        final SGAxis axis = this.getYAxis();
//        final double minValue = axis.getMinValue();
//        final double maxValue = axis.getMaxValue();
//        final double diff = maxValue - minValue;
//        if (diff==0.0) {
//            return 0.0f;
//        }
//        return this.mGraph.getGraphRectHeight() * (float) (shift / diff);
//    }

    @Override
    public Double getShiftMinValue(SGAxis axis) {
        Double shift = null;
        if (axis.equals(this.getXAxis())) {
            if (this.isLineVisible() || this.isSymbolVisible()) {
                shift = Double.valueOf(this.getLineShiftX());
            }
            if (this.isBarVisible()) {
                // minimum left shift size of bars.
                // If this groupset has multiple bar series, minimum shift means shiftx point minus
                // bar widths of series which are enabled to be visible.
                double barLeftShift = this.getBarShiftX();
                if (this.isBarVertical()) {
                    int numberSeries = this.mGraph.getNumberOfVisibleBarSeries(this);
                    barLeftShift = barLeftShift -this.getBarInterval()*(numberSeries*0.5)-this.getBarWidthValue();
                }
                if (null==shift || shift.doubleValue()>barLeftShift) {
                    shift = Double.valueOf(barLeftShift);
                }
            }
        } else if (axis.equals(this.getYAxis())) {
            if (this.isLineVisible() || this.isSymbolVisible()) {
                shift = Double.valueOf(this.getLineShiftY());
            }
            if (this.isBarVisible()) {
                double barBottomShift = this.getBarShiftY();
                if (this.isBarVertical()==false) {
                    int numberSeries = this.mGraph.getNumberOfVisibleBarSeries(this);
                    barBottomShift = barBottomShift-this.getBarInterval()*(numberSeries*0.5)-this.getBarWidthValue();
                }
                if (null==shift || shift.doubleValue()>barBottomShift) {
                    shift = Double.valueOf(barBottomShift);
                }
            }
        }
        
        return shift;
    }

    @Override
    public Double getShiftMaxValue(SGAxis axis) {
        Double shift = null;
        if (axis.equals(this.getXAxis())) {
            if (this.isLineVisible() || this.isSymbolVisible()) {
                shift = Double.valueOf(this.getLineShiftX());
            }
            if (this.isBarVisible()) {
                // maximum right shift size of bars.
                // If this groupset has multiple bar series, maximum shift means shiftx point plus
                // bar widths of series which are enabled to be visible.
                double barRightShift = this.getBarShiftX();
                if (this.isBarVertical()) {
                    int numberSeries = this.mGraph.getNumberOfVisibleBarSeries(this);
                    barRightShift = barRightShift + this.getBarInterval()*(numberSeries*0.5)+this.getBarWidthValue();
                }
                if (null==shift || shift.doubleValue()<barRightShift) {
                    shift = Double.valueOf(barRightShift);
                }
            }
        } else if (axis.equals(this.getYAxis())) {
            if (this.isLineVisible() || this.isSymbolVisible()) {
                shift = Double.valueOf(this.getLineShiftY());
            }
            if (this.isBarVisible()) {
                double barTopShift = this.getBarShiftY();
                if (this.isBarVertical()==false) { 
                    int numberSeries = this.mGraph.getNumberOfVisibleBarSeries(this);
                    barTopShift = barTopShift+this.getBarInterval()*(numberSeries*0.5)+this.getBarWidthValue();
                }
                if (null==shift || shift.doubleValue()<barTopShift) {
                    shift = Double.valueOf(barTopShift);
                }
            }
        }
        
        return shift;
    }
    
    @Override
    public Double getBarBaselineForAxis(final SGAxis axis) {
        if (this.isBarVisible()==false) {
            return null;
        }
        if (axis.equals(this.getXAxis())) {
            if (this.isBarVertical()) {
                return null;
            } else {
                return Double.valueOf(this.getBarBaselineValue());
            }
        } else if (axis.equals(this.getYAxis())) {
            if (this.isBarVertical()) {
                return Double.valueOf(this.getBarBaselineValue());
            } else {
                return null;
            }
        }
        return null;
    }
    
	/**
	 * Sets the direction of error bars.
	 * 
	 * @param vertical
	 *           true to set vertical
	 * @return true if succeeded
	 */
	public boolean setErrorBarDirection(final boolean vertical) {
		return this.setErrorBarVertical(vertical);
	}

	/**
	 * Sets the alignment of tick label.
	 * 
	 * @param horizontal
	 *           true to align horizontally
	 * @return true if succeeded
	 */
	public boolean setTickLabelAlignment(final boolean horizontal) {
		return this.setTickLabelHorizontalAlignment(horizontal);
	}
	
}
