package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.Frame;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.SwingUtilities;

import jp.riken.brain.ni.samuraigraph.base.SGAxis;
import jp.riken.brain.ni.samuraigraph.base.SGData;
import jp.riken.brain.ni.samuraigraph.base.SGData.DataProperties;
import jp.riken.brain.ni.samuraigraph.base.SGDataAxisInfo;
import jp.riken.brain.ni.samuraigraph.base.SGIChildObject;
import jp.riken.brain.ni.samuraigraph.base.SGIConstants;
import jp.riken.brain.ni.samuraigraph.base.SGIDrawingElementConstants;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementAxis;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementForData;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementGraph;
import jp.riken.brain.ni.samuraigraph.base.SGProperties;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyDialog;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyMap;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyResults;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityNumber;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityText;
import jp.riken.brain.ni.samuraigraph.base.SGValueRange;
import jp.riken.brain.ni.samuraigraph.data.SGDataAnimationConfigurationDialog;
import jp.riken.brain.ni.samuraigraph.data.SGISXYData;
import jp.riken.brain.ni.samuraigraph.data.SGISXYTypeData;
import jp.riken.brain.ni.samuraigraph.data.SGISXYTypeMultipleData;
import jp.riken.brain.ni.samuraigraph.data.SGISXYZTypeData;
import jp.riken.brain.ni.samuraigraph.data.SGIThreeDimensionalData;
import jp.riken.brain.ni.samuraigraph.data.SGITwoDimensionalData;
import jp.riken.brain.ni.samuraigraph.data.SGIVXYTypeData;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFData;
import jp.riken.brain.ni.samuraigraph.figure.SGElementGroup;
import jp.riken.brain.ni.samuraigraph.figure.SGIArrowConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGIBarConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGIColorMapConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGIErrorBarConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGIFigureDrawingElementConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGILineConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGISXYDataConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGISymbolConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGITickLabelConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGIVXYDataConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGUtilityForFigureElement;
import jp.riken.brain.ni.samuraigraph.figure.java2d.SGElementGroupSetInFigureElement.ElementGroupSetPropertiesInFigureElement;
import jp.riken.brain.ni.samuraigraph.figure.java2d.SGElementGroupSetInFigureElement.MultipleElementGroupSetProperties;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * The base class of figure element which has data object and related element group set.
 * @author kuromaru
 *
 */
public abstract class SGFigureElementForData extends SGFigureElement2D implements
    SGIFigureDrawingElementConstants, SGIFigureElementForData,
    SGIDrawingElementConstants, SGISXYDataConstants, SGIVXYDataConstants {

    /**
     * The axis element.
     */
    protected SGIFigureElementAxis mAxisElement = null;
    
    /**
     * Default scale reference.
     */
    public static final String DEFAULT_SCALE_REFERENCE = SGIFigureElementAxis.LEFT_BOTTOM;

    /**
     * Default scale reference.
     */
    public static final float DEFAULT_MAGNITUDE_PER_CM = 1.0f;

    /**
     * A map of property dialogs for data.
     */
    private HashMap mDataDialogMap = new HashMap();
    
    private static final String DIALOG_KEY_SXY = "SXY";
    
    private static final String DIALOG_KEY_VXY = "VXY";

    private static final String DIALOG_KEY_SXYZ = "SXYZ";
    
    /**
     * The default constructor.
     *
     */
    public SGFigureElementForData() {
	super();
    }

    /**
     * Returns the group set object related to the given data object.
     * @param data
     *             a data object
     * @return
     *             A group set object related to the given data object.
     *             If it does not exist, returns null.
     */
    protected SGElementGroupSetInFigureElement getElementGroupSet(SGData data) {
        for (int ii = 0; ii < this.mChildList.size(); ii++) {
            SGElementGroupSetInFigureElement groupSet
                = (SGElementGroupSetInFigureElement) this.mChildList.get(ii);
            if (groupSet.getData().equals(data)) {
                return groupSet;
            }
        }
        return null;
    }

    /**
     * Returns the data object related to the given group set object.
     * @param groupSet
     *             a group set object
     * @return
     *             A data object related to the given group set object.
     *             If it does not exist, returns null.
     */
    protected SGData getData(SGElementGroupSetInFigureElement groupSet) {
        for (int ii = 0; ii < this.mChildList.size(); ii++) {
            SGElementGroupSetInFigureElement gs
            	= (SGElementGroupSetInFigureElement) this.mChildList.get(ii);
            if (gs.contains(groupSet)) {
                SGData data = groupSet.getData();
                return data;
            }
        }
        return null;
    }

    /**
     * Returns the name of a given data object.
     * @param data
     *             the data object to get the name
     * @return
     *             the name of a given data object
     */
    public String getDataName(final SGData data) {
        final SGElementGroupSetInFigureElement groupSet = this.getElementGroupSet(data);
        if (groupSet != null) {
            return groupSet.getName();
        }
        return null;
    }

    /**
     * Set the name of a data object.
     * @param name
     *             the name to set
     * @param data
     *             the data object to set the name
     * @return
     *         true if succeeded
     */
    public boolean setDataName(String name, SGData data) {
        final SGElementGroupSetInFigureElement groupSet = this.getElementGroupSet(data);
        if (groupSet != null) {
            groupSet.setName(name);
            return true;
        }
        return false;
    }

    /**
     * Returns properties of a given data object.
     * @param data
     *             a data object
     * @return
     *             the properties
     */
    public SGProperties getDataProperties(SGData data) {
        final SGElementGroupSetInFigureElement groupSet = this.getElementGroupSet(data);
        if (groupSet != null) {
            return groupSet.getProperties();
        }
        return null;
    }

    /**
     * Returns whether a given data is visible.
     * @param data
     *             a data object
     * @return
     *             true if a given data is visible
     */
    public boolean isDataVisible(SGData data) {
    	SGElementGroupSetInFigureElement groupSet = (SGElementGroupSetInFigureElement) this.getElementGroupSet(data);
        return groupSet.isVisible();
    }
    
    /**
     * Sets the visibility of a given data.
     * @param data
     *             a data object
     * @param b
     *             visibility to set to the data
     * @return
     *             true if succeeded
     */
    public boolean setDataVisible(final SGData data, final boolean b) {
    	SGElementGroupSetInFigureElement groupSet = this.getElementGroupSet(data);
        groupSet.setVisible(b);
        return true;
    }
    
    /**
     * Returns the list of visible data objects.
     * 
     * @return a list of visible data objects
     */
    public List<SGData> getVisibleDataList() {
    	List<SGData> list = new ArrayList<SGData>();
        for (int ii = 0; ii < this.mChildList.size(); ii++) {
            SGElementGroupSetInFigureElement groupSet
                = (SGElementGroupSetInFigureElement) this.mChildList.get(ii);
            if (groupSet.isVisible()) {
	        	SGData data = groupSet.getData();
	        	list.add(data);
            }
        }
        return list;
    }
    
    /**
     * Returns the list of visible flags of drawing elements.
     * @param data
     *             a data object
     * @return
     *             the list of visible flags of drawing elements
     */
    public List<Boolean> getVisibleFlagList(final SGData data) {
        if (data == null) {
            throw new IllegalArgumentException("data==null");
        }
        final SGElementGroupSetInFigureElement groupSet = this.getElementGroupSet(data);
        if (groupSet == null) {
            throw new Error("group set is not found");
        }
        List<Boolean> list = groupSet.getVisibleFlagList();
        return list;
    }

    /**
     * Returns whether a given data is visible in the legend.
     * @param data
     *             a data object
     * @return
     *             true if a given data object is visible in the legend
     */
    public boolean getVisibleInLegendFlag(SGData data) {
        if (data == null) {
            throw new IllegalArgumentException("data==null");
        }
        final SGElementGroupSetInFigureElement groupSet = this.getElementGroupSet(data);
        if (groupSet == null) {
            throw new Error("group set is not found");
        }
        boolean flag = groupSet.isVisibleInLegend();
        return flag;
    }

    /**
     * Returns the X axis for a given data.
     * @param data
     *             a data obejct
     * @return
     *             an axis object related to this data for the X-direction
     */
    public SGAxis getXAxis(final SGData data) {
        final SGElementGroupSetInFigureElement groupSet = this.getElementGroupSet(data);
        if (groupSet != null) {
            return groupSet.getXAxis();
        }
        return null;
    }

    /**
     * Returns the Y axis for a given data.
     * @param data
     *             a data obejct
     * @return
     *             an axis object related to this data for the Y-direction
     */
    public SGAxis getYAxis(final SGData data) {
        final SGElementGroupSetInFigureElement groupSet = this.getElementGroupSet(data);
        if (groupSet != null) {
            return groupSet.getYAxis();
        }
        return null;
    }

    /**
     * Returns a list of useless data objects in this figure element.
     * 
     * @return a list of useless data objects in this figure element
     */
    public List<SGData> getUselessDataList() {
		List<SGData> dataList = new ArrayList<SGData>(this.mDataList);
		Set cSet = this.getAvailableChildSet();
		Iterator itr = cSet.iterator();
		while (itr.hasNext()) {
		    SGElementGroupSetInFigureElement gs
		    	= (SGElementGroupSetInFigureElement)itr.next();
		    SGData data = gs.getData();
		    dataList.remove(data);
		}
		return dataList;
    }

    /**
     * Returns the property dialog of data.
     * @param dataType
     *                the type of data
     * @return
     *                the property dialog for given data type
     */
    public SGPropertyDialog getDataDialog(SGData data) {

        String key = null;
        if (data instanceof SGISXYData) {
            key = DIALOG_KEY_SXY;
        } else if (data instanceof SGIVXYTypeData) {
            key = DIALOG_KEY_VXY;
        } else if (data instanceof SGISXYZTypeData) {
            key = DIALOG_KEY_SXYZ;
        } else {
            throw new Error("Unsupported data type: " + data.getDataType());
        }
        
        Object obj = mDataDialogMap.get(key);
        SGPropertyDialog dg = null;
        if (obj != null) {
            dg = (SGPropertyDialog) obj;
        } else {
            if (DIALOG_KEY_SXY.equals(key)) {
                dg = new SGPropertyDialogSXYData(this.mDialogOwner, true);
            } else if (DIALOG_KEY_VXY.equals(key)) {
                dg = new SGPropertyDialogVXYData(this.mDialogOwner, true);
            } else if (DIALOG_KEY_SXYZ.equals(key)) {
                dg = new SGPropertyDialogSXYZData(this.mDialogOwner, true);
            }
            mDataDialogMap.put(key, dg);
        }
        return dg;
    }

    /**
     * Dispose this object.
     */
    public void dispose() {
        super.dispose();

        // dispose dialog map
        Iterator itr = this.mDataDialogMap.keySet().iterator();
        while (itr.hasNext()) {
            Object key = itr.next();
            SGPropertyDialog dg = (SGPropertyDialog) this.mDataDialogMap
                    .get(key);
            dg.dispose();
        }
        this.mDataDialogMap.clear();
        this.mDataDialogMap = null;
    }
 
    /**
     * Create an instance of element group set in this figure element.
     * 
     * @param data
     *            related data
     * @param name
     *            the name of data
     * @return created object
     */
    protected SGIElementGroupSetInFigureElement createGroupSet(SGData data, String name) {

        // get axes
        final SGAxis bAxis = this.mAxisElement
                .getAxisInPlane(SGIFigureElementAxis.AXIS_HORIZONTAL_1);
        final SGAxis tAxis = this.mAxisElement
                .getAxisInPlane(SGIFigureElementAxis.AXIS_HORIZONTAL_2);
        final SGAxis lAxis = this.mAxisElement
                .getAxisInPlane(SGIFigureElementAxis.AXIS_PERPENDICULAR_1);
        final SGAxis rAxis = this.mAxisElement
                .getAxisInPlane(SGIFigureElementAxis.AXIS_PERPENDICULAR_2);
        SGAxis axisX = null;
        SGAxis axisY = null;
        if (DEFAULT_SCALE_REFERENCE.equals(SGIFigureElementAxis.LEFT_BOTTOM)) {
            axisX = bAxis;
            axisY = lAxis;
        } else if (DEFAULT_SCALE_REFERENCE
                .equals(SGIFigureElementAxis.LEFT_TOP)) {
            axisX = tAxis;
            axisY = lAxis;
        } else if (DEFAULT_SCALE_REFERENCE
                .equals(SGIFigureElementAxis.RIGHT_BOTTOM)) {
            axisX = bAxis;
            axisY = rAxis;
        } else if (DEFAULT_SCALE_REFERENCE
                .equals(SGIFigureElementAxis.RIGHT_TOP)) {
            axisX = tAxis;
            axisY = rAxis;
        } else {
            return null;
        }

        // create an instance
        SGIElementGroupSetInFigureElement groupSet = null;
        if (data instanceof SGISXYTypeData) {
            groupSet = this.createGroupSetSXY((SGISXYTypeData) data, axisX,
                    axisY, false, name);
        } else if (data instanceof SGISXYTypeMultipleData) {
            groupSet = this.createGroupSetMultipleSXY(
                    (SGISXYTypeMultipleData) data, axisX, axisY, name);
        } else if (data instanceof SGIVXYTypeData) {
//            if (data instanceof SGArrayData) {
//                groupSet = this.createGroupSetVXY((SGIVXYTypeData) data, axisX,
//                        axisY, name);
//            } else if (data instanceof SGNetCDFData) {
//                groupSet = this.createGroupSetGridVXY((SGIVXYTypeData) data, axisX,
//                        axisY, name);
//            } else {
//                throw new Error("Not supported data type: " + data.getDataType());
//            }
            groupSet = this.createGroupSetVXY((SGIVXYTypeData) data, axisX,
                    axisY, name);
        } else if (data instanceof SGISXYZTypeData) {
            SGAxis axisZ = this.mAxisElement.getZAxis();
//            if (data instanceof SGArrayData) {
//                groupSet = this.createGroupSetSXYZ((SGISXYZTypeData) data, axisX, 
//                        axisY, axisZ, name);
//            } else if (data instanceof SGNetCDFData) {
//                groupSet = this.createGroupSetGridSXYZ((SGISXYZTypeData) data, axisX, 
//                        axisY, axisZ, name);
//            } else {
//                throw new Error("Not supported data type: " + data.getDataType());
//            }
            groupSet = this.createGroupSetSXYZ((SGISXYZTypeData) data, axisX, 
                    axisY, axisZ, name);
        } else {
            throw new Error("Not supported data type: " + data.getDataType());
        }
        
        if (groupSet == null) {
            return null;
        }
        
        // set magnification
        groupSet.setMagnification(this.getMagnification());

        return groupSet;
    }

    // create SXY groupSet
    protected SGIElementGroupSetSXY createGroupSetSXY(
            final SGISXYTypeData data, final SGAxis axisX, final SGAxis axisY,
            final boolean multiple, final String name) {
        SGIElementGroupSetSXY groupSet = this.createSingleGroupSetSXY(
                data, axisX, axisY, multiple);
        if (groupSet == null) {
        	return null;
        }
        groupSet.setName(name);
        return groupSet;
    }

    // create an instance of SXY groupSet
    protected SGIElementGroupSetSXY createSingleGroupSetSXY(
            final SGISXYTypeData data, final SGAxis axisX, final SGAxis axisY,
            final boolean multiple) {

        final SGIElementGroupSetSXY groupSet
            = this.createSXYGroupSetInstance(data);
        if (groupSet == null) {
        	return null;
        }

        // set axes
        groupSet.setXAxis(axisX);
        groupSet.setYAxis(axisY);

        // add drawing element groups of lines, symbols and bars
        if (groupSet.addDrawingElementGroup(SGIElementGroupConstants.RECTANGLE_GROUP) == false) {
            return null;
        }

        // set bar width from the interval of x values and graph width
        if (!multiple) {
            SGElementGroupBar groupBar = (SGElementGroupBar) groupSet.getBarGroup();
			if (data != null) {
				final boolean vertical = groupBar.isVertical();
				final double[] valueArray = vertical ? data.getXValueArray()
						: data.getYValueArray();
				final SGAxis axis = vertical ? axisX : axisY;
				final double widthValue = this.calcDefaultBarWidth(valueArray,
						axis);
				groupBar.setWidthValue(widthValue);
			}
        }

        // create error bar instance
        groupSet.createErrorBars(data);

        if (groupSet
                .addDrawingElementGroup(SGIElementGroupConstants.POLYLINE_GROUP) == false) {
            return null;
        }

        if (groupSet
                .addDrawingElementGroup(SGIElementGroupConstants.SYMBOL_GROUP) == false) {
            return null;
        }

        // create tick label instance
        SGElementGroupTickLabel tickLabels = groupSet.createTickLabels(data);

        if (!multiple) {
            // set decimal places and exponents of data object to tick labels
            final int dp = data.getDecimalPlaces();
            final int exp = data.getExponent();
            tickLabels.setDecimalPlaces(dp);
            tickLabels.setExponent(exp);
        }

        return groupSet;
    }
    
    private double calcDefaultBarWidth(final double[] xArray, final SGAxis axisX) {
        double minDiff = Double.MAX_VALUE;
        for (int ii = 0; ii < xArray.length - 1; ii++) {
            final double diff = Math.abs(xArray[ii + 1] - xArray[ii]);
            if (diff < minDiff) {
                minDiff = diff;
            }
        }
        minDiff = this.getNumberInRangeOrder(minDiff, axisX);
        return minDiff;
    }
    
    /**
     * Return bar width of a given groupSet.
     * @param gs
     * @return
     */
    protected double getBarWidthOfBarSeries(SGIElementGroupSetXY gs) {
        final double[] valueArray;
        SGElementGroupBar bar;
        SGData data;
        boolean vertical;
        SGAxis axis;
        
        if (gs instanceof SGElementGroupSetInGraphSXY) {
            SGElementGroupSetInGraphSXY egs = (SGElementGroupSetInGraphSXY)gs;
            data = egs.getData();
            bar = egs.getBarGroup();
            vertical = bar.isVertical();
            axis = vertical ? egs.getXAxis() : egs.getYAxis();
        } else if (gs instanceof SGElementGroupSetMultipleGraphSXY) {
            SGElementGroupSetMultipleGraphSXY egs = (SGElementGroupSetMultipleGraphSXY)gs;
            data = egs.getData();
            bar = egs.getBarGroup();
            vertical = bar.isVertical();
            axis = vertical ? egs.getXAxis() : egs.getYAxis();
        } else {
            return Double.NaN;
        }
        
        if (bar.isVisible()) {
            if (data instanceof SGISXYTypeData) {
                valueArray = vertical ? ((SGISXYTypeData)data).getXValueArray() : ((SGISXYTypeData)data).getYValueArray();
            } else if (data instanceof SGISXYTypeMultipleData) {
                valueArray = vertical ? ((SGISXYTypeMultipleData)data).getXValueArray()[0] : ((SGISXYTypeMultipleData)data).getYValueArray()[0];
            } else {
                return Double.NaN;
            }
        } else {
            return Double.NaN;
        }
        double widthValue = this.calcDefaultBarWidth(valueArray, axis);
        
        widthValue = this.getNumberInRangeOrder(widthValue, axis);
        return widthValue;
    }
    
    /**
     * Return bar width from the interval of x values and graph width.
     * @param groupBar
     * @param barGroups
     * @param data
     * @param axisX
     * @param axisY
     * @return calculated bar width
     */
    public double getAutoArrangementOfBarWidth(
            final SGElementGroupBar groupBar, final int numberOfBars,
            final SGISXYTypeMultipleData data, final SGAxis axisX, final SGAxis axisY) {
        final boolean vertical = groupBar.isVertical();
        final double[][] valueArray = vertical ? data.getXValueArray() : data.getYValueArray();
        final SGAxis axis = vertical ? axisX : axisY;
        
        double widthValue = this.calcDefaultBarWidth(valueArray[0], axis);
        
        if (numberOfBars>=1) {
            widthValue = widthValue/(numberOfBars+1);
        }
        
        return widthValue;
    }
    
    /**
     * Return bar width from the interval of x values and graph width.
     * 
     * @param groupBar
     * @param numberOfBars
     * @param data
     * @param axisX
     * @param axisY
     * @return calculated bar width
     */
    public double getAutoArrangementOfBarWidth(
            final SGElementGroupBar groupBar, final int numberOfBars,
            final SGISXYTypeData data, final SGAxis axisX, final SGAxis axisY) {
        final boolean vertical = groupBar.isVertical();
        final double[] valueArray = vertical ? data.getXValueArray() : data.getYValueArray();
        final SGAxis axis = vertical ? axisX : axisY;
        
        double widthValue = this.calcDefaultBarWidth(valueArray, axis);
        
        if (numberOfBars>=1) {
            widthValue = widthValue/(numberOfBars+1);
        }
        
        return widthValue;
    }

    // create VXY groupSet
    protected SGIElementGroupSetVXY createGroupSetVXY(
            final SGIVXYTypeData data, final SGAxis axisX, final SGAxis axisY,
            final String name) {
        SGIElementGroupSetVXY groupSet = this.createSingleGroupSetVXY(
                data, axisX, axisY);
        if (groupSet == null) {
        	return null;
        }
        groupSet.setName(name);
        return groupSet;
    }

//    // create Grid VXY groupSet
//    protected SGIElementGroupSetVXY createGroupSetGridVXY(
//            final SGIVXYTypeData data, final SGAxis axisX, final SGAxis axisY,
//            final String name) {
//        SGIElementGroupSetVXY groupSet = this.createSingleGroupSetGridVXY(
//                data, axisX, axisY);
//        if (groupSet == null) {
//        	return null;
//        }
//        groupSet.setName(name);
//        return groupSet;
//    }

    // create SXYZ groupSet
    protected SGIElementGroupSetSXYZ createGroupSetSXYZ(
            final SGISXYZTypeData data, final SGAxis axisX, final SGAxis axisY,
            final String name) {
        SGIElementGroupSetSXYZ groupSet = this.createSingleGroupSetSXYZ(
                data, axisX, axisY);
        if (groupSet == null) {
        	return null;
        }
        groupSet.setName(name);
        return groupSet;
    }

//    // create Grid SXYZ groupSet
//    protected SGIElementGroupSetGridSXYZ createGroupSetGridSXYZ(
//            final SGISXYZTypeData data, final SGAxis axisX, final SGAxis axisY,
//            final String name) {
//        SGIElementGroupSetGridSXYZ groupSet = this.createSingleGroupSetGridSXYZ(
//                data, axisX, axisY);
//        if (groupSet == null) {
//        	return null;
//        }
//        groupSet.setName(name);
//        return groupSet;
//    }

    // create an instance of VXY groupSet
    protected SGIElementGroupSetVXY createSingleGroupSetVXY(
            final SGIVXYTypeData data, final SGAxis axisX, final SGAxis axisY) {

        final SGIElementGroupSetVXY groupSet
            = this.createVXYGroupSetInstance(data);
        if (groupSet == null) {
        	return null;
        }

        // set axes
        groupSet.setXAxis(axisX);
        groupSet.setYAxis(axisY);

        // add drawing element groups of arrows
        if (groupSet
                .addDrawingElementGroup(SGIElementGroupConstants.ARROW_GROUP) == false) {
            return null;
        }

        // set the magnitude per cm
        final float percmReduced = this.getInitialMagnitudePerCM(data);
        groupSet.setMagnitudePerCM(percmReduced);
        groupSet.setDirectionInvariant(data.isPolar());

        return groupSet;
    }

//    // create an instance of grid VXY groupSet
//    protected SGIElementGroupSetVXY createSingleGroupSetGridVXY(
//            final SGIVXYTypeData data, final SGAxis axisX, final SGAxis axisY) {
//
//        final SGIElementGroupSetVXY groupSet
//            = this.createGridVXYGroupSetInstance(data);
//        if (groupSet == null) {
//        	return null;
//        }
//
//        // set axes
//        groupSet.setXAxis(axisX);
//        groupSet.setYAxis(axisY);
//
//        // add drawing element groups of arrows
//        if (groupSet
//                .addDrawingElementGroup(SGIElementGroupConstants.ARROW_GROUP) == false) {
//            return null;
//        }
//
//        // set the magnitude per cm
//        final float percmReduced = this.getInitialMagnitudePerCM(data);
//        groupSet.setMagnitudePerCM(percmReduced);
//        groupSet.setDirectionInvariant(data.isPolar());
//
//        return groupSet;
//    }

    /**
     * Calculates the initial magnitude of the vector per centimeter.
     * 
     * @param data
     *           the vector data
     * @return the initial magnitude of the vector per centimeter
     */
    protected float getInitialMagnitudePerCM(SGIVXYTypeData data) {
        final double[] mArray = data.getMagnitudeArray();
        final float max = (float) SGUtilityNumber.max(mArray);
        if (Float.isNaN(max) || max <= 0.0f) {
        	return DEFAULT_MAGNITUDE_PER_CM;
        }
        final float size = Math.max(this.getGraphRectWidth(), this
                .getGraphRectHeight())
                / this.getMagnification();
        final float len = 0.10f * size;
        final float percm = max / (len * SGIConstants.CM_POINT_RATIO);
        final float percmReduced = SGUtilityForFigureElement
                .roundMagnitudePerCM(percm, data);
        return percmReduced;
    }


    // create an instance of SXYZ groupSet
    protected SGIElementGroupSetSXYZ createSingleGroupSetSXYZ(
            final SGISXYZTypeData data, final SGAxis axisX, final SGAxis axisY) {

        final SGIElementGroupSetSXYZ groupSet
            = this.createSXYZGroupSetInstance(data);
        if (groupSet == null) {
        	return null;
        }

        // set axes
        groupSet.setXAxis(axisX);
        groupSet.setYAxis(axisY);

        // add drawing element groups of rectangles
        if (groupSet
                .addDrawingElementGroup(SGIElementGroupConstants.RECTANGLE_GROUP) == false) {
            return null;
        }
        
        if (groupSet.updateColorMapSize() == false) {
            return null;
        }

        return groupSet;
    }

//    // create an instance of Grid SXYZ groupSet
//    protected SGIElementGroupSetGridSXYZ createSingleGroupSetGridSXYZ(
//            final SGISXYZTypeData data, final SGAxis axisX, final SGAxis axisY) {
//
//        final SGIElementGroupSetGridSXYZ groupSet
//            = this.createGridSXYZGroupSetInstance(data);
//        if (groupSet == null) {
//        	return null;
//        }
//
//        // set axes
//        groupSet.setXAxis(axisX);
//        groupSet.setYAxis(axisY);
//
//        // add drawing element groups of rectangles
//        if (groupSet
//                .addDrawingElementGroup(SGIElementGroupConstants.RECTANGLE_GROUP) == false) {
//            return null;
//        }
//
//        return groupSet;
//    }

    // create multiple SXY groupSet
    protected SGIElementGroupSetMultipleSXY createGroupSetMultipleSXY(
            final SGISXYTypeMultipleData data, final SGAxis axisX,
            final SGAxis axisY, final String name) {

        final SGIElementGroupSetMultipleSXY groupSet
            = this.createMultipleSXYGroupSetInstance(data);
        if (groupSet == null) {
            return null;
        }

        // set the name
        groupSet.setName(name);

        // set axes
        groupSet.setXAxis(axisX);
        groupSet.setYAxis(axisY);
        
        // set up child group sets
        if (this.updateChildGroupSet(groupSet) == false) {
        	return null;
        }
        
        // add element groups to the multiple group set
        if (!groupSet.addDrawingElementGroup(SGIElementGroupConstants.RECTANGLE_GROUP)) {
            return null;
        }
        
        // set bar width from the interval of x values and graph width
        SGElementGroupBar groupBar = (SGElementGroupBar) groupSet.getBarGroup();
        List<SGElementGroupBar> barGroups = groupSet.getBarGroups();
        final int numberOfBars = barGroups.size()-1;
        double barWidth = this.getAutoArrangementOfBarWidth(groupBar, numberOfBars, data, axisX, axisY);
        if (barWidth!=0.0) {
            int digit = BAR_WIDTH_INITIAL_ORDER-1;
            double widthValue = SGUtilityNumber.roundOffNumber(barWidth, digit);
            int count = 20;
            while (widthValue == 0.0 && count-->0) {
                digit = digit -1;
                widthValue = SGUtilityNumber.roundOffNumber(barWidth, digit);
            }
            barWidth = widthValue;
        }
        final double intervalValue = barWidth;
        groupBar.setWidthValue(barWidth);
        groupBar.setInterval(intervalValue);
        for (int ii = 0; ii < barGroups.size(); ii++) {
            SGElementGroupBar barGroup = barGroups.get(ii);
            barGroup.setWidthValue(barWidth);
            barGroup.setInterval(intervalValue);
        }

        // create error bars
        groupSet.createErrorBars(data);
        Boolean errorBarVertical = data.isErrorBarVertical();
        if (errorBarVertical != null) {
        	groupSet.setErrorBarDirection(errorBarVertical.booleanValue());
        }
        
        if (!groupSet.addDrawingElementGroup(SGIElementGroupConstants.POLYLINE_GROUP)) {
        	return null;
        }
        if (!groupSet.addDrawingElementGroup(SGIElementGroupConstants.SYMBOL_GROUP)) {
        	return null;
        }
        
        // create tick labels
        groupSet.createTickLabels(data);
        Boolean tickLabelHorizontal = data.isTickLabelHorizontal();
        if (tickLabelHorizontal != null) {
        	groupSet.setTickLabelAlignment(tickLabelHorizontal.booleanValue());
        }
        
        return groupSet;
    }
    
    // Sets up child group sets of multiple data.
    protected boolean updateChildGroupSet(SGIElementGroupSetMultipleSXY groupSet) {
        
    	// get current values
    	final SGISXYTypeMultipleData dataMult = (SGISXYTypeMultipleData) groupSet.getData();
    	final SGAxis axisX = groupSet.getXAxis();
    	final SGAxis axisY = groupSet.getYAxis();
    	final String name = groupSet.getName();
    	final SGIElementGroupSetInFigureElement[] cArray = groupSet.getChildGroupSetArray();
    	
    	// get new data array
        final SGISXYTypeData[] dArray = dataMult.getSXYDataArray();
        
        try {
            for (int i = 0; i < cArray.length; i++) {
                if (cArray[i] instanceof SGElementGroupSetInGraphSXY) {
                    ((SGElementGroupSetInGraphSXY)cArray[i]).setSeriesIndexAndNumber(i, dArray.length);
                }
            }
            
            if (cArray.length < dArray.length) {

                // set new data
                for (int ii = 0; ii < cArray.length; ii++) {
                	SGData d = (SGData) dArray[ii];
                    cArray[ii].setData((SGData) d.copy());
                }

                // create new child group sets
                for (int ii = cArray.length; ii < dArray.length; ii++) {
                	SGData d = (SGData) dArray[ii];
                    SGIElementGroupSetSXY gs = this.createSingleGroupSetSXY(
                            (SGISXYTypeData) d.copy(), axisX, axisY, true);
                    if (gs == null) {
                        return false;
                    }
                    if (gs instanceof SGElementGroupSetInGraphSXY) {
                        ((SGElementGroupSetInGraphSXY)gs).setSeriesIndexAndNumber(ii, dArray.length);
                    }
                    gs.setName(name);
                    groupSet.addChildGroupSet(gs);
                }

            } else {
            	
            	// remove child group set
            	for (int ii = cArray.length - 1; ii >= dArray.length; ii--) {
            		groupSet.removeChildGroupSet(cArray[ii]);
            	}
            	
                // set new data
                for (int ii = 0; ii < dArray.length; ii++) {
                	SGData d = (SGData) dArray[ii];
                    cArray[ii].setData((SGData) d.copy());
                }
            }
        	
        } finally {

        	if (dArray != null) {
                // disposes of data array
                for (int ii = 0; ii < dArray.length; ii++) {
                	dArray[ii].dispose();
                }
        	}
        }
        
        return true;
    }

    // create SXYZ groupSet
    protected SGIElementGroupSetSXYZ createGroupSetSXYZ(
            final SGISXYZTypeData data, final SGAxis axisX, final SGAxis axisY,
            final SGAxis axisZ, final String name) {
        SGIElementGroupSetSXYZ groupSet = this.createSingleGroupSetSXYZ(
                data, axisX, axisY, axisZ);
        if (groupSet == null) {
        	return null;
        }
        groupSet.setName(name);
        return groupSet;
    }

    // create an instance of SXYZ groupSet
    protected SGIElementGroupSetSXYZ createSingleGroupSetSXYZ(
            final SGISXYZTypeData data, final SGAxis axisX, final SGAxis axisY,
            final SGAxis axisZ) {

        final SGIElementGroupSetSXYZ groupSet
            = this.createSXYZGroupSetInstance(data);
        if (groupSet == null) {
        	return null;
        }

        // set axes
        groupSet.setXAxis(axisX);
        groupSet.setYAxis(axisY);
        groupSet.setZAxis(axisZ);

        // add drawing element groups
        if (groupSet
                .addDrawingElementGroup(SGIElementGroupConstants.RECTANGLE_GROUP) == false) {
            return null;
        }
        
        if (groupSet.updateColorMapSize() == false) {
        	return null;
        }
        
//        final double sizeMinFactor = 0.010;
//        final double xMin = axisX.getMinValue();
//        final double xMax = axisX.getMaxValue();
//        final double wMin = (xMax - xMin) * sizeMinFactor;
//        final double yMin = axisY.getMinValue();
//        final double yMax = axisY.getMaxValue();
//        final double hMin = (yMax - yMin) * sizeMinFactor;
//        
//        // set rectangle size from the interval of x and y values and graph size
//        final double widthValue, heightValue;
//        
//        final double[] xArray = data.getXValueArray();
//        Arrays.sort(xArray);
//        double minDiffX = Double.MAX_VALUE;
//        for (int ii = 0; ii < xArray.length - 1; ii++) {
//            final double diff = Math.abs(xArray[ii + 1] - xArray[ii]);
//            if (diff < wMin) {
//                continue;
//            }
//            if (diff < minDiffX) {
//                minDiffX = diff;
//            }
//        }
//        if (minDiffX == Double.MAX_VALUE) {
//            widthValue = 1.0;
//        } else {
//            minDiffX = this.getNumberInRangeOrder(minDiffX, axisX, BigDecimal.ROUND_HALF_UP);
//            widthValue = minDiffX;
//        }
//        
//        final double[] yArray = data.getYValueArray();
//        Arrays.sort(yArray);
//        double minDiffY = Double.MAX_VALUE;
//        for (int ii = 0; ii < yArray.length - 1; ii++) {
//            final double diff = Math.abs(yArray[ii + 1] - yArray[ii]);
//            if (diff < hMin) {
//                continue;
//            }
//            if (diff < minDiffY) {
//                minDiffY = diff;
//            }
//        }
//        if (minDiffY == Double.MAX_VALUE) {
//            heightValue = 1.0;
//        } else {
//            minDiffY = this.getNumberInRangeOrder(minDiffY, axisY, BigDecimal.ROUND_HALF_UP);
//            heightValue = minDiffY;
//        }
//        
//        SGElementGroupColorMap colorMap = (SGElementGroupColorMap) groupSet.getColorMap();
//        colorMap.setWidthValue(widthValue);
//        colorMap.setHeightValue(heightValue);
        
        return groupSet;
    }
    
//    // create grid SXYZ groupSet
//    protected SGIElementGroupSetGridSXYZ createGroupSetGridSXYZ(
//            final SGISXYZTypeData data, final SGAxis axisX, final SGAxis axisY,
//            final SGAxis axisZ, final String name) {
//        SGIElementGroupSetGridSXYZ groupSet = this.createSingleGroupSetGridSXYZ(
//                data, axisX, axisY, axisZ);
//        if (groupSet == null) {
//        	return null;
//        }
//        groupSet.setName(name);
//        return groupSet;
//    }

//    // create an instance of grid SXYZ groupSet
//    protected SGIElementGroupSetGridSXYZ createSingleGroupSetGridSXYZ(
//            final SGISXYZTypeData data, final SGAxis axisX, final SGAxis axisY,
//            final SGAxis axisZ) {
//
//        final SGIElementGroupSetGridSXYZ groupSet
//            = this.createGridSXYZGroupSetInstance(data);
//        if (groupSet == null) {
//        	return null;
//        }
//
//        // set axes
//        groupSet.setXAxis(axisX);
//        groupSet.setYAxis(axisY);
//        groupSet.setZAxis(axisZ);
//
//        // add drawing element groups
//        if (groupSet
//                .addDrawingElementGroup(SGIElementGroupConstants.RECTANGLE_GROUP) == false) {
//            return null;
//        }
//        
//        return groupSet;
//    }
        
    protected abstract SGIElementGroupSetSXY createSXYGroupSetInstance(SGISXYTypeData dataSXY);
    protected abstract SGIElementGroupSetVXY createVXYGroupSetInstance(SGIVXYTypeData dataVXY);
    protected abstract SGIElementGroupSetMultipleSXY createMultipleSXYGroupSetInstance(SGISXYTypeMultipleData dataMultSXY);
    protected abstract SGIElementGroupSetSXYZ createSXYZGroupSetInstance(SGISXYZTypeData dataSXYZ);
//    protected abstract SGIElementGroupSetGridSXYZ createGridSXYZGroupSetInstance(SGISXYZTypeData dataSXYZ);
//    protected abstract SGIElementGroupSetVXY createGridVXYGroupSetInstance(SGIVXYTypeData dataVXY);
    

    /**
     * 
     */
    public boolean createDataObject(final Element el, final SGData data, final boolean readDataProperty) {

        if (super.createDataObject(el, data, readDataProperty) == false) {
            return false;
        }

//        final int ic = SGIConstants.PROPERTY_FILE_INCORRECT;
        SGIFigureElementAxis aElement = this.mAxisElement;
        String str = null;

        // name of data
        String name = null;
        str = el.getAttribute(KEY_DATA_NAME);
        if (str.length() == 0) {
            return false;
        }
        name = str;

        // x-axis
        SGAxis axisX = null;
        str = el.getAttribute(KEY_X_AXIS_POSITION);
        if (str.length() != 0) {
            axisX = aElement.getAxis(str);
            if (axisX == null) {
                return false;
            }
        }

        // y-axis
        SGAxis axisY = null;
        str = el.getAttribute(KEY_Y_AXIS_POSITION);
        if (str.length() != 0) {
            axisY = aElement.getAxis(str);
            if (axisY == null) {
                return false;
            }
        }
        
        // visibility in legend
        Boolean vLegend = Boolean.FALSE; 
        str = el.getAttribute(KEY_VISIBLE_IN_LEGEND);
        if (str.length() != 0) {
            Boolean b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return false;
            }
            vLegend = b;
        }

//        // create an instance
//        SGIElementGroupSetInFigureElement gs = null;
//        if (data instanceof SGISXYTypeData) {
//            gs = this.createGroupSetSXY((SGISXYTypeData) data, axisX, axisY,
//                    name);
//        } else if (data instanceof SGISXYTypeMultipleData) {
//            gs = this.createGroupSetMultipleSXY((SGISXYTypeMultipleData) data,
//                    axisX, axisY, name);
//        } else if (data instanceof SGIVXYTypeData) {
//            gs = this.createGroupSetVXY((SGIVXYTypeData) data, axisX, axisY,
//                    name);
//        } else {
//            throw new Error("Not supported data type: " + data.getDataType());
//        }
//
//        // set properties
//        if (gs.setVisibleInLegend(vLegend.booleanValue()) == false) {
//            return false;
//        }
//
//        if (data instanceof SGISXYTypeData) {
//            SGISXYTypeData dataSXY = (SGISXYTypeData) data;
//            SGIElementGroupSetSXY groupSet = (SGIElementGroupSetSXY) gs;
//            if (this.setPropertyOfElementGroupSetSXY(el, groupSet, dataSXY) == ic) {
//                return false;
//            }
//        } else if (data instanceof SGISXYTypeMultipleData) {
//            SGISXYTypeMultipleData dataMultSXY = (SGISXYTypeMultipleData) data;
//            SGIElementGroupSetMultipleSXY groupSet = (SGIElementGroupSetMultipleSXY) gs;
//            if (this.setPropertyOfElementGroupSetForMultipleSXY(el,
//                    groupSet, dataMultSXY) == ic) {
//                return false;
//            }
//        } else if (data instanceof SGIVXYTypeData) {
//            SGIVXYTypeData dataVXY = (SGIVXYTypeData) data;
//            SGIElementGroupSetVXY groupSet = (SGIElementGroupSetVXY) gs;
//            if (this.setPropertyOfElementGroupSetVXY(el, groupSet, dataVXY) == ic) {
//                return false;
//            }
//        }
//        
//        // read data properties
//        if (readDataProperty) {
//            if (data.readProperty(el) == false) {
//                return false;
//            }
//        }
        
        SGIElementGroupSetInFigureElement gs = this.createGroupSet(el, data, axisX, axisY, name, readDataProperty);
        if (gs == null) {
            return false;
        }
        
        // set properties
        if (gs.setVisibleInLegend(vLegend.booleanValue()) == false) {
            return false;
        }

        // add to the child list
        this.addToList(gs);

        // initialize the property history
        gs.initPropertiesHistory();
//        this.setDataPropertiesHistory((SGData) data, gs.getProperties());

        return true;
    }
    
    protected SGIElementGroupSetInFigureElement createGroupSet(final Element el, final SGData data, 
            final SGAxis axisX, final SGAxis axisY, final String name, final boolean readDataProperty) {
        
        final int ic = SGIConstants.PROPERTY_FILE_INCORRECT;

        // set data properties
        if (readDataProperty) {
//            if (data.readProperty(el) == false) {
//                return null;
//            }
            
            // Note: do not check the returned value for old data set
            data.readProperty(el);
        }
        
        // create an instance
        SGIElementGroupSetInFigureElement gs = null;
        if (data instanceof SGISXYTypeData) {
            SGISXYTypeData dataSXY = (SGISXYTypeData) data;
            gs = this.createGroupSetSXY(dataSXY, axisX, axisY, true, name);
            SGIElementGroupSetSXY groupSet = (SGIElementGroupSetSXY) gs;
            if (this.setPropertyOfElementGroupSetSXY(el, groupSet, dataSXY) == ic) {
                return null;
            }
        } else if (data instanceof SGISXYTypeMultipleData) {
            SGISXYTypeMultipleData dataMultSXY = (SGISXYTypeMultipleData) data;
            gs = this.createGroupSetMultipleSXY(dataMultSXY, axisX, axisY, name);
            SGIElementGroupSetMultipleSXY groupSet = (SGIElementGroupSetMultipleSXY) gs;
            if (this.setPropertyOfElementGroupSetForMultipleSXY(el,
                    groupSet, dataMultSXY) == ic) {
                return null;
            }
        } else if (data instanceof SGIVXYTypeData) {
            SGIVXYTypeData dataVXY = (SGIVXYTypeData) data;
//            if (data instanceof SGArrayData) {
//                gs = this.createGroupSetVXY((SGIVXYTypeData) data, axisX,
//                        axisY, name);
//            } else if (data instanceof SGNetCDFData) {
//                gs = this.createGroupSetGridVXY((SGIVXYTypeData) data, axisX,
//                        axisY, name);
//            } else {
//                throw new Error("Not supported data type: " + data.getDataType());
//            }
            gs = this.createGroupSetVXY((SGIVXYTypeData) data, axisX,
                    axisY, name);
            SGIElementGroupSetVXY groupSet = (SGIElementGroupSetVXY) gs;
            if (this.setPropertyOfElementGroupSetVXY(el, groupSet, dataVXY) == ic) {
                return null;
            }
        } else if (data instanceof SGISXYZTypeData) {
            SGISXYZTypeData dataSXYZ = (SGISXYZTypeData) data;
//            if (data instanceof SGArrayData) {
//                gs = this.createGroupSetSXYZ(dataSXYZ, axisX, axisY, name);
//                SGIElementGroupSetSXYZ groupSet = (SGIElementGroupSetSXYZ) gs;
//                if (this.setPropertyOfElementGroupSetSXYZ(el, groupSet, dataSXYZ) == ic) {
//                    return null;
//                }
//            } else if (data instanceof SGNetCDFData) {
//                gs = this.createGroupSetGridSXYZ(dataSXYZ, axisX, axisY, name);
//                SGIElementGroupSetGridSXYZ groupSet = (SGIElementGroupSetGridSXYZ) gs;
//                if (this.setPropertyOfElementGroupSetGridSXYZ(el, groupSet, dataSXYZ) == ic) {
//                    return null;
//                }
//            } else {
//                throw new Error("Not supported data type: " + data.getDataType());
//            }
            gs = this.createGroupSetSXYZ(dataSXYZ, axisX, axisY, name);
            SGIElementGroupSetSXYZ groupSet = (SGIElementGroupSetSXYZ) gs;
            if (this.setPropertyOfElementGroupSetSXYZ(el, groupSet, dataSXYZ) == ic) {
                return null;
            }
        } else {
            throw new Error("Not supported data type: " + data.getDataType());
        }

        return gs;
    }
    
    protected int setPropertyOfElementGroupSetSXY(final Element el,
            final SGIElementGroupSetSXY groupSet,
            final SGISXYTypeData data) {

        final int ic = SGIConstants.PROPERTY_FILE_INCORRECT;

//        // create instances of the points
//        final int num = data.getPointsNumber();
//        groupSet.initPointsArray(num);

        //
        // create drawing element groups
        //

        SGElementGroup group = null;
        NodeList nList = null;

        // bar
        nList = el.getElementsByTagName(SGIBarConstants.TAG_NAME_BAR);
        if (nList.getLength() != 1) {
            return ic;
        }
//        if (groupSet
//                .addDrawingElementGroup(SGIElementGroupConstants.RECTANGLE_GROUP) == false) {
//            return ic;
//        }
        Element bar = (Element) nList.item(0);
        group = groupSet.getBarGroup();
        if (group.readProperty(bar) == false) {
            return ic;
        }

        // code for the versions older than 0.9.1
        SGElementGroupBar groupBar = (SGElementGroupBar) group;
        final float rectWidth = groupBar.getRectangleWidth();
        if (rectWidth != 0.0f) {
            SGAxis axisX = groupSet.getXAxis();
            final double max = axisX.getMaxValue();
            final double min = axisX.getMinValue();
            final float ratio = rectWidth / this.mGraphRectWidth;
            double value = (max - min) * ratio;
            value = this.getNumberInRangeOrder(value, axisX);
            groupBar.setWidthValue(value);
        }

        // error bar
        nList = el
                .getElementsByTagName(SGIErrorBarConstants.TAG_NAME_ERROR_BAR);

        // the property file has error bar info
        if (nList.getLength() == 1) {
            // if data has error bars
            if (data.isErrorBarAvailable()) {
//                SGElementGroupErrorBar eGroup = (SGElementGroupErrorBar) groupSet
//                        .createErrorBars(data);
                SGElementGroupErrorBar eGroup = groupSet.getErrorBarGroup();
                Element errorBar = (Element) nList.item(0);
                if (eGroup.readProperty(errorBar) == false) {
                    return ic;
                }

//                if (eGroup.setPropertiesOfDrawingElements() == false) {
//                    return ic;
//                }

                eGroup.updateLocation();

            }
        } else if (nList.getLength() == 0) {
//            // if data has error bars
//            if (data.isErrorBarAvailable()) {
//                SGElementGroupErrorBar eGroup = (SGElementGroupErrorBar) groupSet
//                        .createErrorBars(data);
//                SGElementGroupErrorBar eGroup = groupSet.getErrorBarGroup();
//                if (eGroup.setPropertiesOfDrawingElements() == false) {
//                    return ic;
//                }
//            }
        } else {
            return ic;
        }

        // line
        nList = el.getElementsByTagName(SGILineConstants.TAG_NAME_LINE);
        if (nList.getLength() != 1) {
            return ic;
        }
//        if (groupSet
//                .addDrawingElementGroup(SGIElementGroupConstants.POLYLINE_GROUP) == false) {
//            return ic;
//        }
        Element line = (Element) nList.item(0);
        group = groupSet.getLineGroup();
        if (group.readProperty(line) == false) {
            return ic;
        }

        // symbol
        nList = el.getElementsByTagName(SGISymbolConstants.TAG_NAME_SYMBOL);
        if (nList.getLength() != 1) {
            return ic;
        }
//        if (groupSet
//                .addDrawingElementGroup(SGIElementGroupConstants.SYMBOL_GROUP) == false) {
//            return ic;
//        }
        Element symbol = (Element) nList.item(0);
        group = groupSet.getSymbolGroup();
        if (group.readProperty(symbol) == false) {
            return ic;
        }

        // tick label
        nList = el
                .getElementsByTagName(SGITickLabelConstants.TAG_NAME_TICK_LABELS);

        // the property file has tick label info
        if (nList.getLength() == 1) {
            // if data has tick labels
            if (data.isTickLabelAvailable()) {
//                SGElementGroupTickLabel tGroup = (SGElementGroupTickLabel) groupSet
//                        .createTickLabels(data);
                SGElementGroupTickLabel tGroup = groupSet.getTickLabelGroup();
                Element tickLabel = (Element) nList.item(0);
                if (tGroup.readProperty(tickLabel) == false) {
                    return ic;
                }
//                if (tGroup.setPropertiesOfDrawingElements() == false) {
//                    return ic;
//                }

                // update tick label strings
                groupSet.updateTickLabelStrings();
                
                // update the location
                tGroup.updateLocation();
            }
        } else if (nList.getLength() == 0) {
//            // if data has tick labels
//            if (data.isTickLabelAvailable()) {
////                SGElementGroupTickLabel tGroup = (SGElementGroupTickLabel) groupSet
////                        .createTickLabels(data);
//                SGElementGroupTickLabel tGroup = groupSet.getTickLabelGroup();
//                if (tGroup.setPropertiesOfDrawingElements() == false) {
//                    return ic;
//                }
//            }
        } else {
            return ic;
        }

        return SGIConstants.SUCCESSFUL_COMPLETION;
    }

    protected int setPropertyOfElementGroupSetForMultipleSXY(
            final Element el, final SGIElementGroupSetMultipleSXY groupSet,
            final SGISXYTypeMultipleData data) {

        final int ic = SGIConstants.PROPERTY_FILE_INCORRECT;

        SGISXYTypeData[] dataArray = data.getSXYDataArray();
        try {
            String name = groupSet.getName();
            SGAxis axisX = groupSet.getXAxis();
            SGAxis axisY = groupSet.getYAxis();
            
            SGIElementGroupSetSXY gsDummy = (SGIElementGroupSetSXY) this.createGroupSet(
                    el, (SGData) dataArray[0], axisX, axisY, name, false);
            if (gsDummy == null) {
                return ic;
            }
            SGProperties gp = gsDummy.getProperties();

            SGProperties[] dataProperties = new SGProperties[dataArray.length];
            for (int ii = 0; ii < dataArray.length; ii++) {
    			SGData d = (SGData) dataArray[ii];
    			dataProperties[ii] = d.getProperties();
            }
            
            // set up child group sets
            SGIElementGroupSetInFigureElement[] childArray = groupSet.getChildGroupSetArray();
            for (int ii = 0; ii < dataArray.length; ii++) {
                SGIElementGroupSetSXY gs = (SGIElementGroupSetSXY) childArray[ii];
                if (gs.setProperties(gp) == false) {
                    return ic;
                }
                SGElementGroupSetInFigureElement fgs = (SGElementGroupSetInFigureElement) gs;
                if (fgs.setDataProperties(dataProperties[ii]) == false) {
                	return ic;
                }
                gs.setName(name);
            }

            // set the direction of error bars
            Boolean errorBarVertical = data.isErrorBarVertical();
            if (errorBarVertical != null) {
                for (int ii = 0; ii < dataArray.length; ii++) {
                    SGIElementGroupSetSXY gs = (SGIElementGroupSetSXY) childArray[ii];
                    gs.setErrorBarDirection(errorBarVertical.booleanValue());
                }
            }

            // set the alignment of tick labels
            Boolean tickLabelHorizontal = data.isTickLabelHorizontal();
            if (tickLabelHorizontal != null) {
                for (int ii = 0; ii < dataArray.length; ii++) {
                    SGIElementGroupSetSXY gs = (SGIElementGroupSetSXY) childArray[ii];
                    gs.setTickLabelAlignment(tickLabelHorizontal.booleanValue());
                }
            }

            // initialize the history
            for (int ii = 0; ii < dataArray.length; ii++) {
                SGIElementGroupSetSXY gs = (SGIElementGroupSetSXY) childArray[ii];
                gs.initPropertiesHistory();
            }
            
            // set the properties of element groups
            SGIElementGroupSetInFigureElement first = groupSet.getFirst();
            List elpList = first.getElementGroupPropertiesList();
            if (groupSet.setElementGroupListProperties(elpList) == false) {
                return ic;
            }
        	
        } finally {
        	
        	if (dataArray != null) {
                // disposes temporary data objects
                for (int ii = 0; ii < dataArray.length; ii++) {
                	dataArray[ii].dispose();
                }
        	}
        }
        
        return SGIConstants.SUCCESSFUL_COMPLETION;
    }    
    
    protected int setPropertyOfElementGroupSetVXY(final Element el,
            final SGIElementGroupSetVXY groupSet,
            final SGIVXYTypeData data) {

        final int ic = SGIConstants.PROPERTY_FILE_INCORRECT;
        // SGIFigureElementAxis aElement = this.mAxisElement;
        String str = null;
        Number num = null;
        Boolean b = null;

        //
        // create drawing element groups
        //

        SGElementGroup group = null;
        NodeList nList = null;

//        // create instances of the points
//        final int pointsNum = data.getPointsNumber();
//        groupSet.initPointsArray(pointsNum);

        // arrow
        nList = el.getElementsByTagName(SGIArrowConstants.TAG_NAME_ARROW);
        if (nList.getLength() != 1) {
            return ic;
        }
//        if (groupSet
//                .addDrawingElementGroup(SGIElementGroupConstants.ARROW_GROUP) == false) {
//            return ic;
//        }
        Element arrow = (Element) nList.item(0);
        group = groupSet.getArrowGroup();
        if (group.readProperty(arrow) == false) {
            return ic;
        }

        // magnitude
        str = el.getAttribute(SGIFigureElementGraph.KEY_MAGNITUDE_PER_CM);
        if (str.length() != 0) {
            num = SGUtilityText.getFloat(str);
            if (num == null) {
                return ic;
            }
            groupSet.setMagnitudePerCM(num.floatValue());
        }

        // direction invariance
        str = el.getAttribute(SGIFigureElementGraph.KEY_DIRECTION_INVARIANT);
        if (str.length() != 0) {
            b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return ic;
            }
            groupSet.setDirectionInvariant(b.booleanValue());
        }

//        groupSet.updateDrawingElementsLocation(data);

        return SGIConstants.SUCCESSFUL_COMPLETION;
    }

    protected int setPropertyOfElementGroupSetSXYZ(final Element el,
            final SGIElementGroupSetSXYZ groupSet,
            final SGISXYZTypeData data) {

        final int ic = SGIConstants.PROPERTY_FILE_INCORRECT;
//        String str = null;
//        Number num = null;
//        Boolean b = null;

        //
        // create drawing element groups
        //

        SGElementGroup group = null;
        NodeList nList = null;

        // color map
        nList = el.getElementsByTagName(SGIColorMapConstants.TAG_NAME_COLOR_MAP);
        if (nList.getLength() == 0) {
            // for backward compatibility
            nList = el.getElementsByTagName(SGIColorMapConstants.TAG_NAME_GRID_COLOR_MAP);
        }
        if (nList.getLength() != 1) {
            return ic;
        }
        Element colorMap = (Element) nList.item(0);
        group = groupSet.getColorMap();
        if (group.readProperty(colorMap) == false) {
            return ic;
        }

        return SGIConstants.SUCCESSFUL_COMPLETION;
    }

    /**
     * Sets the dialog owner this figure element.
     * 
     * @param frame
     *            the dialog owner
     * @return true if succeeded
     */
    public boolean setDialogOwner(final Frame frame) {
        if (super.setDialogOwner(frame) == false) {
            return false;
        }
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (mDataDialogMap != null) {
                    if (mDataDialogMap.get(DIALOG_KEY_SXY) == null) {
                        mDataDialogMap.put(DIALOG_KEY_SXY, new SGPropertyDialogSXYData(
                                mDialogOwner, true));
                    }
                    if (mDataDialogMap.get(DIALOG_KEY_VXY) == null) {
                        mDataDialogMap.put(DIALOG_KEY_VXY, new SGPropertyDialogVXYData(
                                mDialogOwner, true));
                    }
                }
                if (mAnimationDialog == null) {
                    mAnimationDialog = new SGDataAnimationConfigurationDialog(mDialogOwner, true);
                }
            }
        });
        return true;
    }

    /**
     * 
     * @param data
     * @return
     */
    public boolean isDataChanged(final SGData data) {
        SGElementGroupSetInFigureElement gs = this.getElementGroupSet(data);
        return gs.isChanged();
    }

    /**
     * Synchronize the data properties with given data properties.
     * 
     * @param p
     *          the properties
     * @param dp
     *          data properties
     * @return synchronized properties
     */
    public SGProperties synchronizeDataProperties(SGProperties p, SGProperties dp) {
        if ((p instanceof ElementGroupSetPropertiesInFigureElement) == false) {
            return null;
        }
        if ((dp instanceof DataProperties) == false) {
            return null;
        }
        ElementGroupSetPropertiesInFigureElement gp = (ElementGroupSetPropertiesInFigureElement) p.clone();
        gp.dataProperties = (DataProperties) dp.clone();
        gp.labelStringList.clear();
        if (gp instanceof MultipleElementGroupSetProperties) {
        	MultipleElementGroupSetProperties mgp = (MultipleElementGroupSetProperties) gp;
        	ElementGroupSetPropertiesInFigureElement first = (ElementGroupSetPropertiesInFigureElement) mgp.mChildPropertyList.get(0);
        	first.dataProperties = (DataProperties) dp.clone();
        	mgp.mChildPropertyList.clear();
        	mgp.mChildPropertyList.add(first);
        }
        return gp;
    }

    /**
     * Returns a list of labels for data objects.
     * 
     * @return a list of labels for data objects 
     */
    public List<DataLabel> getDataLabelList() {
        List<SGData> dList = this.getFocusedDataList();
        List<DataLabel> list = new ArrayList<DataLabel>();
        for (int ii = 0; ii< dList.size(); ii++) {
            SGData data = dList.get(ii);
            SGElementGroupSetInFigureElement gs = this.getElementGroupSet(data);
            DataLabel dl = gs.getDataLabel();
            if (dl != null) {
                list.add(dl);
            } else {
                if (list.size() == 0) {
                    SGUtility.showErrorMessageDialog(this.mDialogOwner, 
                            gs.getName() + ": " + "Coordinate variables with fixed values do not exist.", 
                            SGIConstants.ERROR);
                }
            }
        }
        return list;
    }
    
    /**
     * Hide the selected objects.
     * @return
     *         true if selected
     */
    @Override
    public boolean hideSelectedObjects() {
        ArrayList listBefore = this.getFocusedObjectsList();
        if (listBefore.size()>0) {
            this.notifyToListener(NOTIFY_DATA_WILL_BE_HIDDEN);
        }
        if (super.hideSelectedObjects() == false) {
            return false;
        }
        ArrayList listAfter = this.getFocusedObjectsList();
        if (listBefore.size() != listAfter.size()) {
            this.notifyToListener(NOTIFY_DATA_HIDDEN);
        }
        return true;
    }

    /**
     * Hide the selected data objects.
     * @return
     *         true if selected
     */
    protected boolean hideSelectedData() {
        ArrayList listBefore = this.getFocusedObjectsList();
        if (listBefore.size()>0) {
            this.notifyToListener(NOTIFY_DATA_WILL_BE_HIDDEN);
        }
        if (super.hideSelectedData() == false) {
            return false;
        }
        ArrayList listAfter = this.getFocusedObjectsList();
        if (listBefore.size() != listAfter.size()) {
            this.notifyToListener(NOTIFY_DATA_HIDDEN);
        }
        return true;
    }

    /**
     * A dialog to setup the animation of data objects.
     */
    protected SGDataAnimationConfigurationDialog mAnimationDialog = null;
    
    /**
     * Do the animation for a given group set.
     * 
     * @param gs
     *          a group set to do animation
     */
    protected void doAnimation(SGElementGroupSetInFigureElement gs) {
        SGData data = gs.getData();
        if (data instanceof SGNetCDFData == false) {
            throw new Error("Not supported data type: " + data.getDataType());
        }
        SGNetCDFData nData = (SGNetCDFData) data;
        if (nData.isTimeVariableAvailable()) {
            this.mAnimationDialog.setAnimation(gs);
            this.mAnimationDialog.setCenter(this.mDialogOwner);
            this.mAnimationDialog.setVisible(true);
        } else {
            SGUtility.showErrorMessageDialog(this.mDialogOwner, 
                    "Time variable is not selected.", "Error");
        }
    }

    /**
     * Returns the list of axis information of a given data.
     * 
     * @param data
     *           a data object
     * @return the list of axis information of a given data
     */
    public List<SGDataAxisInfo> getAxisInfoList(SGData data) {
        SGElementGroupSetInFigureElement gs = this.getElementGroupSet(data);
        SGAxis axis = null;
        SGValueRange bounds = null;
        String title = null;
        List<SGDataAxisInfo> aList = new ArrayList<SGDataAxisInfo>();
        if (data instanceof SGITwoDimensionalData) {
            SGITwoDimensionalData tData = (SGITwoDimensionalData) data;
            axis = gs.getXAxis();
            if (axis != null) {
                bounds = tData.getBoundsX();
                title = tData.getTitleX();
                this.addAxisInfo(data, axis, bounds, title, aList);
            }
            axis = gs.getYAxis();
            if (axis != null) {
                bounds = tData.getBoundsY();
                title = tData.getTitleY();
                this.addAxisInfo(data, axis, bounds, title, aList);
            }
        } else if (data instanceof SGIThreeDimensionalData) {
            SGIThreeDimensionalData tData = (SGIThreeDimensionalData) data;
            axis = gs.getXAxis();
            if (axis != null) {
                bounds = tData.getBoundsX();
                title = tData.getTitleX();
                this.addAxisInfo(data, axis, bounds, title, aList);
            }
            axis = gs.getYAxis();
            if (axis != null) {
                bounds = tData.getBoundsY();
                title = tData.getTitleY();
                this.addAxisInfo(data, axis, bounds, title, aList);
            }
            axis = gs.getZAxis();
            if (axis != null) {
                bounds = tData.getBoundsZ();
                title = tData.getTitleZ();
                this.addAxisInfo(data, axis, bounds, title, aList);
            }
        } else {
            throw new IllegalArgumentException("Unsupported data type: " + data.getDataType());
        }
        return aList;
    }
    
    private void addAxisInfo(final SGData data, final SGAxis axis, 
    		final SGValueRange bounds, 
            final String title, final List<SGDataAxisInfo> aList) {
        final int location = this.mAxisElement.getLocationInPlane(axis);
        SGDataAxisInfo info = new SGDataAxisInfo(data, axis, bounds, title, location);
        aList.add(info);
    }
    
    
//    protected SGElementGroupSetInFigureElement getGroupSet(final int id) {
//        SGElementGroupSetInFigureElement el = (SGElementGroupSetInFigureElement) this
//                .getVisibleChild(id);
//        if (el == null) {
//            return null;
//        }
//        return el;
//    }

    /**
     * Returns the data of a given ID.
     * 
     * @param id
     *           the ID of data
     * @return the data object if it exists
     */
    public SGData getData(final int id) {
        SGIChildObject c = this.getVisibleChild(id);
        if (c!=null) {
            SGElementGroupSetInFigureElement gs = (SGElementGroupSetInFigureElement)c;
            return gs.getData();
        }
        
        SGElementGroupSetInFigureElement gs = (SGElementGroupSetInFigureElement) this
                .getChild(id);
        if (gs == null) {
            return null;
        }
        return gs.getData();
    }

    /**
     * Sets the properties of child object.
     * 
     * @param id
     *           the ID of child object
     * @param map
     *           a map properties
     * @return the result of setting properties
     */
    public SGPropertyResults setChildProperties(final int id, SGPropertyMap map) {
        
        SGIChildObject child = this.getVisibleChild(id);
        if (child == null) {
            // returns null when data object is not found
            return null;
        }

        // set properties to the child object
        SGPropertyResults result = child.setProperties(map);

        return result;
    }

	/**
	 * Shows the pop-up menu for a group set.
	 * @param groupSet
	 *                 a group set
	 * @param x
	 *                 x coordinate
	 * @param y
	 *                 y coordinate
	 */
	protected void showDataPopupMenu(SGElementGroupSetInFigureElement groupSet, int x,
			int y) {
		groupSet.createGroupSetPopupMenu().show(this.getComponent(), x, y);
	}

    /**
     * Overrode not to call other "addData" method.
     * 
     * @param data
     *             added data.
     * @param name
     *             the name set to the data
     * @param p
     *             properties set to be data.
     * @return true if succeeded
     */
    public boolean addData(final SGData data, final String name,
            final SGProperties p) {
        if (data == null) {
            throw new Error("data==null");
        }
        // just add to the list
        this.mDataList.add(data);
        return true;
    }

    /**
     * Removes a data object.
     * 
     * @param data
     *             data to be removed
     * @return true if succeeded
     */
    public boolean removeData(SGData data) {
    	if (super.removeData(data) == false) {
    		return false;
    	}
    	for (int ii = 0; ii < this.mChildList.size(); ii++) {
            SGElementGroupSetInFigureElement groupSet = (SGElementGroupSetInFigureElement) this.mChildList.get(ii);
            if (data.equals(groupSet.getData())) {
                if (groupSet.setData(null) == false) {
                	return false;
                }
            }
    	}
    	return true;
    }

}
